﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <nn/pia/clone/clone_SharingClone.h>

namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  無制限に送信のできる双方向の送受信を行う CloneBase 派生クラスです。

    @attention  SequentialSharingClone には EventCloneElement, reckoning::ReckoningCloneElement を登録することはできません。
 */
class SequentialSharingClone : public SharingClone
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    SequentialSharingClone();


    /*!
    @brief デストラクタです。
     */
    virtual ~SequentialSharingClone();


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;


    //! @cond PRIVATE

public:
    static Type GetTypeStatic()
    {
        return Type_Sequential;
    }
    static Key ToKey(Id id)
    {
        return CloneBase::ToKey(Type_Sequential, StationIndex_Invalid, id);
    }
    virtual Type GetTypeVirtual() const
    {
        return GetTypeStatic();
    }

public:
    virtual bool IsReadyToSetValue() const;

protected:
public:
    template <int TEST_ID, typename Arg>
    static void Test(const SequentialSharingClone& obj, Arg* pArg = NULL);

    NN_PIA_DISALLOW_COPY(SequentialSharingClone);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
