﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#if 0
#pragma once

#include <nn/pia/clone/clone_Definitions.h>
#include <memory>

namespace nn
{
namespace pia
{
namespace clone
{


/*!
    @brief  元のデータ構造そのままにシリアライズします。

    @tparam Type_  シリアライズ対象のデータ型です。
 */
template <typename Type_>
class StraightSerializePolicy
{
public:
    /*!
    @brief  シリアライズ対象のデータ型です。
     */
    typedef Type_ Type;


    /*!
    @cond PRIVATE
    @brief  シリアライズします。
    @param[out] pBuffer 出力先バッファです。
    @param[in]  value   シリアライズ対象のデータです。
     */
    static void Serialize(void* pBuffer, const Type& value)
    {
#if NN_PIA_CAFE
        if (reinterpret_cast<uint32_t>(pBuffer) % __alignof(Type) != 0)
        {
            OSBlockMove(pBuffer, &value, sizeof(Type), FALSE);
        }
        else
        {
            *reinterpret_cast<Type*>(pBuffer) = value;
        }
#elif NN_PIA_CTR
        nn::nstd::MemCpy(pBuffer, &value, sizeof(Type));
#elif NN_PIA_NINTENDOSDK
        memcpy(pBuffer, &value, sizeof(Type));
#else
        *reinterpret_cast<Type*>(pBuffer) = value;
#endif
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  デシリアライズします。
    @param[out] pValue  デシリアライズされたデータの保存先です。
    @param[in]  cpData  デシリアライズするバイト列です。
     */
    static void Deserialize(Type* pValue, const void* cpData)
    {
#if NN_PIA_CAFE
        if (reinterpret_cast<uint32_t>(cpData) % __alignof(Type) != 0)
        {
            OSBlockMove(pValue, cpData, sizeof(Type), FALSE);
        }
        else
        {
            *pValue = *reinterpret_cast<const Type*>(cpData);
        }
#elif NN_PIA_CTR
        nn::nstd::MemCpy(pValue, cpData, sizeof(Type));
#elif NN_PIA_NINTENDOSDK
        memcpy(pValue, cpData, sizeof(Type));
#else
        *pValue = *reinterpret_cast<const Type*>(cpData);
#endif
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  シリアライズされたサイズを取得します。
    @details    @ref nn::pia::clone::CloneProtocol::GetElementSizeMax() "CloneProtocol::GetElementSizeMax()" より大きな値は使用できません。
    @return シリアライズされたサイズです。
     */
    static uint32_t GetSize()
    {
        return sizeof(Type);
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  シリアライズされたサイズです。
    @details    @ref nn::pia::clone::ReliableLargeCloneElement で使用するときは @ref GetSize() ではなくこちらの値が使われます。
     */
    static const uint32_t Size = sizeof(Type);
    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
#endif
