﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Definitions.h>
#include <nn/pia/common/common_RootObject.h>

#if NN_PIA_CAFE
#include <sdk_ver.h>

#if CAFE_OS_SDK_VERSION >= 21002
#define NN_PIA_ENABLE_FAST_MUTEX 1
#else
#define NN_PIA_ENABLE_FAST_MUTEX 0
#endif

#elif NN_PIA_A || NN_PIA_B
#include <pthread.h>
#endif

#define PIA_CRITICAL_SECTION_LOCK_CHECKER_ENABLE NN_PIA_UNDER_DEVELOP


namespace nn
{
namespace pia
{
namespace common
{

/*!
@cond PRIVATE
  @brief  CriticalSection のラッパーです。
 */
class CriticalSection : public common::RootObject
{
public:
    /*!
    @brief コンストラクタです。
    @param[in] lockOrder ロック順の値です。CriticalSectionLockChecker::LockOrder の値を指定する必要があります。
     */
    explicit CriticalSection(int lockOrder = -1);


    /*!
    @brief destructorです。
     */
    ~CriticalSection();


    /*!
    @brief    ロックします。
     */
    void Lock();


    /*!
    @brief    ロックを試みます。
    @return   ロックに成功すると true を返します。
     */
    bool TryLock();


    /*!
    @brief    ロックを開放します。
     */
    void Unlock();


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;


/*!
    @brief ロック順取得
     */
#if PIA_CRITICAL_SECTION_LOCK_CHECKER_ENABLE
    int GetLockOrder() const
    {
        return m_LockOrder;
    }
#endif


private:
#if NN_PIA_NINTENDOSDK
    nn::os::MutexType m_MutexInner;
#elif NN_PIA_CTR
    nn::os::CriticalSection m_CriticalSection;
#elif NN_PIA_WIN
    CRITICAL_SECTION m_CriticalSection;
#elif NN_PIA_CAFE
#if NN_PIA_ENABLE_FAST_MUTEX
    OSFastMutex m_FastMutex;
#else
    OSMutex m_Mutex;
#endif
#elif NN_PIA_A || NN_PIA_B
    void initialize_pthread(pthread_mutex_t* p_pthread);
    pthread_mutex_t mMutexInner;
#else
#error "invalid platform"
#endif

#if PIA_CRITICAL_SECTION_LOCK_CHECKER_ENABLE
    int m_LockOrder;
#endif
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
