﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @cond PRIVATE
  @brief  暗号/復号を行うするクラスです。

 */
class Crypto
{
public:
    /*!
    @brief 暗号アルゴリズムの種類を表します。
     */
    enum Mode
    {
        Mode_Nothing = 0,  //!< 無し
        Mode_Aes128        //!< AES-128
    };

    struct Setting
    {
        Mode mode;              //!< 暗号アルゴリズム。
        const void* cpKeyData;  //!< 暗号鍵データのアドレス。
        uint32_t keySize;       //!< 暗号鍵のサイズ
    };

    /*!
    @brief  指定されている設定でデータを暗号化します。
    @return 
     */
    static Result Encrypt(void* pDst, const void* pSrc, uint32_t srcSize, const Setting& setting);

    /*!
    @brief  指定されている設定でデータを復号します。
    @return 
     */
    static Result Decrypt(void* pDst, const void* pSrc, uint32_t srcSize, const Setting& setting);

    /*!
    @brief  暗号アルゴリズムの種類に応じたブロックサイズを返します。
    @return 
     */
    static int GetBlockSize(Mode mode);

private:
};
}
}
}  // end of namespace nn::pia::common

//! @endcond
