﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_Crypto.h>

namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief  暗号化の設定構造体です。

 */
struct CryptoSetting
{
public:
    /*!
    @brief デフォルトコンストラクタです。
    @details 暗号アルゴリズムは暗号なし、暗号鍵はゼロに初期化されます。 
     */
    CryptoSetting();


public:
    /*!
    @brief 暗号アルゴリズムの種類を表します。
     */
    enum Mode
    {
        Mode_Nothing = Crypto::Mode_Nothing,  //!< 無し
        Mode_Aes128 = Crypto::Mode_Aes128     //!< AES-128
    };

    /*!
    @brief  鍵のサイズです。
     */
    static const uint32_t KeySize = 16;

    /*!
    @brief  暗号アルゴリズムの種類です。
     */
    Mode m_Mode;

    /*!
    @brief  暗号鍵です。
    @details    暗号なし（Mode_Nothing）の場合は設定不要です。
     */
    uint8_t m_Key[KeySize];
};
}
}
}  // end of namespace nn::pia::common
