﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>


namespace nn
{
namespace pia
{
namespace common
{

/*!
@cond PRIVATE
  @brief  StringStreamと同等のインタフェースを持ったクラスです。ただし実質的な動作は何もしません。Warning封じが目的です。
 */
class EmptyStream
{
public:
    EmptyStream(void)
    {
    }
    EmptyStream(uint64_t)
    {
    }  // フラグを受け取るだけのダミー的コンストラクタ
    ~EmptyStream(void)
    {
    }

    uint32_t GetLength() const
    {
        return 0;
    }
    uint32_t GetSize() const
    {
        return 0;
    }
    const char* CStr() const
    {
        return NULL;
    }

    EmptyStream& operator<<(const char*)
    {
        return *this;
    }
    EmptyStream& operator<<(const void*)
    {
        return *this;
    }
    EmptyStream& operator<<(bool)
    {
        return *this;
    }
    EmptyStream& operator<<(double)
    {
        return *this;
    }
    EmptyStream& operator<<(float)
    {
        return *this;
    }

    EmptyStream& operator<<(uint8_t)
    {
        return *this;
    }
    EmptyStream& operator<<(int8_t)
    {
        return *this;
    }
    EmptyStream& operator<<(uint16_t)
    {
        return *this;
    }
    EmptyStream& operator<<(int16_t)
    {
        return *this;
    }
    EmptyStream& operator<<(uint32_t)
    {
        return *this;
    }
    EmptyStream& operator<<(int32_t)
    {
        return *this;
    }

    EmptyStream& operator<<(uint64_t)
    {
        return *this;
    }
    EmptyStream& operator<<(int64_t)
    {
        return *this;
    }

    EmptyStream& operator<<(const EmptyStream&)
    {
        return *this;
    }

    // Manipulators
    EmptyStream& operator<<(EmptyStream& (*)(EmptyStream&))
    {
        return *this;
    }

    EmptyStream& Manip_hex()
    {
        return *this;
    }
    EmptyStream& Manip_dec()
    {
        return *this;
    }
    EmptyStream& Manip_zerohex()
    {
        return *this;
    }


    EmptyStream& Manip_showbase()
    {
        return *this;
    }
    EmptyStream& Manip_noshowbase()
    {
        return *this;
    }
    EmptyStream& Manip_boolalpha()
    {
        return *this;
    }
    EmptyStream& Manip_noboolalpha()
    {
        return *this;
    }

    const char& operator[](uint32_t) const;  // not implemented.

private:
    EmptyStream(const EmptyStream&);
    EmptyStream& operator=(const EmptyStream&);
};


inline EmptyStream& hex(EmptyStream& s)
{
    return s;
}
inline EmptyStream& dec(EmptyStream& s)
{
    return s;
}
inline EmptyStream& endl(EmptyStream& s)
{
    return s;
}
inline EmptyStream& showbase(EmptyStream& s)
{
    return s;
}
inline EmptyStream& noshowbase(EmptyStream& s)
{
    return s;
}
inline EmptyStream& boolalpha(EmptyStream& s)
{
    return s;
}
inline EmptyStream& noboolalpha(EmptyStream& s)
{
    return s;
}

inline EmptyStream& zerohex(EmptyStream& s)
{
    return s;
}

//! @endcond
}
}
}  // end of namespace nn::pia::common
