﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_ObjList.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @cond PRIVATE
  @brief  バッファを内部に持つタイプのObjListです。

  @param T    管理するオブジェクトの型。
  @param N    リストに格納できる最大の要素数。
  */
template <typename T, uint32_t N>
class FixedObjList : public nn::pia::common::ObjList<T>
{
public:
    /**
    @brief  デフォルトコンストラクタです。
     */
    FixedObjList()
        : ObjList<T>()
    {
        SetBuffer(N, mBuffer);
    }

private:
    // SetBufferは外から使えなくする
    void SetBuffer(uint32_t limitNum, void* pBuffer)
    {
        ObjList<T>::SetBuffer(limitNum, pBuffer);
    }

private:
    uint8_t mBuffer[ObjList<T>::c_NodeSize * N];

    NN_PIA_DISALLOW_COPY(FixedObjList);
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
