﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

namespace nn
{
namespace pia
{
namespace common
{

class Packet;

/*!
@cond PRIVATE
  @brief  Packet を読み取るインタフェースとなるクラスです。
 */
class IPacketInput
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    IPacketInput(void)
    {
    }


    /*!
    @brief デストラクタです。
     */
    virtual ~IPacketInput(void)
    {
    }


    /*!
    @brief 読み取りインタフェースです。

    @param[out] pPacketData Packetインスタンスへのポインタ。
    @return     読み取りに成功すれば、IsSuccess()がtrueを返すResultが返ります。
    @retval     ResultInvalidArgument  引数が誤っています（NULLポインタが渡されたなど）
    @retval     ResultInvalidState     このAPIを呼び出すタイミングが誤っています（初期化処理が済んでいないなど）
    @retval     ResultNoData           受信データが存在しなかった（データ未到着）ことを表します。
    @retval     ResultNetworkConnectionIsLost ネットワーク接続が失われた状態を示します。
     */
    virtual Result Read(nn::pia::common::Packet* pPacketData) = 0;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
