﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

#include <nn/pia/common/private/common_Packet.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
@cond PRIVATE
  @brief  Packet を書き込むインタフェースとなるクラスです。
 */
class IPacketOutput
{
public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    IPacketOutput(void)
    {
    }


    /*!
    @brief デストラクタです。
     */
    virtual ~IPacketOutput(void)
    {
    }

    virtual Result OnStationConnectionEvent() = 0;

    /*!
    @brief 書き込みインタフェースです。

    @param[in] packetData 書き込むPacket。
    @return    書き込みに成功すれば、IsSuccess()がtrueを返すResultが返ります。
    @retval    ResultInvalidArgument  引数が誤っています（packetの宛先がおかしいなど）
    @retval    ResultInvalidState     このAPIを呼び出すタイミングが誤っています（初期化処理が済んでいないなど）
    @retval    ResultBufferIsFull     内部バッファの一時的な枯渇や処理能力の限界により、送信が失敗したことを示します。
    @retval    ResultNetworkConnectionIsLost ネットワーク接続が失われた状態を示します。
     */
    virtual Result Write(const nn::pia::common::Packet& packetData) = 0;


    /*!
    @brief   マルチキャストで送信できる宛先の最大数を取得します。
    @details 宛先ビットマップで指定される宛先の数がこの値以下になるようにパケット生成を制御します。
    @return  マルチキャストで送信できる宛先の最大数です。
     */
    virtual uint32_t GetMulticastDestinationNum() const
    {
        return 1;
    }


    /*!
    @brief   ブロードキャストをサポートしているかを取得します。
    @details これが true であれば、StationIndex_Allで GetMulticastDestinationNum() 以上のビットの立ったパケット生成をすることがあります。
    @return  ブロードキャストをサポートしていれば true を返します。
     */
    virtual bool IsBroadcastSupported() const
    {
        return false;
    }
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
