﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_ListNode.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @cond PRIVATE
  @brief  リストの基底クラスです。
 */
class ListBase : public nn::pia::common::RootObject
{
public:
    /*!
    @brief  デフォルトコンストラクタです。
     */
    ListBase(void)
    {
        Init();
    }


    /*!
    @brief  リストに含まれている要素数を取得します。
    @return リストに含まれている要素数。
     */
    uint32_t Size() const
    {
        return m_Size;
    }


    /*!
    @brief  リストが空かどうか判定します。
    @return リストが空ならtrue。
     */
    bool IsEmpty() const
    {
        return (Size() == 0);
    }


protected:
    void Init();

    void InitListNode(ListNode* pNode)
    {
        pNode->InitListNode();
    }

    void ClearNode();

    void InsertBeforeNode(ListNode* pBasis, ListNode* pNode);

    void InsertAfterNode(ListNode* pBasis, ListNode* pNode);

    void EraseNode(ListNode* pNode);

    bool IsIncludeNode(const ListNode* pNode) const;


    void PushFrontNode(ListNode* pNode)
    {
        InsertAfterNode(&m_Terminator, pNode);
    }

    void PushBackNode(ListNode* pNode)
    {
        InsertBeforeNode(&m_Terminator, pNode);
    }

    ListNode* PopFrontNode();

    ListNode* PopBackNode();


    ListNode* FrontNode() const
    {
        return (m_Size > 0) ? m_Terminator.m_pNext : NULL;
    }

    ListNode* BackNode() const
    {
        return (m_Size > 0) ? m_Terminator.m_pPrev : NULL;
    }

    void RotateNode(ListNode* pNode);


    const ListNode* Terminator() const
    {
        return &m_Terminator;
    }

private:
    // コピー禁止
    ListBase(const ListBase& rhs);
    ListBase& operator=(const ListBase& rhs);

private:
    ListNode m_Terminator;
    uint32_t m_Size;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
