﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>


namespace nn
{
namespace pia
{
namespace common
{

class ListBase;

/*!
  @cond PRIVATE
  @brief  リストのノードとなるクラスです。
 */
class ListNode : public nn::pia::common::RootObject
{
    friend class ListBase;

public:
    /*!
    @brief デフォルトコンストラクタです。
     */
    ListNode()
    {
        InitListNode();
    }


    /*!
    private:
    @brief  次の要素を取得します。
    @return 次の要素。
    */
    ListNode* Prev() const
    {
        return m_pPrev;
    }


    /*!
    private:
    @brief  前の要素を取得します。
    @return 前の要素。
    */
    ListNode* Next() const
    {
        return m_pNext;
    }


    bool IsFreeListNode() const
    {
        return (m_pPrev == NULL) || (m_pNext == NULL);
    }

private:
    void InitListNode()
    {
        m_pPrev = NULL;
        m_pNext = NULL;
    }

    ListNode* m_pPrev;
    ListNode* m_pNext;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
