﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_Trace.h>

#if NN_PIA_ENABLE_TRACE

/*!
@brief  Result の内容を常にトレース出力します。

@param  result  Result の値。
 */
#define PIA_TRACE_RESULT_ALWAYS(result)                                                                              \
    \
{                                                                                                             \
        PIA_TRACE(nn::pia::common::TraceFlagAlways, "%s, line: %d", PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_LINE); \
        nn::pia::common::ResultUtility::TraceResult(result);                                                         \
    \
}


/*!
@brief  Result の内容をトレース出力します。

@param  flag    トレースフラグ。
@param  result  Result の値。
 */
#define PIA_TRACE_RESULT(flag, result)                                                   \
    \
{                                                                                 \
        PIA_TRACE(flag, "%s, line: %d", PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_LINE); \
        nn::pia::common::ResultUtility::TraceResult(flag, result);                       \
    \
}


#else  // else of NN_PIA_ENABLE_TRACE

#define PIA_TRACE_RESULT_ALWAYS(result) (void)(result)
#define PIA_TRACE_RESULT(flag, result) (void)(flag), (void)(result)

#endif  // end of NN_PIA_ENABLE_TRACE

namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief      Result に関係するユーティリティ類をまとめたクラスです。
  
 */
class ResultUtility
{
public:
    /*!
    @brief                nn::pia::Result 値の詳細な情報を表示します。
    @param[in] result     トレース対象の Result値。
     */
    static void TraceResult(const Result& result);

#if !NN_PIA_WIN && !NN_PIA_A && !NN_PIA_B
    /*!
    @brief                nn::Result 値の詳細な情報を表示します。
    @param[in] result     トレース対象の Result値。
     */
    static void TraceResult(const nn::Result& result);
#endif

    /*!
    @brief                nn::pia::Result 値の詳細な情報を表示します。

    @param[in] traceFlags トレースフラグ。詳細は@ref TraceFlag を参照してください。
    @param[in] result     トレース対象の Result値。
     */
    static void TraceResult(uint64_t traceFlags, const Result& result);

#if !NN_PIA_WIN && !NN_PIA_A && !NN_PIA_B
    /*!
    @brief                nn::Result 値の詳細な情報を表示します。

    @param[in] traceFlags トレースフラグ。詳細は@ref TraceFlag を参照してください。
    @param[in] result     トレース対象の Result値。
     */
    static void TraceResult(uint64_t traceFlags, const nn::Result& result);
#endif


private:
    /*!
    @brief インスタンスを作ることはないのでコンストラクタは封印します。
     */
    ResultUtility(void);

    /*!
    @brief デストラクタは封印します。
     */
    ~ResultUtility(void);

    NN_PIA_DISALLOW_COPY(ResultUtility);
};
}
}
}  // end of namespace nn::pia::common
