﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Definitions.h>

#define NN_PIA_ENABLE_ROOTOBJECT_NEW 1

#if NN_PIA_ENABLE_ROOTOBJECT_NEW
#include <new>
#endif  // end of NN_PIA_ENABLE_ROOTOBJECT_NEW


namespace nn
{
namespace pia
{
namespace common
{


/*!
  @brief      Piaライブラリ内の共通基底クラスです。

*/
class RootObject
{
public:
    /*!
    @brief コンストラクタです。
     */
    RootObject(void)
    {
    }


#if NN_PIA_ENABLE_ROOTOBJECT_NEW
    /*!
    @cond PRIVATE    
    @brief プレーンなnewです。
     */
    static void* operator new(std::size_t size) throw();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief 例外を投げないnewです。
     */
    static void* operator new(std::size_t size, std::nothrow_t) throw();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief 配置newです。
     */
    static void* operator new(std::size_t, void* p) throw()
    {
        return p;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief プレーンなnewです
     */
    static void* operator new[](std::size_t size) throw();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief 例外を投げないnewです（配列版）。
     */
    static void* operator new[](std::size_t size, std::nothrow_t) throw();
    //! @endcond


    /*!
    @cond PRIVATE
    @brief 配置newです（配列版）。
     */
    static void* operator new[](std::size_t, void* p) throw()
    {
        return p;
    }
    //! @endcond


    /*!
    @cond PRIVATE
    @brief delete演算子です。
     */
    static void operator delete(void* p);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief delete演算子です（配列版）。
     */
    static void operator delete[](void* p);
//! @endcond


#if NN_PIA_WIN || NN_PIA_NINTENDOSDK

    // Win では new に対応する delete が必要

    /*!
    @cond PRIVATE
    @brief 例外を投げない new に対応する delete です。
     */
    static void operator delete(void* p, std::nothrow_t) throw();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 配置 new に対応する delete です。
     */
    static void operator delete(void* p, void*) throw();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 例外を投げない new に対応する delete です（配列版）。
     */
    static void operator delete[](void* p, std::nothrow_t) throw();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief 配置 new に対応する delete です（配列版）。
     */
    static void operator delete[](void* p, void*) throw();
//! @endcond

#endif

#endif  // end of NN_PIA_ENABLE_ROOTOBJECT_NEW
};
}
}
}  // end of namespace nn::pia::common
