﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

#include <nn/pia/common/common_Api.h>


// このマクロは、シングルトンに対応する.cppファイルの冒頭付近に記入してください。
#define NN_PIA_COMMON_SINGLETON_DEFINE(CLASSNAME) CLASSNAME* CLASSNAME::s_pInstance = NULL


// このマクロは、シングルトンに対応する.cppファイルで記入してください。
#define NN_PIA_COMMON_SINGLETON_CREATE_INSTANCE_IMPL(CLASSNAME)                                                                                                 \
    \
nn::pia::Result CLASSNAME::CreateInstance(void)                                                                                                                 \
    \
{                                                                                                                                                        \
        if (!IsInitialized())                                                                                                                                   \
        {                                                                                                                                                       \
            PIA_RETURN_RESULT_WITH_REPORT(ResultNotInitialized, "PiaCommon is not initialized.");                                                                  \
        }                                                                                                                                                       \
        if (!IsInSetupMode())                                                                                                                                   \
        {                                                                                                                                                       \
            PIA_RETURN_RESULT_WITH_REPORT(ResultInvalidState, "Singleton must be created between nn::pia::common::BeginSetup() and nn::pia::common::EndSetup()."); \
        }                                                                                                                                                       \
        if (s_pInstance)                                                                                                                                        \
        {                                                                                                                                                       \
            PIA_RETURN_RESULT_WITH_REPORT(ResultAlreadyExists, "Singleton is already created.");                                                                   \
        }                                                                                                                                                       \
                                                                                                                                                                \
        /* RootObjectから派生させていれば、RootObject::new が使われ、*/                                                                         \
        /* グローバルnewは使われないはず。                           */                                                                           \
        s_pInstance = new CLASSNAME();                                                                                                                          \
        PIA_ASSERT(PIA_IS_VALID_POINTER(s_pInstance));                                                                                                          \
                                                                                                                                                                \
        return ResultSuccess();                                                                                                                                 \
    \
}


// このマクロは、シングルトンに対応する.cppファイルで記入してください。
#define NN_PIA_COMMON_SINGLETON_DESTROY_INSTANCE_IMPL(CLASSNAME)                                \
    \
void CLASSNAME::DestroyInstance(void)                                                           \
    \
{                                                                                        \
        if (s_pInstance)                                                                        \
        {                                                                                       \
            /* RootObjectから派生させていれば、RootObject::delete が使われ、 */ \
            /* グローバルdeleteは使われないはず。                            */   \
            delete s_pInstance;                                                                 \
            s_pInstance = NULL;                                                                 \
        }                                                                                       \
    \
}


namespace nn
{
namespace pia
{
namespace common
{
}
}
}  // end of namespace nn::pia::common
