﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>

namespace nn
{
namespace pia
{
namespace common
{


/*!
@cond PRIVATE
  @brief      統計量のクラスです。
  */
class StatisticValue : public nn::pia::common::RootObject
{
public:
    uint32_t m_Value;
    uint32_t m_ValueMax;
    uint32_t m_ValueMaxResult;
    uint32_t m_ValueMin;
    uint32_t m_ValueMinResult;
    uint32_t m_ValueSum;
    uint32_t m_Count;
    uint32_t m_Average;

    /*!
    @brief コンストラクタです。
     */
    StatisticValue(void)
    {
        ResetValue();
    }

    void ResetValue()
    {
        m_Value = 0;
        m_ValueMax = 0;
        m_ValueMaxResult = 0;
        m_ValueMin = 0xFFFFFFFF;
        m_ValueMinResult = 0;
        m_ValueSum = 0;
        m_Count = 0;
        m_Average = 0;
    }

    void AddValue(uint32_t value)
    {
        m_Value = value;
        m_ValueSum += m_Value;
        m_Count++;
        if (m_Value > m_ValueMax)
        {
            m_ValueMax = m_Value;
        }
        if (m_Value < m_ValueMin)
        {
            m_ValueMin = m_Value;
        }
    }

    void CalcAverage()
    {
        m_Average = m_ValueSum / m_Count;
        m_ValueMaxResult = m_ValueMax;
        m_ValueMinResult = m_ValueMin;
    }
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
