﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Platform.h>
#include <nn/pia/common/common_RootObject.h>


//n2788:0埋め16進数表示を追加
#define NN_PIA_ENABLE_ZERO_FILL_HEX 1

namespace nn
{
namespace pia
{
namespace common
{

const uint32_t StringStreamInitialSize = 128;

//n2619:グローバルなnewをBeginSetup()～EndSetup()外で
//行ってしまう処理を有効にするかどうか（基本的に永久に0）
#define PIA_ENABLE_RESIZE_BUFFER 0

/*!
  @cond PRIVATE
  @brief      文字列のストリームクラスです。
              マニピュレータ（<<）を使用して文字列を結合します。
 */
class StringStream : public RootObject
{
public:
    /*!
    @brief コンストラクタです。
     */
    StringStream(void);

    /*!
    @brief デストラクタです。
     */
    virtual ~StringStream(void);

    // Returns the stream length
    uint32_t GetLength() const;

    // Returns the m_szBuffer size (capacity)
    uint32_t GetSize() const
    {
        return m_uiSize;
    }

    const char* CStr() const
    {
        return m_szBuffer;
    }

    void Clear();

    // Streaming of basic types
    StringStream& operator<<(const char* szText);
    StringStream& operator<<(const void* pPointer);
    StringStream& operator<<(bool bValue);
    StringStream& operator<<(double dValue);
    StringStream& operator<<(float fValue);

    StringStream& operator<<(uint8_t uiValue)
    {
        return StreamNumber(uiValue);
    }
    StringStream& operator<<(int8_t iValue)
    {
        return StreamNumber(iValue);
    }
    StringStream& operator<<(uint16_t uiValue)
    {
        return StreamNumber(uiValue);
    }
    StringStream& operator<<(int16_t iValue)
    {
        return StreamNumber(iValue);
    }
    StringStream& operator<<(uint32_t uiValue)
    {
        return StreamNumber(uiValue);
    }
    StringStream& operator<<(int32_t iValue)
    {
        return StreamNumber(iValue);
    }

    StringStream& operator<<(uint64_t uiValue);
    StringStream& operator<<(int64_t uiValue);

    StringStream& operator<<(const StringStream& oSS);

    // Manipulators
    StringStream& operator<<(StringStream& (*pf)(StringStream&))
    {
        return (*pf)(*this);
    }

#if !NN_PIA_ENABLE_ZERO_FILL_HEX
    StringStream& Manip_hex()
    {
        m_IsHex = true;
        return *this;
    }
    StringStream& Manip_dec()
    {
        m_IsHex = false;
        return *this;
    }
#else   // !NN_PIA_ENABLE_ZERO_FILL_HEX
    StringStream& Manip_hex()
    {
        m_IsHex = true;
        m_IsZeroHex = false;
        return *this;
    }
    StringStream& Manip_dec()
    {
        m_IsHex = false;
        m_IsZeroHex = false;
        return *this;
    }
    StringStream& Manip_zerohex()
    {
        m_IsHex = true;
        m_IsZeroHex = true;
        return *this;
    }
#endif  // !NN_PIA_ENABLE_ZERO_FILL_HEX

    StringStream& Manip_showbase()
    {
        m_IsShowBase = true;
        return *this;
    }
    StringStream& Manip_noshowbase()
    {
        m_IsShowBase = false;
        return *this;
    }

    StringStream& Manip_boolalpha()
    {
        m_IsBoolAlpha = true;
        return *this;
    }
    StringStream& Manip_noboolalpha()
    {
        m_IsBoolAlpha = false;
        return *this;
    }

    const char& operator[](uint32_t uiPos) const
    {
        return m_szBuffer[uiPos];
    }

protected:
private:
#if PIA_ENABLE_RESIZE_BUFFER
    void FreeBuffer();
    void FreeBuffer(char* szBuffer);
    void ResizeBuffer(uint32_t uiSize);
#endif
    void AddBaseIfRequired();

    StringStream& StreamNumber(uint32_t uiValue);
    StringStream& StreamNumber(int32_t iValue);
    void TestFreeRoom(uint32_t uiNewSize);

    //n2619
    int32_t AddStringToRoom(const char* szBuffer);

private:
    char* m_szBuffer;
    uint32_t m_uiSize;
    char* m_szCurrentPosition;

    char m_szInitialBuffer[StringStreamInitialSize];
    //n2619
    char m_szBufferToAdd[StringStreamInitialSize];

    bool m_IsHex;
    bool m_IsShowBase;
    bool m_IsBoolAlpha;

#if NN_PIA_ENABLE_ZERO_FILL_HEX
    //n2788
    bool m_IsZeroHex;
#else
    uint8_t m_padding1;  // to avoid compiler warning.
#endif  // NN_PIA_ENABLE_ZERO_FILL_HEX

    NN_PIA_DISALLOW_COPY(StringStream);
};

// Manipulators.

extern StringStream& hex(StringStream& oStringStream);
extern StringStream& dec(StringStream& oStringStream);
extern StringStream& endl(StringStream& oStringStream);
extern StringStream& showbase(StringStream& oStringStream);
extern StringStream& noshowbase(StringStream& oStringStream);
extern StringStream& boolalpha(StringStream& oStringStream);
extern StringStream& noboolalpha(StringStream& oStringStream);

#if NN_PIA_ENABLE_ZERO_FILL_HEX
extern StringStream& zerohex(StringStream& oStringStream);
#endif  // NN_PIA_ENABLE_ZERO_FILL_HEX
}
}
}  // end of namespace nn::pia::common
//! @endcond
