﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_StringStream.h>
#include <nn/pia/common/common_EmptyStream.h>
#include <nn/pia/common/common_Time.h>


namespace nn
{
namespace pia
{
namespace common
{

#if NN_PIA_ENABLE_TRACE
/*!
@cond PRIVATE
@brief トレース出力をします。
 */
#define PIA_TRACE(FLAG, ...)                                                               \
    \
{                                                                                   \
        ::nn::pia::common::Trace* pNnPiaCommonTrace = ::nn::pia::common::Trace::GetInstance(); \
        if (pNnPiaCommonTrace)                                                             \
        {                                                                                  \
            if (pNnPiaCommonTrace->IsFlagSet(FLAG))                                        \
            {                                                                              \
                pNnPiaCommonTrace->WriteCaption();                                         \
                pNnPiaCommonTrace->Write(__VA_ARGS__);                                     \
                pNnPiaCommonTrace->Write("\n");                                            \
            }                                                                              \
        }                                                                                  \
    \
}
//! @endcond

/*!
@cond PRIVATE
@brief  bool値を文字列に変換します。
 */
#define PIA_BOOL_TO_TEXT(VALUE) ((VALUE) ? "true" : "false")
//! @endcond

/*!
@cond PRIVATE
@brief 関数名を追加してトレース出力をします。
 */
#if NN_PIA_A
#define PIA_TRACE_EX(FLAG, ...)                                                                            \
                                                                                                           \
{                                                                                                          \
        ::nn::pia::common::Trace* pNnPiaCommonTrace = ::nn::pia::common::Trace::GetInstance();             \
        if (pNnPiaCommonTrace)                                                                             \
        {                                                                                                  \
            if (pNnPiaCommonTrace->IsFlagSet(FLAG))                                                        \
            {                                                                                              \
                pNnPiaCommonTrace->WriteCaption(PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_FUNC);                                                        \
                pNnPiaCommonTrace->Write(__VA_ARGS__);                                                     \
            }                                                                                              \
        }                                                                                                  \
                                                                                                           \
}
#else
#define PIA_TRACE_EX(FLAG, ...)                                                                     \
    \
{                                                                                            \
        ::nn::pia::common::Trace* pNnPiaCommonTrace = ::nn::pia::common::Trace::GetInstance();          \
        if (pNnPiaCommonTrace)                                                                      \
        {                                                                                           \
            if (pNnPiaCommonTrace->IsFlagSet(FLAG))                                                 \
            {                                                                                       \
                pNnPiaCommonTrace->WriteCaption();                                                  \
                pNnPiaCommonTrace->Write("%s %s:", PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_FUNC); \
                pNnPiaCommonTrace->Write(__VA_ARGS__);                                              \
                pNnPiaCommonTrace->Write("\n");                                                     \
            }                                                                                       \
        }                                                                                           \
    \
}
#endif
//! @endcond


#else  // else of NN_PIA_ENABLE_TRACE

#define PIA_TRACE(FLAG, ...) (void)(FLAG)
#define PIA_BOOL_TO_TEXT(VALUE)
#define PIA_TRACE_EX(FLAG, ...) (void)(FLAG)

#endif  // end of NN_PIA_ENABLE_TRACE

//n1589: モジュールごとに8ビット割り当てています。不足が出てきたら調整を検討します。
/*!
      @brief トレースフラグです。

      @details PIA_TRACE()マクロの第一引数に渡すなどして使用します。
     */

typedef uint64_t TraceFlag;

/*!
     @name トレースフラグ
     @anchor TraceFlag
     @{
*/
static const TraceFlag TraceFlagNever = 0ULL;        //!< 常に表示されないことを示すフラグです。
static const TraceFlag TraceFlagAlways = 1ULL << 0;  //!< 常に表示されることを示すフラグです。

static const TraceFlag TraceFlagCommon = 1ULL << 1;        //!< PiaCommon モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagCommonEvent = 1ULL << 2;   //!< PiaCommon モジュールのイベント関係のトレースを示すフラグです。
static const TraceFlag TraceFlagCommonJob = 1ULL << 3;     //!< PiaCommon モジュールのジョブ関係のトレースを示すフラグです。
static const TraceFlag TraceFlagCommonSocket = 1ULL << 4;  //!< PiaCommon モジュールのソケット関係のトレースを示すフラグです。
static const TraceFlag TraceFlagCommonDetail = 1ULL << 5;  //!< PiaCommon モジュールの詳細トレースを示すフラグです。

static const TraceFlag TraceFlagLocal = 1ULL << 7;           //!< PiaLocal モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagLocalTransport = 1ULL << 8;  //!< PiaLocal モジュールの送受信処理関係のトレースを示すフラグです。
static const TraceFlag TraceFlagLocalMigration = 1ULL << 9;  //!< PiaLocal モジュールのホストマイグレーション機能関係のトレースを示すフラグです。

static const TraceFlag TraceFlagInet = 1ULL << 11;             //!< PiaInet モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagInetInit = 1ULL << 12;         //!< PiaInet モジュールの初期化、終了処理関係のトレースを示すフラグです。
static const TraceFlag TraceFlagInetStream = 1ULL << 13;       //!< PiaInet モジュールの送受信処理関係のトレースを示すフラグです。
static const TraceFlag TraceFlagInetRelayStream = 1ULL << 14;  //!< PiaInet モジュールのサーバーリレー送受信処理関係のトレースを示すフラグです。
static const TraceFlag TraceFlagInetReport = 1ULL << 15;       //!< PiaInet モジュールのレポート処理関係のトレースを示すフラグです。

static const TraceFlag TraceFlagInetNat = 1ULL << 16;          //!< PiaInet モジュールの NAT トラバーサル処理のトレースを示すフラグです
static const TraceFlag TraceFlagInetNatDetail = 1ULL << 17;    //!< PiaInet モジュールの NAT トラバーサル処理の詳細トレースを示すフラグです。
static const TraceFlag TraceFlagInetNatProbe = 1ULL << 18;     //!< PiaInet モジュールの NAT トラバーサル処理のプルーブ関連のトレースを示すフラグです。

static const TraceFlag TraceFlagTransport = 1ULL << 19;                        //!< PiaTransport モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagTransportInit = 1ULL << 20;                    //!< PiaTransport モジュールの初期化処理、終了処理関係のトレースを示すフラグです。
static const TraceFlag TraceFlagTransportKeepAlive = 1ULL << 21;               //!< PiaTransport モジュールのキープアライブ関係のトレースを示すフラグです。
static const TraceFlag TraceFlagTransportRtt = 1ULL << 22;                     //!< PiaTransport モジュールのRTTプロトコル関係のトレースを示すフラグです。
static const TraceFlag TraceFlagTransportBuffer = 1ULL << 23;                  //!< PiaTransport モジュールのバッファ関係のトレースを示すフラグです。
static const TraceFlag TraceFlagTransportPacketSend = 1ULL << 24;              //!< PiaTransport モジュールにおける Packet 送信に関係するトレースフラグです。
static const TraceFlag TraceFlagTransportPacketReceive = 1ULL << 25;           //!< PiaTransport モジュールにおける Packet 受信に関係するトレースフラグです。
static const TraceFlag TraceFlagTransportProtocolmessageSend = 1ULL << 26;     //!< PiaTransport モジュールにおける ProtocolMessage 送信に関係するトレースフラグです。
static const TraceFlag TraceFlagTransportProtocolmessageReceive = 1ULL << 27;  //!< PiaTransport モジュールにおける ProtocolMessage 受信に関係するトレースフラグです。
static const TraceFlag TraceFlagTransportRelay = 1ULL << 28;                   //!< PiaTransport モジュールのリレー機能のトレースを示すフラグです。
static const TraceFlag TraceFlagTransportRelayDetail = 1ULL << 29;             //!< PiaTransport モジュールのリレー機能の詳細のトレースを示すフラグです。

static const TraceFlag TraceFlagSession = 1ULL << 30;           //!< PiaSession モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagSessionConnect = 1ULL << 31;    //!< PiaSession モジュールの接続処理関係のトレースを示すフラグです。
static const TraceFlag TraceFlagSessionLeave = 1ULL << 32;      //!< PiaSession モジュールの切断処理関係のトレースを示すフラグです。
static const TraceFlag TraceFlagSessionUpdate = 1ULL << 33;     //!< PiaSession モジュールの更新処理関係のトレースを示すフラグです。
static const TraceFlag TraceFlagSessionMigration = 1ULL << 34;  //!< PiaSession モジュールのホストマイグレーション処理関係のトレースを示すフラグです。
static const TraceFlag TraceFlagSessionDetail = 1ULL << 35;     //!< PiaSession モジュールの詳細トレースを示すフラグです。
static const TraceFlag TraceFlagSessionInit = 1ULL << 36;       //!< PiaSession モジュールの初期化、終了処理に関係するトレースを示すフラグです。
static const TraceFlag TraceFlagSessionJoint = 1ULL << 37;      //!< PiaSession モジュールのジョイントセッション処理関係のトレースを示すフラグです。
static const TraceFlag TraceFlagSessionClock = 1ULL << 38;      //!< PiaSession モジュールの同期クロック関係のトレースを示すフラグです。

static const TraceFlag TraceFlagSync = 1ULL << 39;  //!< PiaSync モジュールのトレースを示すフラグです。

static const TraceFlag TraceFlagClone = 1ULL << 41;        //!< PiaClone モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagCloneDetail = 1ULL << 42;  //!< PiaClone モジュールの詳細トレースを示すフラグです。

static const TraceFlag TraceFlagReckoning = 1ULL << 43;  //!< PiaReckoning モジュールのトレースを示すフラグです。

static const TraceFlag TraceFlagChat = 1ULL << 45;        //!< PiaChat モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagChatDetail = 1ULL << 46;  //!< PiaChat モジュールの詳細トレースを示すフラグです。

static const TraceFlag TraceFlagService = 1ULL << 49;                  //!< PiaService モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagServiceTransport = 1ULL << 50;         //!< PiaService モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagServiceTransportDetail = 1ULL << 51;   //!< PiaService モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagServiceRelayServer = 1ULL << 52;       //!< PiaService モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagServiceMonitoringServer = 1ULL << 53;  //!< PiaService モジュールのトレースを示すフラグです。

static const TraceFlag TraceFlagBrain = 1ULL << 54;                   //!< PiaBrain モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagBrainDetail = 1ULL << 55;             //!< PiaBrain モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagBrainBehaviorExecution = 1ULL << 56;  //!< PiaBrain モジュールのトレースを示すフラグです。

static const TraceFlag TraceFlagFramework = 1ULL << 58;  //!< PiaFramework モジュールのトレースを示すフラグです。

static const TraceFlag TraceFlagLan = 1ULL << 60;        //!< PiaLan モジュールのトレースを示すフラグです。
static const TraceFlag TraceFlagLanDetail = 1ULL << 61;  //!< PiaLan モジュールの詳細トレースを示すフラグです。

static const TraceFlag TraceFlagPlugin = 1ULL << 62;  //!< PiaPlugin のトレースを示すフラグです。

static const TraceFlag TraceFlagDirect = 1ULL << 63;  //!< PiaDirect のトレースを示すフラグです。

static const TraceFlag TraceFlagAll = TraceFlagAlways;  //!< 全てのトレースで表示されることを示すフラグです。

static const TraceFlag TraceFlagDefault = TraceFlagAlways;  //!< TraceFlag のデフォルト値です。

/*!
     end of name トレースフラグ
     @}
     */


/*!
  @brief      トレースのクラスです。

  @details    Releaseビルドでは、トレースのコードは無効化されます。
  <br>
 */
class Trace : public nn::pia::common::RootObject
{
public:
    /*!
    @brief    Trace クラスのインスタンスを作成します（シングルトンパターン）。

    @details  この API は、 nn::pia::common::BeginSetup() を呼び出してから、
              nn::pia::common::EndSetup() を呼び出すまでの間に呼び出す必要があります。

    @return   インスタンスの作成に成功すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。

    @retval   ResultAlreadyExists      既にインスタンスは作成されています。[:progErr]
    @retval   ResultNotInitialized     common モジュールが初期化されていません。[:progErr]
    @retval   ResultInvalidState       BeginSetup() ～ EndSetup() 間で呼び出されていません。[:progErr]
     */
    static Result CreateInstance(void);


    /*!
    @brief Trace クラスのインスタンスを破棄します（シングルトンパターン）。

    @details Trace クラスのインスタンスを破棄します。インスタンスが作成されて
             いない状態の時にこのAPIを呼び出した場合は、何も起こらずに返ります。
     */
    static void DestroyInstance(void);


    /*!
    @brief    Trace クラスのインスタンスを得ます（シングルトンパターン）

    @details  事前に @ref CreateInstance でインスタンスを作成していなかった場合は、
              NULLポインタが返されます。

    @return   Trace インスタンスへのポインタ。
     */
    static Trace* GetInstance(void)
    {
        return s_pInstance;
    }

    //n1589:IsFlagSetチェックのコードを最短化
    /*!
    @cond PRIVATE
     */
    static bool IsFlagSetStatic(uint64_t flag)
    {
#if NN_PIA_ENABLE_TRACE
        if (Trace::GetInstance() == NULL)
        {
            return false;
        }

        return Trace::GetInstance()->IsFlagSet(flag);
#else
        NN_PIA_UNUSED(flag);
        return false;
#endif
    }
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  文字列を出力します。
            マクロから呼び出されるので直接呼んではいけません。
     */
    void Write(const char* pFmt, ...);
    //! @endcond


    /*!
    @cond PRIVATE
    @brief  キャプション文字列を出力します。
            マクロから呼び出されるので直接呼んではいけません。
     */
    void WriteCaption();
    //! @endcond

    /*!
    @cond PRIVATE
    @brief  キャプション文字列とファイル名、関数名を出力します。
    マクロから呼び出されるので直接呼んではいけません。
    */
    void WriteCaption(const char* pFile, const char* pFunction);
    //! @endcond

    /*!
    @brief    トレースフラグをアクティブに設定します。

    @details  このメソッドを使用して、追加のトレースフラグをアクティブに設定します。
    <br>
    @param[in]    flags    アクティブにする @ref TraceFlag の論理和です。
    */
    void SetFlag(uint64_t flags)
    {
        m_flags |= flags;
    }

    /*!
    @brief    指定したトレースフラグを非アクティブに設定します。

    @details  指定したトレースフラグを非アクティブに設定します。
              ただし、 TraceFlagAlways は除きます。

    @param[in]    flags    非アクティブにする @ref TraceFlag の論理和です。
    */
    void ClearFlag(uint64_t flags)
    {
        m_flags = m_flags & (~flags);
    }

    /*!
    @brief    特定のフラグが設定されているかどうかを返します。

    @details  なお、 IsFlasSet ( TraceFlagNever ) の結果は true となります。
              引数で複数のフラグを指定した際は、指定したフラグのいずれかが設定されていれば
              true が返されます。
    <br>
    @return   特定のフラグが設定されていれば true 、そうでなければ false を返します。

    @param[in]    flag    問い合わせるトレースフラグ。
    */
    bool IsFlagSet(uint64_t flag) const
    {
#if NN_PIA_ENABLE_TRACE
        return (m_flags & flag) != 0;
#else
        NN_PIA_UNUSED(flag);
        return false;
#endif
    }

    /*!
      @brief  TraceFlagAlways を除き、トレースフラグを全てクリアします。
     */
    void ClearFlag(void)
    {
        m_flags = TraceFlagDefault;
    }


    /*!
    @brief 時刻表示の ON / OFF を切り替えます。
           ON の時は "[Pia:12345] ..." のように基準時刻からの経過時間をミリ秒単位で表示します。
    @param[in] isPrint  true なら時刻を表示します。
     */
    void SetPrintTime(bool isPrint)
    {
        m_IsPrintTime = isPrint;
    }


    /*!
    @brief 時刻表示の ON / OFF を取得します。
    @return 時刻表示が ON になっていれば true を返します。
     */
    bool IsPrintTime() const
    {
        return m_IsPrintTime;
    }


    /*!
    @brief 時刻表示の基準時刻を現在の時刻にします。
     */
    void ResetBaseTime()
    {
        m_BaseTime.SetNow();
    }

private:
    /*!
    @brief コンストラクタです。フラグは TraceFlagAlways のみセットされます。
     */
    Trace();

    /*!
    @brief デストラクタです。
     */
    ~Trace(void)
    {
    }


    uint64_t m_flags;
    Time m_BaseTime;
    bool m_IsPrintTime;

    static Trace* s_pInstance;

    NN_PIA_DISALLOW_COPY(Trace);
};
}
}
}  // end of namespace nn::pia::common
