﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_TreeMapNodeBase.h>

namespace nn
{
namespace pia
{
namespace common
{

/*!
    @cond PRIVATE
    @brief  ツリーマップの既定クラスです。
 */
class TreeMapBase : public common::RootObject
{
    typedef TreeMapNodeBase Node;

protected:
    TreeMapBase()
        : m_pRoot(NULL)
    {
    }


public:
    /*!
    @brief  空かどうかを判定します。
    @return 空ならtrueを返します。
     */
    bool IsEmpty() const
    {
        return m_pRoot == NULL;
    }


    /*!
    @brief  要素数を数えます。
    @return 要素数です。
     */
    uint32_t Count() const;


protected:
    bool IsIncludeNodeBase(const Node* cpNode) const;

    void ClearNodeBase();

    Node* FrontNodeBase() const;
    Node* BackNodeBase() const;

    void InsertNodeRoot(Node* pNode);
    void InsertNodeLeft(Node* pNode, Node* pUp);
    void InsertNodeRight(Node* pNode, Node* pUp);
    void ReplaceNode(Node* pNode, Node* pOld);

    void EraseNodeBase(Node* pNode);

    Node* RotateRight(Node* pNode);
    Node* RotateLeft(Node* pNode);

protected:
    Node* GetRootBase() const
    {
        return m_pRoot;
    }

private:
    // コピー禁止
    TreeMapBase(const TreeMapBase& rhs);
    TreeMapBase& operator=(const TreeMapBase& rhs);

    uint32_t CountImpl(const Node* cpNode) const;

    void ClearNodeBaseImpl(Node* pNode);

    void UpdateBalanceInsert(Node* pNode);

private:
    Node* m_pRoot;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
