﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common/common_Definitions.h>
#include <nn/pia/common/common_TreeMapNodeBase.h>


namespace nn
{
namespace pia
{
namespace common
{

//! @cond PRIVATE
template <typename Key>
class TreeMap;
//! @endcond

/*!
    @cond PRIVATE
    @brief  ツリーマップのノードとなるクラスです。
 */
template <typename Key_>
class TreeMapNode : public common::TreeMapNodeBase
{
    friend class TreeMap<Key_>;

public:
    typedef Key_ Key;


    /*!
    @brief デフォルトコンストラクタです。
     */
    TreeMapNode()
        : TreeMapNodeBase(), m_Key()
    {
    }


    /*!
    @brief  キーの値を取得します。
    @return キーの値です。
     */
    const Key& GetKey() const
    {
        return m_Key;
    }


private:
    void InitNode()
    {
        TreeMapNodeBase::InitNodeBase();
        m_Key = Key();
    }

public:
    TreeMapNode* Prev() const
    {
        return static_cast<TreeMapNode*>(PrevBase());
    }
    TreeMapNode* Next() const
    {
        return static_cast<TreeMapNode*>(NextBase());
    }

private:
    TreeMapNode* GetLeft() const
    {
        return static_cast<TreeMapNode*>(GetLeftBase());
    }
    TreeMapNode* GetRight() const
    {
        return static_cast<TreeMapNode*>(GetRightBase());
    }

private:
    Key m_Key;

    TreeMapNode(const TreeMapNode&);
    TreeMapNode& operator=(const TreeMapNode&);
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
