﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/framework/framework_Definitions.h>

/*!
framework モジュールの非メンバ関数群です。
*/
namespace nn
{
namespace pia
{
namespace framework
{
/*!
@cond  PRIVATE
@brief  Initialize() に各種パラメータを渡すための構造体です。
*/
struct CommonSetting
{
    uint8_t* pPiaHeapBuffer;
    uint32_t szPiaHeapBufferSize;
    uint32_t cachedPrintBufferSize;
};
//! @endcond

/*!
@brief                           framework モジュールを初期化します。
@param[in] setting               framework の初期化に必要なパラメータです。
@return                          無事に初期化が完了すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
@retval ResultInvalidState       この関数を呼び出すタイミングが誤っています。[:progErr]
@retval ResultAlreadyInitialized 既に初期化が完了しています。[:progErr]
@retval ResultInvalidArgument    引数が誤っています。[:progErr]
*/
Result Initialize(const InitializeFrameworkSetting& setting);

/*!
@brief   framework モジュールを終了します。
@details framework モジュールが初期化されていない状態の時にこの API を呼び出した場合は、何もしません。
*/
void Finalize();

/*!
@brief        framework モジュールが初期化された状態であるかどうかを返します。
@retval true  初期化されている。
@retval false 初期化されていない。
*/
bool IsInitialized();

/*!
@cond  PRIVATE
@brief BeginSetup() ～ EndSetup() 間の状態であるかどうかを返します。
*/
bool IsInSetupMode();
//! @endcond

/*!
@cond  PRIVATE
@brief             common 初期化処理です。
@param[in] setting 各種パラメータが書き込まれた Setting 構造体です。
@return            成功すれば、 IsSuccess() が true を返す Result が返されます。
*/
Result InitializeCommon(const CommonSetting& setting);
//! @endcond

/*!
@cond  PRIVATE
@brief common 終了処理です。
*/
void FinalizeCommon();
//! @endcond

}
}
}  // end of namespace nn::pia::framework
