﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

// このモジュールの全クラスのヘッダでインクルードするヘッダを列挙します。
#include <nn/pia/pia_Definitions.h>
#include <nn/pia/pia_Assert.h>
#include <nn/pia/common/common_RootObject.h>
#include <nn/pia/common/common_Trace.h>
#include <nn/pia/common/common_Report.h>
#include <nn/pia/session/session_Session.h>
#include <nn/pia/clone/clone_CloneProtocol.h>
#include <nn/pia/sync/sync_SyncProtocol.h>
#include <nn/pia/lan/lan_LanJoinRandomSessionSetting.h>

#if NN_PIA_NINTENDOSDK_WIN
#define NN_PIA_FRAMEWORK_ENABLE_NEX 0
#else

#if NN_PIA_ENABLE_NEX
#define NN_PIA_FRAMEWORK_ENABLE_NEX 1
#else
#define NN_PIA_FRAMEWORK_ENABLE_NEX 0
#endif

#endif

#if NN_PIA_A || NN_PIA_B
#define NN_PIA_FRAMEWORK_ENABLE_PLAY 0
#define NN_PIA_FRAMEWORK_ENABLE_DIRECT 1
#else
#define NN_PIA_FRAMEWORK_ENABLE_PLAY 0
#define NN_PIA_FRAMEWORK_ENABLE_DIRECT 0
#endif

#define NN_PIA_FRAMEWORK_ENABLE_INET (NN_PIA_FRAMEWORK_ENABLE_NEX || NN_PIA_FRAMEWORK_ENABLE_PLAY)

#if NN_PIA_FRAMEWORK_ENABLE_NEX
#include <nn/pia/inet/inet_NexJoinRandomSessionSetting.h>
#endif

//n1589:Settingが必要がなったときに使用する
#define NN_PIA_FRAMEWORK_ENABLE_PLAY_SETTING 0

#if NN_PIA_FRAMEWORK_ENABLE_PLAY
#include <nn/pia/play/play_PlayJoinRandomSessionSetting.h>
#include <jni.h>
#endif

#if NN_PIA_FRAMEWORK_ENABLE_DIRECT
#include <nn/pia/direct/direct_DirectNetworkSetting.h>
#if !NN_PIA_B // iosでもビルドが通るように
#include <jni.h>
#endif
#endif

#if NN_PIA_CTR || NN_PIA_NINTENDOSDK_NX || NN_PIA_FRAMEWORK_ENABLE_DIRECT
#define NN_PIA_ENABLE_LOCAL 1
#else
#define NN_PIA_ENABLE_LOCAL 0
#endif

//n2908: Framework ジョイントセッション有効化マクロ
#define NN_PIA_EXPERIMENT_FRAMEWORK_JOINT_SESSION 1

#if NN_PIA_CTR
#include <nn/pia/local/local_UdsNetworkSetting.h>
#elif NN_PIA_NINTENDOSDK
#include <nn/pia/local/local_LdnNetworkSetting.h>
#include <nn/socket.h>
#include <nn/account.h>
#include <nn/account/account_Selector.h>
#include <nn/err.h>
#include <nn/err/err_Types.h>
#endif

/*!
モジュール内で共通して参照する列挙型やtypedefをまとめたヘッダファイルです。
*/
namespace nn
{
namespace pia
{
namespace framework
{
/*!
@brief ネットワークの種類を列挙しています。
*/
enum NetworkType
{
    NetworkType_Local = 0,
    NetworkType_Inet,
    NetworkType_Lan
};

/*!
@brief framework の初期化に必要なデータを持つ構造体です。
*/
struct InitializeFrameworkSetting
{
    uint8_t* piaBuffer;              //!< Pia 全体のバッファです。
    uint32_t piaBufferSize;          //!< Pia 全体のバッファサイズです。
    uint32_t cachedPrintBufferSize;  //!< @cond CTR_DOC キャッシュするバッファのサイズ（バイト）を指定します。512以上の値を指定する必要があります。 @endcond
    NetworkType networkType;         //!< 使用するインターネットの種類です。
    uint64_t traceFlags;             //!< トレースフラグです。

    /*!
    @brief デフォルトコンストラクタです。
    */
    InitializeFrameworkSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
        piaBuffer = NULL;
        piaBufferSize = 0;
        cachedPrintBufferSize = 0;
        networkType = framework::NetworkType_Inet;
        traceFlags = common::TraceFlagDefault;
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeFrameworkSetting] piaBufferSize:%u, cachedPrintBufferSize:%u, networkType:%d, traceFlags:%llu", piaBufferSize, cachedPrintBufferSize, networkType, traceFlags);
    }
};

/*!
@brief inet の初期化に必要なデータを持つ構造体です。
*/
struct InitializeInetSetting
{
    // InitializeInetService のための設定項目
    bool isAutoInitializeAndLogin;         //!< InitializeInetService とログインを自動で行うかどうか指定します。
    uint32_t gameId;                   //!< ゲームサーバーにログインする際に指定したゲームサーバー ID です。

#if NN_PIA_FRAMEWORK_ENABLE_NEX
    uint8_t* nexMemory;            //!< NEX で使用するメモリ。@if NIN_DOC 4 KB アラインメントされている必要があります。また、16 KB 以上である必要があります。 @endif
    uint32_t totalMemorySize;      //!< @ref nexMemory のサイズです。
    uint32_t emergencyMemorySize;  //!< @ref nexMemory の内、NEX のヒープ枯渇時に使用する緊急用メモリとして割り当てるサイズです。
    common::FixedString<8> accessKey;  //!< ゲームサーバーにログインする際に指定したアクセスキーです
#endif

#if NN_PIA_FRAMEWORK_ENABLE_PLAY
    jobject appActivity; //!< isAutoInitializeAndLogin が true の場合のみ設定する必要があります。
    gpg::GameServices* pGameServices; //!< isAutoInitializeAndLogin が false の場合のみ設定する必要があります。true の場合は設定しないでください。
#endif

#if NN_PIA_NINTENDOSDK
    common::FixedString<15> hostName;   //!< デバッグログイン用のアドレスです。
    uint16_t port;                      //!< デバッグログイン用のポート番号です。
    common::FixedString<64> hostUri;    //!< デバッグログイン用のホスト Uriです。
    common::FixedString<20> userName;   //!< デバッグログイン用のユーザーネームです。
    common::FixedString<32> password;   //!< デバッグログイン用のパスワードです。
    nn::account::UserHandle userHandle; //!< ログインするユーザのハンドルを指定します。
#endif

    /*!
    @brief デフォルトコンストラクタです。
    */
    InitializeInetSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
        isAutoInitializeAndLogin = false;
        gameId = 0;

#if NN_PIA_FRAMEWORK_ENABLE_NEX
        nexMemory = NULL;
        totalMemorySize = 0;
        emergencyMemorySize = 0;
        accessKey.Clear();
#endif

#if NN_PIA_FRAMEWORK_ENABLE_PLAY
        pGameServices = NULL;
#endif

#if NN_PIA_NINTENDOSDK
        hostName.Clear();
        port = 0;
        hostUri.Clear();
        userName.Clear();
        password.Clear();
#endif
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeInetSetting] totalMemorySize:%u, emergencyMemorySize:%u, isAutoInitialize:%d, gameId:%u, accessKey:%s", totalMemorySize, emergencyMemorySize, isAutoInitializeAndLogin, gameId, accessKey.GetCharArray());
#endif
#if NN_PIA_FRAMEWORK_ENABLE_PLAY
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeInetSetting] pGameServices:%p", pGameServices);
#endif
#if NN_PIA_NINTENDOSDK
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeInetSetting] hostName:%s, port:%u, hostUri:%s, userName:%s, password:%s", hostName.GetCharArray(), port, hostUri.GetCharArray(), userName.GetCharArray(), password.GetCharArray());
#endif
    }

    /*!
    @brief コピーします。
    */
    void Copy(const InitializeInetSetting& setting)
    {
        isAutoInitializeAndLogin = setting.isAutoInitializeAndLogin;
        gameId = setting.gameId;

#if NN_PIA_FRAMEWORK_ENABLE_NEX
        nexMemory = setting.nexMemory;
        totalMemorySize = setting.totalMemorySize;
        emergencyMemorySize = setting.emergencyMemorySize;
        accessKey.Copy(setting.accessKey);
#endif

#if NN_PIA_FRAMEWORK_ENABLE_PLAY
        appActivity = setting.appActivity;
        pGameServices = setting.pGameServices;
#endif

#if NN_PIA_NINTENDOSDK
        hostName.Copy(setting.hostName);
        port = setting.port;
        hostUri.Copy(setting.hostUri);
        userName.Copy(setting.userName);
        password.Copy(setting.password);
        userHandle = setting.userHandle;
#endif
    }

};

/*!
@brief lan の初期化に必要なデータを持つ構造体です。
*/
struct InitializeLanSetting
{
    /*!
    @brief デフォルトコンストラクタです。
    */
    InitializeLanSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeLanSetting] ");
    }
};

/*!
@brief local の初期化に必要なデータを持つ構造体です。
*/
struct InitializeLocalSetting
{
#if NN_PIA_CTR
    uint8_t applicationVersion;  //!< アプリケーションのバージョンです。構築されたネットワークは、この値が異なるバージョンのアプリケーションから発見はされますが、接続には失敗します。
    uint32_t receiveBufferSize;  //!< 受信バッファサイズです。@ref local::UdsReceiveBufferSizeMin 以上、かつ、4096 の倍数である必要があります。
    uint32_t scanBufferSize;     //!< スキャン用バッファサイズです。@ref local::UdsScanBufferSizeMin 以上である必要があります。
#elif NN_PIA_NINTENDOSDK
    int localCommunicationVersion;  //!< ローカル通信バージョンです。アプリケーションへのパッチの適用などで通信の仕様が変わる場合に、ローカル通信バージョンをインクリメントしておくことで、通信の互換性がないことを表明できます。このバージョンが異なるネットワークは、探索結果として取得することはできますが、接続しようとすると失敗します。@ref local::LdnLocalCommunicationVersionMin 以上、@ref local::LdnLocalCommunicationVersionMax 以下の値を指定する必要があります。
#elif NN_PIA_FRAMEWORK_ENABLE_DIRECT
#if NN_PIA_A
    JavaVM *pJavaVm; // Java のオブジェクトを扱うために必要
    jclass piaDirectClass; // java の PiaDirect クラス
    jobject contextObject; // Java の Context
#elif NN_PIA_B
#endif
#endif

    /*!
    @brief デフォルトコンストラクタです。
    */
    InitializeLocalSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
#if NN_PIA_CTR
        applicationVersion = 0;
        receiveBufferSize = local::UdsReceiveBufferSizeMin;
        scanBufferSize = local::UdsScanBufferSizeMin;
#elif NN_PIA_NINTENDOSDK
        localCommunicationVersion = local::LdnLocalCommunicationVersionMin;
#elif NN_PIA_FRAMEWORK_ENABLE_DIRECT
#if NN_PIA_A
        pJavaVm = NULL;
        piaDirectClass = 0;
        contextObject = 0;
#elif NN_PIA_B
#endif
#endif
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
#if NN_PIA_CTR
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeLocalSetting] applicationVersion:%u, receiveBufferSize:%u, scanBufferSize:%u", applicationVersion, receiveBufferSize, scanBufferSize);
#elif NN_PIA_NINTENDOSDK
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeLocalSetting] localCommunicationVersion:%d", localCommunicationVersion);
#endif
    }
};

#if 0
#if NN_PIA_FRAMEWORK_ENABLE_PLAY
/*!
@brief play の初期化に必要なデータを持つ構造体です。
*/
struct InitializePlaySetting
{
    InitializePlaySetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
    }
};
#endif
#endif

/*!
@brief transport の初期化に必要なデータを持つ構造体です。
*/
struct InitializeTransportSetting
{
    uint16_t maxStationNum;    //!< 一つのネットワークに参加可能な最大 Station 数を指定します。
    int32_t analysisInterval;  //!< TransportAnalyzer 機能による測定結果を自動的にコンソールに出力する間隔です。単位は秒です。0 を指定した場合、出力は自動では行われなくなり、データの取得はアプリケーションに委ねられます。

    /*!
    @brief デフォルトコンストラクタです。
    */
    InitializeTransportSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
        maxStationNum = 8;
        analysisInterval = 0;
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeTransportSetting] maxStationNum:%u, analysisInterval:%d", maxStationNum, analysisInterval);
    }
};

/*!
@brief clone の初期化に必要なデータを持つ構造体です。
*/
struct InitializeCloneSetting
{
    bool isAutoInitialize;          //!< InitializeClone を行うかどうか
    float clockPerSec;              //!< 1 秒あたりに進める時刻の値を指定します。
    uint32_t dataCompressionLevel;  //!< 送信データの圧縮レベルを指定します。
    int ackBufferNum;               //!< Ack 用バッファの数を指定します。

    /*!
    @brief デフォルトコンストラクタです。
    */
    InitializeCloneSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
        isAutoInitialize = false;
        clockPerSec = 1000.0f;
        dataCompressionLevel = clone::CloneProtocol::CompressionLevelMiddle;
        ackBufferNum = 32;
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeCloneSetting] clockPerSec:%f, dataCompressionLevel:%u, isAutoInitialize:%d, ackBufferNum:%d", clockPerSec, dataCompressionLevel, isAutoInitialize, ackBufferNum);
    }
};

/*!
@brief sync の初期化に必要なデータを持つ構造体です。
*/
struct InitializeSyncSetting
{
    bool isAutoInitialize;                                 //!< InitializeSync を行うかどうか指定します。
    bool isChangeDelayEnabled;                             //!< 同期通信中の入力遅延変更機能を使用できるようにするかどうか指定します。
    uint32_t dataUnitSize[sync::SyncProtocol::DataIdNum];  //!< 各データ ID ごとに、同期データのサイズを設定します。使用しないデータ ID には 0 を指定します。
    uint32_t timeoutFrame;                                 //!< タイムアウトまでのフレーム数を設定します。
    uint32_t dataCompressionLevel;                         //!< 同期データの圧縮レベルを指定します。

    /*!
    @brief デフォルトコンストラクタです。
    */
    InitializeSyncSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
        isAutoInitialize = false;
        isChangeDelayEnabled = false;
        for (int i = 0; i < sync::SyncProtocol::DataIdNum; ++i)
        {
            dataUnitSize[i] = 0;
        }
        timeoutFrame = 240;
        dataCompressionLevel = sync::SyncProtocol::CompressionLevelMiddle;
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
        nn::pia::common::StringStream ss;
        ss << "{";
        for (int i = 0; i < sync::SyncProtocol::DataIdNum; ++i)
        {
            ss << dataUnitSize[i];
            if (i + 1 < sync::SyncProtocol::DataIdNum)
            {
                ss << ", ";
            }
            else
            {
                ss << "}";
            }
        }
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeSyncSetting] dataUnitSize:%s, timeoutFrame:%u, dataCompressionLevel:%u, isAutoInitialize:%d, isChangeDelayEnabled:%d", ss.CStr(), timeoutFrame, dataCompressionLevel, isAutoInitialize, isChangeDelayEnabled);
    }
};

/*!
@brief session の初期化に必要なデータを持つ構造体です。
*/
struct InitializeSessionSetting
{
    session::NetworkTopology networkTopology;                     //!< ネットワークトポロジの設定をします。
    session::Session::SessionEventCallback sessionEventCallback;  //!< セッションの状態が変化した場合に呼び出したいコールバック関数を設定します。
    uint16_t browsedSessionInfoListNum;                           //!< ブラウズ可能なセッションの数です。

    /*!
    @brief デフォルトコンストラクタです。
    */
    InitializeSessionSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
        networkTopology = session::NetworkTopology_FullMesh;
        sessionEventCallback = NULL;
        browsedSessionInfoListNum = 8;
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeSessionSetting] networkTopology:%d, browsedSessionInfoListNum:%u", networkTopology, browsedSessionInfoListNum);
    }
};

/*!
@brief ネットワーク（ネットワークインターフェース、ソケット）のスタートアップに必要なデータを持つ構造体です。
*/
struct StartupNetworkSetting
{
    bool isAutoInitializeNetworkInterface;  //!< InitializeNetworkInterface を行うかどうかを指定します。
    bool isAutoInitializeSocket;            //!< InitializeSocket を行うかどうか指定します。
    void* memoryPool;                       //!< ソケットライブラリが使用するワーク領域のアドレスを指定します。BUFFER_ALIGNMENT(4096 バイト) 整合である必要があります。デバイスメモリは使用できません。
    size_t memoryPoolSize;                  //!< ワーク領域のサイズを指定します。4 KB の倍数である必要があります。
    size_t allocatorPoolSize;               //!< ワーク領域のうち、ソケットの送受信バッファの割り当てに使用するサイズを指定します。BUFFER_UNITSIZE_FOR_SOCKETS(4096 バイト) の倍数である必要があります。

    /*!
    @brief デフォルトコンストラクタです。
    */
    StartupNetworkSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
        isAutoInitializeNetworkInterface = false;
        isAutoInitializeSocket = false;
        memoryPool = NULL;
        memoryPoolSize = 0;
        allocatorPoolSize = 0;
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "[InitializeSocketSetting] isAutoInitializeNetworkInterface:%d, isAutoInitializeSocket:%d, memoryPoolSize:%u, allocatorPoolSize:%u", isAutoInitializeNetworkInterface, isAutoInitializeSocket, memoryPoolSize, allocatorPoolSize);
    }
};

/*!
@brief session のスタートアップに必要なデータを持つ構造体です。
@details 各設定項目については session::Session::Startup も併せて参照してください。
*/
struct StartupSessionSetting
{
    uint16_t maxNumOfRelayRoutesAssignedToStation;                                            //!< リレー接続有効時に、ステーション1台に割り当てられる転送依頼数の最大制限値です。
    uint16_t maxRttOfRelayRoute;                                                              //!< リレー接続有効時に、1つのリレー接続経路の合計 RTT の最大制限値(msec)です。
    uint32_t maxSilenceTime;                                                                  //!< 許容される最大無通信時間 [msec] を指定します。デフォルトは session::SessionMaxSilenceTimeDefault です。設定可能な最大値は session::SessionMaxSilenceTimeMax, 最小値は session::SessionMaxSilenceTimeMin です。
    uint32_t keepAliveSendingInterval;                                                        //!< キープアライブの送信間隔 [msec] を指定します。デフォルトは session::SessionKeepAliveIntervalDefault です。
    transport::Station::PlayerInfo playerInfo;                                                //!< Station に結び付けて保存されるプレイヤー情報です。
    common::FixedString<common::SignatureSetting::SignatureKeyDataSizeMax> signatureKeyData;  //!< 署名鍵です。
    uint32_t signatureKeyDataSize;                                                            //!< 署名鍵のサイズです。署名を無効にする場合は 0 を指定する必要があります。@ref common::SignatureSetting::SignatureKeyDataSizeMax 以下である必要があります。
#if NN_PIA_NINTENDOSDK
    bool isAutoInitializeLdn;                                                                 //!< LDN の初期化を自動で行うかどうか指定します。
#endif
    bool isInetHostMigrationEnabled;                                                          //!< インターネット通信時にホストマイグレーション機能を有効にするかどうかを指定します。デフォルトは true (有効) になっています。 

    /*!
    @brief デフォルトコンストラクタです。
    */
    StartupSessionSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
        maxNumOfRelayRoutesAssignedToStation = 10;
        maxRttOfRelayRoute = 400;
        maxSilenceTime = 10000;
        keepAliveSendingInterval = 500;
        {
            playerInfo.screenName.Clear();
            playerInfo.nameStringLanguage = 0xff;
            playerInfo.playerId.id[0] = 0;
            playerInfo.playerId.id[1] = 0;
        }
        signatureKeyData.Copy("framework key");
        signatureKeyDataSize = sizeof("framework key");
#if NN_PIA_NINTENDOSDK
        isAutoInitializeLdn = false;
#endif
        isInetHostMigrationEnabled = true;
    }

    /*!
    @brief コピーします。
    */
    void Copy(const StartupSessionSetting& setting)
    {
        maxNumOfRelayRoutesAssignedToStation = setting.maxNumOfRelayRoutesAssignedToStation;
        maxRttOfRelayRoute = setting.maxRttOfRelayRoute;
        maxSilenceTime = setting.maxSilenceTime;
        keepAliveSendingInterval = setting.keepAliveSendingInterval;
        playerInfo.nameStringLanguage = setting.playerInfo.nameStringLanguage;
        playerInfo.playerId = setting.playerInfo.playerId;
        playerInfo.screenName.Copy(setting.playerInfo.screenName);
        signatureKeyData.Copy(setting.signatureKeyData);
        signatureKeyDataSize = setting.signatureKeyDataSize;
#if NN_PIA_NINTENDOSDK
        isAutoInitializeLdn = setting.isAutoInitializeLdn;
#endif
        isInetHostMigrationEnabled = setting.isInetHostMigrationEnabled;
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
#if NN_PIA_NINTENDOSDK
        PIA_TRACE_EX(common::TraceFlagFramework, "[SessionStartupSetting] maxNumOfRelayRoutesAssignedToStation:%u, maxRttOfRelayRoute:%u, playerInfo.nameStringLanguage:%u, playerInfo.playerId.id[0]:%llu, playerInfo.playerId.id[1]:%llu, playerInfo.screenName:%s, signatureKeyData:%s, signatureKeyDataSize:%u, isAutoInitializeLdn:%d, isInetHostMigrationEnabled:%d", maxNumOfRelayRoutesAssignedToStation, maxRttOfRelayRoute, playerInfo.nameStringLanguage, playerInfo.playerId.id[0], playerInfo.playerId.id[1], playerInfo.screenName.GetCharArray(), signatureKeyData.GetCharArray(), signatureKeyDataSize, isAutoInitializeLdn, isInetHostMigrationEnabled);
#else
        PIA_TRACE_EX(common::TraceFlagFramework, "[SessionStartupSetting] maxNumOfRelayRoutesAssignedToStation:%u, maxRttOfRelayRoute:%u, playerInfo.nameStringLanguage:%u, playerInfo.playerId.id[0]:%llu, playerInfo.playerId.id[1]:%llu, playerInfo.screenName:%s, signatureKeyData:%s, signatureKeyDataSize:%u, isInetHostMigrationEnabled:%d", maxNumOfRelayRoutesAssignedToStation, maxRttOfRelayRoute, playerInfo.nameStringLanguage, playerInfo.playerId.id[0], playerInfo.playerId.id[1], playerInfo.screenName.GetCharArray(), signatureKeyData.GetCharArray(), signatureKeyDataSize, isInetHostMigrationEnabled);
#endif
    }

private:
    StartupSessionSetting& operator=(const StartupSessionSetting& rhs) const;
};

/*!
@brief セッションを構築する際に必要なデータを持つ構造体です。
*/
struct JoinRandomSessionSetting
{
protected:
    JoinRandomSessionSetting()
    {
        Reset();
    }

public:
    /*!
    @brief  デストラクタです。
    */
    virtual ~JoinRandomSessionSetting()
    {
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    virtual void Reset()
    {
    }
};

/*!
@brief NEX を使用してセッションを構築する際に必要なデータを持つ構造体です。
*/
struct NexJoinRandomSessionSetting : public JoinRandomSessionSetting
{
#if NN_PIA_FRAMEWORK_ENABLE_NEX
    inet::NexJoinRandomSessionSetting nexJoinRandomSessionSetting;    //!< ランダムマッチメイク処理に関する設定です。
#endif
    /*!
    @brief デフォルトコンストラクタです。
    */
    NexJoinRandomSessionSetting()
        : JoinRandomSessionSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    virtual void Reset()
    {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
        JoinRandomSessionSetting::Reset();
        nexJoinRandomSessionSetting.Reset();
#endif
    }

    /*!
    @brief コピーします。
    */
    void Copy(const NexJoinRandomSessionSetting& setting)
    {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
        nexJoinRandomSessionSetting.Copy(setting.nexJoinRandomSessionSetting);
#else
        NN_PIA_UNUSED(setting);
#endif
    }

private:
    NexJoinRandomSessionSetting& operator=(const NexJoinRandomSessionSetting& rhs) const;
};

/*!
@brief lan を使用してセッションを構築する際に必要なデータを持つ構造体です。
*/
struct LanJoinRandomSessionSetting : public JoinRandomSessionSetting
{
    lan::LanJoinRandomSessionSetting lanJoinRandomSessionSetting;    //!< ランダムマッチメイク処理に関する設定です。

    /*!
    @brief デフォルトコンストラクタです。
    */
    LanJoinRandomSessionSetting()
        : JoinRandomSessionSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    virtual void Reset()
    {
        JoinRandomSessionSetting::Reset();
        lanJoinRandomSessionSetting.Reset();
    }

    /*!
    @brief コピーします。
    */
    void Copy(const LanJoinRandomSessionSetting& setting)
    {
        lanJoinRandomSessionSetting.Copy(setting.lanJoinRandomSessionSetting);
    }

private:
    LanJoinRandomSessionSetting& operator=(const LanJoinRandomSessionSetting& rhs) const;
};

#if NN_PIA_FRAMEWORK_ENABLE_PLAY
/*!
@brief gpg を使用してセッションを構築する際に必要なデータを持つ構造体です。
*/
struct PlayJoinRandomSessionSetting : public JoinRandomSessionSetting
{
    play::PlayJoinRandomSessionSetting playJoinRandomSessionSetting;    //!< ランダムマッチメイク処理に関する設定です。

    /*!
    @brief デフォルトコンストラクタです。
    */
    PlayJoinRandomSessionSetting()
        : JoinRandomSessionSetting()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    virtual void Reset()
    {
        JoinRandomSessionSetting::Reset();
        playJoinRandomSessionSetting.Reset();
    }

    /*!
    @brief コピーします。
    */
    void Copy(const PlayJoinRandomSessionSetting& setting)
    {
        playJoinRandomSessionSetting.Copy(setting.playJoinRandomSessionSetting);
    }

private:
    PlayJoinRandomSessionSetting& operator=(const PlayJoinRandomSessionSetting& rhs) const;
};
#endif

/*!
@brief レイテンシエミュレーション、およびパケロスエミュレーションに必要なデータを持つ構造体です。
*/
struct SetEmulationParamArg
{
    int32_t sendThreadPacketLossRatio;           //!< 送信スレッドがエミュレートするパケロスの確率を設定します。
    int32_t receiveThreadPacketLossRatio;        //!< 受信スレッドがエミュレートするパケロスの確率を設定します。
    int32_t sendThreadLatencyEmulationMin;       //!< 送信レイテンシエミュレーション時間の最小値を設定します（単位はミリ秒）。
    int32_t sendThreadLatencyEmulationMax;       //!< 送信レイテンシエミュレーション時間の最大値を設定します（単位はミリ秒）。
    int32_t receiveThreadLatencyEmulationMin;    //!< 受信レイテンシエミュレーション時間の最小値を設定します（単位はミリ秒）。
    int32_t receiveThreadLatencyEmulationMax;    //!< 受信レイテンシエミュレーション時間の最大値を設定します（単位はミリ秒）。

    /*!
    @brief デフォルトコンストラクタです。
    */
    SetEmulationParamArg()
    {
        Reset();
    }

    /*!
    @brief デフォルトの値を設定します。
    */
    void Reset()
    {
        sendThreadPacketLossRatio = 0;
        receiveThreadPacketLossRatio = 0;
        sendThreadLatencyEmulationMin = 0;
        sendThreadLatencyEmulationMax = 0;
        receiveThreadLatencyEmulationMin = 0;
        receiveThreadLatencyEmulationMax = 0;
    }

    /*!
    @brief コピーします。
    */
    void Copy(const SetEmulationParamArg& arg)
    {
        sendThreadPacketLossRatio = arg.sendThreadPacketLossRatio;
        receiveThreadPacketLossRatio = arg.receiveThreadPacketLossRatio;
        sendThreadLatencyEmulationMin = arg.sendThreadLatencyEmulationMin;
        sendThreadLatencyEmulationMax = arg.sendThreadLatencyEmulationMax;
        receiveThreadLatencyEmulationMin = arg.receiveThreadLatencyEmulationMin;
        receiveThreadLatencyEmulationMax = arg.receiveThreadLatencyEmulationMax;
    }

    /*!
    @brief 構造体の中身をトレースします。
    */
    void Trace()
    {
        PIA_TRACE_EX(common::TraceFlagFramework, "[SetEmulationParamArg] sendThreadPacketLossRatio:%d, receiveThreadPacketLossRatio:%d, sendThreadLatencyEmulationMin:%d, sendThreadLatencyEmulationMax:%d, receiveThreadLatencyEmulationMin:%d, receiveThreadLatencyEmulationMax%d", sendThreadPacketLossRatio, receiveThreadPacketLossRatio, sendThreadLatencyEmulationMin, sendThreadLatencyEmulationMax, receiveThreadLatencyEmulationMin, receiveThreadLatencyEmulationMax);
    }
};

/*!
@cond  PRIVATE
@brief CPU 負荷測定の設定を行う構造体です。
*/
struct InitializeTuneSetting
{
    bool isAutoInitialize;             //!< CpuProfiling を行うかどうか指定します。

    InitializeTuneSetting()
    {
        Reset();
    }

    void Reset()
    {
        isAutoInitialize = false;
    }
};
//! @endcond

/*!
@brief 処理の結果を示す Result にネットワークのエラーコードを持たせた構造体です。
*/
struct ResultInfo
{
    Result result;              //!< 処理の結果を格納します。
    uint32_t networkErrorCode;  //!< ネットワークエラーコードです。

    /*!
    @brief デフォルトコンストラクタです。
    */
    ResultInfo()
        : result(ResultSuccess()), networkErrorCode(0)
    {
    }

    /*!
    @brief デフォルトコンストラクタです。
    */
    ResultInfo(Result res, uint32_t code)
    {
        result = res;
        networkErrorCode = code;
    }
};

}
}
}  // end of namespace common
