﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/framework/framework_Definitions.h>

namespace nn
{
namespace pia
{
namespace framework
{
/*!
@brief inet モジュールの管理クラスです。
*/
class InetService
{
public:
    /*!
    @brief Initialize() に各種パラメータを渡すための構造体です。
    */
    struct InitializeInetServiceSetting
    {

    };

    InetService()
    {

    }

    virtual ~InetService()
    {

    }
    virtual Result LoginFriendServer() = 0;
    virtual Result LogoutFriendServer() = 0;

    virtual Result InitializeInet(const InitializeInetSetting& setting) = 0;
    virtual void FinalizeInet() = 0;

    virtual Result InitializeInetService(const InitializeInetServiceSetting& setting) = 0;
    virtual void FinalizeInetService() = 0;

    virtual Result Dispatch() = 0;
    virtual Result CheckConnection() = 0;
    virtual transport::NetworkFactory* GetNetworkFactory() = 0;

    virtual Result LoginGameServer(const InitializeInetSetting* pSetting, uint32_t timeout = 30 * 1000) = 0;
    virtual bool IsInLoginProcess() = 0;
    virtual bool IsInLogoutProcess() = 0;

    virtual Result PreLoginProcess(const InitializeInetSetting* pSetting) = 0;
    virtual bool IsInPreLoginProcess() = 0;

    virtual Result PostLoginProcess() = 0;
    virtual Result LogoutGameServer(bool isBlock, bool isInChangeStateAsync) = 0;
    virtual Result GetLogoutOutcome() = 0;
private:

    NN_PIA_DISALLOW_COPY(InetService);
};
}
}
}  // end of namespace nn::pia::framework
