﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/framework/framework_Definitions.h>

namespace nn
{
namespace pia
{
namespace framework
{
/*!
@brief ネットワーク機能の管理クラスです。
*/
class NetworkService
{
public:
    /*!
    @cond  PRIVATE
    @brief コンストラクタです。
    */
    NetworkService();
    //! @endcond

    /*!
    @cond  PRIVATE
    @brief デストラクタです。
    */
    ~NetworkService();
    //! @endcond

    /*!
    @brief  ネットワークインターフェースの初期化処理です。
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    Result InitializeNetworkInterface();

    /*!
    @brief  ソケットの初期化処理です。
    @return 成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    Result InitializeSocket(const StartupNetworkSetting& setting);

    /*!
    @brief  ネットワークインターフェースの終了処理です。
    */
    void FinalizeNetworkInterface();

    /*!
    @brief ソケットの終了処理です。
    */
    void FinalizeSocket();

    /*!
    @brief                  ローカルアドレスを取得します。
    @param[out] pIpAddr     IP アドレス値です。
    @param[out] pSubnetMask サブネットマスク値です。
    @return                 成功すれば、 IsSuccess() が true を返す Result が返されます。
    */
    Result GetLocalAddress(uint32_t* pIpAddr, uint32_t* pSubnetMask);

    bool IsNetworkInterfaceConnected();

private:
    bool m_IsNetworkIfInitialized;
    bool m_IsNetworkIfUp;
    bool m_IsInitializedSocket;
    static const int SocketSessionMax = 5;
    // ネットワークエラーコード
    uint32_t m_NetworkErrorCode;
#if NN_PIA_CTR
    nn::os::Event m_ACDisconnectedEvent;  // 切断通知を受ける Event
#elif NN_PIA_CAFE
    bool m_ACConnected;  // 接続状態フラグ
    bool m_UpdateACStatusRun;
#elif NN_PIA_NINTENDOSDK
#endif

    NN_PIA_DISALLOW_COPY(NetworkService);
};
}
}
}  // end of namespace nn::pia::framework
