﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>

#include <nn/pia/common/common_FixedString.h>
#include <nn/pia/session/session_JoinCommunitySetting.h>

namespace nn
{
namespace pia
{
namespace inet
{
/*!
  @brief  コミュニティ参加処理に関する設定を管理するクラスです。

  @see session::JoinCommunitySetting
 */
class NexJoinCommunitySetting : public nn::pia::session::JoinCommunitySetting
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexJoinCommunitySetting();

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexJoinCommunitySetting();

    /*!
    @brief      参加するコミュニティのユーザーパスワードを設定します。
    @param[in]  userPassword   設定するユーザーパスワード。パスワードの文字列は SessionUserPasswordLengthMax 以下である必要があります。
    @return     ユーザーパスワードの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはユーザーパスワードの最大長を超えています。[:progErr]
     */
    Result SetCommunityUserPassword(const common::String& userPassword);

    /*!
    @cond PRIVATE
    @brief 指定したユーザーパスワードを取得します。
    @return 指定したユーザーパスワードを返します。
    */
    const common::String& GetCommunityUserPassword() const;
    //! @endcond

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    NexJoinCommunitySetting& operator=(const NexJoinCommunitySetting& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexJoinCommunitySetting& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexJoinCommunitySetting(const NexJoinCommunitySetting& rhs);

    /*!
    @brief 設定をリセットします。
     */
    virtual void Reset();

    // デバッグに有用な情報をプリント。
    void Trace(uint64_t flag) const;

protected:
    common::FixedString<SessionUserPasswordLengthMax> m_UserPassword;

private:
};
}
}
}  // end of namespace nn::pia::inet

#endif
