﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/inet/inet_NexSessionTypes.h>
#include <nn/pia/session/session_SessionSearchCriteria.h>

namespace nn
{
namespace pia
{
namespace inet
{

/*!
  @brief  セッション ID を検索条件に指定するクラスです。
  @details nn::pia::session::Session::CreateInstance 呼び出し時に指定する
           nn::pia::session::Session::Setting 構造体の browsedSessionInfoListNum は 1 以上の値を
           指定する必要があります。

  @see nn::pia::session::SessionSearchCriteria
 */
class NexSessionSearchCriteriaSessionId : public nn::pia::session::SessionSearchCriteria
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    NexSessionSearchCriteriaSessionId();

    /*!
    @brief      コンストラクタ。
    @param[in]  sessionId    検索条件として指定するセッション ID。
    */
    NexSessionSearchCriteriaSessionId(uint32_t sessionId);

    /*!
    @brief      デストラクタ。
     */
    virtual ~NexSessionSearchCriteriaSessionId();

    /*!
    @brief      検索条件として指定するセッション ID をセットします。
    @param[in]  sessionId    検索条件として指定するセッション ID。
     */
    void SetSessionId(uint32_t sessionId);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
     */
    NexSessionSearchCriteriaSessionId& operator=(const NexSessionSearchCriteriaSessionId& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const NexSessionSearchCriteriaSessionId& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    NexSessionSearchCriteriaSessionId(const NexSessionSearchCriteriaSessionId& rhs);

    /*!
    @cond PRIVATE
     */
    uint32_t GetSessionId() const;
    //! @endcond

    /*!
    @brief      検索条件をリセットします。
     */
    virtual void Reset();

private:
    // SetResultRange は封印。
    virtual Result SetResultRange(uint32_t offset, uint32_t size);

    uint32_t m_SessionId;
};
}
}
}  // end of namespace nn::pia::inet

#endif
