﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <nn/pia/session/session_Session.h>

namespace nn
{
namespace pia
{
namespace inet
{
/*!
@brief  @ref nn::pia::session::Session::Startup に渡す設定用構造体です。
 */
struct NexSessionStartupSetting : public nn::pia::session::Session::StartupSetting
{
    NexSessionStartupSetting()
        : mtuSize(DefaultMtuSize),
          isPlayersCountedAsParticipants(true),
          isHostMigrationEnabled(true),
          isBitRateCheckSkipped(false),
          uplinkBitRateLowerLimit(session::SessionInvalidUplinkBitRateLowerLimit),
          bitRateMeasuringSpan(1000),
          bitRateCheckPacketSize(0),
          cryptoMode(common::CryptoSetting::Mode_Aes128),
          isAddRecentPlayRecordEnabled(true)
    {
    }

    uint32_t mtuSize;                                              //!< パケットの MTU を指定します。単位はバイトです。@ref inet::MtuSizeMin から @ref inet::MtuSizeMax までの値を指定します。
    transport::Station::PlayerInfo playerInfo[MaxPlayerInfoSize];  //!< 自分自身のステーションと関連付けるプレイヤー情報を設定します。他のセッション参加者から参照されます。@if NIN_DOC 設定された値が「いっしょにあそんだ人の記録」に使用されます。 @endif
    bool isPlayersCountedAsParticipants;                           //!< 自分自身のステーションに複数のプレイヤーを関連付ける際、プレイヤー数をセッション参加者数としてカウントするか指定します。デフォルトは true です。
    bool isHostMigrationEnabled;                                   //!< ホストマイグレーション機能を設定します。デフォルトは true (有効) になっています。
    bool isBitRateCheckSkipped;                                    //!< ビットレート判定機能を利用する際に、判定に関わる処理をスキップするかどうかを設定します。スキップする場合には true を設定します。デフォルトは false です。
    int32_t uplinkBitRateLowerLimit;                               //!< ビットレート判定機能を使用する際に、アプリケーションが要求する水準をこのメンバで指定します。単位は bps です。デフォルトは session::SessionInvalidUplinkBitRateLowerLimit です。
    int32_t bitRateMeasuringSpan;                                  //!< ビットレート判定機能を利用する際に、計測にかける時間を設定します。単位は msec です。デフォルトは 1000 msec です。デフォルト値からの変更はお勧めしません。
    uint32_t bitRateCheckPacketSize;                               //!< ビットレート判定機能を利用する際に、判定に用いる IP パケットサイズを指定します。単位はバイトです。アプリケーションが送受信する IP パケットの平均的なサイズを指定することを想定しています。4 の倍数でなくてはなりません。デフォルトは 0 です。
    common::CryptoSetting::Mode cryptoMode;                        //!< 暗号化設定モードを指定します。デフォルトは common::CryptoSetting::Mode_Aes128 です。
    bool isAddRecentPlayRecordEnabled;                             //!< セッション参加者を「いっしょにあそんだ人の記録」に追加するかどうかを指定します。デフォルトは true（有効）になっています。本体機能が「いっしょにあそんだ人の記録」に対応していない場合は無視されます。
};
}
}
}  // end of namespace nn::pia::inet

#endif
