﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/inet/inet_Definitions.h>

#if NN_PIA_ENABLE_NEX

#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>

namespace nn
{
namespace pia
{
namespace inet
{
const uint32_t InvalidSessionId = INVALID_GATHERINGID;
const uint16_t InvalidParticipantsNum = 0xffff;
#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 0, 2))
const uint32_t SessionInfoListSizeMax = 1000;       //!< 検索可能なセッション数の最大長
#else
const uint32_t SessionInfoListSizeMax = 300;        //!< 検索可能なセッション数の最大長
#endif
const uint32_t CommunityInfoListSizeMax = 100;      //!< 検索可能なコミュニティ数の最大長
#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 2, 0))
const uint32_t ParticipantArraySizeMax = nn::nex::MAX_PRINCIPALID_SIZE_TO_FIND_MATCHMAKE_SESSION;   //!< 検索条件に指定できるユーザーのプリンシパル ID の配列の最大長
#elif (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(4, 0, 2))
const uint32_t ParticipantArraySizeMax = 300;       //!< 検索条件に指定できるユーザーのプリンシパル ID の配列の最大長
#else
const uint32_t ParticipantArraySizeMax = 100;       //!< 検索条件に指定できるユーザーのプリンシパル ID の配列の最大長
#endif
const uint32_t AttributeSizeMax = 6;                //!< セッションに指定可能な属性の数
const uint32_t AttributeArraySizeMax = 100;         //!< 検索条件に指定できる属性値の配列の最大長
const uint32_t ApplicationDataBufferSizeMax = 512;  //!< アプリケーション定義のデータとして設定できる最大サイズ
const uint32_t DescriptionLengthMax = 256;          //!< セッションに設定できる説明文字列の最大長
const uint8_t SearchCriteriaListSizeMax = 2;        //!< ランダムマッチメイク時に設定できる検索条件の最大数
const uint32_t SessionMatchmakeKeywordLength = 32;  //!< あいことばマッチメイク時に設定できるあいことばの文字列長

const uint32_t SessionSystemPasswordLengthMax = nn::nex::MATCHMAKE_SESSION_SYSTEM_PASSWORD_LENGTH;                      //!< システムパスワードの文字列長
const uint32_t SessionUserPasswordLengthMax = nn::nex::MAX_MATCHMAKE_SESSION_USER_PASSWORD_LENGTH;                      //!< ユーザーパスワードの文字列長
const uint32_t SessionUpdateSelectionPriorityIntervalTimeMin = nn::nex::UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME;    //!< ゲーム進行度の最小アップデート間隔
const uint8_t SessionSelectionPriorityMax = nn::nex::MAX_PROGRESS_SCORE;                                                //!< 設定可能なゲーム進行度の最大値
const uint32_t SessionBroadenRangeAttributeIndex = nn::nex::MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX;            //!< レンジ拡大形式でセッションを選ぶ際の基準とする属性値のインデックス

const uint32_t AdditionalParamKeyLength = 8;
const uint32_t AdditionalParamKeySizeMax = 3;
const uint32_t AdditionalParamStringLength = 64;
const uint32_t AdditionalParamStringSizeMax = 1;

const uint32_t SearchCriteriaResultRandomOffset = 0xffffffff;
const uint32_t SearchCriteriaResultSizeMax = 100;

const uint32_t CountryCodeLengthMax = 2;            //!< @if NIN_DOC 国コードとして設定できる文字列長 @endif

#if NN_PIA_CTR
const uint32_t PrincipalIdArraySizeMax = 100; //!< ブロックリスト管理に指定可能なユーザーのプリンシパル ID の配列の最大長
#endif

/*!
    @brief  セッションタイプを表す列挙型です。
 */
enum SessionType
{
    SessionType_Anybody = 0   //!< 誰でも参加可能
#if NN_PIA_ENABLE_SESSION_TYPE_FRIEND
    ,
    SessionType_Friend       //!< ホストとフレンド関係を結んでいる場合のみ参加可能
#endif
};

/*!
    @brief  ランダムマッチメイク実行時に参加するセッションの選択方法を表す列挙型です。
 */
enum SelectionMethod
{
    SelectionMethod_Random = 0,  //!< ランダムに選択
    //SELECTION_METHOD_NEAREST_NEIGHBOR,                //!< 属性値が最も近いセッションを選択
    //SELECTION_METHOD_BROADEN_RANGE,                   //!< レンジ拡大形式で選択
    SelectionMethod_BroadenRangeWithSelectionPriority,  //!< レンジ拡大形式 + マッチメイク優先度により選択
    SelectionMethod_ScoreBased                          //!< スコア換算形式により選択
};

/*!
@cond PRIVATE
@brief  パラメータを格納する構造体です。
*/
struct ParamVariant
{
    enum ValueType
    {
        ValueType_Invalid = 0,
        ValueType_Uint64,
        ValueType_Int64,
        ValueType_Uint32,
        ValueType_Int32,
        ValueType_Bool,
        ValueType_Double
    };

    ParamVariant()
    {
        value.valUint64 = 0;
        valueType = ValueType_Invalid;
    }

    ParamVariant(uint64_t val)
    {
        value.valUint64 = val;
        valueType = ValueType_Uint64;
    }
    ParamVariant(int64_t val)
    {
        value.valInt64 = val;
        valueType = ValueType_Int64;
    }
    ParamVariant(uint32_t val)
    {
        value.valUint32 = val;
        valueType = ValueType_Uint32;
    }
    ParamVariant(int32_t val)
    {
        value.valInt32 = val;
        valueType = ValueType_Int32;
    }
    ParamVariant(bool val)
    {
        value.valBool = val;
        valueType = ValueType_Bool;
    }
    ParamVariant(double val)
    {
        value.valDouble = val;
        valueType = ValueType_Double;
    }

    union Value
    {
        uint64_t valUint64;
        int64_t valInt64;
        uint32_t valUint32;
        int32_t valInt32;
        bool valBool;
        double valDouble;
    } value;

    ValueType valueType;
};
//! @endcond
}
}
}  // end of namespace nn::pia::inet

#endif
