﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/lan/lan_Definitions.h>


// lan モジュールの非メンバ関数群です。

namespace nn
{
namespace pia
{
namespace lan
{


/*!
    @brief  lan モジュールが初期化された状態であるかどうかを返します。

    @return 初期化されていれば true、されていなければ false が返されます。
*/
bool IsInitialized(void);


/*!
    @brief  lan モジュールを初期化します。

    @return 無事に初期化が完了すれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultAlreadyInitialized  既に初期化は終了しています。[:progErr]
    @see    Setting
*/
Result Initialize();


/*!
    @brief  lanモジュールを終了します。
*/
void Finalize(void);


/*!
    @brief  セットアップの開始を宣言します。
    @details lanモジュールのシングルトンなどを作成する前に呼び出す必要があります。
            アプリケーションが lan モジュールのシングルトンを一つも作成しない場合であっても、
            このAPIと @ref EndSetup は必ず呼び出す必要があります。

    @return この API を適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized  lan モジュールの初期化関数が呼ばれていません。[:progErr]
    @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。[:progErr]
*/
Result BeginSetup(void);


/*!
    @brief  セットアップの終了を宣言します。
    @details lan モジュールのシングルトンなどを作成し終えた後に呼び出す必要があります。。
            この API 呼び出しにより、lan モジュールのシングルトンなどが消費する
            メモリを確定し、ライブラリ内部でのメモリ管理を最適化します。
            アプリケーションが lan モジュールのシングルトンを一つも作成しない場合であっても、
            @ref BeginSetup とこの API は必ず呼び出す必要があります。

    @return この API を適切なタイミングで呼び出していれば、成功の Result が返されます。この関数がエラーを返さないようにアプリケーションを実装する必要があります。
    @retval ResultNotInitialized  lan モジュールの初期化関数が呼ばれていません。[:progErr]
    @retval ResultInvalidState    この関数を呼び出すタイミングが誤っています。[:progErr]
*/
Result EndSetup(void);


/*!
    @cond PRIVATE
    @brief BeginSetup() ～ EndSetup()間の状態であるかどうかを返します。
    @endcond
*/
bool IsInSetupMode(void);
}
}
}  // end of namespace nn::pia::lan
