﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/lan/lan_Definitions.h>

#include <nn/pia/session/session_JoinRandomSessionSetting.h>
#include <nn/pia/lan/lan_LanCreateSessionSetting.h>
#include <nn/pia/lan/lan_LanSessionSearchCriteria.h>

namespace nn
{
namespace pia
{
namespace lan
{
/*!
  @brief  ランダムマッチメイク処理に関する設定を管理するクラスです。

  @see nn::pia::session::JoinRandomSessionSetting
 */
class LanJoinRandomSessionSetting : public session::JoinRandomSessionSetting
{
public:
    /*!
    @brief      デフォルトコンストラクタ。
     */
    LanJoinRandomSessionSetting();

    /*!
    @brief      デストラクタ。
     */
    virtual ~LanJoinRandomSessionSetting();

    /*!
    @brief      ランダムマッチメイクのセッション作成時の設定を指定します。
    @param[in]  pSetting セッション作成時の設定。
    */
    virtual Result SetCreateSessionSetting(const session::CreateSessionSetting* pSetting);

    /*!
    @brief      ランダムマッチメイクのセッション検索時の設定を指定します。
    @param[in]  pSearchCriteria     セッション検索設定のリスト。
    @param[in]  searchCriteriaSize  セッション検索設定のリストサイズ。LanSessionSearchCriteriaListSizeMax 以下である必要があります。
    */
    virtual Result SetSessionSearchCriteria(const session::SessionSearchCriteria* pSearchCriteria, uint32_t searchCriteriaSize);

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
    */
    LanJoinRandomSessionSetting& operator=(const LanJoinRandomSessionSetting& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const LanJoinRandomSessionSetting& rhs);

    /*!
    @brief コピーコンストラクタ。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    LanJoinRandomSessionSetting(const LanJoinRandomSessionSetting& rhs);

    /*!
    @brief      設定をリセットします。
    */
    virtual void Reset();

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    virtual void Trace(uint64_t flag) const;
    //! @endcond

    /*!
    @cond PRIVATE
    */
    virtual session::CreateSessionSetting* GetCreateSessionSetting();
    virtual session::SessionSearchCriteria* GetSessionSearchCriteria(uint8_t index);
    virtual const session::CreateSessionSetting* GetCreateSessionSetting() const;
    virtual const session::SessionSearchCriteria* GetSessionSearchCriteria(uint8_t index) const;
    virtual uint32_t GetSessionSearchCriteriaSize() const
    {
        return m_SessionSearchCriteriaSize;
    }
    //! @endcond

protected:

    LanCreateSessionSetting m_CreateSessionSetting;
    LanSessionSearchCriteria m_SessionSearchCriteria[LanSessionSearchCriteriaListSizeMax];
    uint32_t m_SessionSearchCriteriaSize;

private:
};
}
}
}  // end of namespace nn::pia::lan
