﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/lan/lan_Definitions.h>

#include <nn/pia/session/session_Session.h>

namespace nn
{
namespace pia
{
namespace lan
{
/*!
@brief  @ref nn::pia::session::Session::Startup に渡す設定用構造体です。
 */
struct LanSessionStartupSetting : public session::Session::StartupSetting
{
    LanSessionStartupSetting()
        : session::Session::StartupSetting(),
          isPlayersCountedAsParticipants(true),
          cryptoMode(common::CryptoSetting::Mode_Aes128)
    {
        maxSilenceTime = lan::MaxSilenceTime;
        keepAliveSendingInterval = lan::KeepAliveSendingInterval;
    }

    transport::Station::PlayerInfo playerInfo[MaxPlayerInfoSize];  //!< 自分自身のステーションと関連付けるプレイヤー情報を設定します。他のセッション参加者から参照されます。
    bool isPlayersCountedAsParticipants;                           //!< 自分自身のステーションに複数のプレイヤーを関連付ける際、プレイヤー数をセッション参加者数としてカウントするか指定します。デフォルトは true です。
    common::CryptoSetting::Mode cryptoMode;                        //!< 暗号化設定モードを指定します。デフォルトは common::CryptoSetting::Mode_Aes128 です。
};
}
}
}  // end of namespace nn::pia::lan
