﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/lan/lan_Definitions.h>
#include <nn/pia/session/session_SessionStationInfo.h>
#include <nn/pia/transport/transport_Station.h>

namespace nn
{
namespace pia
{
namespace lan
{

/*!
    @brief  セッションに参加中のステーション情報です。

    @see session::SessionStationInfo
 */
class LanStationInfo : public session::SessionStationInfo
{
public:
    LanStationInfo();
    virtual ~LanStationInfo();

    /*!
    @brief  ステーションのユーザー名を取得します。
    @return ステーションのユーザー名を返します。
    */
    const transport::Station::UserNameString& GetUserName() const;

    /*!
    @brief  StationId を取得します。
    @return 対応づけられている StationId が返されます。
    */
    StationId GetStationId() const;

    /*!
    @cond PRIVATE
    */
    void SetUserName(const transport::Station::UserNameString& name);
    void SetStationId(const StationId stationId);
    virtual void Clear();
    static uint32_t GetSerializedSize();
    //! @endcond

protected:
    transport::Station::UserNameString m_UserName;
    StationId m_StationId;                                         //!< 対応する StationId です。
};
}
}
}  // end of namespace lan
