﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Definitions.h>
#include <nn/pia/common/common_Trace.h>

// Pia ライブラリ開発者用
#if NN_PIA_CTR
#define NN_PIA_LOCAL_ENABLE_UDS 1
#define NN_PIA_LOCAL_ENABLE_LDN 0
#elif NN_PIA_NINTENDOSDK
#define NN_PIA_LOCAL_ENABLE_UDS 0
#define NN_PIA_LOCAL_ENABLE_LDN 1
#else
#define NN_PIA_LOCAL_ENABLE_UDS 0
#define NN_PIA_LOCAL_ENABLE_LDN 0
#endif

#define NN_PIA_EXPERIMENT_LDN_HOST_MIGRATION 1
#define NN_PIA_ENABLE_BACKGROUND_EVENT_CHECK 0

namespace nn
{
namespace pia
{
namespace local
{

/*!
@brief      通信が切断した理由を表します。
 */
enum DisconnectReason
{
    DisconnectReason_None = 0,              //!< まだ通信を開始していない、または接続は維持されています。
    DisconnectReason_RequestFromMyself,     //!< 自身の操作でネットワークから切断しました。
    DisconnectReason_RequestFromSystem,     //!< システムからの要求 (無線オフモードへの遷移、スリープ遷移) でネットワークから切断しました。
    DisconnectReason_DiscardedFromNetwork,  //!< ホストがネットワークを破棄しました。または、ホストからの指示でネットワークから追放されました。または、通信状況が悪化し、ホストの内部状態でクライアントが切断されたことになりました。
    DisconnectReason_ConnectionLost,        //!< 通信状況が悪化し、ホストからのビーコンが一定期間受信できず、接続が維持できなくなりました。
    DisconnectReason_Unknown                //!< 未知の理由で切断されました。
};


/*!
@cond PRIVATE
@brief      新規参加許可/拒否の状態を表します。
 */
enum AllowParticipatingState
{
    AllowParticipatingState_None = 0,              //!< ネットワークに接続していません。
    AllowParticipatingState_Allow,                 //!< 新規参加を許可しています。
    AllowParticipatingState_Disallow,              //!< 新規参加を拒否しています。
    AllowParticipatingState_AllowOnlyReconnection  //!< 再接続のみ許可しています。
};
//! @endcond

/*!
@cond PRIVATE
*/
enum InternalRole
{
    InternalRole_None = 0,
    InternalRole_Host,
    InternalRole_Client
};
//! @endcond

// Pia ライブラリ開発者用
enum LocalFunctionBit
{
    LocalFunctionBit_AroundNetwork = (0x01 << 0),
    LocalFunctionBit_HostMigration = (0x01 << 1)
};

#if NN_PIA_LOCAL_ENABLE_LDN
const uint32_t MtuSize = 1500;
#elif NN_PIA_LOCAL_ENABLE_UDS
const uint32_t MtuSize = 1478;
#else
const uint32_t MtuSize = 0;
#endif

const uint32_t MaxSilenceTime = 4000;           //!< ステーションとの通信が断絶したと判断する無通信時間 [msec] のデフォルト値です。
const uint32_t KeepAliveSendingInterval = 400;  //!< キープアライブの送信間隔 [msec] のデフォルト値です。
const int DefaultRttInitialPulseInterval = 600; //!< 初期段階での RTT 計測パルス送信間隔のデフォルト値です。単位はミリ秒です。
}
}
}  // end of namespace nn::pia::local
