﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>

#include <nn/pia/session/session_CreateSessionSetting.h>
#include <nn/pia/local/local_LocalSessionTypes.h>

#include <nn/pia/common/common_SignatureSetting.h>
#include <nn/pia/common/common_FixedString.h>

namespace nn
{
namespace pia
{
namespace local
{

// 先行宣言
class LocalCreateNetworkSetting;

/*!
  @brief  LocalFacade の CreateSession に関する設定を管理する基底クラスです。

  @details アプリケーションが本クラスのインスタンスを作成することはありません。
           本クラスを継承したクラスを使用してください。
  @if CTR_DOC
  @see nn::pia::local::UdsCreateSessionSetting
  @elseif NIN_DOC
  @see nn::pia::local::LdnCreateSessionSetting
  @endif
 */
class LocalCreateSessionSetting : public nn::pia::session::CreateSessionSetting
{
public:
    /*!
    @cond PRIVATE
    @brief      デストラクタ。
     */
    virtual ~LocalCreateSessionSetting();
    //! @endcond

    virtual const LocalCreateNetworkSetting* GetLocalCreateNetworkSetting() const = 0;

    /*!
    @brief  設定をリセットします。
     */
    virtual void Reset();

    /*!
    @brief      作成したセッションにユーザーパスワードを設定します。
    @details    初期値は空文字列です。設定されなかった場合や空文字列が設定された場合はユーザーパスワードの設定されていないセッションとなります。
    @param[in]  userPassword   設定するユーザーパスワード
    @return     ユーザーパスワードの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはユーザーパスワードの最大長を超えています。[:progErr]
    */
    virtual Result SetSessionUserPassword(const common::String& userPassword);
    
    /*!
    @cond PRIVATE
    */
    const common::String& GetSessionUserPassword() const;
    //bool IsSetSessionUserPassword() const;
    //! @endcond

    // デバッグに有用な情報をプリント。
    virtual void Trace(uint64_t flag) const;

protected:
    /*!
    @cond PRIVATE
    @brief      デフォルトコンストラクタ。
     */
    LocalCreateSessionSetting();
    //! @endcond

    void Copy(const LocalCreateSessionSetting& rhs);

private:
    NN_PIA_DISALLOW_COPY(LocalCreateSessionSetting);
    common::FixedString<SessionUserPasswordLengthMax> m_UserPassword;
};
}
}
}  // end of namespace nn::pia::local
