﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>

#include <nn/pia/session/session_JoinSessionSetting.h>
#include <nn/pia/local/local_LocalSessionTypes.h>

#include <nn/pia/common/common_SignatureSetting.h>
#include <nn/pia/common/common_FixedString.h>

namespace nn
{
namespace pia
{
namespace local
{

/*!
  @brief  LocalFacade の JoinSession に関する設定を管理する基底クラスです。

  @details アプリケーションが本クラスのインスタンスを作成することはありません。
           本クラスを継承したクラスを使用してください。

  @if CTR_DOC
  @see nn::pia::local::UdsJoinSessionSetting
  @elseif NIN_DOC
  @see nn::pia::local::LdnJoinSessionSetting
  @endif
 */
class LocalJoinSessionSetting : public nn::pia::session::JoinSessionSetting
{
public:
    /*!
    @cond PRIVATE
    @brief      デストラクタ。
     */
    virtual ~LocalJoinSessionSetting();
    //! @endcond

    /*!
    @cond PRIVATE
     */
    virtual const char* GetWirelessCryptoKey() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
     */
    uint8_t GetWirelessCryptoKeySize() const
    {
        return m_WirelessCryptoKeySize;
    }
    //! @endcond

    virtual Result SetWirelessCryptoKey(const char* pWirelessCryptoKey, uint8_t wirelessCryptoKeySize) = 0;

    /*!
    @cond PRIVATE
     */
    virtual uint32_t GetSessionId() const;
    //! @endcond

    /*!
    @brief      セッションのユーザーパスワードを設定します。
    @details    セッションにユーザーパスワードが設定されている場合に一致が確認されます。 初期状態は空文字列です。
    @param[in]  userPassword  ユーザーパスワード
    @return     ユーザーパスワードの指定に成功すれば、成功の Result が返されます。

    @retval     nn::pia::ResultInvalidArgument  文字列が NULL、文字コードが不正、またはユーザーパスワードの最大長を超えています。[:progErr]
    */
    virtual Result SetSessionUserPassword(const common::String& userPassword);

    /*!
    @cond PRIVATE
    */
    const common::String& GetSessionUserPassword() const;
    //bool IsSetSessionUserPassword() const;
    //! @endcond

    /*!
    @brief  設定をリセットします。
     */
    virtual void Reset();

protected:
    uint8_t m_WirelessCryptoKeySize;

    /*!
    @cond PRIVATE
    @brief      デフォルトコンストラクタ。
     */
    LocalJoinSessionSetting();
    //! @endcond

private:
    // コピーコンストラクタは封印。
    LocalJoinSessionSetting(const LocalJoinSessionSetting&);

    // 代入演算子は封印。
    LocalJoinSessionSetting& operator=(const LocalJoinSessionSetting&);

    common::FixedString<SessionUserPasswordLengthMax> m_UserPassword;
};
}
}
}  // end of namespace nn::pia::local
