﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>

#include <nn/pia/session/session_Session.h>
#include <nn/pia/common/common_SignatureSetting.h>

namespace nn
{
namespace pia
{
namespace local
{

/*!
@brief  @ref nn::pia::session::Session::Startup に渡す設定用構造体です。
 */
struct LocalSessionStartupSetting : public nn::pia::session::Session::StartupSetting
{
    LocalSessionStartupSetting()
        : playerInfo(),
        signatureKeyDataSize(0),
        isAddRecentPlayRecordEnabled(true)
    {
        maxSilenceTime = local::MaxSilenceTime;
        keepAliveSendingInterval = local::KeepAliveSendingInterval;
        memset(signatureKeyData, 0, sizeof(signatureKeyData));
    }

    transport::Station::PlayerInfo playerInfo;                                 //!< 自分自身のステーションと関連付けるプレイヤー情報を設定します。他のセッション参加者から参照されます。@if NIN_DOC 設定された値が「いっしょにあそんだ人の記録」に使用されます。 @endif
    char signatureKeyData[common::SignatureSetting::SignatureKeyDataSizeMax];  //!< 署名鍵です。
    uint32_t signatureKeyDataSize;                                             //!< 署名鍵のサイズです。署名を無効にする場合は 0 を指定する必要があります。@ref common::SignatureSetting::SignatureKeyDataSizeMax 以下である必要があります。
    bool isAddRecentPlayRecordEnabled;                                         //!< セッション参加者を「いっしょにあそんだ人の記録」に追加するかどうかを指定します。デフォルトは true（有効）になっています。本体機能が「いっしょにあそんだ人の記録」に対応していない場合は無視されます。
};
}
}
}  // end of namespace nn::pia::local
