﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#include <nn/pia/local/local_LocalSessionTypes.h>

#include <nn/pia/session/session_UpdateSessionSetting.h>

namespace nn
{
namespace pia
{
namespace local
{

/*!
  @brief   セッションの条件更新処理に関する設定を管理するクラスです。
  @details アプリケーションが本クラスのインスタンスを作成することはありません。 本クラスを継承したクラスを使用してください。

  @if CTR_DOC
  @see local::UdsUpdateSessionSetting
  @elseif NIN_DOC
  @see local::LdnUpdateSessionSetting
  @endif
 */

class LocalUpdateSessionSetting : public session::UpdateSessionSetting
{
public:
    /*!
    @brief    デストラクタ
     */
    virtual ~LocalUpdateSessionSetting();

    /*!
    @cond PRIVATE
    @brief      アプリケーション定義データのバッファへのポインタを取得します。
    @return     アプリケーション定義データのバッファへのポインタが返されます。
    */
    virtual const uint8_t* GetApplicationDataPtr() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      アプリケーション定義データのバッファサイズを取得します。
    @return     アプリケーション定義データのバッファサイズが返されます。
    */
    virtual uint32_t GetApplicationDataSizeMax() const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    @brief      指定したアプリケーション定義のデータサイズを取得します。
    @return     アプリケーション定義データサイズが返されます。
    */
    virtual uint32_t GetApplicationDataSize() const;
    //! @endcond

    /*!
    @brief 設定をリセットします。
    */
    virtual void Reset();

protected:
    // コンストラクタ。
    LocalUpdateSessionSetting();

    void Copy(const LocalUpdateSessionSetting& rhs);

    uint32_t m_ApplicationDataSize;

private:
    NN_PIA_DISALLOW_COPY(LocalUpdateSessionSetting);
};
}
}
}  // end of namespace local
