﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#if NN_PIA_LOCAL_ENABLE_UDS
#include <nn/pia/local/local_LocalJoinSessionSetting.h>

namespace nn
{
namespace pia
{
namespace local
{
/*!
  @brief  UDS ライブラリ（ローカル通信）を利用したネットワークの JoinSession に関する設定を管理するクラスです。
 */

class UdsJoinSessionSetting : public LocalJoinSessionSetting
{
public:
    //! 無線レイヤの暗号化に使用する暗号鍵の最小サイズです。
    static const uint8_t WirelessCryptoKeyLengthMin = nn::uds::UDS_PASSPHRASE_LENGTH_MIN;

    //! 無線レイヤの暗号化に使用する暗号鍵の最大サイズです。
    static const uint8_t WirelessCryptoKeyLengthMax = nn::uds::UDS_PASSPHRASE_LENGTH_MAX;

    /*!
    @brief  デフォルトコンストラクタ。
     */
    UdsJoinSessionSetting();

    /*!
    @brief  デストラクタ。
     */
    virtual ~UdsJoinSessionSetting() {};

    /*!
    @cond PRIVATE
     */
    virtual const char* GetWirelessCryptoKey() const
    {
        return m_WirelessCryptoKey;
    }
    //! @endcond

    /*!
    @brief  無線レイヤの暗号化に使用する暗号鍵とそのサイズを設定します。

    @details 暗号鍵のサイズは、@ref WirelessCryptoKeyLengthMin 以上、 @ref WirelessCryptoKeyLengthMax 以下である必要があります。
            
    @param[in]  pWirelessCryptoKey 無線レイヤの暗号化に使用する暗号鍵です。
    @param[in]  wirelessCryptoKeySize  無線レイヤの暗号化に使用する暗号鍵のサイズです。

    @return     成功すれば、 IsSuccess() が true を返す Result が返されます。
    
    @retval ResultInvalidArgument   引数の指定が不正です。[:progErr]
     */
    virtual Result SetWirelessCryptoKey(const char* pWirelessCryptoKey, uint8_t wirelessCryptoKeySize);

    /*!
    @brief  設定をリセットします。
     */
    virtual void Reset();

protected:
    char m_WirelessCryptoKey[WirelessCryptoKeyLengthMax];

private:
    // コピーコンストラクタは封印。
    UdsJoinSessionSetting(const UdsJoinSessionSetting&);

    // 代入演算子は封印。
    UdsJoinSessionSetting& operator=(const UdsJoinSessionSetting&);
};
}
}
}  // end of namespace nn::pia::local
#endif
