﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/local/local_Definitions.h>
#if NN_PIA_LOCAL_ENABLE_UDS
#include <nn/pia/local/local_LocalSessionSearchCriteria.h>
#include <nn/pia/local/local_UdsNetworkSetting.h>

namespace nn
{
namespace pia
{
namespace local
{
/*!
  @brief  UDS ライブラリ（ローカル通信）を利用したネットワークのセッションを検索する際に使用する検索条件のクラスです。
  @details 検索結果として得られるセッションの最大数は nn::pia::session::Session::CreateInstance 呼び出し時に
           指定する nn::pia::session::Session::Setting 構造体の browsedSessionInfoListNum の値になります。
 */
class UdsSessionSearchCriteria : public LocalSessionSearchCriteria
{
public:
    // コンストラクタ。
    UdsSessionSearchCriteria()
        : LocalSessionSearchCriteria(),
          m_UdsScanNetworkSetting()
    {
    }

    // デストラクタ。
    virtual ~UdsSessionSearchCriteria()
    {
    }

    /*!
    @brief      検索条件にローカル通信 ID を指定します。
    @param[in]  localCommunicationId 検索条件に指定するローカル通信 ID です。@ref LocalFacade::CreateLocalCommunicationId で生成した値を指定する必要があります。
    */
    void SetLocalCommunicationId(uint32_t localCommunicationId)
    {
        m_UdsScanNetworkSetting.localCommunicationId = localCommunicationId;
    }

    /*!
    @brief      検索条件に通信モード識別用 ID を指定します。
    @param[in]  subId   検索条件に指定する通信モード識別用 ID です。0xff を指定するとあらゆる ID を探索します。
    */
    void SetSubId(uint8_t subId)
    {
        m_UdsScanNetworkSetting.subId = subId;
    }

    virtual const LocalScanNetworkSetting* GetLocalScanNetworkSetting() const
    {
        return &m_UdsScanNetworkSetting;
    }

    /*!
    @cond PRIVATE
     */
    virtual bool IsHit(const LocalNetworkDescription* pDescription) const;
    //!@endcond

    /*!
    @brief      代入演算子です。
    @param[in]  rhs 代入するオブジェクトです。
    @return     本オブジェクトへの参照。
     */
    UdsSessionSearchCriteria& operator=(const UdsSessionSearchCriteria& rhs);

    /*!
    @brief 設定をコピーします。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    void Copy(const UdsSessionSearchCriteria& rhs);

    /*!
    @brief コピーコンストラクタ。。
    @param[in]  rhs コピー元のオブジェクトです。
    */
    UdsSessionSearchCriteria(const UdsSessionSearchCriteria& rhs);

    /*!
    @brief  設定をリセットします。
     */
    virtual void Reset();

    // デバッグに有用な情報をプリント。
    virtual void Trace(uint64_t flag) const
    {
        NN_PIA_UNUSED(flag);
    }

protected:
    UdsScanNetworkSetting m_UdsScanNetworkSetting;

private:
};
}
}
}  // end of namespace nn::pia::local
#endif
