﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Definitions.h>
#include <nn/pia/pia_Assert.h>

//n1589:EDMのポートインクリメントを1に固定する（1以外は発見されたことはない）
#define NN_PIA_ENABLE_EDM_PORT_INCREMENT_FIX 1

//n1872:EDMのポート予測時に予測ポートの前後も NAT トラバーサルを試みる
#define NN_PIA_ENABLE_EDM_PORT_SPRAY 1

//n1872: ヘアピン環境のテストを強制的に再現させるためのマクロ
#define NN_PIA_EXPERIMENT_FORCED_HAIR_PIN 0
//n1589:ヘアピンエミュレーション
#define NN_PIA_EXPERIMENT_PROBE_LIST_HAIR_PIN_EMULATION 0

namespace nn
{
namespace pia
{
namespace nat
{
/*!
@cond PRIVATE
@brief    NAT ポートマッピングのタイプ
*/
enum Mapping
{
    Mapping_Unknown = 0,  //!< 不明なタイプです
    Mapping_Eim,          //!< EIM（Endpoint Independent Mapping）です
    Mapping_Edm           //!< EDM (Endpoint Dependent Mapping, Symmetric）です
};
//! @endcond

/*!
@cond PRIVATE
@brief    NAT フィルタリングのタイプ
*/
enum Filtering
{
    Filtering_Unknown = 0,  //!< 不明なタイプです
    Filtering_Pif,          //!< PIF（Port Independent Filtering）です
    Filtering_Pdf           //!< PDF（Port Dependent Filtering）です
};
//! @endcond

/*!
@cond PRIVATE
@brief    NAT プロパティの構造体です。
*/
struct NatProperty
{
public:
    NatProperty() :
        interfacePort(0),
        publicPort(0),
        mapping(Mapping_Unknown),
        filtering(Filtering_Unknown),
        portIncrement(0),
        isPd(false)
    {
    };


public:
    uint16_t interfacePort;
    uint16_t publicPort;
    Mapping mapping;
    Filtering filtering;
    uint32_t portIncrement;
    bool isPd;
};
//! @endcond

const uint32_t NatCheckServerAddressListSize = 4;
const uint32_t NatCheckProbeDuplicationNum = 5; //!< NatCheckProbe は複製して送信する。その個数。
}
}
}  // end of namespace nn::pia::nat

#include <nn/pia/common/common_RootObject.h>
#include <nn/pia/common/common_Report.h>
#include <nn/pia/common/common_Trace.h>
