﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Definitions.h>

namespace nn
{
namespace pia
{
/*!
  @cond PRIVATE
  @brief デバッグ用の機能を扱うクラスです。
 */
class Debug
{
#if NN_PIA_CAFE
public:
    /*!
          @brief   ネットワークインターフェースを停止します。
          @details 本 API は、通信時にネットワークインターフェースが異常切断された場合の
                   ハンドリングをデバッグするために使用してください。

                   ネットワークインターフェースを停止した場合、自動的に再起動されます。
                   しかしながら、停止時に通信関係のライブラリは継続して動作できない状態に陥るため、
                   再起動後にそれらライブラリの再起動も必要になりますので、注意してください。
    */
    static void ControlNetworkIfDown();
#elif NN_PIA_NINTENDOSDK
public:
    /*!
    @cond PRIVATE
    @brief  ネットワークインターフェースの種類
    */
    enum NetworkInterfaceType
    {
        NetworkInterfaceType_Invalid = 0,
        NetworkInterfaceType_Ieee80211,
        NetworkInterfaceType_Ethernet
    };
    //! @endcond

    /*!
    @cond PRIVATE
    @brief              （デバッグ用）ネットワークインターフェースを停止します。
    @details            本 API は、通信時にネットワークインターフェースが異常切断された場合のハンドリングをデバッグするために使用してください。
    @param[in] type     停止するネットワークインターフェースの種類
    */
    static void NetworkInterfaceDown(NetworkInterfaceType type);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief              （デバッグ用）ネットワークインターフェースを起動します。
    @details            本 API は、通信時にネットワークインターフェースを異常切断した後、復帰させるために使用してください。
    @param[in] type     起動するネットワークインターフェースの種類
    */
    static void NetworkInterfaceUp(NetworkInterfaceType type);
    //! @endcond

    /*!
    @cond PRIVATE
    @brief              （デバッグ用）ネットワークインターフェースの起動状態を取得します。
    @param[in] type      起動状態を取得するネットワークインターフェースの種類
    @return              ネットワークインターフェースの起動状態を返します。起動状態の場合は true を返します。
    */
    static bool IsNetworkInterfaceUp(NetworkInterfaceType type);
    //! @endcond

private:
    static void ControlNetworkInterface(NetworkInterfaceType type, bool isEnabled);
#endif
};
//! @endcond
}
}  // end of namespace nn::pia
