﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Settings.h>


/*!
    @brief 未使用変数に対して発せられるWarningを抑制するためのマクロです。
*/
#define NN_PIA_UNUSED(var) ((void)&var)


/*!
    @brief 未使用変数に対して発せられるWarningを抑制するためのマクロですが、
           リリースビルド時のみ機能します。
           例えばアサートでのみチェックしている変数に対してWarningが通知されてしまう
           煩雑さを解消するために用意しました。
*/
#if NN_PIA_RELEASE
#define NN_PIA_UNUSED_IN_RELEASE(var) ((void)&var)
#else
#define NN_PIA_UNUSED_IN_RELEASE(var)
#endif


/*!
    @brief  コピーコンストラクタとコピー代入演算子を禁止します。
            引数にはクラス名を記述します。
            このマクロを使用する事でアクセス制限が private に変化するので、
            クラス定義の最後尾に記述するのが良いかもしれません。
*/
#define NN_PIA_DISALLOW_COPY(type) \
    \
private:                           \
    type(const type&);             \
    type& operator=(const type&)


// warning抑制マクロ
#if NN_PIA_NINTENDOSDK_NX || NN_PIA_A || NN_PIA_B
#define NN_PIA_PRAGMA_PUSH_WARNINGS _Pragma("clang diagnostic push")
#define NN_PIA_PRAGMA_POP_WARNINGS _Pragma("clang diagnostic pop")
#define NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF _Pragma("clang diagnostic ignored  \"-Winvalid-offsetof\"")
#define NN_PIA_DISABLE_WARNING_CLANG_FORMAT_SECURITY _Pragma("clang diagnostic ignored  \"-Wformat-security\"")
#define NN_PIA_DISABLE_WARNING_CLANG_UNUSED_PRIVATE_FIELD _Pragma("clang diagnostic ignored  \"-Wunused-private-field\"")
#define NN_PIA_DISABLE_WARNING_CLANG_UNUSED_VARIABLE _Pragma("clang diagnostic ignored  \"-Wunused-variable\"")
#define NN_PIA_DISABLE_WARNING_CLANG_UNUSED_FUNCTION _Pragma("clang diagnostic ignored  \"-Wunused-function\"")
#define NN_PIA_DISABLE_WARNING_CLANG_UNINITIALIZED _Pragma("clang diagnostic ignored  \"-Wuninitialized\"")
#define NN_PIA_DISABLE_WARNING_WIN_4005
#define NN_PIA_DISABLE_WARNING_WIN_4307
#define NN_PIA_DISABLE_WARNING_WIN_4355
#define NN_PIA_DISABLE_WARNING_WIN_4996
#elif NN_PIA_NINTENDOSDK_WIN || NN_PIA_WIN
#define NN_PIA_PRAGMA_PUSH_WARNINGS __pragma(warning(push))
#define NN_PIA_PRAGMA_POP_WARNINGS __pragma(warning(pop))
#define NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
#define NN_PIA_DISABLE_WARNING_CLANG_FORMAT_SECURITY
#define NN_PIA_DISABLE_WARNING_CLANG_UNUSED_PRIVATE_FIELD
#define NN_PIA_DISABLE_WARNING_CLANG_UNUSED_VARIABLE
#define NN_PIA_DISABLE_WARNING_CLANG_UNUSED_FUNCTION
#define NN_PIA_DISABLE_WARNING_CLANG_UNINITIALIZED
#define NN_PIA_DISABLE_WARNING_WIN_4005 __pragma(warning(disable : 4005))
#define NN_PIA_DISABLE_WARNING_WIN_4307 __pragma(warning(disable : 4307))
#define NN_PIA_DISABLE_WARNING_WIN_4355 __pragma(warning(disable : 4355))
#define NN_PIA_DISABLE_WARNING_WIN_4996 __pragma(warning(disable : 4996))
#else
#define NN_PIA_PRAGMA_PUSH_WARNINGS
#define NN_PIA_PRAGMA_POP_WARNINGS
#define NN_PIA_DISABLE_WARNING_CLANG_INVALID_OFFSETOF
#define NN_PIA_DISABLE_WARNING_CLANG_FORMAT_SECURITY
#define NN_PIA_DISABLE_WARNING_CLANG_UNUSED_PRIVATE_FIELD
#define NN_PIA_DISABLE_WARNING_CLANG_UNUSED_VARIABLE
#define NN_PIA_DISABLE_WARNING_CLANG_UNUSED_FUNCTION
#define NN_PIA_DISABLE_WARNING_CLANG_UNINITIALIZED
#define NN_PIA_DISABLE_WARNING_WIN_4005
#define NN_PIA_DISABLE_WARNING_WIN_4307
#define NN_PIA_DISABLE_WARNING_WIN_4355
#define NN_PIA_DISABLE_WARNING_WIN_4996
#endif
