﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once


#define NN_PIA_PLATFORM_WIN (1)
#define NN_PIA_PLATFORM_CTR (2)
#define NN_PIA_PLATFORM_CAFE (3)
#define NN_PIA_PLATFORM_NINTENDOSDK (4)
#define NN_PIA_PLATFORM_A (5)
#define NN_PIA_PLATFORM_B (6)

#define NN_PIA_BUILD_DEBUG (1)
#define NN_PIA_BUILD_DEVELOP (2)
#define NN_PIA_BUILD_RELEASE (3)

#if defined(NN_NINTENDO_SDK)
// ※_WIN32判定の前にNN_NINTENDO_SDK判定すること
#define NN_PIA_PLATFORM NN_PIA_PLATFORM_NINTENDOSDK

#if defined(NN_SDK_BUILD_DEBUG)
#define NN_PIA_BUILD NN_PIA_BUILD_DEBUG
#elif defined(NN_SDK_BUILD_DEVELOP)
#define NN_PIA_BUILD NN_PIA_BUILD_DEVELOP
#elif defined(NN_SDK_BUILD_RELEASE)
#define NN_PIA_BUILD NN_PIA_BUILD_RELEASE
#else
#error "PIA_BUILD_TARGET_XXX is not defined."
#endif

// NN_PIA_NINTENDOSDKのときの更なる分岐マクロ
#if defined(_WIN32)
#define NN_PIA_NINTENDOSDK_WIN 1
#define NN_PIA_NINTENDOSDK_NX 0
#else
#define NN_PIA_NINTENDOSDK_WIN 0
#define NN_PIA_NINTENDOSDK_NX 1
#endif

#elif defined(NN_PLATFORM_CTR)

#define NN_PIA_PLATFORM NN_PIA_PLATFORM_CTR

#if defined(NN_BUILD_NOOPT)
#define NN_PIA_BUILD NN_PIA_BUILD_DEBUG
#elif defined(NN_BUILD_VERBOSE)
#define NN_PIA_BUILD NN_PIA_BUILD_DEVELOP
#else
#define NN_PIA_BUILD NN_PIA_BUILD_RELEASE
#endif

#elif defined(PLATFORM) && (PLATFORM == CAFE)

#define NN_PIA_PLATFORM NN_PIA_PLATFORM_CAFE

//n1769 PIA_BUILD_TARGET_* は、
//ビルドシステムの commondefs.mk によって定義されます。
#if defined(PIA_BUILD_TARGET_DEBUG)
#define NN_PIA_BUILD NN_PIA_BUILD_DEBUG
#elif defined(PIA_BUILD_TARGET_DEVELOP)
#define NN_PIA_BUILD NN_PIA_BUILD_DEVELOP
#elif defined(PIA_BUILD_TARGET_RELEASE)
#define NN_PIA_BUILD NN_PIA_BUILD_RELEASE
#else
#error "PIA_BUILD_TARGET_XXX is not defined."
#endif

#elif defined(_WIN32)
// CTR-VSIが_WIN32マクロを勘違いしてインテリセンスに反映するので
// WINプラットフォーム判定はCTR判定のあとに書くこと
#define NN_PIA_PLATFORM NN_PIA_PLATFORM_WIN

#if defined(PIA_BUILD_TARGET_DEBUG)
#define NN_PIA_BUILD NN_PIA_BUILD_DEBUG
#elif defined(PIA_BUILD_TARGET_DEVELOP)
#define NN_PIA_BUILD NN_PIA_BUILD_DEVELOP
#elif defined(PIA_BUILD_TARGET_RELEASE)
#define NN_PIA_BUILD NN_PIA_BUILD_RELEASE
#else
#error "PIA_BUILD_TARGET_XXX is not defined."
#endif

#elif defined(ANDROID)

#define NN_PIA_PLATFORM NN_PIA_PLATFORM_A
#if defined(PIA_BUILD_TARGET_DEBUG)
#define NN_PIA_BUILD NN_PIA_BUILD_DEBUG
#elif defined(PIA_BUILD_TARGET_DEVELOP)
#define NN_PIA_BUILD NN_PIA_BUILD_DEVELOP
#elif defined(PIA_BUILD_TARGET_RELEASE)
#define NN_PIA_BUILD NN_PIA_BUILD_RELEASE
#else
#error "PIA_BUILD_TARGET_XXX is not defined."
#endif

#elif defined(__APPLE__)

#define NN_PIA_PLATFORM NN_PIA_PLATFORM_B
#if defined(PIA_BUILD_TARGET_DEBUG)
#define NN_PIA_BUILD NN_PIA_BUILD_DEBUG
#elif defined(PIA_BUILD_TARGET_DEVELOP) || defined(UE_BUILD_DEVELOPMENT)
#define NN_PIA_BUILD NN_PIA_BUILD_DEVELOP
#elif defined(PIA_BUILD_TARGET_RELEASE)
#define NN_PIA_BUILD NN_PIA_BUILD_RELEASE
#else
#error "PIA_BUILD_TARGET_XXX is not defined."
#endif

#else
#error "This platform is not supported."
#endif


#define NN_PIA_CTR (NN_PIA_PLATFORM == NN_PIA_PLATFORM_CTR)
#define NN_PIA_WIN (NN_PIA_PLATFORM == NN_PIA_PLATFORM_WIN)
#define NN_PIA_CAFE (NN_PIA_PLATFORM == NN_PIA_PLATFORM_CAFE)
#define NN_PIA_NINTENDOSDK (NN_PIA_PLATFORM == NN_PIA_PLATFORM_NINTENDOSDK)
#define NN_PIA_A (NN_PIA_PLATFORM == NN_PIA_PLATFORM_A)
#define NN_PIA_B (NN_PIA_PLATFORM == NN_PIA_PLATFORM_B)
#if !NN_PIA_NINTENDOSDK
#define NN_PIA_NINTENDOSDK_WIN 0
#define NN_PIA_NINTENDOSDK_NX 0
#endif

#define NN_PIA_DEBUG (NN_PIA_BUILD == NN_PIA_BUILD_DEBUG)
#define NN_PIA_DEVELOP (NN_PIA_BUILD == NN_PIA_BUILD_DEVELOP)
#define NN_PIA_RELEASE (NN_PIA_BUILD == NN_PIA_BUILD_RELEASE)

#define NN_PIA_UNDER_DEBUG (NN_PIA_BUILD <= NN_PIA_BUILD_DEBUG)
#define NN_PIA_UNDER_DEVELOP (NN_PIA_BUILD <= NN_PIA_BUILD_DEVELOP)

#define NN_PIA_ENABLE_TRACE (NN_PIA_UNDER_DEVELOP)
#define NN_PIA_ENABLE_ASSERT (NN_PIA_UNDER_DEVELOP)
