﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

// プラットフォーム毎に定義を移しました。
#include <nn/pia/pia_Platform.h>
#include <stdint.h>


namespace nn
{
namespace pia
{

typedef uint64_t PrincipalId;  //!< プリンシパル ID

/*!
@brief      セッション内の Station 識別子 StationId の定義です。
@details    セッションに参加中のステーション毎に固有の値が割り当てられます。
 */
typedef uint64_t StationId;

// StationIndex を StationId に変換する際、値を StationIdInvalid (= 0) と
// バッティングさせないことを目的としたオフセット値です。
const uint64_t StationIdOffset = 1;

// NEX において、無効であることが保証されている PID が 0 であることに倣っています。
const uint64_t StationIdInvalid = 0;  //!< 無効な StationId を表す定数です。

typedef uint32_t ServerConnectionId;
const ServerConnectionId ServerConnectionIdInvalid = 0;


/*!
 @brief Pia の各モジュールを示す列挙型です。
 */
enum ModuleType
{
    ModuleType_Reserved = 0,  //!< 予約値です。
    ModuleType_Pia,           //!< Pia のモジュールには属さないタイプであることを示します。
    ModuleType_Common,        //!< PiaCommon のモジュールであることを示します。
    ModuleType_Local,         //!< PiaLocal のモジュールであることを示します。
    ModuleType_Transport,     //!< PiaTransport のモジュールであることを示します。
    ModuleType_Sync,          //!< PiaSync のモジュールであることを示します。
    ModuleType_Inet,          //!< PiaInet のモジュールであることを示します。
    ModuleType_Session,       //!< PiaSession のモジュールであることを示します。
    ModuleType_Util,          //!< @cond PRIVATE PiaUtil のモジュールであることを示します（廃止されました）。@endcond
    ModuleType_Service,       //!< @cond PRIVATE PiaService のモジュールであることを示します。@endcond
    ModuleType_Db,            //!< @cond PRIVATE PiaDB のモジュールであることを示します。@endcond
    ModuleType_Chat,          //!< PiaChat のモジュールであることを示します。
    ModuleType_Clone,         //!< PiaClone のモジュールであることを示します。
    ModuleType_Reckoning,     //!< PiaReckoning のモジュールであることを示します。
    ModuleType_Brain,         //!< @cond PRIVATE PiaBrain のモジュールであることを示します。@endcond
    ModuleType_Framework,     //!< @cond PRIVATE PiaFramework のモジュールであることを示します。@endcond
    ModuleType_Lan,           //!< PiaLan のモジュールであることを示します。
    ModuleType_Tune,          //!< PiaTune のモジュールであることを示します。
    ModuleType_Play,          //!< @cond PRIVATE PiaPlay のモジュールであることを示します。@endcond
    ModuleType_Direct,        //!< @cond PRIVATE PiaDirect のモジュールであることを示します。@endcond
    ModuleType_Nat,           //!< @cond PRIVATE PiaNat のモジュールであることを示します。@endcond
    ModuleType_Star,          //!< @cond PRIVATE PiaStar のモジュールであることを示します。@endcond
    ModuleType_Max            //!< 最大値です。アプリケーションがこの値を利用する必要はありません。
};
}
}  // end of namespace nn::pia
