﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/types.h>
#include <nn/Result.h>
#include <nn/pia/platform/pia_BasicMacrosCtr.h>
#include <nn/uds.h>
#include <nn/version.h>

#define NN_PIA_ENDIAN_BIG 0
#define NN_PIA_ENABLE_NEX 1
#define NN_PIA_ENABLE_WINSOCK 0
#define NN_PIA_ENABLE_CRYPT 1
#define NN_PIA_ENABLE_CACHEDPRINT 1
#define NN_PIA_ENABLE_MONITORING 1
#define NN_PIA_EXPERIMENT_SERVICE_NODE 0

namespace nn
{
namespace pia
{
//! @cond CTR_DOC

typedef wchar_t charUtf16_t;

const uint16_t MaxStationNum = 12;                                          //!< Piaのセッションに参加可能なStation数の最大値です。
const uint32_t MaxUserNameLength = nn::cfg::CTR::CFG_USER_NAME_LENGTH - 1;  //!< ユーザー名の最大文字数です。（NULL終端文字は含みません）
const uint32_t MaxPlayerNameLength = 20;                                    //!< プレイヤー名の最大文字数です。
const uint8_t MaxPlayerInfoSize = 1;                                        //!< プレイヤー情報の最大サイズです。

/*!
    @brief StationIndex の列挙型です。
    
    @details Pia の各モジュールは、ネットワークの種別に依らずセッション内の
           Station の識別子として StationIndex を使用します。
           この ID は Station 毎の情報をアプリケーションが配列として扱えることを
           意図して作られています。

           Pia ライブラリがサポートする最大同時接続数が変更される度に最大値は
           変化します。

 */
enum StationIndex
{
    StationIndex_1 = 0,                  //!< Player  1 を示す ID です
    StationIndex_2,                      //!< Player  2 を示す ID です
    StationIndex_3,                      //!< Player  3 を示す ID です
    StationIndex_4,                      //!< Player  4 を示す ID です
    StationIndex_5,                      //!< Player  5 を示す ID です
    StationIndex_6,                      //!< Player  6 を示す ID です
    StationIndex_7,                      //!< Player  7 を示す ID です
    StationIndex_8,                      //!< Player  8 を示す ID です
    StationIndex_9,                      //!< Player  9 を示す ID です
    StationIndex_10,                     //!< Player 10 を示す ID です
    StationIndex_11,                     //!< Player 11 を示す ID です
    StationIndex_12,                     //!< Player 12 を示す ID です
    StationIndex_Max = StationIndex_12,  //!< StationIndex の最大値です
    StationIndex_Invalid = 0xfd,         //!< Session 内に存在しない Station を示す ID です
    StationIndex_Host = 0xfe,            //!< Hostの役割を持つ Station を示す ID です
    StationIndex_All = 0xff              //!< Session 内のすべての Station を示す ID です
};
}
}  // end of namespace nn::pia

//! @endcond
