﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/reckoning/reckoning_Definitions.h>


namespace nn
{
namespace pia
{
namespace reckoning
{

/*
n1589:memo
T2にvoidを指定した時に余計なメモリを使わないためのラッパーテンプレートです。
*/

//! @cond PRIVATE
template <typename T1, typename T2>
struct VoidMemberConcealer
{
public:
    VoidMemberConcealer()
        : m_T1(), m_T2()
    {
    }
    T1* GetT1Ptr()
    {
        return &m_T1;
    }
    T2* GetT2Ptr()
    {
        return &m_T2;
    }

private:
    T1 m_T1;
    T2 m_T2;
};

template <typename T1>
struct VoidMemberConcealer<T1, void>
{
public:
    VoidMemberConcealer()
        : m_T1()
    {
    }
    T1* GetT1Ptr()
    {
        return &m_T1;
    }
    void* GetT2Ptr()
    {
        return NULL;
    }

private:
    T1 m_T1;
};
//! @endcond
}
}
}  // end of namespace nn::pia::reckoning
