﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>

#include <nn/pia/session/session_Types.h>

namespace nn
{
namespace pia
{
namespace session
{
/*!
  @brief   コミュニティの作成処理に関する設定を管理する基底クラスです。
  @details アプリケーションが本クラスのインスタンスを作成することはありません。 本クラスを継承したクラスを使用する必要があります。

  @see inet::NexCreateCommunitySetting
 */
class CreateCommunitySetting : public common::RootObject
{
public:
    /*!
    @brief      デストラクタ。
     */
    virtual ~CreateCommunitySetting();

    /*!
    @brief      作成するコミュニティの最小参加人数を指定します。
    @param[in]  min 作成するコミュニティに指定する最小参加人数。
    */
    void SetMinParticipantNum(uint16_t min);

    /*!
    @cond PRIVATE
    @brief      指定した最小参加人数を取得します。
    @return     指定した最小参加人数を返します。
    */
    uint16_t GetMinParticipantNum() const;
    //! @endcond

    /*!
    @brief      作成するコミュニティの最大参加人数を指定します。
    @param[in]  max 作成するコミュニティに指定する最大参加人数。
    */
    void SetMaxParticipantNum(uint16_t max);

    /*!
    @cond PRIVATE
    @brief      指定した最大参加人数を取得します。
    @return     指定した最大参加人数を返します。
    */
    uint16_t GetMaxParticipantNum() const;
    //! @endcond

    /*!
    @brief      設定をリセットします。
     */
    virtual void Reset();


protected:
    // デフォルトコンストラクタ
    CreateCommunitySetting();

    uint16_t m_MinParticipantNum;
    uint16_t m_MaxParticipantNum;

private:
    NN_PIA_DISALLOW_COPY(CreateCommunitySetting);
};
}
}
}  // end of namespace session
