﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>
#include <nn/pia/session/session_ISessionInfo.h>

namespace nn
{
namespace pia
{
namespace session
{

/*!
    @brief  セッションの情報へのインターフェースのリストです。
 */
class ISessionInfoList : public common::RootObject
{
public:
    /*!
    @brief    デフォルトコンストラクタ
     */
    ISessionInfoList()
    {
    }
    /*!
    @brief    デストラクタ
     */
    virtual ~ISessionInfoList()
    {
    }

    typedef ISessionInfo** Iterator;
    typedef const ISessionInfo** ConstIterator;

    /*!
    @brief    先頭にある ISessionInfo* のイテレータを得ます（const版）。
    @return   先頭にある要素のイテレータを返します。
     */
    virtual ConstIterator Begin() const = 0;
    /*!
    @brief    先頭にある ISessionInfo* のイテレータを得ます（非const版）。
    @return   先頭にある要素のイテレータを返します。
     */
    virtual Iterator Begin() = 0;
    /*!
    @brief    末尾の次に対応する ISessionInfo* のイテレータを得ます（const版）。
    @return   末尾の次に対応する要素のイテレータを返します。
     */
    virtual ConstIterator End() const = 0;
    /*!
    @brief    末尾の次に対応する ISessionInfo* のイテレータを得ます（非const版）。
    @return   末尾の次に対応する要素のイテレータを返します。
     */
    virtual Iterator End() = 0;
    /*!
    @brief    リスト内の要素数を取得します。
    @return   リスト内の要素数を返します。
     */
    virtual uint16_t GetSize() const = 0;
    /*!
    @brief    リストの最大数を取得します。
    @return   リストの最大数を返します。
     */
    virtual uint16_t GetMaxSize() const = 0;
    /*!
    @brief    リストをクリアします。
     */
    virtual void Clear() = 0;
};
}
}
}  // end of namespace nn::pia::session
