﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>

#include <nn/pia/session/session_Types.h>

namespace nn
{
namespace pia
{
namespace session
{
/*!
  @brief   コミュニティの参加処理に関する設定を管理する基底クラスです。
  @details アプリケーションが本クラスのインスタンスを作成することはありません。 本クラスを継承したクラスを使用する必要があります。

  @see nn::pia::inet::NexJoinCommunitySetting
 */
class JoinCommunitySetting : public nn::pia::common::RootObject
{
public:
    /*!
    @brief      デストラクタ。
     */
    virtual ~JoinCommunitySetting();

    /*!
    @brief      参加するコミュニティを指定します。
    @param[in]  communityId 参加するコミュニティID。
    */
    void SetCommunityId(uint32_t communityId);

    /*!
    @cond PRIVATE
    @brief      指定したコミュニティIDを取得します。
    @return     指定したコミュニティIDを返します。
    */
    uint32_t GetCommunityId() const;
    //! @endcond

    /*!
    @brief      設定をリセットします。
     */
    virtual void Reset();

protected:
    // デフォルトコンストラクタ
    JoinCommunitySetting();

    void Copy(const JoinCommunitySetting& rhs);

    uint32_t m_CommunityId;

private:
    NN_PIA_DISALLOW_COPY(JoinCommunitySetting);
};
}
}
}  // end of namespace nn::pia::session
