﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>

#include <nn/pia/session/session_CreateSessionSetting.h>
#include <nn/pia/session/session_SessionSearchCriteria.h>

namespace nn
{
namespace pia
{
namespace session
{
/*!
  @brief   ランダムマッチメイク処理に関する設定を管理する基底クラスです。
  @details アプリケーションが本クラスのインスタンスを作成することはありません。 本クラスを継承したクラスを使用する必要があります。

  @see inet::NexJoinRandomSessionSetting
  @see lan::LanJoinRandomSessionSetting
 */
class JoinRandomSessionSetting : public common::RootObject
{
public:
    /*!
    @brief      デストラクタ。
     */
    virtual ~JoinRandomSessionSetting() {}

    /*!
    @brief      ランダムマッチメイクのセッション作成時の設定を指定します。
    @param[in]  pSetting セッション作成時の設定。
    */
    virtual Result SetCreateSessionSetting(const CreateSessionSetting* pSetting) = 0;

    /*!
    @brief      ランダムマッチメイクのセッション検索時の設定を指定します。
    @param[in]  pSearchCriteria     セッション検索設定のリスト。
    @param[in]  searchCriteriaSize  セッション検索設定のリストサイズ。
    */
    virtual Result SetSessionSearchCriteria(const SessionSearchCriteria* pSearchCriteria, uint32_t searchCriteriaSize) = 0;

    /*!
    @brief      設定をリセットします。
     */
    virtual void Reset() = 0;

    /*!
    @cond PRIVATE
    @brief デバッグに有用な情報をプリント。
    */
    virtual void Trace(uint64_t flag) const = 0;
    //! @endcond

    /*!
    @cond PRIVATE
    */
    virtual CreateSessionSetting* GetCreateSessionSetting() = 0;
    virtual SessionSearchCriteria* GetSessionSearchCriteria(uint8_t index) = 0;
    virtual const CreateSessionSetting* GetCreateSessionSetting() const = 0;
    virtual const SessionSearchCriteria* GetSessionSearchCriteria(uint8_t index) const = 0;
    virtual uint32_t GetSessionSearchCriteriaSize() const = 0;
    //! @endcond

protected:
    // デフォルトコンストラクタ
    JoinRandomSessionSetting() {}

private:
    NN_PIA_DISALLOW_COPY(JoinRandomSessionSetting);
};
}
}
}  // end of namespace session
