﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>

#include <nn/pia/session/session_Types.h>

namespace nn
{
namespace pia
{
namespace session
{

class ISessionInfo;
/*!
  @brief   セッション参加処理に関する設定を管理する基底クラスです。
  @details アプリケーションが本クラスのインスタンスを作成することはありません。 本クラスを継承したクラスを使用する必要があります。

@if CTR_DOC
  @see nn::pia::local::UdsJoinSessionSetting
@endif
@if NIN_DOC
  @see nn::pia::local::LdnJoinSessionSetting
@endif
  @see nn::pia::inet::NexJoinSessionSetting
  @see nn::pia::lan::LanJoinSessionSetting
 */
class JoinSessionSetting : public nn::pia::common::RootObject
{
public:
    /*!
    @brief    デストラクタ
     */
    virtual ~JoinSessionSetting();

    /*!
        @cond PRIVATE
        @brief      参加先のセッションの情報を取得します。

        @return     参加先のセッションの情報を返します。
     */
    virtual const ISessionInfo* GetSessionInfoPtr() const;
    //! @endcond

    /*!
        @brief  参加先のセッションの ID を設定します。
     */
    virtual void SetSessionId(uint32_t id);

    /*!
        @cond PRIVATE
        @brief      参加先のセッションの ID を取得します。
        @return     参加先のセッションの ID を返します。
     */
    virtual uint32_t GetSessionId() const;
    //! @endcond

    /*!
        @cond PRIVATE
        @brief  参加先のセッションの最大参加者数を設定します。
     */
    virtual void SetMaxParticipantNum(uint16_t num);
    //! @endcond

    /*!
        @cond PRIVATE
        @brief      参加先のセッションの最大参加者数を取得します。
        @return     参加先のセッションの最大参加者数を返します。
     */
    virtual uint16_t GetMaxParticipantNum() const;
    //! @endcond

    /*!
        @brief      参加先のセッションの情報を指定します。
        @details    @ref Session::BrowseSessionAsync で検索したセッションの情報を指定します。

        @param[in]  pSessionInfo   参加先のセッションの情報。

        @see        Session::JoinSessionAsync, Session::BrowseSessionAsync
     */
    virtual void SetSessionInfoPtr(const ISessionInfo* pSessionInfo);

    /*!
    @brief      設定をリセットします。
     */
    virtual void Reset();

    /*!
        @cond PRIVATE
        @brief デバッグに有用な情報をプリント。
     */
    void Trace(uint64_t flag) const;
    //! @endcond

protected:
    const ISessionInfo* m_pSessionInfo;
    uint32_t m_SessionId;
    uint16_t m_MaxParticipantNum;

    // コンストラクタ。
    JoinSessionSetting();

    void Copy(const JoinSessionSetting& rhs);

private:
    NN_PIA_DISALLOW_COPY(JoinSessionSetting);
};
}
}
}  // end of namespace nn::pia::session
