﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>

namespace nn
{
namespace pia
{
namespace session
{

/*!
    @brief  セッションに参加中のステーション情報です。
    @see nn::pia::lan::LanStationInfo
    @if CTR_DOC
    @see nn::pia::local::UdsStationInfo
    @endif
    @if NIN_DOC
    @see nn::pia::local::LdnStationInfo
    @endif
    */
class SessionStationInfo : public common::RootObject
{
public:
    /*!
    @brief  ステーションの役割（接続していない/ホスト/クライアント）を表します。
    */
    enum StationInfoRole
    {
        StationInfoRole_None = 0,  //!< 接続していない
        StationInfoRole_Host,      //!< ホスト
        StationInfoRole_Client     //!< クライアント
    };

    /*!
    @brief  ステーションの役割を取得します。
    @return ステーションの役割を返します。
    */
    StationInfoRole GetRole() const;

    /*!
    @cond PRIVATE
    */
    void SetRole(StationInfoRole role);
    virtual void Clear();
    static uint32_t GetSerializedSize();
    //! @endcond

protected:
    SessionStationInfo();
    virtual ~SessionStationInfo();

    StationInfoRole m_Role;
};
}
}
}  // end of namespace session
