﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/session/session_Definitions.h>

namespace nn
{
namespace pia
{
namespace session
{

static const uint32_t SessionJoinTimeoutDefault = 10000;    //!< JoinMesh 処理のタイムアウト時間 [msec] のデフォルト値です。
static const uint32_t SessionDestroyTimeoutDefault = 5000;  //!< DestroyMesh 処理のタイムアウト時間 [msec] のデフォルト値です。
static const uint32_t SessionLeaveTimeoutDefault = 5000;    //!< LeaveMesh 処理のタイムアウト時間 [msec] のデフォルト値です。

static const int32_t SessionKeepAliveIntervalDefault = 1000;  //!< キープアライブの送信間隔 [msec] のデフォルト値です。
static const int32_t SessionMaxSilenceTimeDefault = 10000;    //!< ステーションとの通信が断絶したと判断する無通信時間 [msec] のデフォルト値です。
static const int32_t SessionMaxSilenceTimeMax = 30000;        //!< ステーションとの通信が断絶したと判断する無通信時間 [msec] の設定可能な最大値です。
static const int32_t SessionMaxSilenceTimeMin = 1000;         //!< ステーションとの通信が断絶したと判断する無通信時間 [msec] の設定可能な最小値です。

static const int32_t SessionUnicastUpdateMeshIntervalDefault = 5000;   //!< セッション更新情報のブロードキャスト送信できない場合の送信間隔 [msec] のデフォルト値です。
static const int32_t SessionBroadcastUpdateMeshIntervalDefault = 2000;  //!< セッション更新情報をブロードキャスト送信できる場合の送信間隔 [msec] のデフォルト値です。

static const uint16_t SessionMaxRttDefault = 500;  //!< 1つのリレー接続経路の合計 RTT の最大制限値 [msec] のデフォルト値です。

static const int32_t SessionInvalidUplinkBitRateLowerLimit = -1;  //!< ビットレート判定機能において、無効なビットレート値を表現する定数です。

static const uint16_t SessionDefaultResultSize = 20;  //!< 検索範囲のエントリー数のデフォルト値です。

/*!
@brief Session クラス、Mesh クラスで構築するメッシュのネットワークトポロジーを設定するための列挙型です。

@details 今後、Session クラス、Mesh クラスの機能追加に伴って追加する予定です。
 */
enum NetworkTopology
{
    NetworkTopology_FullMesh = 0,  //!< フルメッシュ型（完全接続型）ネットワークトポロジー
    NetworkTopology_RelayMesh  //!< リレー接続有効メッシュ型ネットワークトポロジー
};


/*!
@brief ビットレート判定機能の設定を行うための列挙型です。
 */
enum BitRateCheckMode
{
    BitRateCheckMode_Disable = 0,  //!< ビットレートの測定を行いません。
    BitRateCheckMode_Enable        //!< ビットレートの測定を行います。
};

/*!
@cond PRIVATE

@brief Session クラス、Mesh クラスで構築するメッシュのホストマイグレーションを設定するための列挙型です。

@details 今後、Session クラス、Mesh クラスの機能追加に伴って追加する予定です。
 */
enum HostMigrationMode
{
    HostMigrationMode_None = 0,       //!< ホストマイグレーションを使用しません。
    HostMigrationMode_Solecandidate,  //!< ホストマイグレーションを使用します。次のセッションホスト候補は単独候補です。
    HostMigrationMode_Multicandidate  //!< ホストマイグレーションを使用します。次のセッションホスト候補は複数候補です。
};
//! @endcond

/*!
  @brief    無効な SessionId を示す定数です。
 */
static const uint32_t InvalidSessionId = 0;

#if NN_PIA_NINTENDOSDK
static const int MaxFriendsAppScreenNameSize = 64;
#endif
}
}
}
