﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>


namespace nn
{
namespace pia
{
namespace transport
{


/*!
  @brief  ConnectionAnalysisData は、接続品質に関係するパラメータ（RTT、パケロス率）をまとめた構造体です。
 */
struct ConnectionAnalysisData
{
    static const int32_t MaxEntries = MaxStationNum;

    /*!
    @brief メンバの内容をクリアします。
     */
    void Clear();


    /*!
    @brief 解析結果を Station ごとに集計するためのエントリ構造体です。
     */
    struct Entry
    {
        int32_t rtt;  //!< RTT。
        int32_t rttMin;  //!< 最小 RTT。
        int32_t rttMax;  //!< 最大 RTT。

        uint32_t prevTotalPacketNum;  //!< 前回のパケット数。
        uint32_t currTotalPacketNum;  //!< 今回のパケット数。

        uint32_t prevPacketLossNum;  //!< 前回のパケットロス数
        uint32_t currPacketLossNum;  //!< 今回のパケットロス数

        bool isValid;  //!< このエントリが有効なエントリであれば true がセットされます。
    };

    struct Entry entry[MaxEntries];  //!< エントリ。
    int32_t passedMilliSec;          //!< 経過時間（単位はミリ秒）


    /*!
    @brief データをコンソールに出力します（リリースビルド以外）。

    @param[in] bDetail Pia 内部実装に関係するデータも出力する場合に true に設定します。
     */
    void Print(bool bDetail) const;


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;
};
}
}
}  // end of namespace nn::pia::transport
