﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/transport/transport_ProtocolId.h>

#include <nn/pia/common/common_Time.h>


namespace nn
{
namespace pia
{
namespace transport
{


/*!
  @brief  PacketAnalysisData は、送受信されるパケットの内部データを解析した結果を集計するために用意された構造体です。

 */
struct PacketAnalysisData
{
    static const int32_t MaxEntries = 32;  //!< 最大エントリ数です。
    static const int32_t MaxNameLen = 32;  //!< 本オブジェクトに付けられる名前の最大長です。


    /*!
    @brief 解析結果を ProtocolId ごとに集計するためのエントリ構造体です。
     */
    struct Entry
    {
        ProtocolId protocolId;      //!< ProtocolId
        uint32_t totalNum;          //!< ProtocolId に対応するプロトコルメッセージの個数
        uint32_t totalDataSize;     //!< ProtocolId に対応するデータ量の総和（単位はバイト）
        uint32_t sumTotalNum;       //!< ProtocolId に対応するプロトコルメッセージの個数。ClearCounters() ではクリアされません。
        uint64_t sumTotalDataSize;  //!< ProtocolId に対応するデータ量の総和（単位はバイト）。ClearCounters() ではクリアされません。
    };

    struct Entry entry[MaxEntries];  //!< エントリ配列。
    int32_t nowEntryNum;             //!< 現時点でのエントリ数。
    int32_t passedMilliSec;          //!< PiaTransport のスタートアップ処理実行時点からの経過時間（単位はミリ秒）
    char name[MaxNameLen];           //!< インスタンスに付ける名前です。'\0' 終端されている必要があります。
    uint32_t totalPacketNum;         //!< 解析したパケットの個数。
    uint32_t totalPacketSize;        //!< 解析したパケットのサイズの総和。

    uint32_t sumTotalPacketNum;   //!< 解析が開始されてからカウントしたパケットの個数。ClearCounters() ではクリアされません。
    uint64_t sumTotalPacketSize;  //!< 解析が開始されてからカウントしたパケットのサイズの総和。ClearCounters() ではクリアされません。

    /*!
    @brief name メンバ以外の内容を全てクリアします。
     */
    void ClearExceptName();


    /*!
    @brief エントリの totalNum、totalDataSize と、passedMilliSec、totalPacketNum、totalPacketSize をクリアします。
     */
    void ClearCounters();


    /*!
    @brief データの内容をプリントします。

    @param[in] bDetail 詳細な出力を得る場合は true を指定します。
     */
    void Print(bool bDetail) const;


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;


    static const int32_t InvalidIdx = -1;   // 無効な添え字を表す。
    int32_t GetIndex(ProtocolId id) const;  // id に対応するエントリが存在していればその添え字を、存在しなければ InvalidIdx を返す。
    int32_t CreateIndex(ProtocolId id);     // エントリが満杯で、新たな添え字を割り振ることができなければ、InvalidIdx を返す。
};
}
}
}  // end of namespace nn::pia::transport
