﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>
#include <nn/pia/common/common_StationAddress.h>

namespace nn
{
namespace pia
{
namespace transport
{


/*!
@cond PRIVATE
  @brief  Protocolのイベントクラスです。
 */
class ProtocolEvent : public nn::pia::common::RootObject
{
public:
    /*!
    @brief イベントの種類です。
     */
    enum Type
    {
        Type_Connect,
        Type_Disconnect
    };


    /*!
    @brief コンストラクタです。
     */
    ProtocolEvent(Type type, const common::StationAddress& stationAddress)
        : m_Type(type), m_StationAddress(stationAddress)
    {
    }


    /*!
    @brief  イベントの種類を取得します。
     */
    Type GetType() const
    {
        return m_Type;
    }


    /*!
    @brief  StationAddress を取得します。
     */
    const common::StationAddress& GetStationAddress() const
    {
        return m_StationAddress;
    }


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は@ref TraceFlag 型を参照してください。
     */
    virtual void Trace(uint64_t flag) const;

private:
    Type m_Type;
    common::StationAddress m_StationAddress;
};
//! @endcond
}
}
}  // end of namespace nn::pia::transport
