﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>


namespace nn
{
namespace pia
{
namespace transport
{


/*!
@cond PRIVATE
  @brief  プロトコルIDを表す構造体です。
 */
struct ProtocolId
{
public:
    struct Item
    {
#if NN_PIA_ENDIAN_BIG
        uint16_t m_Type;
        uint16_t m_Port;
#else
        uint16_t m_Port;
        uint16_t m_Type;
#endif
    };

    union
    {
        uint32_t m_Value;
        Item m_Item;
    };


    /*!
    @brief デフォルトコンストラクタです。
     */
    ProtocolId()
        : m_Value(INVALID.m_Value)
    {
    }


    /*!
    @brief uint32_tの値からの変換
     */
    ProtocolId(uint32_t value)
        : m_Value(value)
    {
    }

    /*!
    @brief タイプの取得
     */
    uint16_t GetType()
    {
        return m_Item.m_Type;
    }

    /*!
    @brief タイプ、ポートから生成
     */
    ProtocolId(uint16_t type, uint16_t port)
    {
        m_Item.m_Type = type;
        m_Item.m_Port = port;
    }


    /*!
    @brief uint32_tの値への変換
     */
    operator uint32_t() const
    {
        return m_Value;
    }


    /*!
    @brief 無効なプロトコルID
     */
    static const ProtocolId INVALID;
};
//! @endcond
}
}
}  // end of namespace nn::pia::transport
