﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/transport/transport_Definitions.h>

#include <nn/pia/transport/transport_PacketAnalysisData.h>
#include <nn/pia/transport/transport_ConnectionAnalysisData.h>


namespace nn
{
namespace pia
{
namespace transport
{


/*!
  @brief  TransportAnalysisData は、送受信パケットの分析データと、
          各ステーションとの接続品質を表すパラメータ（RTT、パケロス率）をまとめた構造体です。

 */
struct TransportAnalysisData
{
    struct PacketAnalysisData sendPacketAnalysisData;           //!< 送信したパケットの分析データ。
    struct PacketAnalysisData sendUnicastPacketAnalysisData;    //!< ユニキャストで送信したパケットの分析データ。
    struct PacketAnalysisData sendBroadcastPacketAnalysisData;  //!< ブロードキャストで送信したパケットの分析データ。
    struct PacketAnalysisData recvPacketAnalysisData;           //!< 受信したパケットの分析データ。
    struct ConnectionAnalysisData connectionAnalysisData;       //!< 接続品質の分析データ。
    uint32_t dispatchCount;                                     //!< ディスパッチ回数

    /*!
    @brief データをコンソールに出力します。

    @param[in] isDetail     Pia 内部で使用されているプロトコルも出力する場合は true を設定します。
    @param[in] isTotalOnly  ユニキャスト送信とブロードキャスト送信を分けた分析データも出力する場合は false を設定します。
    */
    void Print(bool isDetail, bool isTotalOnly) const;


    /*!
    @brief デバッグに有用な情報をプリントします。

    @param[in] flag  トレースフラグの論理和。詳細は @ref TraceFlag 型を参照してください。
     */
    void Trace(uint64_t flag) const;
};
}
}
}  // end of namespace nn::pia::transport
