﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/tune/tune_Thread.h>

#if NN_PIA_NINTENDOSDK
#include <nn/os.h>


namespace nn
{
namespace pia
{
namespace tune
{

// インスタンス生成式のワンショットアラームのクラスです。
class NonStaticAlarm
{
public:
    NonStaticAlarm();
    ~NonStaticAlarm();

    typedef void (*Handler)(void* param);

    // 初期化を行います。
    void Initialize(uint32_t coreIndex);

    // 終了処理を行います。
    void Finalize();

    // アラームを仕掛けます。
    // このメンバ関数を呼び出してから msec ミリ秒後に handler 関数が
    // 呼ばれます。呼び出された handler 関数には param が引数として渡されます。
    void Set(int32_t msec, Handler handler, void* param);

private:
    static const size_t s_ThreadStackSize = 1024 * 8;  // スレッド操作スレッドのスタックサイズ
    uint8_t* m_pThreadStack;

    nn::os::TimerEvent m_TimerEvent;
    Thread m_Thread;
    void* m_Param;
    static void TimerThreadFunc(void* param);
    void TimerThreadFuncCore();

    Handler m_Handler;

    bool m_IsThreadRun;
};
}
}
}  // end of namespace nn::pia::tune
#endif
