﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/tune/tune_Definitions.h>

#if NN_PIA_CAFE
#include <cafe.h>
#endif

#if NN_PIA_CTR
typedef void (*ThreadFunc)(void* pArg);
#elif NN_PIA_WIN
// #error "not implemented."
#elif NN_PIA_NINTENDOSDK
typedef void (*ThreadFunc)(void*);
#elif NN_PIA_CAFE
typedef int (*ThreadFunc)(int args, void* pArg);
#elif NN_PIA_A || NN_PIA_B
//n1849:todo:not implemented
#else
#error "invalid platform"
#endif

namespace nn
{
namespace pia
{
namespace tune
{

class Thread
{
public:
    Thread(void);
    ~Thread(void);

#if NN_PIA_CTR
    void Start(ThreadFunc pFunc, void* pArg, size_t stackSize, int32_t priority = nn::os::DEFAULT_THREAD_PRIORITY);
#elif NN_PIA_WIN

#elif NN_PIA_NINTENDOSDK
    void Start(ThreadFunc pFunc, void* pArg, void* pStackAddr, size_t stackSize, int32_t priority);
    void Start(ThreadFunc pFunc, void* pArg, void* pStackAddr, size_t stackSize, int32_t priority, int32_t coreIndex);
#elif NN_PIA_CAFE
    void Start(ThreadFunc pFunc, void* pArg, void* pStackAddr, size_t stackSize, uint16_t attr = 0, OSPriority priority = OS_PRIORITY_APP_DEFAULT);
#elif NN_PIA_A || NN_PIA_B
//n1849:todo:not implemented
#else
#error "invalid platform"
#endif

    bool IsAlive();

    void Join(void);

    void Detach(void);

    void Finalize(void);

    ///< 現在のスレッドを指定された時間だけ休眠状態とします。時間の単位はミリ秒です。
    static void Sleep(int32_t msec);


private:
    ///< コピーコンストラクタ、代入演算子は封印します。
    Thread(const Thread&);
    Thread& operator=(const Thread&);

    bool m_IsCreated;

#if NN_PIA_CTR
    nn::os::Thread m_Thread;
#elif NN_PIA_WIN
// #error "not implemented."
#elif NN_PIA_NINTENDOSDK
    nn::os::ThreadType m_ThreadType;
    bool m_ThreadIsAlive;
#elif NN_PIA_CAFE
    PIA_ATTRIBUTE_ALIGN(8) OSThread m_Thread;
#elif NN_PIA_A || NN_PIA_B
//n1849:todo:not implemented
#else
#error "invalid platform"
#endif
};
}
}
}  // end of namespace nn:pia:tune
