﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "Alarm.h"

namespace nns
{
namespace pia
{

Alarm::Handler Alarm::s_Handler = NULL;

#if NN_PIA_CTR
nn::os::Alarm Alarm::s_Alarm;
#elif NN_PIA_CAFE
OSAlarm Alarm::s_Alarm;
#elif NN_PIA_NINTENDOSDK
nn::os::TimerEvent Alarm::s_TimerEvent(nn::os::EventClearMode_AutoClear);
Thread Alarm::s_Thread;
void* Alarm::s_Param = NULL;
const size_t StackSize = 8192;
PIA_ATTRIBUTE_ALIGN(4096) static uint8_t s_StackMemory[StackSize];  // nn::os::ThreadStackAlignment = 4096

void Alarm::TimerThreadFunc(void*)
{
    for (;;)
    {
        s_TimerEvent.Wait();
        if (s_Handler)
        {
            s_Handler(s_Param);
        }
    }
}
#endif


void Alarm::Initialize()
{
    s_Handler = NULL;

#if NN_PIA_CTR
    s_Alarm.Initialize();
#elif NN_PIA_CAFE
    OSCreateAlarm(&s_Alarm);
#elif NN_PIA_NINTENDOSDK
    s_Thread.Start(TimerThreadFunc, NULL, s_StackMemory, StackSize, nn::os::HighestThreadPriority);
#endif
}


void Alarm::Finalize()
{
#if NN_PIA_CTR
    s_Alarm.Cancel();
    s_Alarm.Finalize();
#elif NN_PIA_CAFE
    OSCancelAlarm(&s_Alarm);
#elif NN_PIA_NINTENDOSDK
    s_Thread.Finalize();
#endif

    s_Handler = NULL;
}


void Alarm::Set(int32_t msec, Handler handler, void* param)
{
    s_Handler = handler;
#if NN_PIA_CTR
    nn::fnd::TimeSpan span = nn::fnd::TimeSpan::FromMilliSeconds(msec);
    s_Alarm.SetOneShot(span, AlarmHandler, param);
#elif NN_PIA_CAFE
    OSSetAlarmUserData(&s_Alarm, param);
    OSTick span = OSMillisecondsToTicks(msec);
    OSSetAlarm(&s_Alarm, span, AlarmHandler);
#elif NN_PIA_NINTENDOSDK
    s_Param = param;
    nn::TimeSpan span = nn::TimeSpan::FromMilliSeconds(msec);
    s_TimerEvent.StartOneShot(span);
#endif
}


#if NN_PIA_CTR
void Alarm::AlarmHandler(void* param, bool canceled)
{
    (void)canceled;

    if (s_Handler)
    {
        s_Handler(param);
    }
}
#elif NN_PIA_CAFE
void Alarm::AlarmHandler(OSAlarm* alarm, OSContext* context)
{
    (void)context;

    if (s_Handler)
    {
        void* param = OSGetAlarmUserData(alarm);
        s_Handler(param);
    }
}
#endif
}
}  // end of namespace nns::pia
