﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common.h>

#include "Thread.h"

namespace nns
{
namespace pia
{

// ワンショットアラームのクラスです。
class Alarm
{
public:
    typedef void (*Handler)(void* param);

    // 初期化を行います。
    static void Initialize();

    // 終了処理を行います。
    static void Finalize();

    // アラームを仕掛けます。
    // このメンバ関数を呼び出してから msec ミリ秒後に handler 関数が
    // 呼ばれます。呼び出された handler 関数には param が引数として渡されます。
    static void Set(int32_t msec, Handler handler, void* param);

private:
#if NN_PIA_CTR
    static void AlarmHandler(void* param, bool canceled);
    static nn::os::Alarm s_Alarm;
#elif NN_PIA_CAFE
    static void AlarmHandler(OSAlarm* alarm, OSContext* context);
    static OSAlarm s_Alarm;
#elif NN_PIA_NINTENDOSDK
    static nn::os::TimerEvent s_TimerEvent;
    static Thread s_Thread;
    static void* s_Param;
    static void TimerThreadFunc(void* param);
#endif

    static Handler s_Handler;

    Alarm();
    ~Alarm();
};
}
}  // end of namespace nns::pia
