﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common.h>

#if NN_PIA_UNDER_DEVELOP
/*!
@brief サンプル用のアサートです。
 */
#define PIA_SAMPLE_ASSERT(exp) \
    (static_cast<void>((exp) || (nns::pia::SampleAssert(PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_LINE, #exp), true)))

/*!
@brief サンプル用のメッセージ付きアサートです。
 */
#define PIA_SAMPLE_ASSERTMSG(exp, ...) \
    (static_cast<void>((exp) || (nns::pia::SampleAssert(PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_LINE, __VA_ARGS__), true)))

#else
#define PIA_SAMPLE_ASSERT(exp) ((void)0)
#define PIA_SAMPLE_ASSERTMSG(exp, ...) ((void)0)
#endif

/*!
@brief result.IsFailure() が true となる場合、アサート停止します。
 */
#define PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result) \
    if (result.IsFailure())                         \
    {                                               \
        PIA_SAMPLE_PRINT_RESULT(result);              \
        PIA_SAMPLE_ASSERT(false);                   \
    }

namespace nns
{
namespace pia
{

void SampleAssert(const char* cpPos, uint32_t line, const char* cpMsg, ...);
}
}  // end of namespace nns::pia

/*!
@brief プログラムを強制的に停止させます。
 */
#define PIA_SAMPLE_PANIC(...) PIA_PANIC(__VA_ARGS__)
