﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Clone_Setup.h"

namespace
{
uint32_t s_hClone = 0;
}

namespace nns
{
namespace pia
{

void Clone_InitializePiaClone()
{
    nn::pia::clone::CloneProtocol::Setting setting;
    setting.clockType = nn::pia::clone::CloneProtocol::Setting::ClockType_Rtc;
    setting.clockPerSec = 1000.0f;
    setting.clockPriorityJudgeMethod = nn::pia::clone::CloneProtocol::Setting::ClockPriorityJudgeMethod_Simple;
    Clone_InitializePiaClone(setting);
}

void Clone_InitializePiaClone(const nn::pia::clone::CloneProtocol::Setting& setting)
{
    PIA_SAMPLE_PRINT("Clone_InitializePiaClone\n");

    nn::pia::Result r;

    // 事前に transport モジュールの初期化を済ませておいてください。
    PIA_SAMPLE_ASSERT(nn::pia::transport::IsInitialized());

    r = nn::pia::clone::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);

    r = nn::pia::clone::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);

    s_hClone = nn::pia::transport::Transport::GetInstance()->CreateProtocol<nn::pia::clone::CloneProtocol>();
    PIA_SAMPLE_ASSERT(s_hClone != 0);

    r = Clone_GetCloneProtocol()->Initialize(setting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);

    r = nn::pia::clone::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
}

void Clone_FinalizePiaClone()
{
    PIA_SAMPLE_PRINT("Clone_FinalizePiaClone\n");

    if (Clone_GetCloneProtocol() != NULL)
    {
        Clone_GetCloneProtocol()->Finalize();
    }

    if (nn::pia::transport::Transport::GetInstance() != NULL)
    {
        nn::pia::transport::Transport::GetInstance()->DestroyProtocol(s_hClone);
    }

    nn::pia::clone::Finalize();
}

nn::pia::clone::CloneProtocol* Clone_GetCloneProtocol()
{
    if (nn::pia::transport::Transport::GetInstance() != NULL)
    {
        return nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::clone::CloneProtocol>(s_hClone);
    }
    else
    {
        return NULL;
    }
}
}
}  // end of namespace nns::pia
