﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Platform.h>

// トレース関係。
#define SAMPLE_TRACE_ALWAYS (1 << 0)

#define SAMPLE_TRACE_COMMON_ALWAYS (1 << 1)
#define SAMPLE_TRACE_COMMON_DETAIL (1 << 2)
#define SAMPLE_TRACE_LOCAL_ALWAYS (1 << 3)
#define SAMPLE_TRACE_LOCAL_DETAIL (1 << 4)
#define SAMPLE_TRACE_TRANSPORT_ALWAYS (1 << 5)
#define SAMPLE_TRACE_TRANSPORT_DETAIL (1 << 6)
#define SAMPLE_TRACE_SYNC_ALWAYS (1 << 7)
#define SAMPLE_TRACE_SYNC_DETAIL (1 << 8)
#define SAMPLE_TRACE_INET_ALWAYS (1 << 9)
#define SAMPLE_TRACE_INET_DETAIL (1 << 10)
#define SAMPLE_TRACE_SESSION_ALWAYS (1 << 11)
#define SAMPLE_TRACE_SESSION_DETAIL (1 << 12)

#define SAMPLE_TRACE_APP1 (1 << 24)
#define SAMPLE_TRACE_APP2 (1 << 25)
#define SAMPLE_TRACE_APP3 (1 << 26)
#define SAMPLE_TRACE_APP4 (1 << 27)
#define SAMPLE_TRACE_APP5 (1 << 28)
#define SAMPLE_TRACE_APP6 (1 << 29)
#define SAMPLE_TRACE_APP7 (1 << 30)
#define SAMPLE_TRACE_APP8 (1 << 31)

#if 1
#define SAMPLE_TRACE_ERROR_ (1 << 13)
#define SAMPLE_TRACE_BUTTON_ (1 << 14)
#define SAMPLE_TRACE_UDS_ (1 << 15)
#define SAMPLE_TRACE_TRANSPORT_ (1 << 16)
#define SAMPLE_TRACE_UPDATE_ (1 << 17)
#define SAMPLE_TRACE_ROUTE_ (1 << 18)

#define SAMPLE_TRACE_SCENE_ SAMPLE_TRACE_APP1
#define SAMPLE_TRACE_AVATAR_ SAMPLE_TRACE_APP2
#define SAMPLE_TRACE_POSITION_ SAMPLE_TRACE_APP3
#define SAMPLE_TRACE_WORLD_ SAMPLE_TRACE_APP4
#define SAMPLE_TRACE_LATENCY_ SAMPLE_TRACE_APP5
#endif

#ifdef PIA_SAMPLE_WITHOUT_NEX
#define NNS_PIA_ENABLE_INET 0
#else
#define NNS_PIA_ENABLE_INET (1 && NN_PIA_ENABLE_NEX)
#endif

#ifdef PIA_TEST_BUILD
// 自動検証用のコードです。サンプルとしては無視してください。
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif

//マクロ定義後でincludeしないとマクロが反映されない
#include "SampleLib/VersionInfo.h"

#if NN_PIA_CTR
#include <nn/applet.h>

#define NNS_PIA_ENABLE_LOCAL 1
#define SAMPLE_STRING(x) L##x
typedef wchar_t SampleStringChar_t;

// CTR では nn::applet::Enable() を呼び出してからサンプルプログラムのメイン関数を呼び出します
#define PIA_SAMPLE_MAIN_FUNC() \
    \
/* サンプルプログラムのメイン関数の宣言       */ \
void SampleMainFunc(void);     \
    \
void nnMain()                  \
    \
{                       \
        nns::pia::VersionInfo::TraceBuildEnvironment();  \
        nn::applet::Enable();  \
        SampleMainFunc();      \
    \
}                       \
    \
void SampleMainFunc()

#elif NN_PIA_WIN

#define NNS_PIA_ENABLE_LOCAL 0
#define SAMPLE_STRING(x) L##x
typedef wchar_t SampleStringChar_t;

// Console/GUI 毎の関数に変換する
#ifdef _CONSOLE
#define PIA_SAMPLE_MAIN_FUNC() int main()
#else
#define PIA_SAMPLE_MAIN_FUNC() int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
#endif  // _CONSOLE

#elif NN_PIA_NINTENDOSDK
#if NN_PIA_NINTENDOSDK_NX
#include <nn/oe/oe_NotificationMessage.h>
#endif

#define NNS_PIA_ENABLE_LOCAL 1
#define NNS_PIA_USE_SHOWUSERSELECTOR 1
#define NNS_PIA_USE_ENSURENETWORKSERVICEACCOUTAVAILABLE 1
#define NNS_PIA_USE_HANDLENETWORKREQUESTRESULT 1
#define SAMPLE_STRING(x) x
typedef char SampleStringChar_t;

#if NN_PIA_NINTENDOSDK_NX
#define PIA_SAMPLE_MAIN_FUNC() \
    \
/* サンプルプログラムのメイン関数の宣言       */ \
void SampleMainFunc(void);     \
    \
extern "C" void nnMain()                  \
    \
{                       \
        nns::pia::VersionInfo::TraceBuildEnvironment();  \
        nn::oe::SetFocusHandlingMode(nn::oe::FocusHandlingMode_Notify);  \
        SampleMainFunc();      \
    \
}                       \
    \
void SampleMainFunc()

#else
#define PIA_SAMPLE_MAIN_FUNC() \
    \
/* サンプルプログラムのメイン関数の宣言       */ \
void SampleMainFunc(void);     \
    \
extern "C" void nnMain()                  \
    \
{                       \
        nns::pia::VersionInfo::TraceBuildEnvironment();  \
        SampleMainFunc();      \
    \
}                       \
    \
void SampleMainFunc()

#endif

#elif NN_PIA_CAFE

#define NNS_PIA_ENABLE_LOCAL 0
#define SAMPLE_STRING(x) L##x
typedef wchar_t SampleStringChar_t;

// サンプルプログラムのメイン関数をどのコアで実行するか
#define PIA_SAMPLE_MAIN_CORE 2

// Cafe ではサンプルプログラムのメイン関数をコアを指定して実行できるように
// メイン関数で動作コアを確認し、サンプルプログラムのメイン関数を目的のコアで
// 実行されるようにスレッドの設定を行います
#define PIA_SAMPLE_MAIN_FUNC()                                                                                                                                          \
    \
/* サンプルプログラムのメイン関数の宣言       */ \
void SampleMainFunc(void);                                                                                                                                              \
    \
/* サンプルプログラムのスレッドのエントリ関数 */ \
int SampleEntryFunc(int argc, void* argv)                                                                                                                               \
    {                                                                                                                                                                   \
        /* 引数未使用警告を回避するために SampleMainFunc 関数を別途呼び出すようにしています */                                          \
        NN_PIA_UNUSED(argc);                                                                                                                                       \
        NN_PIA_UNUSED(argv);                                                                                                                                       \
        SampleMainFunc();                                                                                                                                               \
        return 0;                                                                                                                                                       \
    \
}                                                                                                                                                                \
    \
const uint32_t SAMPLE_THREAD_STACK_SIZE = 2 * 1024 * 1024;                                                                                                              \
    \
OSThread g_SampleThread;                                                                                                                                                \
    \
uint8_t g_SampleThreadStack[SAMPLE_THREAD_STACK_SIZE];                                                                                                                  \
    \
void RunSampleThread(void)                                                                                                                                              \
    {                                                                                                                                                                   \
        uint16_t sampleThreadAttribute = OS_THREAD_ATTR_DETACH;                                                                                                         \
                                                                                                                                                                        \
        if (PIA_SAMPLE_MAIN_CORE == 0)                                                                                                                                  \
            sampleThreadAttribute |= OS_THREAD_ATTR_AFFINITY_CORE0;                                                                                                     \
        if (PIA_SAMPLE_MAIN_CORE == 1)                                                                                                                                  \
            sampleThreadAttribute |= OS_THREAD_ATTR_AFFINITY_CORE1;                                                                                                     \
        if (PIA_SAMPLE_MAIN_CORE == 2)                                                                                                                                  \
            sampleThreadAttribute |= OS_THREAD_ATTR_AFFINITY_CORE2;                                                                                                     \
                                                                                                                                                                        \
        OSCreateThread(&g_SampleThread, SampleEntryFunc, 0, NULL, g_SampleThreadStack + SAMPLE_THREAD_STACK_SIZE, SAMPLE_THREAD_STACK_SIZE, 16, sampleThreadAttribute); \
        OSResumeThread(&g_SampleThread);                                                                                                                                \
    \
}                                                                                                                                                                \
    \
/* メイン関数 */ \
int main()                                                                                                                                                              \
    {                                                                                                                                                                   \
        nns::pia::VersionInfo::TraceBuildEnvironment();  \
        if (OSGetCoreId() == PIA_SAMPLE_MAIN_CORE)                                                                                                                      \
        {                                                                                                                                                               \
            SampleMainFunc();                                                                                                                                           \
        }                                                                                                                                                               \
        else                                                                                                                                                            \
        {                                                                                                                                                               \
            RunSampleThread();                                                                                                                                          \
                                                                                                                                                                        \
            while (OSIsThreadTerminated(&g_SampleThread) == FALSE)                                                                                                      \
            {                                                                                                                                                           \
                OSSleepSeconds(1);                                                                                                                                      \
            }                                                                                                                                                           \
        }                                                                                                                                                               \
        return 0;                                                                                                                                                       \
    \
}                                                                                                                                                                \
    \
void SampleMainFunc()

#elif NN_PIA_A

#define PIA_SAMPLE_MAIN_FUNC() int main()
typedef char SampleStringChar_t;
#define SAMPLE_STRING(x) x

#elif NN_PIA_B

#define NNS_PIA_ENABLE_LOCAL 0
#define SAMPLE_STRING(x) x
typedef char SampleStringChar_t;

// Console/GUI 毎の関数に変換する
#define PIA_SAMPLE_MAIN_FUNC() int main()

#else
#error "invalid platform";
#endif
