﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/Demo.h"

#include "SampleLib/SampleLib.h"

#include <nn/pia/session.h>

namespace nns
{
namespace pia
{
#if PIA_TEST_CODE
const int32_t IndexRelayEmulation = 0;
const int32_t IndexPacketLossEmulation = 1;
const int32_t IndexSendPacketLoss = 2;
const int32_t IndexReceivePacketLoss = 3;
const int32_t IndexRelayPreset = 4;
const int32_t IndexLatencyEmulation = 5;
const int32_t IndexLatencyPreset = 6;
const int32_t IndexDemandingBitRate = 7;
const int32_t IndexEnd = 8;

struct ParameterSetting
{
    int32_t value;      // この値が操作される。
    int32_t minimum;    // value の最小値。
    int32_t maximum;    // value の最大値。
    int32_t increment;  // 増分
    const char* pName;  // 項目の名称。
};

ParameterSetting s_ParameterSetting[IndexEnd] =
    {
        { 0, 0, 1, 1, "Relay emulation" },
        { 0, 0, 1, 1, "Packet loss emulation" },
        { 0, 0, 100, 1, "Send packet loss percentage" },
        { 0, 0, 100, 1, "Recv packet loss percentage" },
        { 0, 0, 10, 1, "Relay preset" },
        { 0, 0, 1, 1, "Latency emulation" },
        { 0, 0, 10, 1, "Latency preset" },
        { 200 * 1000, 100 * 1000, 1000 * 1000, 100 * 1000, "Demanding bit rate" },
    };

int32_t s_CursorIdx = 0;


void initializePiaTest()
{
    s_CursorIdx = 0;
}


void operatePiaTest(OptionalParameter* pOpt)
{
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pOpt));

    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pSampleFramework));

    // 十字ボタンの上下でカーソル移動（ループ可能）
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
    {
        s_CursorIdx = (s_CursorIdx + IndexEnd - 1) % IndexEnd;
    }
    else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
    {
        s_CursorIdx = (s_CursorIdx + 1) % IndexEnd;
    }

    // 十字ボタンの左右で値の増減
    struct ParameterSetting& param = s_ParameterSetting[s_CursorIdx];
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
    {
        if (param.value < param.maximum)
        {
            param.value += param.increment;
        }
    }
    else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
    {
        if (param.value > param.minimum)
        {
            param.value -= param.increment;
        }
    }
}


void updatePiaTestData(int32_t offset)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    pSampleFramework->SetProgramParam(offset + 0, s_CursorIdx);
    pSampleFramework->SetProgramParam(offset + 1, s_ParameterSetting[s_CursorIdx].value);
}


void drawPiaTest()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pSampleFramework));

    // XY 座標の始点と高さ
    int32_t x = 10;
    int32_t y = 110;
    const int32_t H = 10;

    // カーソル描画（アニメーション付き）
    static int calls = 0;
    int shift = (calls / 10) % 2;
    pSampleFramework->DrawString(x - 7 + shift, y + H * s_CursorIdx, ">");
    ++calls;

    // 各項目の描画
    for (int i = 0; i < IndexEnd; ++i)
    {
        const struct ParameterSetting& param = s_ParameterSetting[i];
        pSampleFramework->DrawString(x, y, "%s: %d", param.pName, param.value);
        y += H;
    }
}
#endif  // end of PIA_TEST_CODE


#if !NN_PIA_A && !NN_PIA_B
// 実行するモードを決定します。
GameMode DecideGameMode(const char* pTitle, OptionalParameter* pOpt)
{
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTitle));

#if PIA_TEST_CODE
    initializePiaTest();
#else
    NN_PIA_UNUSED(pOpt);
#endif

    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Please select game mode.\n [START]Quit\n [Y]LanRandom\n");
#if NNS_PIA_ENABLE_INET
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" [X]InetRandom\n");
#endif
#if NNS_PIA_ENABLE_LOCAL
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" [A]LocalHost\n [B]LocalClient\n");
#endif
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" Waiting input\n");

    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    GameMode gameMode = GameMode_Exit;
    bool isDecided = false;
    while (!isDecided)
    {
        pSampleFramework->BeginFrame();

        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Exit\n");
            gameMode = GameMode_Exit;
            isDecided = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LanRandom\n");
            gameMode = GameMode_LanRandom;
            isDecided = true;
        }
#if NNS_PIA_ENABLE_INET
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: InetRandom\n");
            gameMode = GameMode_InetRandom;
            isDecided = true;
        }
#if PIA_TEST_CODE
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: InetHost\n");
            gameMode = GameMode_InetHost;
            isDecided = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: InetClient\n");
            gameMode = GameMode_InetClient;
            isDecided = true;
        }
#endif
#endif
#if NNS_PIA_ENABLE_LOCAL
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LocalHost\n");
            gameMode = GameMode_LocalHost;
            isDecided = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LocalClient\n");
            gameMode = GameMode_LocalClient;
            isDecided = true;
        }
#endif

#if PIA_TEST_CODE
        operatePiaTest(pOpt);
        updatePiaTestData(pOpt->offset);
#endif

        int y = 10;
        pSampleFramework->DrawString(10, y, pTitle);
        y += 10;
        pSampleFramework->DrawString(10, y, "Select game mode.");
        y += 10;
#if NNS_PIA_ENABLE_LOCAL
        pSampleFramework->DrawString(10, y, "A: LocalHost");
        y += 10;
        pSampleFramework->DrawString(10, y, "B: LocalClient");
        y += 10;
#endif
#if NNS_PIA_ENABLE_INET
        pSampleFramework->DrawString(10, y, "X: InetRandom");
        y += 10;
#endif
        pSampleFramework->DrawString(10, y, "Y: LanRandom");
        y += 10;
        pSampleFramework->DrawString(10, y, "START: Quit this application");

#if PIA_TEST_CODE
        drawPiaTest();

        pSampleFramework->SetProgramStatus(isDecided ? 0 : 1, false);
#endif

        pSampleFramework->EndFrame();

        if (isDecided)
        {
#if PIA_TEST_CODE
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pOpt));
            pOpt->bRelayEmulation = (s_ParameterSetting[IndexRelayEmulation].value == 0) ? false : true;
            pOpt->bPacketLossEmulation = (s_ParameterSetting[IndexPacketLossEmulation].value == 0) ? false : true;
            pOpt->sendPacketLossPercentage = s_ParameterSetting[IndexSendPacketLoss].value;
            pOpt->recvPacketLossPercentage = s_ParameterSetting[IndexReceivePacketLoss].value;
            pOpt->relayPreset = s_ParameterSetting[IndexRelayPreset].value;
            pOpt->bLatencyEmulation = (s_ParameterSetting[IndexLatencyEmulation].value == 0) ? false : true;
            pOpt->latencyPreset = s_ParameterSetting[IndexLatencyPreset].value;
            pOpt->demandingBitRate = s_ParameterSetting[IndexDemandingBitRate].value;
#endif
        }
    }
    return gameMode;
}
#endif


void SetRelayEmulation(nn::pia::StationIndex c1, nn::pia::StationIndex c2)
{
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pSession));

    // 接続経路エミュレーション設定前に、以前の設定が残っているかもしれないので初期化
    pSession->ResetRelayEmulation();

    // リレー接続
    PIA_SAMPLE_PRINT("Set RelayEmulation: %d - %d\n", c1, c2);
    nn::pia::Result r = pSession->RelayEmulation(c1, c2);
    PIA_SAMPLE_ASSERT(r.IsSuccess());
    NN_PIA_UNUSED_IN_RELEASE(r);
}
}
}  // end of namespace nns::pia
