﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Platform.h>
#include <SampleLib/Defines.h>

namespace nns
{
namespace pia
{

// 実行中のモードを表します。
// inet, lan では、当初ランダムマッチメイクで始まって、マッチング後にホストになるかクライアントになるか決まります。
enum GameMode
{
#if NNS_PIA_ENABLE_LOCAL
    GameMode_LocalHost,
    GameMode_LocalClient,
    GameMode_LocalError,
#endif
#if NNS_PIA_ENABLE_INET
    GameMode_InetRandom,
    GameMode_InetHost,
    GameMode_InetClient,
    GameMode_InetError,
#endif
    GameMode_LanRandom,
    GameMode_LanHost,
    GameMode_LanClient,
    GameMode_LanError,
    GameMode_Exit
};


struct OptionalParameter
{
    int32_t offset;  // この値は DecideGameMode() を呼ぶ側が前もってセットしておかなければなりません。

    // 以下の値は DecideGameMode() 内でセットされます。
    bool bRelayEmulation;
    bool bPacketLossEmulation;
    int32_t sendPacketLossPercentage;
    int32_t recvPacketLossPercentage;
    int32_t relayPreset;
    bool bLatencyEmulation;
    int32_t latencyPreset;
    int32_t demandingBitRate;

    OptionalParameter()
    {
        Initialize(0);
    }

    void Initialize(int32_t ofs)
    {
        offset = ofs;

        bRelayEmulation = false;
        bPacketLossEmulation = false;
        sendPacketLossPercentage = 0;
        recvPacketLossPercentage = 0;
        relayPreset = 0;
        bLatencyEmulation = false;
        latencyPreset = 0;
        demandingBitRate = 0;
    }
};


GameMode DecideGameMode(const char* pTitle, OptionalParameter* pOpt = NULL);


void SetRelayEmulation(nn::pia::StationIndex c1, nn::pia::StationIndex c2);
}
}  // end of namespace nns::pia
