﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/common.h>


namespace nn
{
class Result;
}

namespace nn
{
namespace pia
{
class Result;
}
}

namespace nns
{
namespace pia
{

///< これはエラー処理のために用意されたクラスです。
class ErrorHandling
{
public:
    // 初期化処理。
    static void Initialize(void);

    // 終了処理。
    static void Finalize(void);

    // エラーが検出されたか？
    static bool IsErrorDetected(void);

    // エラーが検出されたことが明らかになってから呼んでもらうことを想定。
    // 発生したエラーに関する情報を得る。
    static nn::pia::Result GetResultCode(void);
    static const char* GetFilename(void);
    static uint32_t GetLineNo(void);

#if NN_PIA_CTR || NN_PIA_CAFE || NN_PIA_NINTENDOSDK
    static void PrintSdkResult(nn::Result result);
#endif
    static void PrintResult(nn::pia::Result result);

    // デバッグ出力に本クラスで保持しているリザルトコードの文字列表現をプリント。
    static void Print(void);

    // 本クラスで保持しているnn::pia::Result型の値をクリアする。
    static void Clear(void);

    // {secret}
    // n1769
    // リザルトコードを強引にセットする。わざとエラーを発生させ、
    // エラー処理シーケンスが正常に機能するかどうかをチェックするために用意した。
    static void SetResultCode(nn::pia::Result result);

    static const size_t MaxFilenameLength = 256;

    // nn::pia::Result型のエラーコード関連。
    static nn::pia::Result s_result;
    static char s_filename[MaxFilenameLength];
    static uint32_t s_lineNo;

private:
    ///< コピーコンストラクタ、代入演算子は封印します。
    ErrorHandling(void)
    {
    }
    ~ErrorHandling(void)
    {
    }
    ErrorHandling(const ErrorHandling&);
    ErrorHandling& operator=(const ErrorHandling&);

    static bool s_bInitialized;
};
}
}  // end of namespace nns::pia
