﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Nex_Setup.h"

#if NNS_PIA_ENABLE_INET
#if NN_PIA_CTR
#include <nn/version.h>
#endif

namespace
{
nn::pia::inet::NexNetworkFactory s_NetworkFactory;

// ログインネットワークエラーコード
#if NN_PIA_NINTENDOSDK
nn::Result s_LoginError;
#else
int32_t s_LoginError = 0;
#endif

}  // end of namespace

namespace nns
{
namespace pia
{

//-----------------------------------------------------------------------------
// ゲームサーバにログインします
// フレンドサーバへのログインも同時に行います
//-----------------------------------------------------------------------------
bool Inet_Login(uint32_t timeout)
{
    PIA_SAMPLE_PRINT("Inet_Login\n");
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(nn::pia::inet::NexFacade::GetInstance()));

#if NN_PIA_NINTENDOSDK
    s_LoginError = nn::ResultSuccess();
#else
    s_LoginError = 0;
#endif

    NN_PIA_COMMON_SET_PROGRAM_INFORMATION();

    if (!Nex_Login(&s_LoginError, timeout))
    {
        PIA_SAMPLE_PRINT("Inet_Login:Login failed\n");
        return false;
    }

    // NexFacade に NEX ゲームサーバ情報を設定
    nn::pia::inet::NexFacade::LoginInfo loginInfo;
    nn::pia::common::String accessKey(NNS_PIA_LOGIN_ACCESS_KEY);

    loginInfo.gameServerId = NNS_PIA_LOGIN_GAME_ID;
    loginInfo.pAccessKey = &accessKey;
    loginInfo.pNgsFacade = Nex_GetNgsFacadeInstance();

    nn::pia::Result result = nn::pia::inet::NexFacade::GetInstance()->Bind(&loginInfo);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    PIA_SAMPLE_PRINT("Inet_Login:Login succeeded\n");

    return true;
}

//-----------------------------------------------------------------------------
// ゲームサーバからログアウトします
//-----------------------------------------------------------------------------
bool Inet_Logout()
{
    // NexFacade から NEX ゲームサーバ情報を解除
    if (nn::pia::inet::NexFacade::GetInstance())
    {
        nn::pia::inet::NexFacade::GetInstance()->Unbind();
    }

    bool result = Nex_Logout();
    if (result == true)
    {
        PIA_SAMPLE_PRINT("Inet_Logout:Logout succeeded\n");
    }
    else
    {
        PIA_SAMPLE_PRINT("Inet_Logout:Logout failed\n");
    }
    return true;
}

#if NN_PIA_NINTENDOSDK
nn::Result Inet_GetLoginError()
#else
int32_t Inet_GetLoginError()
#endif
{
    return s_LoginError;
}

nn::pia::Result Inet_InitializePiaInet()
{
    PIA_SAMPLE_PRINT("Inet_InitializePiaInet\n");
    nn::pia::Result result = nn::pia::ResultSuccess();

    result = nn::pia::inet::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::inet::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::inet::NexFacade::CreateInstance();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::inet::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // メモリ消費量をプリント
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Inet_InitializePiaInet:Pia current memory usage: %d\n", nn::pia::common::GetMemoryUsage());
    return result;
}

void Inet_FinalizePiaInet()
{
    PIA_SAMPLE_PRINT("Inet_FinalizePiaInet\n");

    // PiaではFinalize、DestroyInstanceは常に成功します
    nn::pia::inet::NexFacade::DestroyInstance();
    nn::pia::inet::Finalize();

    PIA_SAMPLE_PRINT("Inet_FinalizePiaInet:finished\n");
}

nn::pia::transport::NetworkFactory* Inet_GetNetworkFactory()
{
    return &s_NetworkFactory;
}
}
}  // end of namespace nns::pia
#endif
