﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Lan_Setup.h"

namespace
{
nn::pia::lan::LanNetworkFactory s_NetworkFactory;
}

namespace nns
{
namespace pia
{

nn::pia::Result Lan_StartupPiaLan(int localCommunicationVersion)
{
    if (!Net_IsInitialized())
    {
        PIA_SAMPLE_PRINT("Lan_StartupPiaLan: Network is not initialized yet.\n");
        return nn::pia::ResultNetworkConnectionIsLost();
    }

    if (!PIA_IS_VALID_POINTER(nn::pia::lan::LanFacade::GetInstance()))
    {
        PIA_SAMPLE_PRINT("Lan_StartupPiaLan: LanFacade is not created yet.\n");
        return nn::pia::ResultInvalidState();
    }

    nn::pia::lan::LanFacade::LanNetworkSetting setting;
    bool isSuccess = Net_GetLocalAddress(&setting.localAddress, &setting.subnetmask);
    if (!isSuccess)
    {
        PIA_SAMPLE_PRINT("Lan_StartupPiaLan: Net_GetLocalAddress() failed.\n");
        return nn::pia::ResultNetworkConnectionIsLost();
    }
    else if (setting.localAddress == 0)
    {
        PIA_SAMPLE_PRINT("Lan_StartupPiaLan: localAddress == 0\n");
        return nn::pia::ResultNetworkConnectionIsLost();
    }
    setting.localCommunicationVersion = localCommunicationVersion;

    nn::pia::Result result = nn::pia::lan::LanFacade::GetInstance()->Bind(setting);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("Lan_StartupPiaLan: LanFacade::Bind failed\n");
        return result;
    }

    return nn::pia::ResultSuccess();
}

void Lan_CleanupPiaLan()
{
    if (nn::pia::lan::LanFacade::GetInstance())
    {
        nn::pia::lan::LanFacade::GetInstance()->Unbind();
    }
}

nn::pia::Result Lan_InitializePiaLan()
{
    PIA_SAMPLE_PRINT("Lan_InitializePiaLan\n");
    nn::pia::Result result;

    result = nn::pia::lan::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::lan::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::lan::LanFacade::CreateInstance();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::lan::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // メモリ消費量をプリント
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Lan_InitializePiaLan:Pia current memory usage: %d\n", nn::pia::common::GetMemoryUsage());
    return result;
}

void Lan_FinalizePiaLan()
{
    PIA_SAMPLE_PRINT("Lan_FinalizePiaLan\n");

    nn::pia::lan::LanFacade::DestroyInstance();

    nn::pia::lan::Finalize();

    PIA_SAMPLE_PRINT("Lan_FinalizePiaLan:finished\n");
}

nn::pia::transport::NetworkFactory* Lan_GetNetworkFactory()
{
    return &s_NetworkFactory;
}
}
}  // end of namespace nns::pia
