﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include "SampleLib/SampleLib.h"

#if NNS_PIA_ENABLE_LOCAL

#include <nn/pia/transport.h>
#include <nn/pia/local.h>

namespace nns
{
namespace pia
{
#if NN_PIA_NINTENDOSDK
// ローカル通信識別子です。
const uint64_t LocalCommunicationId = 0x01004cf00004c000;
#endif

struct LocalSetting
{
    LocalSetting()
    {
        idKey1 = 0;
        idKey2 = 0;
        isAroundNetworkSearchEnabled = false;
#if NN_PIA_CTR
        sendOption = nn::uds::NO_WAIT;
#endif
    }
    LocalSetting(uint64_t key1, uint16_t key2)
    {
        idKey1 = key1;
        idKey2 = key2;
        isAroundNetworkSearchEnabled = false;
#if NN_PIA_CTR
        sendOption = nn::uds::NO_WAIT;
#endif
    }
    LocalSetting(uint64_t key1, uint16_t key2, bool ans)
    {
        idKey1 = key1;
        idKey2 = key2;
        isAroundNetworkSearchEnabled = ans;
#if NN_PIA_CTR
        sendOption = nn::uds::NO_WAIT;
#endif
    }
#if NN_PIA_CTR
    LocalSetting(uint64_t key1, uint16_t key2, bool ans, uint8_t sendOpt)
    {
        idKey1 = key1;
        idKey2 = key2;
        isAroundNetworkSearchEnabled = ans;
        sendOption = sendOpt;
    }
#endif
    uint64_t idKey1;
    uint16_t idKey2;
    bool isAroundNetworkSearchEnabled;
#if NN_PIA_CTR
    uint8_t sendOption;
#endif
};

// PiaLocal の初期化処理です。
nn::pia::Result Local_InitializePiaLocal(const LocalSetting& setting);

// PiaLocal の終了処理です。
void Local_FinalizePiaLocal();

// ローカル通信機能の初期化処理です。
bool Local_InitializeLocalCommunication();

// ローカル通信機能の終了処理です。
void Local_FinalizeLocalCommunication();

// 親機としてネットワークを構築します。
bool Local_CreateNetwork(uint16_t maxStations);
bool Local_CreateNetwork(nn::pia::local::LocalCreateNetworkSetting* pCreateNetworkSetting);

// 子機としてネットワークに参加します。
bool Local_SearchAndConnectToNetwork(nn::pia::transport::StationConnectionInfo* pStationConnectionInfo);

bool Local_ScanNetwork(nn::pia::local::LocalScanNetworkSetting* pScanNetworkSetting);
bool Local_ConnectNetwork(nn::pia::local::LocalConnectNetworkSetting* pConnectNetworkSetting);
bool Local_DisconnectNetwork(bool isMigration = false);
nn::pia::Result Local_StartupPiaLocal();
void Local_CleanupPiaLocal();

// ネットワークを終了します。
void Local_CleanupNetwork();

// Transport の初期化に必要な NetworkFactory を取得します。
nn::pia::transport::NetworkFactory* Local_GetNetworkFactory();

// ネットワークへ接続しているかを取得します。
bool Local_IsConnected();
}
}  // end of namespace nns::pia

#endif  // NN_PIA_CTR || NN_PIA_NINTENDOSDK
