﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Definitions.h>

#if NN_PIA_NINTENDOSDK
#include <nn/account.h>
#endif

#include "Defines.h"

namespace
{
}

namespace nns
{
namespace pia
{

enum NetResult
{
    NetResult_Success = 0,
    NetResult_NicDown,
    NetResult_AcError,
    NetResult_SocketError,
    NetResult_AccountError,
    NetResult_FriendsError
};

/*!
    @brief ネットワーク種別を示す列挙型です。
*/
enum NetworkType
{
    NetworkType_Local,     //!< ローカル通信を示します。
    NetworkType_Internet,  //!< インターネット通信を示します。
    NetworkType_Lan        //!< LAN通信を示します。
};

// ネットワーク機能の初期化処理
NetResult Net_InitializeNetwork(NetworkType networkType);

// アカウント機能の初期化処理
NetResult Net_InitializeAccount(NetworkType networkType);

// フレンド機能の初期化処理
NetResult Net_InitializeFriends();

// ネットワーク機能の終了処理
void Net_FinalizeNetwork();

// アカウント機能の終了処理
void Net_FinalizeAccount();

// フレンド機能の終了処理
void Net_FinalizeFriends();

// ネットワーク機能の初期化済みかどうか
bool Net_IsInitialized();

// ネットワークの接続状態チェック
bool Net_IsConnected();

// 自身のアドレス取得
bool Net_GetLocalAddress(uint32_t* pIpAddr, uint32_t* pSubnetMask);

// ネットワークエラーコード取得
uint32_t Net_GetNetworkErrorCode();

#if NN_PIA_NINTENDOSDK
struct UserAccountInfo
{
    nn::account::Uid userUid;
    nn::account::UserHandle userHandle;
};

int Net_GetUserAccountInfoSize();
UserAccountInfo* Net_GetMainUserAccountInfo();
UserAccountInfo* Net_GetUserAccountInfo(int index);
#endif
}
}  // end of namespace nns::pia
