﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Nex_Setup.h"

#if NNS_PIA_ENABLE_INET

#if NN_PIA_CTR
#pragma diag_suppress 1300
#endif

#include <OnlineCore/src/version/nex_version.h>

#if NN_PIA_CTR
#pragma diag_warning 1300
#endif


// 開発時に NEX パッケージに応じたログイン設定を取得します。
// このソースコードは Pia パッケージに含めてはいけません。

namespace nns
{
namespace pia
{
uint32_t Nex_GetLoginGameId()
{
    nn::pia::common::String nexPatchVersion(NEX_VERSION_PATCH);
    uint32_t nexVersion = NEX_CURRENT_VERSION_NUMBER;
#if NN_PIA_CTR
    if (nexPatchVersion == nn::pia::common::String("AGQ"))
    {
        return 0x2984E403;
    }
    if (nexVersion >= NEX_VERSION_NUMBER(4, 1, 1))
    {
        return 0x2CC90F00;
    }
    if (nexVersion >= NEX_VERSION_NUMBER(3, 10, 0))
    {
        return 0x1F900000;
    }
    if (nexVersion >= NEX_VERSION_NUMBER(3, 6, 0))
    {
        return 0x1F906E00;
    }
#elif NN_PIA_CAFE
    if (nexVersion == NEX_VERSION_NUMBER(4, 0, 0))
    {
        return 0x1F900D01;
    }
    if (nexVersion >= NEX_VERSION_NUMBER(3, 10, 0))
    {
        return 0x1F900000;
    }
    if (nexVersion >= NEX_VERSION_NUMBER(3, 6, 0))
    {
        return 0x1F906E00;
    }
#elif NN_PIA_NINTENDOSDK
    if (nexPatchVersion == nn::pia::common::String("AGQ"))
    {
        return 0x2984E403;
    }
    if (nexVersion >= NEX_VERSION_NUMBER(4, 0, 0))
    {
        return 0x2122D301;
    }
#endif
    PIA_SAMPLE_ASSERT(false);
    return 0;
}

const SampleStringChar_t* Nex_GetLoginAccessKey()
{
    nn::pia::common::String nexPatchVersion(NEX_VERSION_PATCH);
    uint32_t nexVersion = NEX_CURRENT_VERSION_NUMBER;

#if NN_PIA_CTR
    if (nexPatchVersion == nn::pia::common::String("AGQ"))
    {
        return SAMPLE_STRING("1a855aea");
    }
    if (nexVersion >= NEX_VERSION_NUMBER(4, 1, 1))
    {
        return SAMPLE_STRING("e8a3de21");
    }
    if (nexVersion >= NEX_VERSION_NUMBER(3, 10, 0))
    {
        return SAMPLE_STRING("3c273382");
    }
    if (nexVersion >= NEX_VERSION_NUMBER(3, 6, 0))
    {
        return SAMPLE_STRING("3c273382");
    }
#elif NN_PIA_CAFE
    if (nexVersion == NEX_VERSION_NUMBER(4, 0, 0))
    {
        return SAMPLE_STRING("9cc6afe8");
    }
    if (nexVersion >= NEX_VERSION_NUMBER(3, 10, 0))
    {
        return SAMPLE_STRING("3c273382");
    }
    if (nexVersion >= NEX_VERSION_NUMBER(3, 6, 0))
    {
        return SAMPLE_STRING("3c273382");
    }
#elif NN_PIA_NINTENDOSDK
    if (nexPatchVersion == nn::pia::common::String("AGQ"))
    {
        return SAMPLE_STRING("1a855aea");
    }
    if (nexVersion >= NEX_VERSION_NUMBER(4, 0, 0))
    {
        return SAMPLE_STRING("b968e53f");
    }
#endif
    PIA_SAMPLE_ASSERT(false);
    return NULL;
}
}
}  // end of namespace nns::pia
#endif  // end of NNS_PIA_ENABLE_INET
