﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include "SampleLib/Nex_Setup.h"

#if NNS_PIA_ENABLE_INET

#if NN_PIA_CTR
#pragma diag_suppress 1300
#endif

#include <nex.h>

#include <OnlineCore/src/version/nex_version.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <OnlineCore/src/Platform/Core/SystemErrorResult.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/Type.h>
#include <OnlineCore/src/Platform/Core/SystemClock.h>
#include <OnlineCore/src/Platform/Core/UpdatableTimeProvider.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Transport/UDP/Packet.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPoint.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#include <OnlineCore/src/Transport/Interface/TransportError.h>
#include <OnlineCore/src/Transport/Interface/TransportAdapter.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <NetZ/src/Core/DOCore/Session.h>
#include <NetZ/src/Core/DOCore/SessionDescription.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DirectStream.h>
#include <NetZ/src/Core/DOCore/DirectTransport.h>
#include <NetZ/src/Extensions/SessionSelection/ConsoleSessionEnumerator.h>
#include <NetZ/HostMigration.h>
#include <RendezVous/Services/NATTraversal/src/Client/NATTraversalClient.h>
#include <RendezVous/Services/Jugem/Debug/src/Client/DebugClient.h>

#if NN_PIA_CTR
#pragma diag_warning 1300
#endif

namespace nns
{
namespace pia
{

bool Nex_InitializeMatchmake();
bool Nex_FinalizeMatchmake();
nn::nex::MatchmakeExtensionClient* Nex_GetMatchmakeClient();

bool Nex_AutoMatchmake(
    nn::nex::GatheringID* pJoinedGatheringId,
    nn::nex::PrincipalID* pOwnerPID,
    uint32_t matchmakeMode,
    uint16_t matchmakeMaxParticipants,
    const char* matchmakeDescription = NULL);

bool Nex_OpenParticipation(nn::nex::GatheringID gatheringID);

bool Nex_LeaveMatchmakeSession(
    nn::nex::GatheringID gatheringID,
    nn::nex::PrincipalID ownerPID,
    bool hostMigration);

bool Nex_GetHostStationURLs(
    nn::nex::GatheringID joinedGatheringId,
    nn::nex::qList<nn::nex::StationURL>* pStationURLs);

bool Nex_SearchMatchmakeSession(
    uint32_t gameMode,
    uint16_t maxParticipants,
    uint32_t rangeSize,
    nn::nex::qList<nn::nex::GatheringHolder>* plistGatheringHolder);

bool Nex_CreateMatchmakeSession(
    nn::nex::GatheringID* pJoinedGatheringID,
    uint32_t gameMode,
    uint16_t maxParticipants,
    const char* description);

bool Nex_JoinMatchmakeSession(nn::nex::GatheringID gatheringID);

uint32_t Nex_GetMatchmakeNetworkErrorCode();
void Nex_ResetMatchmakeNetworkErrorCode();

nn::nex::qVector<nn::nex::qByte>* Nex_GetMatchmakeSessionSignatureKey();
nn::pia::common::SignatureSetting* Nex_GetMatchmakeSessionSignatureSetting();

nn::nex::PrincipalID Nex_GetOwnerPID();
void Nex_SetOwnerPID(nn::nex::PrincipalID id);

// プロトコルコールコンテキストのエラーを表示します
void Nex_ReportProtocolCallContext(nn::nex::ProtocolCallContext& context, const char* pHeaderText = NULL);
}
}  // end of namespace nns::pia

#endif
