﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Settings.h>
#include <nn/pia/pia_Platform.h>
#include <nn/pia/pia_Result.h>

#include "SampleLib/Defines.h"

#if NNS_PIA_ENABLE_INET

#pragma diag_suppress 1300
#include <OnlineCore/src/version/nex_version.h>
#include <RendezVous/Services/Jugem/Login/src/Client/NgsFacade.h>
#pragma diag_warning 1300

#define NEX_VERSION_NUMBER(major, minor, micro) (((major) << 16) | ((minor) << 8) | ((micro) << 0))
#define NEX_CURRENT_VERSION_NUMBER NEX_VERSION_NUMBER(NEX_VERSION_MAJOR, NEX_VERSION_MINOR, NEX_VERSION_MICRO)


// ログインに使用するパラメータ
#define NNS_PIA_LOGIN_GAME_ID 0x25E5BE00
#define NNS_PIA_LOGIN_ACCESS_KEY SAMPLE_STRING("12345678")

namespace
{
}

namespace nns
{
namespace pia
{

// NEX の初期化処理を行います
nn::pia::Result Nex_InitializeNex(size_t memorySize, size_t emergencyMemorySize);

// NEX の終了処理を行います
void Nex_FinalizeNex();

// ゲームサーバーにログインします
bool Nex_Login(int32_t* pLoginError, uint32_t timeout = 30 * 1000);

// ゲームサーバーからログアウトします
bool Nex_Logout();

// ゲームサーバに接続しているか取得します
nn::pia::Result Nex_IsNgsConnected();

// 自分のプリンシパルIDを取得します
nn::pia::PrincipalId Nex_GetLocalPrincipalId();

// NEX のスケジューラーディスパッチを実行します
void Nex_SchedulerDispatch();

// NEX の NgsFacade のインスタンスを取得します
nn::nex::NgsFacade* Nex_GetNgsFacadeInstance();

}
} // end of namespace nns::pia


#endif

