﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Platform.h>
#include <nn/pia/session.h>
#include <nn/pia/transport.h>

namespace nns
{
namespace pia
{

// サンプル内で nn::pia::Result で表現されていないエラーを大別するための nn::pia::Result 定義群
enum SAMPLE_RESULT_MESH_DESCRIPTION
{
    MESH_STATE_DISCONNECTED = 20  // セッションから切断された
};

PIA_DEFINE_RESULT(
    SampleMeshStateDisconnected,
    nn::pia::Result::HandlingType_Void,
    nn::pia::Result::ViewerType_Void,
    (nn::pia::Result::Description)MESH_STATE_DISCONNECTED);

// メッシュ直接操作時の PiaSession の初期化処理です。
nn::pia::Result Session_InitializePiaSession_ForMesh(const nn::pia::session::Mesh::Setting& setting);
nn::pia::Result Session_InitializePiaSession_ForMesh(const nn::pia::session::Mesh::Setting& setting, bool isRelayEmulationEnabled);

// メッシュ直接操作時の PiaSession の終了処理です。
void Session_FinalizePiaSession_ForMesh();

// メッシュを開始します。
nn::pia::Result Session_StartupMesh(nn::pia::session::Mesh::StartupSetting startupSetting);

// メッシュを終了します。
void Session_CleanupMesh();

uint8_t Session_GetLocalPlayerInfo_ForMesh(nn::pia::transport::Station::PlayerInfo* pPlayerInfo, uint8_t playerInfoSize, bool isUserNameInAppilcationUsed = false);

// Hostが呼び出します。メッシュを作成します。
bool Session_CreateMesh();

// Hostが呼び出します。メッシュを破棄します。
void Session_DestroyMesh();

// Hostが呼び出します。ホストマイグレーション開始指示を出してメッシュを離脱します。
void Session_LeaveMeshWithHostMigration();

// Clientが呼び出します。メッシュに参加します。
bool Session_JoinMesh(const nn::pia::transport::StationConnectionInfo& stationConnectionInfo);

// Clientが呼び出します。メッシュから離脱します。
void Session_LeaveMesh();
}
}  // end of namespace nns::pia
