﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Platform.h>
#include <nn/pia/session.h>
#include <nn/pia/transport.h>

namespace nns
{
namespace pia
{

// (Pia 開発者向け関数）
// セッション構築／参加中に毎フレーム実行されるコールバック型
typedef void (*SampleSessionCallback)();

// コールバックを登録するAPI（SampleFramework::EndFrame() の後のタイミングで呼び出されます）
SampleSessionCallback RegisterEndFrameSampleSessionCallback(SampleSessionCallback cb);

// コールバックを登録するAPI（common::Dispatch() の後のタイミングで呼び出されます）
SampleSessionCallback RegisterDispatchSampleSessionCallback(SampleSessionCallback cb);

// PiaSession の初期化処理です。
nn::pia::Result Session_InitializePiaSession(const nn::pia::session::Session::Setting& setting);

// PiaSession の終了処理です。
void Session_FinalizePiaSession();

nn::pia::Result Session_StartupSession(const nn::pia::session::Session::StartupSetting& startupSetting);


// セッションを終了します。
void Session_CleanupSession();

uint8_t Session_GetLocalPlayerInfo(nn::pia::transport::Station::PlayerInfo* pPlayerInfo, uint8_t playerInfoSize, bool isUserNameInAppilcationUsed = false);

// セッションのランダムマッチメイクを行います。
nn::pia::Result Session_JoinRandomSession(nn::pia::session::JoinRandomSessionSetting* pJoinRandomSessionSetting, bool bScore = false, bool bRender = true);

// セッションを構築します。
nn::pia::Result Session_CreateSession(nn::pia::session::CreateSessionSetting* pCreateSessionSetting);

// セッションを検索します。
nn::pia::Result Session_BrowseSession(nn::pia::session::SessionSearchCriteria* pSearchCriteria);

// セッションへ参加します。
nn::pia::Result Session_JoinSession(nn::pia::session::JoinSessionSetting* pJoinSessionSetting);

// セッションから離脱します。
nn::pia::Result Session_LeaveSession();
}
}  // end of namespace nns::pia
