﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "StringUtil.h"

namespace
{
}  // end of anonymous namespace

namespace nns
{
namespace pia
{

void StringUtil::ConvertToAscii(char* pBuffer, uint32_t bufferSize, const nn::pia::common::String& string)
{
    uint32_t stringLength = string.GetStringLength();

    // 初期化
    for (uint32_t i = 0; i < bufferSize; i++)
    {
        pBuffer[i] = '\0';
    }

    // エンコードタイプに応じて文字を取り出す
    if (string.GetEncodingType() == nn::pia::common::String::EncodingType_Utf16)
    {
        const nn::pia::charUtf16_t* p = static_cast<const nn::pia::charUtf16_t*>(string.GetBuffer());
        for (uint32_t i = 0; i < bufferSize && i < stringLength; i++)
        {
            if (*p >= 0x20 && *p <= 0x7f)
            {
                pBuffer[i] = static_cast<char>(*p);
            }
            else
            {
                pBuffer[i] = '?';
            }
            ++p;
        }
    }
    else
    {
        const char* p = string.GetCharArray();
        for (uint32_t i = 0; i < bufferSize && i < stringLength; i++)
        {
            if (*p >= 0x20 && *p <= 0x7f)
            {
                pBuffer[i] = static_cast<char>(*p);
                ++p;
            }
            else
            {
                pBuffer[i] = '?';
                if (*p > 0x00 && *p <= 0x1f)
                {
                    p += 1;
                }
                else if (*p >= 0xc2 && *p <= 0xdf)
                {
                    p += 2;
                }
                else if (*p >= 0xe0 && *p <= 0xef)
                {
                    p += 3;
                }
                else if (*p >= 0xf0 && *p <= 0xf7)
                {
                    p += 4;
                }
                else if (*p >= 0xf8 && *p <= 0xfb)
                {
                    p += 5;
                }
                else if (*p >= 0xfc && *p <= 0xfd)
                {
                    p += 6;
                }
            }
        }
    }
}
}
}  // end of namespace nns::pia
