﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "SampleLib/Sync_Setup.h"

namespace
{
uint32_t s_hSyncProtocol = 0;
}

namespace nns
{
namespace pia
{

void Sync_InitializePiaSync(const nn::pia::sync::SyncProtocol::Setting& setting)
{
    PIA_SAMPLE_PRINT("Sync_InitializePiaSync\n");

    // syncモジュールを初期化します。
    // 事前にcommonモジュールと、transportモジュールのセットアップは済ませておいてください。
    nn::pia::Result r;
    r = nn::pia::sync::Initialize();
    PIA_SAMPLE_ASSERT(r.IsSuccess());

    r = nn::pia::sync::BeginSetup();
    PIA_SAMPLE_ASSERT(r.IsSuccess());

    // Transportシングルトン取得。
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTransport));

    // SyncProtocol （キーデータ同期） の作成と初期化。
    {
        s_hSyncProtocol = pTransport->CreateProtocol<nn::pia::sync::SyncProtocol>();
        PIA_SAMPLE_ASSERT(s_hSyncProtocol != 0);  // CreateProtocol()は、既に同じプロトコルIDが存在していた場合は0を返すので、そうではないことを確認

        // 送信するデータ 1 単位のサイズをチェック
        size_t dataUnitSizeMax = Sync_GetSyncProtocol()->GetDataUnitSizeMax();
        NN_PIA_UNUSED_IN_RELEASE(dataUnitSizeMax);
        for (uint32_t i = 0; i < nn::pia::sync::SyncProtocol::DataIdNum; i++)
        {
            PIA_SAMPLE_ASSERT(setting.dataUnitSize[i] <= dataUnitSizeMax);
        }

        r = Sync_GetSyncProtocol()->Initialize(setting);
        PIA_SAMPLE_ASSERT(r.IsSuccess());
    }

    r = nn::pia::sync::EndSetup();
    PIA_SAMPLE_ASSERT(r.IsSuccess());
}


void Sync_FinalizePiaSync()
{
    PIA_SAMPLE_PRINT("Sync_FinalizePiaSync\n");

    // Transportシングルトン取得。
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTransport));

    // SyncProtocolの終了処理
    nn::pia::sync::SyncProtocol* pSyncProtocol = pTransport->GetProtocol<nn::pia::sync::SyncProtocol>(s_hSyncProtocol);
    if (PIA_IS_VALID_POINTER(pSyncProtocol))
    {
        pSyncProtocol->Finalize();
        pTransport->DestroyProtocol(s_hSyncProtocol);
        s_hSyncProtocol = 0;
    }

    nn::pia::sync::Finalize();
}
nn::pia::sync::SyncProtocol* Sync_GetSyncProtocol()
{
    return nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::sync::SyncProtocol>(s_hSyncProtocol);
}
}
}  // end of namespace nns::pia
