﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Platform.h>
#include <nn/pia/transport.h>

namespace nns
{
namespace pia
{

typedef nn::pia::Result (*InitializeAdditionalProtocolFunction)();
typedef void (*FinalizeAdditionalProtocolFunction)();

#if NN_PIA_CTR
const uint32_t MaxSendThreadBufferNum = 32;
const uint32_t MaxReceiveThreadBufferNum = 32;
#else
const uint32_t MaxSendThreadBufferNum = 64;
const uint32_t MaxReceiveThreadBufferNum = 64;
#endif

nn::pia::Result Transport_InitializePiaTransport(const nn::pia::transport::Transport::Setting& setting);

nn::pia::Result Transport_InitializePiaTransport(
    const nn::pia::transport::Transport::Setting& setting,
    uint16_t unreliableProtocolPort,
    uint32_t* pUnreliableProtocolHandle,  //n1769: NULL を渡すと、UnreliableProtocol の初期化処理をしない。
    uint16_t reliableProtocolPort,
    uint32_t* pReliableProtocolHandle,  //n1769: NULL を渡すと、ReliableProtocol の初期化処理をしない。
    uint32_t reliableSendBufNum = 64,
    uint32_t reliableRecvBufNum = 64,
    InitializeAdditionalProtocolFunction pAdditionalFunction = NULL,
    uint32_t unreliableRecvBufNum = 32);

nn::pia::Result Transport_InitializePiaTransport(
    nn::pia::transport::NetworkFactory* pFactory,
    uint16_t unreliableProtocolPort,
    uint32_t* pUnreliableProtocolHandle,  //n1769: NULL を渡すと、UnreliableProtocol の初期化処理をしない。
    uint16_t reliableProtocolPort,
    uint32_t* pReliableProtocolHandle,  //n1769: NULL を渡すと、ReliableProtocol の初期化処理をしない。
    uint32_t reliableSendBufNum = 64,
    uint32_t reliableRecvBufNum = 64,
    InitializeAdditionalProtocolFunction pAdditionalFunction = NULL,
    uint32_t unreliableRecvBufNum = 32);

void Transport_FinalizePiaTransport();

void Transport_FinalizePiaTransport(
    uint32_t unreliableProtocolHandle,  //n1769: 0 を渡すと、UnreliableProtocol の終了処理をしない。
    uint32_t reliableProtocolHandle,    //n1769: 0 を渡すと、ReliableProtocol の終了処理をしない。
    FinalizeAdditionalProtocolFunction pAdditionalFunction = NULL);
}
}  // end of namespace nns::pia
