﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Settings.h>
#include <nn/pia/pia_Version.h>
#include "SampleLib/SampleLib.h"
#include "SampleLib/Print.h"

#if NN_PIA_CTR
#include <nn.h>
#include <nn/version.h>
#endif

#if NN_PIA_WIN
#include <nn/pia/platform/pia_PlatformWin.h>
#endif

#if NN_PIA_CAFE
#include <cafe.h>
#include <sdk_ver.h>
#endif

#if NNS_PIA_ENABLE_INET
#include <OnlineCore/src/version/nex_version.h>
#endif

namespace nns
{
namespace pia
{
class VersionInfo
{
public:
    // ビルド時の SDK, NEX のバージョンを表示します。
    static inline void TraceBuildEnvironment() {

        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("-------- Version information begin --------\n");

#if NN_PIA_CTR
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("[VERSION_INFO] <SDK version>: %s\n", NN_CURRENT_SDK_VERSION);
#elif NN_PIA_CAFE
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("[VERSION_INFO] <SDK version>: %s\n", CAFE_OS_SDK_VERSION_STRING);
#elif NN_PIA_NINTENDOSDK
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("[VERSION_INFO] <SDK version>: %d.%d.%d\n", NN_SDK_VERSION_MAJOR, NN_SDK_VERSION_MINOR, NN_SDK_VERSION_MICRO);
#if NN_PIA_NINTENDOSDK_NX
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("[VERSION_INFO] <NXAddon version>: %d.%d.%d\n", NN_NX_ADDON_VERSION_MAJOR, NN_NX_ADDON_VERSION_MINOR, NN_NX_ADDON_VERSION_MICRO);
#endif
#endif

#if NNS_PIA_ENABLE_INET
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("[VERSION_INFO] <NEX version>: %s\n", NEX_VERSION_STRING);
#endif

        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("[VERSION_INFO] <PIA version>: %s\n", PIA_VERSION_STRING);

        //BuildType
#if NN_PIA_DEBUG
        const char* strBuildType = "Debug";
#elif NN_PIA_DEVELOP
        const char* strBuildType = "Develop";
#elif NN_PIA_RELEASE
        const char* strBuildType = "Release";
#else
#error "Invalid buildType"
#endif
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("[VERSION_INFO] <BuildType>: %s\n", strBuildType);

        //PLATFORM_ADDRESS(32 or 64)
#if NN_PIA_NINTENDOSDK_NX
#if NN_BUILD_TARGET_PLATFORM_ADDRESS_32
        const char* strPlatformAddress = "32";
#elif NN_BUILD_TARGET_PLATFORM_ADDRESS_64
        const char* strPlatformAddress = "64";
#else
#error "Invalid PLATFORM_ADDRESS"
#endif
        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("[VERSION_INFO] <PlatformAddress>: %s\n", strPlatformAddress);
#endif

        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("-------- Version information end --------\n");
    }
};
}
}  // end of namespace nns::pia
