﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "ReckoningStrategy.h"

bool ReckoningStrategy::Estimate(Value* pValue, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const
{
    if (accessor.GetNum() <= 0)
    {
        return false;
    }
    if (clock > accessor.GetClock(0))
    {
        *pValue = accessor.GetSample(0);
        return true;
    }
    if (clock < accessor.GetClock(accessor.GetNum() - 1))
    {
        *pValue = accessor.GetSample(accessor.GetNum() - 1);
        return true;
    }

    float tt = static_cast<float>(static_cast<int32_t>(clock - cpWork->baseClock));
    *pValue = (cpWork->paramA * tt + cpWork->paramB) * tt + cpWork->paramC;
    return true;
}

bool ReckoningStrategy::CheckSample(bool* pIsReliable, const Value& value, const SetValueArg* cpSetValueArg, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const
{
    NN_PIA_UNUSED(cpSetValueArg);

    if (accessor.GetNum() <= 0)
    {
        *pIsReliable = true;
        return true;
    }

    float estimatedValue;
    bool r = Estimate(&estimatedValue, accessor, clock, cpWork);
    if (r == false)
    {
        return true;
    }

    float diff = value - estimatedValue;
    return (diff <= -5.0f || diff >= 5.0f);
}

void ReckoningStrategy::MakeSample(Sample* pSample, const Value& value, const SetValueArg* cpSetValueArg, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const
{
    NN_PIA_UNUSED(cpSetValueArg);
    NN_PIA_UNUSED(accessor);
    NN_PIA_UNUSED(clock);
    NN_PIA_UNUSED(cpWork);

    *pSample = value;
}

void ReckoningStrategy::OnUpdateSample(const Accessor& accessor, int index, Work* pWork) const
{
    NN_PIA_UNUSED(index);

    if (accessor.GetNum() <= 0)
    {
        pWork->paramA = 0.0f;
        pWork->paramB = 0.0f;
        pWork->paramC = 0.0f;
        pWork->baseClock = 0;
        return;
    }

    float v0 = accessor.GetSample(0);
    nn::pia::clone::ClockValue c0 = accessor.GetClock(0);

    pWork->paramC = v0;
    pWork->baseClock = c0;

    if (accessor.GetNum() >= 3)
    {
        nn::pia::clone::ClockValue c1 = accessor.GetClock(1);
        nn::pia::clone::ClockValue c2 = accessor.GetClock(2);

        PIA_SAMPLE_ASSERT(c0 != c1 && c1 != c2);
        {
            float t1 = static_cast<float>(static_cast<int32_t>(c1 - pWork->baseClock));
            float t2 = static_cast<float>(static_cast<int32_t>(c2 - pWork->baseClock));
            float v1 = accessor.GetSample(1) - pWork->paramC;
            float v2 = accessor.GetSample(2) - pWork->paramC;

            pWork->paramA = (t2 * v1 - t1 * v2) / (t1 * t2 * (t1 - t2));
            pWork->paramB = v1 / t1 - pWork->paramA * t1;
            return;
        }
    }

    pWork->paramA = 0.0f;

    if (accessor.GetNum() >= 2)
    {
        nn::pia::clone::ClockValue c1 = accessor.GetClock(1);

        PIA_SAMPLE_ASSERT(c0 != c1);
        {
            float t1 = static_cast<float>(static_cast<int32_t>(c1 - pWork->baseClock));
            float v1 = accessor.GetSample(1) - pWork->paramC;

            pWork->paramB = v1 / t1;
            return;
        }
    }

    pWork->paramB = 0.0f;
}
