﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include "SampleLib/SampleLib.h"
#include <cstdio>

#if NN_PIA_CTR || NN_PIA_NINTENDOSDK_NX
#define ENABLE_PIA_LOCAL 1
#else
#define ENABLE_PIA_LOCAL 0
#endif

#ifdef PIA_TEST_BUILD
// 自動検証用のコードです。サンプルとしては無視してください。
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif

#if ENABLE_PIA_LOCAL
#include <nn/pia/local.h>
#include "SampleLib/Local_Setup.h"
#endif

#include "SampleLib/Basic_Setup.h"
#include "SampleLib/Common_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Lan_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Reckoning_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleLib/Transport_Setup.h"
#include "SampleLib/Clone_Setup.h"

#define PROGRAM_TITLE "CloneProtocol"

 // 最大 Station 数
#if PIA_TEST_CODE
#if NN_PIA_CTR
const int MaxStations = 12;
#elif NN_PIA_CAFE
const int MaxStations = 16;
#elif NN_PIA_NINTENDOSDK
const int MaxStations = 16;
#else
const int MaxStations = 8;
#endif
#else
const int MaxStations = 8;
#endif
#if ENABLE_PIA_LOCAL
const int LocalMaxStations = 8;
#endif

const bool IsHostMigrationEnabled = true;  //!< 親交代機能の ON/OFF。

extern nn::pia::StationId g_StationIdList[MaxStations];
nn::pia::StationIndex ConvertStationIdToStationIndex(nn::pia::StationId stationId);

#if PIA_TEST_CODE
const uint32_t MAX_SILENT_TIME = 90000;

// PiaTestで使用する配列に格納する順番
enum ParamKey
{
    LocalMaxStationNum = 0x00,
    InetMaxStationNum,
    LanMaxStationNum,
    STATION_INDEX,
    SESSION_PROCESSING,
    CLONE_PROTOCOL_STATE = 0x10,
    CLOCK_TYPE,
    CLOCK_VALUE,
    CURSOR_POS = 0x20,
    SEND_UNRELIABLE_SEND = 0x30,
    SEND_RELIABLE_SEND,
    SEND_EVENT_SEND,
    SEND_RECKONING_SEND,
    SEND_IS_REGISTERED,
    SEND_CONNECTED,
    ATOMIC_UNRELIABLE = 0x40,
    ATOMIC_RELIABLE,
    ATOMIC_EVENT,
    ATOMIC_RECKONING,
    ATOMIC_UNRELIABLE_SEND,
    ATOMIC_RELIABLE_SEND,
    ATOMIC_EVENT_SEND,
    ATOMIC_RECKONING_SEND,
    ATOMIC_IS_REGISTERED,
    ATOMIC_CONNECTED,
    ATOMIC_LOCK_STATUS,
    SEQUENTIAL_UNRELIABLE = 0x50,
    SEQUENTIAL_RELIABLE,
    SEQUENTIAL_UNRELIABLE_SEND,
    SEQUENTIAL_RELIABLE_SEND,
    SEQUENTIAL_IS_REGISTERED,
    SEQUENTIAL_CONNECTED,
    RECEIVE_UNRELIABLE = 0x60,
    RECEIVE_RELIABLE = 0x70,
    RECEIVE_EVENT = 0x80,
    RECEIVE_RECKONING = 0x90,
    RECEIVE_IS_REGISTERED = 0xa0,
    RECEIVE_IS_CONNECTED = 0xb0
};

#endif
