﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "Object.h"
#include "SampleConfig.h"
#include <stdlib.h>

#if NN_PIA_A
#include "SampleFramework/include/AdrMainHelper.h"
#endif

nn::pia::StationId g_StationIdList[MaxStations];

nn::pia::StationIndex ConvertStationIdToStationIndex(nn::pia::StationId stationId)
{
    nn::pia::StationIndex stationIndex;
    nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&stationIndex, stationId);
    if (result.IsFailure())
    {
        stationIndex = nn::pia::StationIndex_Invalid;
    }
    return stationIndex;
}

namespace
{
#if PIA_TEST_CODE
static nn::pia::clone::CloneProtocol::Setting::ClockType s_ClockType = nn::pia::clone::CloneProtocol::Setting::ClockType_Rtc;
#endif

nns::pia::NetworkType s_NetworkType = nns::pia::NetworkType_Internet;

// 実行中のモードを表します。
// inet では、当初 automatch で始まって、マッチング後に master になるか client になるか決まります。
enum GameMode
{
#if ENABLE_PIA_LOCAL
    GameMode_LocalHost,
    GameMode_LocalClient,
#endif
#if NNS_PIA_ENABLE_INET
    GameMode_InetRandom,
    GameMode_InetHost,
    GameMode_InetClient,
#endif
    GameMode_LanRandom,
    GameMode_LanHost,
    GameMode_LanClient,
#if ENABLE_PIA_LOCAL
    GameMode_LocalError,
#endif
#if NNS_PIA_ENABLE_INET
    GameMode_InetError,
#endif
    GameMode_LanError,
    GameMode_Exit
};


// 実行するモードを決定します。
GameMode DecideGameMode()
{
#if ENABLE_PIA_LOCAL
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Please select game mode.\n [A]LocalHost\n [B]LocalClient\n [X]InetRandom\n [Y]LanRandom\n [START]Quit\n");
#else
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Please select game mode.\n [X]InetRandom\n [Y]LanRandom\n [START]Quit\n");
#endif
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(" Waiting input\n");

    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    GameMode gameMode = GameMode_Exit;
    bool isDecided = false;
    while (true)
    {
        pSampleFramework->BeginFrame();

        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Exit\n");
            gameMode = GameMode_Exit;
            isDecided = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LanRandom\n");
            gameMode = GameMode_LanRandom;
            isDecided = true;
        }
#if NNS_PIA_ENABLE_INET
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: InetRandom\n");
            gameMode = GameMode_InetRandom;
            isDecided = true;
        }
#endif
#if ENABLE_PIA_LOCAL
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LocalHost\n");
            gameMode = GameMode_LocalHost;
            isDecided = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("GameMode: LocalClient\n");
            gameMode = GameMode_LocalClient;
            isDecided = true;
        }
#endif
#if PIA_TEST_CODE
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR) && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            if (s_ClockType == nn::pia::clone::CloneProtocol::Setting::ClockType_Rtc)
            {
                s_ClockType = nn::pia::clone::CloneProtocol::Setting::ClockType_Frame;
            }
            else
            {
                s_ClockType = nn::pia::clone::CloneProtocol::Setting::ClockType_Rtc;
            }
        }
        pSampleFramework->SetProgramParam(CLOCK_TYPE, s_ClockType);
        pSampleFramework->SetProgramParam(SESSION_PROCESSING, isDecided ? 1 : 0);
#endif

        int32_t y = 10;
        pSampleFramework->DrawString(10, y, PROGRAM_TITLE);
        y += 10;
        pSampleFramework->DrawString(10, y, "Select game mode.");
        y += 10;
        pSampleFramework->DrawString(10, y, "START: Quit this application");
        y += 10;
        pSampleFramework->DrawString(10, y, "Y: LanRandom");
        y += 10;
#if NNS_PIA_ENABLE_INET
        pSampleFramework->DrawString(10, y, "X: InetRandom");
        y += 10;
#endif
#if ENABLE_PIA_LOCAL
        pSampleFramework->DrawString(10, y, "A: LocalHost");
        y += 10;
        pSampleFramework->DrawString(10, y, "B: LocalClient");
        y += 10;
#endif

        pSampleFramework->EndFrame();

        if (isDecided)
        {
            return gameMode;
        }
    }
}
}

namespace
{
static nn::pia::clone::CloneProfiler<MaxStations, 120> s_ProtocolSendProfiler;
static nn::pia::clone::CloneProfiler<MaxStations, 120> s_ProtocolReceivedProfiler;

const uint32_t SendReceiveObjectNum = 1;
const uint32_t AtomiObjectNum = 2;
const uint32_t SequentialObjectNum = 1;
static uint32_t s_ObjectNum = 0;
static SendObject s_SendObject[SendReceiveObjectNum];
static ReceiveObject s_ReceiveObject[SendReceiveObjectNum][MaxStations];
static AtomicObject s_AtomicObject[AtomiObjectNum];
static SequentialObject s_SequentialObject[SequentialObjectNum];
static Object* s_apObject[(1 + MaxStations) * SendReceiveObjectNum + AtomiObjectNum + SequentialObjectNum];

static int s_Cursor = 0;

static nn::pia::common::Time s_StartTime;
static bool s_IsClockInvalid;

uint32_t s_SendPeriod = 1;

int GetStationNumMax(nns::pia::NetworkType networkType)
{
    int stationNumMax = 0;
#if PIA_TEST_CODE
    switch (networkType)
    {
    case nns::pia::NetworkType_Internet:
    case nns::pia::NetworkType_Lan:
    {
#if NN_PIA_CTR
        stationNumMax = 12;
#elif NN_PIA_CAFE
        stationNumMax = 16;
#elif NN_PIA_NINTENDOSDK
        stationNumMax = 16;
#else
        stationNumMax = 8;
#endif
    }
    break;
    case nns::pia::NetworkType_Local:
    {
#if NN_PIA_CTR
        stationNumMax = 8;
#elif NN_PIA_NINTENDOSDK
        stationNumMax = 8;
#else
        stationNumMax = 8;
#endif
    }
    break;
    default:
    {
        PIA_SAMPLE_ASSERT(false);
    }
    break;
    }
#else
    NN_PIA_UNUSED(networkType);
    stationNumMax = MaxStations;
#endif

    PIA_SAMPLE_ASSERT(stationNumMax <= MaxStations);
    return stationNumMax;
}

// プロファイリング用
struct ProfilingInfo
{
    static const float ProfilingTimeSec;

    ProfilingInfo()
        : isStart(false), startTime(), loopCount(0)
    {
    }

    bool isStart;
    nn::pia::common::Time startTime;
    int loopCount;
};
const float ProfilingInfo::ProfilingTimeSec = 10.0f;
ProfilingInfo s_ProfilingInfo;

void StartProfiling()
{
#if PIA_TEST_CODE
    return;
#else
    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Profiling Start (time:%f[s])\n", s_ProfilingInfo.ProfilingTimeSec);

    // TransportAnalyzer を更新
    nn::pia::transport::TransportAnalyzer* pAnalyzer = nn::pia::transport::TransportAnalyzer::GetInstance();
    if (pAnalyzer)
    {
        nn::pia::Result r = pAnalyzer->Update();
        PIA_SAMPLE_ASSERT(r.IsSuccess());
        NN_PIA_UNUSED(r);
    }

    // clone のプロファイリングを開始
    nns::pia::Clone_GetCloneProtocol()->ResetProfiler();
    for (uint32_t i = 0; i < s_ObjectNum; ++i)
    {
        s_apObject[i]->ResetProfiler();
    }

    s_ProfilingInfo.isStart = true;
    s_ProfilingInfo.startTime.SetNow();
    s_ProfilingInfo.loopCount = 0;
#endif
}

void UpdateProfiling()
{
    if (s_ProfilingInfo.isStart)
    {
        ++s_ProfilingInfo.loopCount;

        nn::pia::common::Time curTime;
        curTime.SetNow();
        if ((curTime - s_ProfilingInfo.startTime).GetSecF32() >= s_ProfilingInfo.ProfilingTimeSec)
        {
            // プロファイリング終了
            s_ProfilingInfo.isStart = false;
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Profiling End (time:%f[s] loop:%d)\n", (curTime - s_ProfilingInfo.startTime).GetSecF32(), s_ProfilingInfo.loopCount);

            // TransportAnalyzer の結果を出力
            nn::pia::transport::TransportAnalyzer* pAnalyzer = nn::pia::transport::TransportAnalyzer::GetInstance();
            if (pAnalyzer)
            {
                nn::pia::Result r = pAnalyzer->Update();
                PIA_SAMPLE_ASSERT(r.IsSuccess());
                NN_PIA_UNUSED(r);
                nn::pia::transport::TransportAnalysisData data = pAnalyzer->GetTransportAnalysisData();
                data.Print(true, false);
            }

            // clone のプロファイリング結果を出力
            {
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("==== Profiling Result (CloneProtocol) Begin ====\n");
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("                        Latest          Total\n");
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("                        Size  Count     Size  Count\n");

                nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();
                if (pCloneProtocol->GetSendProfiler())
                {
                    for (uint32_t i = 0; i < MaxStations; ++i)
                    {
                        nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send                (%02d) %8d %6d %8d %6d\n",
                                         i,
                                         pCloneProtocol->GetProfiledLatestSendSize(stationIndex),
                                         pCloneProtocol->GetProfiledLatestSendCount(stationIndex),
                                         pCloneProtocol->GetProfiledTotalSendSize(stationIndex),
                                         pCloneProtocol->GetProfiledTotalSendCount(stationIndex));
                    }
                    for (uint32_t i = 0; i < MaxStations; ++i)
                    {
                        nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send [NoCompressed] (%02d) %8d %6d %8d %6d\n",
                                         i,
                                         pCloneProtocol->GetProfiledLatestSendNoCompressedSize(stationIndex),
                                         pCloneProtocol->GetProfiledLatestSendCount(stationIndex),
                                         pCloneProtocol->GetProfiledTotalSendNoCompressedSize(stationIndex),
                                         pCloneProtocol->GetProfiledTotalSendCount(stationIndex));
                    }
                    for (uint32_t i = 0; i < MaxStations; ++i)
                    {
                        nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send      [Unicast] (%02d) %8d %6d %8d %6d\n",
                                         i,
                                         pCloneProtocol->GetProfiledLatestSendUnicastSize(stationIndex),
                                         pCloneProtocol->GetProfiledLatestSendUnicastCount(stationIndex),
                                         pCloneProtocol->GetProfiledTotalSendUnicastSize(stationIndex),
                                         pCloneProtocol->GetProfiledTotalSendUnicastCount(stationIndex));
                    }
                    for (uint32_t i = 0; i < MaxStations; ++i)
                    {
                        nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Send       [System] (%02d) %8d %6d %8d %6d\n",
                                         i,
                                         pCloneProtocol->GetProfiledLatestSendSystemSize(stationIndex),
                                         pCloneProtocol->GetProfiledLatestSendSystemCount(stationIndex),
                                         pCloneProtocol->GetProfiledTotalSendSystemSize(stationIndex),
                                         pCloneProtocol->GetProfiledTotalSendSystemCount(stationIndex));
                    }
#if NN_PIA_ENABLE_CACHEDPRINT
                    if (nn::pia::common::CachedPrint::GetInstance())
                    {
                        nn::pia::common::CachedPrint::GetInstance()->Flush();
                    }
#endif // NN_PIA_ENABLE_CACHEDPRINT
                }
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("--------------------------------------------------------\n");

                if (pCloneProtocol->GetReceiveProfiler())
                {
                    for (uint32_t i = 0; i < MaxStations; ++i)
                    {
                        nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
                        PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Recv                (%02d) %8d %6d %8d %6d\n",
                                         i,
                                         pCloneProtocol->GetProfiledLatestReceiveSize(stationIndex),
                                         pCloneProtocol->GetProfiledLatestReceiveCount(stationIndex),
                                         pCloneProtocol->GetProfiledTotalReceiveSize(stationIndex),
                                         pCloneProtocol->GetProfiledTotalReceiveCount(stationIndex));
                    }
#if NN_PIA_ENABLE_CACHEDPRINT
                    if (nn::pia::common::CachedPrint::GetInstance())
                    {
                        nn::pia::common::CachedPrint::GetInstance()->Flush();
                    }
#endif // NN_PIA_ENABLE_CACHEDPRINT
                }
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("==== Profiling Result (CloneProtocol) End ====\n");
            }

            for (uint32_t i = 0; i < s_ObjectNum; ++i)
            {
                s_apObject[i]->TraceProfiling();
            }
        }
    }
}

void ClearStationIdList()
{
    for (int i = 0; i < MaxStations; ++i)
    {
        g_StationIdList[i] = nn::pia::StationIdInvalid;
    }
}

void RegisterStationId(nn::pia::StationId stationId)
{
    nn::pia::StationIndex stationIndex = ConvertStationIdToStationIndex(stationId);
    PIA_SAMPLE_ASSERT(stationIndex < MaxStations);
    PIA_SAMPLE_ASSERT(g_StationIdList[stationIndex] == nn::pia::StationIdInvalid);
    g_StationIdList[stationIndex] = stationId;
}

void UnregisterStationId(nn::pia::StationId stationId)
{
    nn::pia::StationIndex stationIndex = ConvertStationIdToStationIndex(stationId);
    PIA_SAMPLE_ASSERT(stationIndex < MaxStations);
    PIA_SAMPLE_ASSERT(g_StationIdList[stationIndex] != nn::pia::StationIdInvalid);
    g_StationIdList[stationIndex] = nn::pia::StationIdInvalid;
}

// コールバック関数の宣言
void SampleSessionCallback(nn::pia::session::Session::EventType eventType, nn::pia::StationId stationId, void* pApplicationParam)
{
    NN_PIA_UNUSED(pApplicationParam);
    switch (eventType)
    {
    case nn::pia::session::Session::EventType_EventJoin:
    {
        PIA_SAMPLE_PRINT("Sample Event JOIN detect. StationId(0x%016llx)\n", stationId);
        RegisterStationId(stationId);
    }
    break;
    case nn::pia::session::Session::EventType_EventLeave:
    {
        PIA_SAMPLE_PRINT("Sample Event LEAVE detect. StationId(0x%016llx)\n", stationId);
        UnregisterStationId(stationId);
    }
    break;
    default:
        break;
    }
}

// 各オブジェクトを初期化します。
void InitObjects(GameMode gameMode)
{
    NN_PIA_UNUSED(gameMode);

    s_ObjectNum = (1 + GetStationNumMax(s_NetworkType)) * SendReceiveObjectNum + AtomiObjectNum + SequentialObjectNum;

    uint32_t i = 0;
    for (uint32_t j = 0; j < SendReceiveObjectNum; ++j)
    {
        s_SendObject[j].m_CloneId = j;
        s_apObject[i] = &s_SendObject[j];
        ++i;
    }

    uint32_t maxStations = GetStationNumMax(s_NetworkType);

    for (uint32_t k = 0; k < maxStations; ++k)
    {
        for (uint32_t j = 0; j < SendReceiveObjectNum; ++j)
        {
            s_ReceiveObject[j][k].m_CloneId = j;
            s_ReceiveObject[j][k].SetStationIndex(static_cast<nn::pia::StationIndex>(k));
            s_ReceiveObject[j][k].InitCloneTypeText();
            s_apObject[i] = &s_ReceiveObject[j][k];
            ++i;
        }
    }
    for (uint32_t j = 0; j < AtomiObjectNum; ++j)
    {
        s_AtomicObject[j].m_CloneId = j;
        s_apObject[i] = &s_AtomicObject[j];
        ++i;
    }
    for (uint32_t j = 0; j < SequentialObjectNum; ++j)
    {
        s_SequentialObject[j].m_CloneId = j;
        s_apObject[i] = &s_SequentialObject[j];
        ++i;
    }
    PIA_SAMPLE_PRINT("i: %d  ObjectNum: %d\n", i, s_ObjectNum);
    PIA_SAMPLE_ASSERT(i == s_ObjectNum);
}


// キー入力による操作です。
bool Operate(GameMode gameMode)
{
    NN_PIA_UNUSED(gameMode);
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    // START で GameLoop() を抜けます。
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
    {
        return true;
    }

    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
    {
        --s_Cursor;
        if (s_Cursor < 0)
        {
            s_Cursor = s_ObjectNum;
        }
    }
    else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
    {
        ++s_Cursor;
        if (s_Cursor >= static_cast<int>(s_ObjectNum + 1))
        {
            s_Cursor = 0;
        }
    }


    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
    {
        StartProfiling();
    }

#if PIA_TEST_CODE
    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL) && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
    {
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            for (int i = 0; i < s_ObjectNum; ++i)
            {
                if (!s_apObject[i]->IsCloneRegistered())
                {
                    s_apObject[i]->RegisterClone();
                }
            }
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            for (int i = 0; i < s_ObjectNum; ++i)
            {
                if (s_apObject[i]->IsCloneRegistered())
                {
                    s_apObject[i]->UnregisterClone();
                }
            }
        }
    }
    else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
    {
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            PIA_SAMPLE_PRINT("Send packet loss 100\n");
            nn::pia::transport::ThreadStreamManager* pThreadStreamMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pThreadStreamMgr));
            nn::pia::Result result = pThreadStreamMgr->SetSendThreadPacketLossRatio(100);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            PIA_SAMPLE_PRINT("Send packet loss 0\n");
            nn::pia::transport::ThreadStreamManager* pThreadStreamMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pThreadStreamMgr));
            nn::pia::Result result = pThreadStreamMgr->SetSendThreadPacketLossRatio(0);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
        }
    }
    else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
    {
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            PIA_SAMPLE_PRINT("Receive packet loss 100\n");
            nn::pia::transport::ThreadStreamManager* pThreadStreamMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pThreadStreamMgr));
            nn::pia::Result result = pThreadStreamMgr->SetReceiveThreadPacketLossRatio(100);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            PIA_SAMPLE_PRINT("Receive packet loss 0\n");
            nn::pia::transport::ThreadStreamManager* pThreadStreamMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pThreadStreamMgr));
            nn::pia::Result result = pThreadStreamMgr->SetReceiveThreadPacketLossRatio(0);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
        }
    }
#endif

    if (s_Cursor < static_cast<int>(s_ObjectNum))
    {
        s_apObject[s_Cursor]->Operate();
    }
    else
    {
        nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();

        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            switch (pCloneProtocol->GetState())
            {
            case nn::pia::clone::CloneProtocol::State_Inactive:
            {
                nn::pia::Result r = pCloneProtocol->Start();
                PIA_SAMPLE_ASSERT(r.IsSuccess());
                NN_PIA_UNUSED_IN_RELEASE(r);
                s_StartTime = nn::pia::common::Time::GetTime();
                s_IsClockInvalid = false;
            }
            break;

            case nn::pia::clone::CloneProtocol::State_Start:
            case nn::pia::clone::CloneProtocol::State_Active:
            {
                nn::pia::Result r = pCloneProtocol->Stop();
                PIA_SAMPLE_ASSERT(r.IsSuccess());
                NN_PIA_UNUSED_IN_RELEASE(r);
            }
            break;

            default:
                break;
            }
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            if (pCloneProtocol->GetState() == nn::pia::clone::CloneProtocol::State_Active &&
                pCloneProtocol->IsRegulatingClock() == false)
            {
                nn::pia::Result r = pCloneProtocol->RegulateClock();
                PIA_SAMPLE_ASSERT(r.IsSuccess());
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Called RegulateClock()\n");
                NN_PIA_UNUSED_IN_RELEASE(r);
            }
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
        {
            if (s_SendPeriod > 1)
            {
                --s_SendPeriod;
            }
            nn::pia::Result r = nns::pia::Clone_GetCloneProtocol()->SetSendPeriod(s_SendPeriod);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Called SetSendPeriod(%d)\n", s_SendPeriod);
            NN_PIA_UNUSED_IN_RELEASE(r);
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
        {
            ++s_SendPeriod;
            nn::pia::Result r = nns::pia::Clone_GetCloneProtocol()->SetSendPeriod(s_SendPeriod);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Called SetSendPeriod(%d)\n", s_SendPeriod);
            NN_PIA_UNUSED_IN_RELEASE(r);
        }
#if PIA_TEST_CODE
#else
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            // Search関数のチェック
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Search Clone\n");
            nn::pia::StationIndex sidx;
            nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&sidx, nn::pia::session::Session::GetInstance()->GetLocalStationId());
            if (result.IsFailure())
            {
                sidx = nn::pia::StationIndex_Invalid;
            }
            for (uint32_t i = 0; i < s_ObjectNum; ++i)
            {
                nn::pia::clone::CloneBase::Id cid = static_cast<nn::pia::clone::CloneBase::Id>(i);
                nn::pia::clone::SendClone* pSendClone = pCloneProtocol->SearchSendClone(cid);
                nn::pia::clone::ReceiveClone* pReceiveClone = pCloneProtocol->SearchReceiveClone(sidx, cid);
                nn::pia::clone::AtomicSharingClone* pAtomicClone = pCloneProtocol->SearchAtomicSharingClone(cid);
                nn::pia::clone::SequentialSharingClone* pSequentialClone = pCloneProtocol->SearchSequentialSharingClone(cid);
                if (pSendClone)
                {
                    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("index:%d Clone Type: Send\n", cid);
                }
                if (pReceiveClone)
                {
                    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("index:%d local station index:%d Clone Type: Receive\n", cid, sidx);
                }
                if (pAtomicClone)
                {
                    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("index:%d Clone Type: Atomic\n", cid);
                }
                if (pSequentialClone)
                {
                    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("index:%d Clone Type: Sequential\n", cid);
                }
            }
            // IsActive関数のチェック
            for (uint32_t i = 0; i < MaxStations; ++i)
            {
                nn::pia::StationId sid = g_StationIdList[i];
                if (pCloneProtocol->IsActiveStation(sid))
                {
                    PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Station%d (%016llx) is active.\n", i, sid);
                }
            }
        }
#endif
    }

    return false;
}


#if PIA_TEST_CODE
void UpdatePiaTestData(GameMode gameMode, bool isExit)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    bool isInet = false;
    switch (gameMode)
    {
#if NNS_PIA_ENABLE_INET
    case GameMode_InetHost:
    case GameMode_InetClient:
    case GameMode_InetRandom:
    {
        isInet = true;
    }
    break;
#endif
    default:
        break;
    }
    PIA_SAMPLE_ASSERT(pSession);
    pSampleFramework->SetProgramStatus(
        isExit ? 0 : (isInet ? 4 : 3),
        (pSession->CheckConnectionError().IsSuccess() &&
         pSession->GetStatus() == nn::pia::session::Session::Status_ConnectedSession),
        pSession->GetLocalStationId() == pSession->GetHostStationId(),
        pSession->GetStationNum());
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();

    pSampleFramework->SetProgramParam(STATION_INDEX, pSession->GetLocalStationIndex());
    pSampleFramework->SetProgramParam(CLONE_PROTOCOL_STATE, pCloneProtocol->GetState());
    pSampleFramework->SetProgramParam(CLOCK_TYPE, s_ClockType);
    pSampleFramework->SetProgramParam(CLOCK_VALUE, pCloneProtocol->GetClock());
    pSampleFramework->SetProgramParam(CURSOR_POS, s_Cursor);
    PIA_SAMPLE_PRINT("s_Cursor: %d\n", s_Cursor);

    for (uint32_t i = 0; i < s_ObjectNum; ++i)
    {
        s_apObject[i]->UpdatePiaTestData();
    }
}
#endif

// 画面の描画処理です。
void Draw(GameMode gameMode)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();

    static const nns::pia::sampleframework::Color DefaultColor(1.0f, 1.0f, 1.0f, 1.0f);
    static const nns::pia::sampleframework::Color CursorColor(1.0f, 0.3f, 0.3f, 1.0f);
    static const nns::pia::sampleframework::Color InvalidColor(0.4f, 0.4f, 0.4f, 1.0f);
    static const nns::pia::sampleframework::Color ColorR(1.0f, 0.4f, 0.4f, 1.0f);
    static const nns::pia::sampleframework::Color ColorG(0.4f, 1.0f, 0.4f, 1.0f);
    static const nns::pia::sampleframework::Color ColorB(0.4f, 0.4f, 1.0f, 1.0f);

    pSampleFramework->SetFontColor(DefaultColor);
    {
        pSampleFramework->DrawString(10, 10, PROGRAM_TITLE);
        const char* GameModeString = NULL;
        switch (gameMode)
        {
#if ENABLE_PIA_LOCAL
        case GameMode_LocalHost:
        case GameMode_LocalClient:
        {
            if (pSession->IsHost())
            {
                GameModeString = "LocalHost";
            }
            else
            {
                GameModeString = "LocalClient";
            }
        }
        break;
#endif
#if NNS_PIA_ENABLE_INET
        case GameMode_InetRandom:
        {
            GameModeString = "InetRandom";
        }
        break;
        case GameMode_InetHost:
        case GameMode_InetClient:
        {
            if (pSession->IsHost())
            {
                GameModeString = "InetHost";
            }
            else
            {
                GameModeString = "InetClient";
            }
        }
        break;
#endif
        case GameMode_LanRandom:
        {
            GameModeString = "LanRandom";
        }
        break;
        case GameMode_LanHost:
        case GameMode_LanClient:
        {
            if (pSession->IsHost())
            {
                GameModeString = "LanHost";
            }
            else
            {
                GameModeString = "LanClient";
            }
        }
        break;
        default:
            break;
        }
        PIA_SAMPLE_ASSERT(GameModeString != NULL);
        pSampleFramework->DrawString(10, 20, "GameMode:%s", GameModeString);
        pSampleFramework->DrawString(180, 20, "StationNum:%d", pSession->GetStationNum());
        pSampleFramework->DrawString(300, 20, "Index:%d", ConvertStationIdToStationIndex(pSession->GetLocalStationId()));
    }

    {
        uint32_t idx = (pSampleFramework->GetFrameCount() / 4) % 4;
        const char AnimLetter[] = { '-', '\\', '|', '/' };
        pSampleFramework->DrawString(380, 10, "%c", AnimLetter[idx]);
    }

    {
        pSampleFramework->SetFontColor(s_Cursor == s_ObjectNum ? CursorColor : DefaultColor);

        const char* cpStateText = "";
        switch (pCloneProtocol->GetState())
        {
        case nn::pia::clone::CloneProtocol::State_Inactive:
        {
            cpStateText = "Inactive";
        }
        break;
        case nn::pia::clone::CloneProtocol::State_Start:
        {
            cpStateText = "Start";
        }
        break;
        case nn::pia::clone::CloneProtocol::State_Active:
        {
            cpStateText = "Active";
        }
        break;
        case nn::pia::clone::CloneProtocol::State_Stop:
        {
            cpStateText = "Stop";
        }
        break;
        }
        pSampleFramework->DrawString(10, 225, "State:%s", cpStateText);

        pSampleFramework->SetFontColor(DefaultColor);
    }

    {
        const int ClockX = 140;
        nn::pia::clone::ClockValue clock = pCloneProtocol->GetClock();
        if (clock == nn::pia::clone::InvalidClock)
        {
            pSampleFramework->DrawString(ClockX, 225, "Clock:--------");
        }
        else
        {
#if PIA_TEST_CODE
            if (s_ClockType == nn::pia::clone::CloneProtocol::Setting::ClockType_Rtc)
            {
                pSampleFramework->DrawString(ClockX, 225, "Clock:%8.2f%c", static_cast<float>(clock) / 1000.0f, pCloneProtocol->IsRegulatingClock() ? 'R' : ' ');
            }
            else
            {
                pSampleFramework->DrawString(ClockX, 225, "Clock:%08d%c", clock, pCloneProtocol->IsRegulatingClock() ? 'R' : ' ');
            }
#else
            pSampleFramework->DrawString(ClockX, 225, "Clock:%8.2f%c", static_cast<float>(clock) / 1000.0f, pCloneProtocol->IsRegulatingClock() ? 'R' : ' ');
#endif
            if (s_IsClockInvalid == false)
            {
                nn::pia::common::Time time = nn::pia::common::Time::GetTime();
                int32_t diff = time - s_StartTime;
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("span of start to invalid clock : %d\n", diff);
                s_IsClockInvalid = true;
            }
        }
    }

    {
        pSampleFramework->DrawString(270, 225, "SPeriod:%d", s_SendPeriod);
    }

    {
        static uint32_t printPacketNum = 0;
        if (pSampleFramework->GetFrameCount() % 60 == 0)
        {
            static uint32_t lastPacketNum = 0;
            uint32_t packetNum = nn::pia::transport::ThreadStreamManager::GetInstance()->GetSendThreadSendPacketNum();
            printPacketNum = packetNum - lastPacketNum;
            lastPacketNum = packetNum;
        }
        pSampleFramework->DrawString(365, 225, "%3d", printPacketNum);
    }

    {
        int y = 30;
        for (uint32_t i = 0; i < s_ObjectNum; ++i)
        {
            y = s_apObject[i]->Print(y, (static_cast<int>(i) == s_Cursor) ? CursorColor : DefaultColor);
        }
    }

    pSampleFramework->SetFontColor(DefaultColor);
}


void Update()
{
    nn::pia::clone::CloneProtocol* pCloneProtocol = nns::pia::Clone_GetCloneProtocol();

    if (pCloneProtocol->GetState() != nn::pia::clone::CloneProtocol::State_Inactive)
    {
        pCloneProtocol->UpdateClock();
    }

    if (pCloneProtocol->GetState() == nn::pia::clone::CloneProtocol::State_Start ||
        pCloneProtocol->GetState() == nn::pia::clone::CloneProtocol::State_Active)
    {
        if (pCloneProtocol->GetError() != nn::pia::clone::CloneProtocol::ErrorType_Nothing)
        {
            // エラー発生を検知したので終了
            const char* cpErrorText = "";
            switch (pCloneProtocol->GetError())
            {
            case nn::pia::clone::CloneProtocol::ErrorType_ClockOverflow:
            {
                cpErrorText = "ClockOverflow";
            }
            break;
            case nn::pia::clone::CloneProtocol::ErrorType_EventDropped:
            {
                cpErrorText = "EventDropped";
            }
            break;
            default:
                break;
            }
            PIA_SAMPLE_DIRECT_PRINT_ALWAYS("Error: %s\n", cpErrorText);
            pCloneProtocol->Stop();
        }
    }

#if PIA_TEST_CODE
    static int frameCnt = 0;
    const int SendPeriod = 10;
    bool isSend = (frameCnt % SendPeriod == 0);
    ++frameCnt;
    if (isSend)
    {
        for (uint32_t i = 0; i < s_ObjectNum; ++i)
        {
            s_apObject[i]->UpdateForPiaTest();
        }
    }
#else
    for (uint32_t i = 0; i < s_ObjectNum; ++i)
    {
        s_apObject[i]->Update();
    }
#endif
}

bool Cleanup()
{
    // CloneProtocolからCloneをUnregisterし、その後CloneからElementをUnregisterします
    bool isCleanupAll = true;
    for (uint32_t i = 0; i < s_ObjectNum; ++i)
    {
        isCleanupAll &= s_apObject[i]->Cleanup();
    }
    return isCleanupAll;
}

// 通信中のメインループです。
void MainLoop(GameMode gameMode, bool* pIsLeaveNeeded)
{
    nn::pia::Result result;
    while (true)
    {
        bool isExit = false;

        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

#if NNS_PIA_ENABLE_INET
        // GameMode_InetXXX の場合は nn::nex::Scheduler が作られていて、そのディスパッチも行う必要があります。
        nns::pia::Nex_SchedulerDispatch();
#endif

        // nn::pia::common::Scheduler::Dispatch() を定期的に呼び出す必要があります。
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        UpdateProfiling();

        isExit |= Operate(gameMode);
        if (isExit)
        {
            *pIsLeaveNeeded = true;
        }

#if NNS_PIA_ENABLE_INET
        if (gameMode == GameMode_LanHost || gameMode == GameMode_LanClient || gameMode == GameMode_InetHost || gameMode == GameMode_InetClient)
#else
        if (gameMode == GameMode_LanHost || gameMode == GameMode_LanClient)
#endif
        {
            if (nns::pia::Net_IsConnected() == false)
            {
                // AC が切断したので終了します。
                isExit = true;
                *pIsLeaveNeeded = false;
            }
        }

        result = nn::pia::session::Session::GetInstance()->CheckConnectionError();
        if (result.IsSuccess())
        {
            Update();
        }
        else
        {
            // セッションが切れたので終了します。
            isExit = true;
            nn::pia::Result::HandlingType handlingType = result.GetHandlingType();
            if (handlingType == nn::pia::Result::HandlingType_CleanupWithLeave)
            {
                *pIsLeaveNeeded = true;
            }
            else if (handlingType == nn::pia::Result::HandlingType_ShutdownNetwork)
            {
                *pIsLeaveNeeded = false;
            }
        }

        // プロトコルに対して送信したものをすぐに処理するために、もう一度 nn::pia::common::Scheduler::Dispatch() を呼びます。
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

        Draw(gameMode);


#if PIA_TEST_CODE
        UpdatePiaTestData(gameMode, isExit);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SESSION_PROCESSING, isExit ? 1 : 0);
#endif

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

        if (isExit)
        {
            if (nns::pia::Clone_GetCloneProtocol())
            {
                PIA_SAMPLE_DIRECT_PRINT_ALWAYS("AckBufferUsedNumMax:%d\n", nns::pia::Clone_GetCloneProtocol()->GetAckBufferUsedNumMax());
            }
            return;
        }
    }
}


}  // end of anonymous namespace

namespace
{
const uint32_t MemoryBlockSize = 10 * 1024 * 1024;                // nn::pia::common::Initialize()で供給するメモリサイズ（単位はバイト）
PIA_ATTRIBUTE_ALIGN(8) static uint8_t s_Memory[MemoryBlockSize];  // nn::pia::common::Initialize()で供給するメモリ
}


namespace
{

#if NNS_PIA_ENABLE_INET
// アプリケーションの仕様に応じて調整して下さい。
const size_t MemoryBlockSizeForNex = 1 * 1024 * 1024;
const size_t MemoryBlockSizeForNexEmergency = 50 * 1024;
#endif

const uint32_t MatchmakeGameModeOffset = 0;
const uint32_t MatchmakeGameModeBase = 45000;
const uint32_t MatchmakeGameMode = MatchmakeGameModeBase + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;
#if ENABLE_PIA_LOCAL
#if NN_PIA_CTR
const uint32_t LocalTitleId = 0xFF330;
const uint8_t LocalSubId = MatchmakeGameModeOffset;
#elif NN_PIA_NINTENDOSDK_NX
const uint16_t LocalSceneId = MatchmakeGameModeOffset + 7000 + 30000 * PIA_TEST_CODE;
const int16_t LocalChannel = 0;
#endif
const char LocalWirelessCryptoKey[] = "aaabbccc0123456789";
const char SignatureKey[] = "cloneprotocol_key";
#endif
}

// エントリポイント。
PIA_SAMPLE_MAIN_FUNC()
{
    nn::pia::Result result;
    {
        nns::pia::sampleframework::SampleFramework::Setting setting;
        setting.isUpperScreenOnly = true;
        nns::pia::sampleframework::SampleFramework::CreateInstance(setting);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFramePerSecond(60);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetClearColor(nns::pia::sampleframework::Color(0.1f, 0.1f, 0.1f, 1.0f));

#if PIA_TEST_CODE
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontSize(5);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(LocalMaxStationNum, GetStationNumMax(nns::pia::NetworkType_Local));
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(InetMaxStationNum, GetStationNumMax(nns::pia::NetworkType_Internet));
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(LanMaxStationNum, GetStationNumMax(nns::pia::NetworkType_Lan));
#endif
    }

    PIA_SAMPLE_DIRECT_PRINT_ALWAYS(PROGRAM_TITLE "\n");
    nns::pia::Basic_SetProgramTitle(PROGRAM_TITLE);

    int randomSeed;
#if NN_PIA_CTR
    randomSeed = static_cast<int>(nn::os::Tick::GetSystemCurrent());
#elif NN_PIA_CAFE
    randomSeed = static_cast<int>(OSGetTick());
#elif NN_PIA_NINTENDOSDK
    nn::os::Tick tick = nn::os::GetSystemTick();
    randomSeed = static_cast<int>(tick.GetInt64Value());
#endif
    srand(randomSeed);

    while (true)
    {
        ClearStationIdList();

        GameMode gameMode = DecideGameMode();
        if (gameMode == GameMode_Exit)
        {
            break;
        }

        switch (gameMode)
        {
#if ENABLE_PIA_LOCAL
        case GameMode_LocalHost:
        case GameMode_LocalClient:
        {
            s_NetworkType = nns::pia::NetworkType_Local;
        }
        break;
#endif
#if NNS_PIA_ENABLE_INET
        case GameMode_InetRandom:
        {
            s_NetworkType = nns::pia::NetworkType_Internet;
        }
        break;
#endif
        case GameMode_LanRandom:
        {
            s_NetworkType = nns::pia::NetworkType_Lan;
        }
        break;
        default:
        {
            PIA_SAMPLE_ASSERT(false);
        }
        break;
        }

        // Piaの各モジュールの初期化処理
        // この順番で行ってください。
        {
#if PIA_TEST_CODE
            uint64_t traceFlags =
                nn::pia::common::TraceFlagClone |
                nn::pia::common::TraceFlagCloneDetail |
                nn::pia::common::TraceFlagLocal |
                nn::pia::common::TraceFlagTransport |
                nn::pia::common::TraceFlagSession |
                nn::pia::common::TraceFlagSessionConnect |
                nn::pia::common::TraceFlagSessionLeave |
                nn::pia::common::TraceFlagSessionUpdate;
#else
            uint64_t traceFlags = nn::pia::common::TraceFlagClone | nn::pia::common::TraceFlagCloneDetail;
#endif
            result = nns::pia::Common_InitializePiaCommon(s_Memory, MemoryBlockSize, traceFlags, true);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

            nn::pia::common::Report::GetGlobalObject()->SetMask(nn::pia::common::Report::LevelError);

            nn::pia::transport::NetworkFactory* pFactory = NULL;
            switch (gameMode)
            {
#if ENABLE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            {
#if NN_PIA_CTR
                result = nns::pia::Local_InitializePiaLocal(nns::pia::LocalSetting(LocalTitleId, LocalSubId, false, nn::uds::NO_WAIT | nn::uds::FORCE_DIRECT_BC));
#elif NN_PIA_NINTENDOSDK_NX
                result = nns::pia::Local_InitializePiaLocal(nns::pia::LocalSetting(nns::pia::LocalCommunicationId, LocalSceneId));
#endif
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                pFactory = nns::pia::Local_GetNetworkFactory();
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetRandom:
            {
                // PiaInet を初期化するより前に NEX を初期化してください。
                result = nns::pia::Nex_InitializeNex(
                    MemoryBlockSizeForNex,
                    MemoryBlockSizeForNexEmergency);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                result = nns::pia::Inet_InitializePiaInet();
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                pFactory = nns::pia::Inet_GetNetworkFactory();
            }
            break;
#endif
            case GameMode_LanRandom:
            {
                result = nns::pia::Lan_InitializePiaLan();
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                pFactory = nns::pia::Lan_GetNetworkFactory();
            }
            break;
            default:
                PIA_SAMPLE_ASSERT(false);
                break;
            }

            // Factoryを利用して、Transportをセットアップします。
            nn::pia::transport::Transport::Setting setting;
            setting.pFactory = pFactory;
            setting.maxStationNum = GetStationNumMax(s_NetworkType);
            setting.maxSendThreadBufferNum = nns::pia::MaxSendThreadBufferNum;
            setting.maxReceiveThreadBufferNum = nns::pia::MaxReceiveThreadBufferNum;
#if PIA_TEST_CODE
            nn::pia::transport::Transport::DebugSetting debugSetting;
            debugSetting.sendThreadLatencyEmulationPacketNum = 128;
            debugSetting.receiveThreadLatencyEmulationPacketNum = 256;
            debugSetting.isPacketLossEmulationEnabled = true;
            result = nn::pia::transport::Transport::SetDebugSetting(debugSetting);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
#endif

            result = nns::pia::Transport_InitializePiaTransport(setting);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

            // cloneのセットアップを行います。
            InitObjects(gameMode);
            nn::pia::clone::CloneProtocol::Setting cloneSetting;
            {
#if PIA_TEST_CODE
                cloneSetting.clockType = s_ClockType;
                cloneSetting.clockPerSec = 1000.0f;
                cloneSetting.clockPerFrame = 1;
                cloneSetting.clockPriorityJudgeMethod = nn::pia::clone::CloneProtocol::Setting::ClockPriorityJudgeMethod_Simple;
                cloneSetting.dataCompressionLevel = nn::pia::clone::CloneProtocol::CompressionLevelNone;
#else
#if 1
                cloneSetting.clockType = nn::pia::clone::CloneProtocol::Setting::ClockType_Rtc;
                cloneSetting.clockPerSec = 1000.0f;
#else
                cloneSetting.clockType = nn::pia::clone::CloneProtocol::Setting::ClockType_Frame;
                cloneSetting.clockPerFrame = 1;
#endif
                cloneSetting.clockPriorityJudgeMethod = nn::pia::clone::CloneProtocol::Setting::ClockPriorityJudgeMethod_Simple;
                cloneSetting.dataCompressionLevel = nn::pia::clone::CloneProtocol::CompressionLevelMiddle;
                cloneSetting.ackBufferNum = 32;
#endif
            }
            nns::pia::Clone_InitializePiaClone(cloneSetting);

            result = nns::pia::Clone_GetCloneProtocol()->SetProfiler(&s_ProtocolSendProfiler, &s_ProtocolReceivedProfiler);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

            result = nns::pia::Clone_GetCloneProtocol()->SetSendPeriod(s_SendPeriod);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

            result = nns::pia::Clone_GetCloneProtocol()->SetResendSpanRatio(1.3f);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
#if 0
            // デバッグ用機能です
            result = nns::pia::Clone_GetCloneProtocol()->SetClockOverFlowSpanForDebug(100);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
#endif

            // reckoningのセットアップを行います。
            nns::pia::Reckoning_InitializePiaReckoning();

            // sessionのセットアップを行います。
            nn::pia::session::Session::Setting sessionSetting;
            sessionSetting.pNetworkFactory = pFactory;
            sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;
            sessionSetting.browsedSessionInfoListNum = 8;
#if PIA_TEST_CODE
            {
                nn::pia::session::Session::DebugSetting sessionDebugSetting(false, false, false);
                nn::pia::session::Session::SetDebugSetting(sessionDebugSetting);
            }
#endif
            result = nns::pia::Session_InitializePiaSession(sessionSetting);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            result = nn::pia::session::Session::GetInstance()->RegisterSessionEventCallback(SampleSessionCallback);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
        }

        // ネットワークの接続、セッションの確立の処理
        {
            // ネットワークの初期化
            nns::pia::NetResult netResult = nns::pia::Net_InitializeNetwork(s_NetworkType);
            if (netResult != nns::pia::NetResult_Success)
            {
                PIA_SAMPLE_PRINT("nns::pia::Net_InitializeNetwork:failed. (netResult = %d)\n", netResult);

                switch (gameMode)
                {
#if ENABLE_PIA_LOCAL
                case GameMode_LocalHost:
                case GameMode_LocalClient:
                {
                    gameMode = GameMode_LocalError;
                }
                break;
#endif
#if NNS_PIA_ENABLE_INET
                case GameMode_InetRandom:
                {
                    gameMode = GameMode_InetError;
                }
                break;
#endif
                case GameMode_LanRandom:
                {
                    gameMode = GameMode_LanError;
                }
                break;
                default:
                {
                    PIA_SAMPLE_ASSERT(false);
                }
                break;
                }
            }

            switch (gameMode)
            {
#if ENABLE_PIA_LOCAL
            case GameMode_LocalHost:
            {
                bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
                if (!isSuccess)
                {
                    gameMode = GameMode_LocalError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::local::LocalSessionStartupSetting startupSetting;
                memcpy(startupSetting.signatureKeyData, SignatureKey, sizeof(SignatureKey));
                startupSetting.signatureKeyDataSize = sizeof(SignatureKey);

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo.Copy(playerName);

#if PIA_TEST_CODE
                startupSetting.maxSilenceTime = MAX_SILENT_TIME;
#endif
                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    gameMode = GameMode_LocalError;
                    break;
                }

// セッションを構築します。
// 作成するマッチメイクセッションの設定
#if NN_PIA_CTR
                nn::pia::local::UdsCreateSessionSetting createSessionSetting;
                {
                    createSessionSetting.SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    createSessionSetting.SetSubId(LocalSubId);
                    createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    createSessionSetting.SetMaxParticipantNum(GetStationNumMax(s_NetworkType));
                }
#elif NN_PIA_NINTENDOSDK_NX
                nn::pia::local::LdnCreateSessionSetting createSessionSetting;
                {
                    createSessionSetting.SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    createSessionSetting.SetSceneId(LocalSceneId);
                    createSessionSetting.SetChannel(LocalChannel);
                    createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    createSessionSetting.SetMaxParticipantNum(GetStationNumMax(s_NetworkType));
                }
#endif

                result = nns::pia::Session_CreateSession(&createSessionSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_CreateSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LocalError;
                    break;
                }
            }
            break;

            case GameMode_LocalClient:
            {
                bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
                if (!isSuccess)
                {
                    gameMode = GameMode_LocalError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::local::LocalSessionStartupSetting startupSetting;
                memcpy(startupSetting.signatureKeyData, SignatureKey, sizeof(SignatureKey));
                startupSetting.signatureKeyDataSize = sizeof(SignatureKey);

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo.Copy(playerName);

#if PIA_TEST_CODE
                startupSetting.maxSilenceTime = MAX_SILENT_TIME;
#endif
                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    gameMode = GameMode_LocalError;
                    break;
                }
                // セッションを検索します。
                // 検索条件の設定
                const int searchSettingNum = 1;
#if NN_PIA_CTR
                nn::pia::local::UdsSessionSearchCriteria searchCriteriaList[searchSettingNum];
                {
                    searchCriteriaList[0].SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    searchCriteriaList[0].SetSubId(LocalSubId);
                    searchCriteriaList[0].SetMaxParticipantNum(GetStationNumMax(s_NetworkType));
                }
#elif NN_PIA_NINTENDOSDK_NX
                nn::pia::local::LdnSessionSearchCriteria searchCriteriaList[searchSettingNum];
                {
                    searchCriteriaList[0].SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    searchCriteriaList[0].SetSceneId(LocalSceneId);
                    searchCriteriaList[0].SetMaxParticipantNum(GetStationNumMax(s_NetworkType));
                }
#endif
                searchCriteriaList[0].SetResultRange(0, 8);
                result = nns::pia::Session_BrowseSession(&searchCriteriaList[0]);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LocalError;
                    break;
                }

                // 検索したセッションに参加します。
                uint32_t foundSessionNum = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->GetSize();
                if (foundSessionNum > 0)
                {
                    PIA_SAMPLE_PRINT("session found\n");
                    nn::pia::session::ISessionInfoList::Iterator it = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->Begin();
                    nn::pia::session::ISessionInfo* pTargetSessionInfo = *it;
                    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTargetSessionInfo));

#if NN_PIA_CTR
                    nn::pia::local::UdsJoinSessionSetting joinSessionSetting;
#elif NN_PIA_NINTENDOSDK_NX
                    nn::pia::local::LdnJoinSessionSetting joinSessionSetting;
#endif
                    joinSessionSetting.SetSessionInfoPtr(pTargetSessionInfo);
                    joinSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    result = nns::pia::Session_JoinSession(&joinSessionSetting);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("nns::pia::Session_JoinSession:failed\n");
                        PIA_SAMPLE_PRINT_RESULT(result);
                        gameMode = GameMode_LocalError;
                        break;
                    }
                }
                else
                {
                    PIA_SAMPLE_PRINT("no session found\n");
                    gameMode = GameMode_LocalError;
                    break;
                }
            }
            break;
#endif  // ENABLE_PIA_LOCAL

#if NNS_PIA_ENABLE_INET
            case GameMode_InetRandom:
            {
                // ゲームサーバにログインします。
                if (!nns::pia::Inet_Login())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Inet_Login:failed\n");
                    gameMode = GameMode_InetError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::inet::NexSessionStartupSetting startupSetting;
                startupSetting.isHostMigrationEnabled = IsHostMigrationEnabled;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo[0].Copy(playerName);

#if PIA_TEST_CODE
                startupSetting.maxSilenceTime = MAX_SILENT_TIME;
#endif
                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_StartupSession failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_InetError;
                    break;
                }
                // 自動的にマッチングします。
                nn::pia::inet::NexCreateSessionSetting setting;
                {
                    setting.SetGameMode(MatchmakeGameMode);
                    setting.SetMaxParticipantNum(GetStationNumMax(s_NetworkType));
                    setting.SetOpenSession(true);
                }

                const uint32_t SearchSettingNum = 1;
                nn::pia::inet::NexSessionSearchCriteria criteria[SearchSettingNum];
                {
                    criteria[0].SetGameMode(MatchmakeGameMode);
                    criteria[0].SetMaxParticipantNum(GetStationNumMax(s_NetworkType));
                }

                nn::pia::inet::NexJoinRandomSessionSetting joinRandomSetting;
                joinRandomSetting.SetCreateSessionSetting(&setting);
                joinRandomSetting.SetSessionSearchCriteria(criteria, SearchSettingNum);

                result = nns::pia::Session_JoinRandomSession(&joinRandomSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinRandomSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_InetError;
                    break;
                }

                bool isHost = nn::pia::session::Session::GetInstance()->IsHost();
                if (isHost)
                {
                    // マッチングの結果ホストになりました。
                    gameMode = GameMode_InetHost;
                }
                else
                {
                    // マッチングの結果クライアントになりました。
                    gameMode = GameMode_InetClient;
                }
            }
            break;
#endif
            case GameMode_LanRandom:
            {
                result = nns::pia::Lan_StartupPiaLan();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Lan_StartupLan:failed\n");
                    gameMode = GameMode_LanError;
                    break;
                }

                // Session クラスのスタートアップ処理。
                nn::pia::lan::LanSessionStartupSetting startupSetting;

                nn::pia::transport::Station::PlayerInfo playerName;
                nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
                startupSetting.playerInfo[0].Copy(playerName);

#if PIA_TEST_CODE
                startupSetting.maxSilenceTime = MAX_SILENT_TIME;
#endif
                result = nns::pia::Session_StartupSession(startupSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_StartupSession failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LanError;
                    break;
                }
                // 自動的にマッチングします。
                nn::pia::lan::LanCreateSessionSetting setting;
                {
                    setting.SetGameMode(MatchmakeGameMode);
                    setting.SetMaxParticipantNum(GetStationNumMax(s_NetworkType));
                    setting.SetOpenSession(true);
                }

                const uint32_t SearchSettingNum = 1;
                nn::pia::lan::LanSessionSearchCriteria criteria[SearchSettingNum];
                {
                    criteria[0].SetGameMode(MatchmakeGameMode);
                    criteria[0].SetMaxParticipantNum(GetStationNumMax(s_NetworkType));
                }

                nn::pia::lan::LanJoinRandomSessionSetting joinRandomSetting;
                joinRandomSetting.SetCreateSessionSetting(&setting);
                joinRandomSetting.SetSessionSearchCriteria(criteria, SearchSettingNum);

                result = nns::pia::Session_JoinRandomSession(&joinRandomSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nns::pia::Session_JoinRandomSession:failed\n");
                    PIA_SAMPLE_PRINT_RESULT(result);
                    gameMode = GameMode_LanError;
                    break;
                }

                bool isHost = nn::pia::session::Session::GetInstance()->IsHost();
                if (isHost)
                {
                    // マッチングの結果ホストになりました。
                    gameMode = GameMode_LanHost;
                }
                else
                {
                    // マッチングの結果クライアントになりました。
                    gameMode = GameMode_LanClient;
                }
            }
            break;

#if NNS_PIA_ENABLE_INET
            case GameMode_InetError:
            {
            }
            break;
#endif
#if ENABLE_PIA_LOCAL
            case GameMode_LocalError:
            {
            }
            break;
#endif
            case GameMode_LanError:
            {
            }
            break;

            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }
        }

        bool isLeaveNeeded = false;

        // メインの処理
        switch (gameMode)
        {
#if ENABLE_PIA_LOCAL
        case GameMode_LocalHost:
        case GameMode_LocalClient:
#endif
#if NNS_PIA_ENABLE_INET
        case GameMode_InetHost:
        case GameMode_InetClient:
#endif
        case GameMode_LanHost:
        case GameMode_LanClient:
        {
            MainLoop(gameMode, &isLeaveNeeded);

            // Watermark機能による計測結果プリント。
            if (PIA_IS_VALID_POINTER(nn::pia::common::WatermarkManager::GetInstance()))
            {
                nn::pia::common::WatermarkManager::GetInstance()->Trace(nn::pia::common::TraceFlagAlways);
            }
        }
        break;
        default:
            break;
        }


        // セッションの終了、ネットワークからの切断の処理
        {
            // Session 終了処理。
            switch (gameMode)
            {
#if ENABLE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetHost:
            case GameMode_InetClient:
#endif
            case GameMode_LanHost:
            case GameMode_LanClient:
            {
                if (isLeaveNeeded)
                {
                    // セッションから離脱します。
                    // 本サンプルでは、この後終了処理を行うため、返り値は無視しています。
                    nns::pia::Session_LeaveSession();
                }
            }
            break;
#if ENABLE_PIA_LOCAL
            case GameMode_LocalError:
            {
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetError:
            {
            }
            break;
#endif
            case GameMode_LanError:
            {
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            // Session クラスのクリーンアップ処理。
            nns::pia::Session_CleanupSession();

            switch (gameMode)
            {
#if ENABLE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            case GameMode_LocalError:
            {
                nns::pia::Local_FinalizeLocalCommunication();
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetHost:
            case GameMode_InetClient:
            case GameMode_InetError:
            {
                nns::pia::Inet_Logout();
            }
            break;
#endif
            case GameMode_LanHost:
            case GameMode_LanClient:
            case GameMode_LanError:
            {
                nns::pia::Lan_CleanupPiaLan();
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }

            nns::pia::Net_FinalizeNetwork();
        }

#if PIA_TEST_CODE
        UpdatePiaTestData(gameMode, true);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SESSION_PROCESSING, 0);
#endif

        // Piaの各モジュールの終了処理
        // 初期化と逆順で行ってください。
        {
            nns::pia::Session_FinalizePiaSession();
            nns::pia::Reckoning_FinalizePiaReckoning();
            nns::pia::Clone_FinalizePiaClone();
            nns::pia::Transport_FinalizePiaTransport();
            switch (gameMode)
            {
#if ENABLE_PIA_LOCAL
            case GameMode_LocalHost:
            case GameMode_LocalClient:
            case GameMode_LocalError:
            {
                nns::pia::Local_FinalizePiaLocal();
            }
            break;
#endif
#if NNS_PIA_ENABLE_INET
            case GameMode_InetHost:
            case GameMode_InetClient:
            case GameMode_InetError:
            {
                nns::pia::Inet_FinalizePiaInet();
                nns::pia::Nex_FinalizeNex();
            }
            break;
#endif
            case GameMode_LanHost:
            case GameMode_LanClient:
            case GameMode_LanError:
            {
                nns::pia::Lan_FinalizePiaLan();
            }
            break;
            default:
            {
                PIA_SAMPLE_ASSERT(false);
            }
            break;
            }
            nns::pia::Common_FinalizePiaCommon();
        }
    }
    while (!Cleanup())
    {
    }
    nns::pia::sampleframework::SampleFramework::DestroyInstance();

    PIA_SAMPLE_PRINT("end.\n");
}
