﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "Object.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Clone_Setup.h"

SendObject::SendObject()
{
    clone.RegisterElement(&elementIncrementNumber, 0x01);
    clone.RegisterElement(&elementRectPosition, 0x02);
    clone.RegisterElement(&elementStationNumber, 0x03);
    clone.RegisterElement(&elementIsJointSessionStarted, 0x04);
}

void SendObject::RegisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->RegisterSendClone(&clone, cloneId);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_UNUSED_IN_RELEASE(result);
}

void SendObject::UnregisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->UnregisterSendClone(&clone);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_UNUSED_IN_RELEASE(result);
}

ReceiveObject::ReceiveObject()
{
    clone.RegisterElement(&elementIncrementNumber, 0x01);
    clone.RegisterElement(&elementRectPosition, 0x02);
    clone.RegisterElement(&elementStationNumber, 0x03);
    clone.RegisterElement(&elementIsJointSessionStarted, 0x04);
}

void ReceiveObject::RegisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->RegisterReceiveClone(&clone, stationIndex, cloneId);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
}

void ReceiveObject::UnregisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->UnregisterReceiveClone(&clone);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
}


AtomicObject::AtomicObject()
{
    clone.RegisterElement(&elementIncrementNumber, 0x01);
    clone.RegisterElement(&elementRectPosition, 0x02);
}

void AtomicObject::RegisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->RegisterAtomicSharingClone(&clone, cloneId);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_UNUSED_IN_RELEASE(result);
}

void AtomicObject::UnregisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->UnregisterAtomicSharingClone(&clone);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_UNUSED_IN_RELEASE(result);
}


SequentialObject::SequentialObject()
{
    clone.RegisterElement(&elementIncrementNumber, 0x01);
    clone.RegisterElement(&elementRectPosition, 0x02);
}

void SequentialObject::RegisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->RegisterSequentialSharingClone(&clone, cloneId);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_UNUSED_IN_RELEASE(result);
}

void SequentialObject::UnregisterClone()
{
    nn::pia::Result result = nns::pia::Clone_GetCloneProtocol()->UnregisterSequentialSharingClone(&clone);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_UNUSED_IN_RELEASE(result);
}

// シリアル化のために SerializePolicy を定義します。
namespace nn
{
namespace pia
{
namespace clone
{

template<>
void SerializePolicy<RectPosition>::Serialize(void* pBuffer, const Type& value)
{
    uint8_t* offset = reinterpret_cast<uint8_t*>(pBuffer);
    SerializePolicy<int32_t>::Serialize(offset, value.x);
    offset += 4;
    SerializePolicy<int32_t>::Serialize(offset, value.y);
    offset += 4;
}

template<>
void SerializePolicy<RectPosition>::Deserialize(Type* pValue, const void* cpData)
{
    const uint8_t* offset = reinterpret_cast<const uint8_t*>(cpData);
    SerializePolicy<int32_t>::Deserialize(&pValue->x, offset);
    offset += 4;
    SerializePolicy<int32_t>::Deserialize(&pValue->y, offset);
    offset += 4;
}

template<>
uint32_t SerializePolicy<RectPosition>::GetSerializedSize()
{
    return 8;
}

}
}
}  // end of namespace nn::pia::clone

#endif
