﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include "SampleConfig.h"

#if NNS_PIA_ENABLE_INET

namespace
{
}

// Clone に対応するゲームオブジェクトの基底クラスです。
class Object
{
public:
    virtual void RegisterClone()
    {
    }
    virtual void UnregisterClone()
    {
    }
    virtual nn::pia::Result TryLock()
    {
        PIA_SAMPLE_ASSERT(false);
        PIA_RETURN_RESULT(nn::pia::ResultInvalidState);
    }
    virtual nn::pia::Result Unlock()
    {
        PIA_SAMPLE_ASSERT(false);
        PIA_RETURN_RESULT(nn::pia::ResultInvalidState);
    }
    virtual nn::pia::clone::AtomicSharingClone::LockStatus GetLockStatus() const
    {
        PIA_SAMPLE_ASSERT(false);
        return nn::pia::clone::AtomicSharingClone::LockStatus_Unlocked;
    }
    nn::pia::clone::CloneBase::Id cloneId;
    nn::pia::clone::UnreliableCloneElement<uint32_t> elementIncrementNumber;
    nn::pia::clone::UnreliableCloneElement<uint16_t> elementStationNumber;
    nn::pia::clone::UnreliableCloneElement<RectPosition> elementRectPosition;
    nn::pia::clone::ReliableCloneElement<uint8_t> elementIsJointSessionStarted;

protected:
};


// SendClone を使ったゲームオブジェクトです。
class SendObject : public Object
{
public:
    SendObject();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    nn::pia::clone::SendClone clone;

protected:
};

// ReceiveClone を使ったゲームオブジェクトです。
class ReceiveObject : public Object
{
public:
    ReceiveObject();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    nn::pia::clone::ReceiveClone clone;
    nn::pia::StationIndex stationIndex;

protected:
};

// AtomicSharingClone を使ったゲームオブジェクトです。
class AtomicObject : public Object
{
public:
    AtomicObject();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    nn::pia::clone::AtomicSharingClone clone;
    virtual nn::pia::Result TryLock()
    {
        return clone.TryLock();
    }
    virtual nn::pia::Result Unlock()
    {
        return clone.Unlock();
    }
    virtual nn::pia::clone::AtomicSharingClone::LockStatus GetLockStatus() const
    {
        return clone.GetLockStatus();
    }

protected:
};

// SequentialSharingClone を使ったゲームオブジェクトです。
class SequentialObject : public Object
{
public:
    SequentialObject();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    nn::pia::clone::SequentialSharingClone clone;

protected:
};

#endif
