﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "ObjectManager.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Clone_Setup.h"

namespace
{
}

ObjectManager g_ObjectManager;

ObjectManager::ObjectManager()
{
    m_SendObject.cloneId = CloneIdOffset_Station + 0x0;

    for (int i = 0; i < GamePlayerMax; ++i)
    {
        m_ReceiveObject[i].cloneId = CloneIdOffset_Station + 0x0;
        m_ReceiveObject[i].stationIndex = static_cast<nn::pia::StationIndex>(i);
    }

    for (int i = 0; i < TeamMax + 1; ++i)
    {
        m_AtomicObject[i].cloneId = CloneIdOffset_CommonObject + 0x10 * i + 0x0;
        m_SequentialObject[i].cloneId = CloneIdOffset_CommonObject + 0x10 * i + 0x1;
    }
}

void ObjectManager::RegisterAll()
{
    m_SendObject.RegisterClone();
    for (int i = 0; i < GamePlayerMax; ++i)
    {
        m_ReceiveObject[i].RegisterClone();
    }
    for (int i = 0; i < TeamMax + 1; ++i)
    {
        m_AtomicObject[i].RegisterClone();
        m_SequentialObject[i].RegisterClone();
    }
}

void ObjectManager::UnregisterAll()
{
    m_SendObject.UnregisterClone();
    for (int i = 0; i < GamePlayerMax; ++i)
    {
        m_ReceiveObject[i].UnregisterClone();
    }
    for (int i = 0; i < TeamMax + 1; ++i)
    {
        m_AtomicObject[i].UnregisterClone();
        m_SequentialObject[i].UnregisterClone();
    }
}

SendObject* ObjectManager::GetSendObject()
{
    return &m_SendObject;
}


ReceiveObject* ObjectManager::GetReceiveObject(nn::pia::StationId stationId)
{
    if (stationId == nn::pia::StationIdInvalid)
    {
        return NULL;
    }

    nn::pia::StationIndex stationIndex;
    nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&stationIndex, stationId);
    if (result.IsFailure())
    {
        return NULL;
    }
    PIA_SAMPLE_ASSERT(stationIndex < GamePlayerMax);
    return &m_ReceiveObject[stationIndex];
}

AtomicObject* ObjectManager::GetTeamAtomicObject(int teamIndex)
{
    if (teamIndex < 0 || teamIndex >= TeamMax)
    {
        return NULL;
    }
    return &m_AtomicObject[teamIndex];
}

SequentialObject* ObjectManager::GetTeamSequentialObject(int teamIndex)
{
    if (teamIndex < 0 || teamIndex >= TeamMax)
    {
        return NULL;
    }
    return &m_SequentialObject[teamIndex];
}

AtomicObject* ObjectManager::GetCommonAtomicObject()
{
    return &m_AtomicObject[TeamMax];
}

SequentialObject* ObjectManager::GetCommonSequentialObject()
{
    return &m_SequentialObject[TeamMax];
}

#endif
