﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include "SampleConfig.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Clone_Setup.h"
#include "Object.h"

enum SampleObjectStatusType
{
    SampleObjectStatusType_None = 0,
    SampleObjectStatusType_Station,
    SampleObjectStatusType_TeamAtomic,
    SampleObjectStatusType_TeamSequential,
    SampleObjectStatusType_CommonAtomic,
    SampleObjectStatusType_CommonSequential
};

struct SampleObjectStatus
{
    nn::pia::StationId stationId;
    RectPosition pos;
    uint32_t latestIncrementNumber;
    uint16_t stationNumber;
    SampleObjectStatusType type;
    Object* pObject;
    int teamIndex;

    bool isStartJointSession;
    bool isUpdataStartJointSession;

    void Clear()
    {
        stationId = nn::pia::StationIdInvalid;
        pos.x = 0;
        pos.y = 0;
        latestIncrementNumber = 0;
        stationNumber = 0;
        type = SampleObjectStatusType_None;
        pObject = NULL;
        teamIndex = -1;

        isStartJointSession = false;
        isUpdataStartJointSession = false;
    }

    void Update()
    {
        if (nns::pia::Clone_GetCloneProtocol()->GetState() != nn::pia::clone::CloneProtocol::State_Active)
        {
            return;
        }

        if (type == SampleObjectStatusType_Station && stationId == nn::pia::StationIdInvalid)
        {
            return;
        }
        if (type != SampleObjectStatusType_Station && teamIndex == -1)
        {
            return;
        }

        PIA_SAMPLE_ASSERT(pObject != NULL);
        if (pObject->elementIncrementNumber.IsValidValue())
        {
            latestIncrementNumber = pObject->elementIncrementNumber.GetValue();
        }
        if (pObject->elementRectPosition.IsValidValue())
        {
            pos = pObject->elementRectPosition.GetValue();
        }
        if (pObject->elementStationNumber.IsValidValue())
        {
            stationNumber = pObject->elementStationNumber.GetValue();
        }

        if (pObject->elementIsJointSessionStarted.IsValidValue())
        {
            if (!isUpdataStartJointSession)
            {
                isStartJointSession = static_cast<bool>(pObject->elementIsJointSessionStarted.GetValue());
                isUpdataStartJointSession = false;
            }
        }
    }
};

#endif
