﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include <nn/pia/common.h>
#include <nn/pia/common/common_Event.h>
#include <nn/pia/inet.h>
#include <nn/pia/session.h>

#include "SampleLib/Defines.h"
#include "SampleLib/ErrorHandling.h"
#include "SampleLib/SampleLib.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleLib/Clone_Setup.h"

#include "SampleConfig.h"
#include "EventChecker.h"
#include "TeamConstitutionManager.h"


namespace
{
// チーム合流・分断処理時に待っている非同期処理を示すための列挙型
enum SampleWaitingAsyncProcessing
{
    SampleWaitingAsyncProcessing_Passive = 0,
    SampleWaitingAsyncProcessing_JoinRandomJointSession,
    SampleWaitingAsyncProcessing_CreateJointSession,
    SampleWaitingAsyncProcessing_JoinJointSession,
    SampleWaitingAsyncProcessing_BrowseJointSession,
    SampleWaitingAsyncProcessing_DestroyJointSession,
    SampleWaitingAsyncProcessing_LeaveJointSession,
    SampleWaitingAsyncProcessing_LeaveSession,
    SampleWaitingAsyncProcessing_OpenSession,
    SampleWaitingAsyncProcessing_OpenJointSession
};
}

// セッションマイグレーション開始前の処理
SampleSceneNo SceneChangingSessionPreprocessing()
{
    nn::Result result;

    uint32_t frameNo = 0;
    bool isExit = false;

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_SessionMigrationPreprocess,
        true,
        false,
        0,
        0);

    nn::pia::StationId localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
    nn::pia::StationId hostStationId = nn::pia::session::Session::GetInstance()->GetHostStationId();
    nn::pia::StationId jointHostStationId = nn::pia::session::Session::GetInstance()->GetJointSessionHostStationId();

    switch (g_GameState)
    {
    case GameState_Team:
    {
        // チームセッション ==> ゲームセッションへの移行開始前
        if (localStationId != hostStationId)
        {
            // このデモの仕様上、非ホストが STATUS_MIGRATING_SESSION 以外でココに来ることは無いはず
            PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_MigratingSession);
            // 受動的なセッション移行処理が開始されてこのシーンへ来た
        }

        // セッション参加／拒否判定コールバックを登録
        nn::pia::session::Session::GetInstance()->RegisterJoiningApprovalCallback(SampleJoinApprovalCallback);
        isExit = true;
    }
    break;
    case GameState_Game:
    {
        // ゲームセッション ==> チームセッションへの移行開始前
        if ((localStationId != jointHostStationId && g_JointSessionType == JointSessionType_Destroy) || (localStationId != hostStationId && g_JointSessionType == JointSessionType_Leave))
        {
            // このデモの仕様上、非ホストが STATUS_MIGRATING_SESSION 以外でココに来ることは無いはず
            PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_MigratingSession);
            // 受動的なセッション移行処理が開始されてこのシーンへ来た
        }

        isExit = true;
    }
    break;
    default:
    {
        PIA_SAMPLE_ASSERT(false);
    }
    break;
    }

    while (!isExit)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        // Dispatch 処理
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        nns::pia::Nex_SchedulerDispatch();

#if PIA_TEST_CODE
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(TestParamIndex_JointSessionSequence, nn::pia::session::Session::GetInstance()->GetJointSequence());

        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));
            PIA_SAMPLE_PRINT("[JointSequence] %d\n", nn::pia::session::Session::GetInstance()->GetJointSequence());

            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                pMgr->SetSendThreadPacketLossRatio(100);
                pMgr->SetReceiveThreadPacketLossRatio(10);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 100%, Receive 10%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                pMgr->SetSendThreadPacketLossRatio(10);
                pMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 10%, Receive 100%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                pMgr->SetSendThreadPacketLossRatio(10);
                pMgr->SetReceiveThreadPacketLossRatio(10);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 10%).\n");
            }
            else
            {
                pMgr->SetSendThreadPacketLossRatio(100);
                pMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 100%).\n");
            }
        }
#endif

        //画面描画処理
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

        {
            //自分自身の役割チェック
            int roleIndex = 0;  //Unknown;
            localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
            if (localStationId != nn::pia::StationIdInvalid)
            {
                if (nn::pia::session::Session::GetInstance()->IsHost())
                {
                    roleIndex = 1;  // 1:Host
                }
                else
                {
                    roleIndex = 2;  // 2:Client
                }
            }

            int line = 0;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Changing Session Preprocessing");
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationNum:%2d  Role=%s", nn::pia::session::Session::GetInstance()->GetValidStationIdNum(), GetSessionRoleString(roleIndex));
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationId:0x%016llX", localStationId);
            line += FontSize;
            line += FontSize;

            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   Processing is in progress...");
            line += FontSize;

            line = ScreenHeight - FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
            line += FontSize;
        }

        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
        frameNo++;
    }

    // 処理後の SessionStatus でシーン遷移先を分岐
    switch (nn::pia::session::Session::GetInstance()->GetStatus())
    {
    case nn::pia::session::Session::Status_NotConnected:
    {
        // チーム選択へ戻る
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_SelectTeam;
    }
    case nn::pia::session::Session::Status_ConnectedSession:
    case nn::pia::session::Session::Status_ConnectedJointSession:
    case nn::pia::session::Session::Status_MigratingSession:
    {
        return SampleSceneNo_SessionMigration;
    }
    case nn::pia::session::Session::Status_DisconnectedSession:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedSession\n");
        SetCommunicationErrorType(SampleErrorType_SessionLost);
        SetNetworkErrorResult(nn::pia::ResultSessionConnectionIsLost());
        nn::pia::Result leaveResult = nns::pia::Session_LeaveSession();
        if (leaveResult.IsSuccess())
        {
            // Session のクリーンアップ＆再スタートアップをすればマッチメイク処理に戻れる状態であるため、ネットワークのリセット不要状態にしておく
            SetResetOfNetworkIsNecessary(false);
        }
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetwork:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetwork\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultNetworkConnectionIsLost());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetworkByDuplicateLogin:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetworkByDuplicateLogin\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetworkByWifiOff:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetworkByWifiOff\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultWifiOff());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetworkBySleep:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetworkBySleep\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultSleep());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    default:
    {
        // このデモの仕様上、ありえないはず
        PIA_SAMPLE_ASSERTMSG(false, "Unexpected MatchmakeStatus:%d", nn::pia::session::Session::GetInstance()->GetStatus());

        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    }
}

// セッションマイグレーション中の非同期処理を待つシーン
SampleSceneNo SceneChangingSession()
{
    nn::pia::Result result = nn::pia::ResultSuccess();

    uint32_t frameNo = 0;
    bool isExit = false;

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_SessionMigration,
        true,
        false,
        0,
        0);

    nn::pia::Result::HandlingType handlingType = nn::pia::Result::HandlingType_Void;
    SampleWaitingAsyncProcessing waitingProcess = SampleWaitingAsyncProcessing_Passive;

    // 移行するための非同期処理起動
    switch (g_GameState)
    {
    case GameState_Team:
    {  // チームセッション ==> ゲームセッションへの移行
        if (nn::pia::session::Session::GetInstance()->IsHost())
        {
// ホスト
#if PIA_TEST_CODE
            uint16_t maxParticipantNum = GetMaxPlayerNum();
#else
            uint16_t maxParticipantNum = static_cast<uint16_t>(GamePlayerMax);
#endif

            // 作成するマッチメイクセッションの設定（合流用）
            nn::pia::inet::NexCreateSessionSetting nexCreateSessionSettingForGame;
            {
                // ゲームモードは "ゲーム用" にする
                nexCreateSessionSettingForGame.SetGameMode(MatchmakeGameModeForGame);
                // 合流するメンバ全員が参加するので、最大参加人数は "合流後の最大人数" にする
                nexCreateSessionSettingForGame.SetMaxParticipantNum(maxParticipantNum);
                nexCreateSessionSettingForGame.SetMinParticipantNum(1);
                nexCreateSessionSettingForGame.SetAttribute(MatchmakeAttributeIndexTeamType, static_cast<uint32_t>(GetCurrentTeamType()));
#if PIA_TEST_CODE
                // テストシーケンス上の都合で構築時点では参加可状態になるように設定
                nexCreateSessionSettingForGame.SetOpenSession(true);
#else
                // 構築時点では参加不可状態になるように設定
                nexCreateSessionSettingForGame.SetOpenSession(false);
#endif
            }
            // ランダムマッチの検索条件の設定
            const uint32_t SearchCriteriaNum = 1;
            nn::pia::inet::NexSessionSearchCriteria nexSearchCriteriaArray[SearchCriteriaNum];
            nexSearchCriteriaArray[0].SetGameMode(MatchmakeGameModeForGame);
            nexSearchCriteriaArray[0].SetMaxParticipantNum(maxParticipantNum);
            // 自チームとは異なるチームタイプを検索する
            const uint32_t AttributeArraySize = SampleTeamType_Max - 1;
            uint32_t attributeArray[AttributeArraySize];
            int idx = 0;
            for (int i = 0; i < SampleTeamType_Max; ++i)
            {
                if (static_cast<SampleTeamType>(i) != GetCurrentTeamType())
                {
                    attributeArray[idx] = i;
                    ++idx;
                }
            }
            nexSearchCriteriaArray[0].SetAttribute(MatchmakeAttributeIndexTeamType, attributeArray, AttributeArraySize);

            nn::pia::inet::NexJoinRandomSessionSetting nexJoinRandomSessionSetting;
            nexJoinRandomSessionSetting.SetCreateSessionSetting(&nexCreateSessionSettingForGame);
            nexJoinRandomSessionSetting.SetSessionSearchCriteria(nexSearchCriteriaArray, SearchCriteriaNum);

            if (g_JointSessionType == JointSessionType_Random)
            {
                result = nn::pia::session::Session::GetInstance()->JoinRandomJointSessionAsync(&nexJoinRandomSessionSetting);
                PIA_SAMPLE_PRINT("Call JoinRandomJointSessionAsync.\n");
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("Call JoinRandomJointSessionAsync is failure.\n");
                    PIA_SAMPLE_ASSERT(false);
                    isExit = true;
                }
                else
                {
                    g_SessionMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_SessionMigration);
                    g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_StartJoinRandomJointSession);
                    g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_EndJoinRandomJointSession);

                    waitingProcess = SampleWaitingAsyncProcessing_JoinRandomJointSession;
                }
            }
#if PIA_TEST_CODE
            else if (g_JointSessionType == JointSessionType_Create || g_JointSessionType == JointSessionType_CreatePrivate)
            {
                nexCreateSessionSettingForGame.SetAttribute(MatchmakeAttributeIndexUserPassword, (g_JointSessionType == JointSessionType_CreatePrivate) ? 1 : 0);
                if (g_JointSessionType == JointSessionType_CreatePrivate)
                {
                    nexCreateSessionSettingForGame.SetSessionUserPassword(UserPasswordString[g_UserPasswordIndexForJoint]);
                }
                result = nn::pia::session::Session::GetInstance()->CreateJointSessionAsync(&nexCreateSessionSettingForGame);
                PIA_SAMPLE_PRINT("Call CreateJointSessionAsync.\n");
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("Call CreateJointSessionAsync is failure.\n");
                    PIA_SAMPLE_ASSERT(false);
                    isExit = true;
                }
                else
                {
                    g_SessionMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_SessionMigration);
                    g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_StartCreateJointSession);
                    g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_EndCreateJointSession);

                    waitingProcess = SampleWaitingAsyncProcessing_CreateJointSession;
                }
            }
            else if (g_JointSessionType == JointSessionType_Join || g_JointSessionType == JointSessionType_JoinPrivate)
            {
                nexSearchCriteriaArray[0].SetAttribute(MatchmakeAttributeIndexUserPassword, (g_JointSessionType == JointSessionType_JoinPrivate) ? 1 : 0);
                nexSearchCriteriaArray[0].SetExcludeUserPasswordSet(!(g_JointSessionType == JointSessionType_JoinPrivate));
                nexSearchCriteriaArray[0].SetResultRange(0, 4);
#if PIA_TEST_CODE
                nexSearchCriteriaArray[0].SetOpenedOnly(false);
#endif

                result = nn::pia::session::Session::GetInstance()->BrowseSessionAsync(&nexSearchCriteriaArray[0]);
                PIA_SAMPLE_PRINT("Call BrowseSessionAsync.\n");
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("Call BrowseSessionAsync is failure.\n");
                    PIA_SAMPLE_ASSERT(false);
                    isExit = true;
                }
                else
                {
                    waitingProcess = SampleWaitingAsyncProcessing_BrowseJointSession;
                }
            }
#endif
        }
        else
        {
            // このデモの仕様上、非ホストが STATUS_MIGRATING_SESSION 以外でココに来ることは無いはず
            PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_MigratingSession);
            // 受動的なセッション移行処理が開始されてこのシーンへ来た
            // 非同期処理はライブラリ内部で開始されているので進行を待つだけ
            waitingProcess = SampleWaitingAsyncProcessing_Passive;
        }
        PIA_SAMPLE_PRINT("SceneChangingSession: GameState %d -> %d\n", g_GameState, GameState_PreparingGame);
        g_GameState = GameState_PreparingGame;
    }
    break;
    case GameState_Game:
    {  // ゲームセッション ==> チームセッションへの移行
        if (g_JointSessionType == JointSessionType_Destroy)
        {
            // ジョイントセッションホスト
            if (nn::pia::session::Session::GetInstance()->IsJointSessionHost())
            {
                // ゲームセッションから離脱してチームセッションへ戻る
                PIA_SAMPLE_PRINT("Call DestroyJointSessionAsync.\n");
                result = nn::pia::session::Session::GetInstance()->DestroyJointSessionAsync();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("Call DestroyJointSessionAsync is failure.\n");
                    PIA_SAMPLE_ASSERT(false);
                    isExit = true;
                }
                else
                {
                    // 想定されるイベントをセット
                    g_SessionMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_SessionMigration);
                    g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_StartDestroyJointSession);
                    g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_EndDestroyJointSession);
                    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
                    nn::pia::session::Session::StationIdIterator itEnd = pSession->End();
                    for (nn::pia::session::Session::StationIdIterator it = pSession->Begin(); it != itEnd; ++it)
                    {
                        if (g_TeamConstitutionManager.GetTeamIndex(*it) != g_TeamConstitutionManager.GetTeamIndex(pSession->GetLocalStationId()))
                        {
                            g_SessionMigrationEventChecker.SetExpectedLeaveEventStationId(*it);
                        }
                    }

                    waitingProcess = SampleWaitingAsyncProcessing_DestroyJointSession;
                }
            }
            else
            {
                // DestroyJointSession 以外のイベントフラグが立っているのはおかしい
                PIA_SAMPLE_ASSERTMSG((g_SessionMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_StartDestroyJointSession) != 0,
                                     "DestroyJointSession is not processing!?");
                // このデモの仕様上、同行者が STATUS_MIGRATING_SESSION 以外でココに来ることは無いはず
                PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_MigratingSession);
            }
        }
#if PIA_TEST_CODE
        else if (g_JointSessionType == JointSessionType_Leave)
        {
            if (nn::pia::session::Session::GetInstance()->IsHost())
            {
                // ジョイントセッションが先に DestroyJointSession を開始しているかチェック
                if (g_SessionMigrationEventChecker.IsCheckEnabled() == false)
                {
                    PIA_SAMPLE_PRINT("Call LeaveJointSessionAsync.\n");
                    result = nn::pia::session::Session::GetInstance()->LeaveJointSessionAsync();
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("Call LeaveJointSessionAsync is failure.\n");
                        PIA_SAMPLE_ASSERT(false);
                        isExit = true;
                    }
                    else
                    {
                        // 想定されるイベントをセット
                        g_SessionMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_SessionMigration);
                        g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_StartLeaveJointSession);
                        g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_EndLeaveJointSession);
                        nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
                        nn::pia::session::Session::StationIdIterator itEnd = pSession->End();
                        for (nn::pia::session::Session::StationIdIterator it = pSession->Begin(); it != itEnd; ++it)
                        {
                            if (g_TeamConstitutionManager.GetTeamIndex(*it) != g_TeamConstitutionManager.GetTeamIndex(pSession->GetLocalStationId()))
                            {
                                g_SessionMigrationEventChecker.SetExpectedLeaveEventStationId(*it);
                            }
                        }

                        waitingProcess = SampleWaitingAsyncProcessing_LeaveJointSession;
                    }
                }
                else
                {
                    // DestroyJointSession 以外のイベントフラグが立っているのはおかしい
                    PIA_SAMPLE_ASSERTMSG((g_SessionMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_StartDestroyJointSession) != 0,
                                         "DestroyJointSession is not processing!?");
                    // このデモの仕様上、同行者が STATUS_MIGRATING_SESSION 以外でココに来ることは無いはず
                    PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_MigratingSession);
                    PIA_SAMPLE_PRINT("Received DestroyJointSession Invitation before calling LeaveJointSessionAsync.\n");
                }
            }
            else
            {
                PIA_SAMPLE_ASSERTMSG((g_SessionMigrationEventChecker.GetEventFlag() & EventChecker::EventBit_StartLeaveJointSession) != 0,
                                     "LeaveJointSession is not processing!?");
                // このデモの仕様上、同行者が STATUS_MIGRATING_SESSION 以外でココに来ることは無いはず
                PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_MigratingSession);
            }
        }
#endif
        PIA_SAMPLE_PRINT("SceneChangingSession: GameState %d -> %d\n", g_GameState, GameState_PreparingTeam);
        g_GameState = GameState_PreparingTeam;
    }
    break;
    default:
    {
        PIA_SAMPLE_ASSERT(false);
    }
    break;
    }

#if PIA_TEST_CODE
    bool isStopJointSessionProcess = false;
#endif
    while (!isExit)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

#if PIA_TEST_CODE
        bool isSkipDispatch = isStopJointSessionProcess && (nn::pia::session::Mesh::GetInstance()->GetMeshState() == nn::pia::session::Mesh::State_Disconnected);
        if (!isSkipDispatch)
        {
            nn::pia::common::Scheduler::GetInstance()->Dispatch();
            nns::pia::Nex_SchedulerDispatch();
        }

        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(TestParamIndex_JointSessionSequence, nn::pia::session::Session::GetInstance()->GetJointSequence());

        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));
            PIA_SAMPLE_PRINT("[JointSequence] %d\n", nn::pia::session::Session::GetInstance()->GetJointSequence());

            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                pMgr->SetSendThreadPacketLossRatio(100);
                pMgr->SetReceiveThreadPacketLossRatio(10);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 100%, Receive 10%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                pMgr->SetSendThreadPacketLossRatio(10);
                pMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 10%, Receive 100%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                pMgr->SetSendThreadPacketLossRatio(10);
                pMgr->SetReceiveThreadPacketLossRatio(10);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 10%).\n");
            }
            else
            {
                pMgr->SetSendThreadPacketLossRatio(100);
                pMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 100%).\n");
            }
        }

        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            // ジョイントセッションに受動的に参加させられたクライアントは LeaveSession を呼ぶことが可能ですが非推奨です。
            if (waitingProcess == SampleWaitingAsyncProcessing_Passive)
            {
                PIA_SAMPLE_PRINT("[JointSequence] %d\n", nn::pia::session::Session::GetInstance()->GetJointSequence());
                nn::pia::session::Session::GetInstance()->LeaveSessionAsync();
                waitingProcess = SampleWaitingAsyncProcessing_LeaveSession;
            }
        }
        else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonStart))
        {
            // テストのために異常終了させます。
            // アプリケーションでは非同期処理の完了を待ち、成否に応じて適切な処理を行ってください。
            result = nn::pia::ResultNetworkConnectionIsLost();
            isExit = true;
        }
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonRight) &&
            nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonLeft))
        {
            PIA_SAMPLE_PRINT("isStopJointSessionProcess is set.\n");
            isStopJointSessionProcess = true;
        }
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp) &&
            nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            PIA_SAMPLE_PRINT("isStopJointSessionProcess is unset.\n");
            isStopJointSessionProcess = false;
        }
#else
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        nns::pia::Nex_SchedulerDispatch();
#endif


        // 非同期処理の完了チェック
        switch (waitingProcess)
        {
        case SampleWaitingAsyncProcessing_JoinRandomJointSession:
        {
            if (nn::pia::session::Session::GetInstance()->IsJoinRandomJointSessionCompleted())
            {
                isExit = true;
                result = nn::pia::session::Session::GetInstance()->GetJoinRandomJointSessionResult();
            }
        }
        break;
#if PIA_TEST_CODE
        case SampleWaitingAsyncProcessing_CreateJointSession:
        {
            if (nn::pia::session::Session::GetInstance()->IsCreateJointSessionCompleted())
            {
                isExit = true;
                result = nn::pia::session::Session::GetInstance()->GetCreateJointSessionResult();
                if (result.IsSuccess())
                {
#if PIA_TEST_CODE
                    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(TestParamIndex_JoinJointSessionId, nn::pia::session::Session::GetInstance()->GetJointSessionId());
#endif
                }
            }
        }
        break;
        case SampleWaitingAsyncProcessing_BrowseJointSession:
        {
            if (nn::pia::session::Session::GetInstance()->IsBrowseSessionCompleted())
            {
                result = nn::pia::session::Session::GetInstance()->GetBrowseSessionResult();
                nn::pia::session::ISessionInfoList* pSessionInfoList = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("BrowseSession is failed.\n");
                    isExit = true;
                }
                else if (pSessionInfoList->GetSize() == 0)
                {
                    PIA_SAMPLE_PRINT("Session is not found.\n");
                    result = nn::pia::ResultSessionIsNotFound();
                    isExit = true;
                }
                else
                {
#if PIA_TEST_CODE
                    nn::pia::inet::NexSessionInfo* info = static_cast<nn::pia::inet::NexSessionInfo*>(*(pSessionInfoList->Begin()));
                    uint32_t bufSize = 0;
                    const uint32_t ApplicationDataBufSize = 16;
                    uint8_t applicationData[ApplicationDataBufSize];

                    bufSize = info->GetApplicationDataSize();
                    info->GetApplicationData(&applicationData, ApplicationDataBufSize);

                    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(TestParamIndex_ApplicationDataHead, applicationData[0]);
                    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(TestParamIndex_ApplicationDataSize, bufSize);
                    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(TestParamIndex_JoinJointSessionId, info->GetSessionId());

                    PIA_SAMPLE_PRINT("JointSessionId: %d\n", info->GetSessionId());
                    for (uint32_t i = 0; i < ApplicationDataBufSize; ++i)
                    {
                        PIA_SAMPLE_PRINT("App Data buf[%d]: %d\n", i, applicationData[i]);
                    }
                    PIA_SAMPLE_PRINT("App Data Size: %d\n", bufSize);
#endif

                    nn::pia::inet::NexJoinSessionSetting joinSessionSetting;
                    joinSessionSetting.SetSessionInfoPtr(*(pSessionInfoList->Begin()));
                    if (g_JointSessionType == JointSessionType_JoinPrivate)
                    {
                        joinSessionSetting.SetSessionUserPassword(UserPasswordString[g_UserPasswordIndexForJoint]);
                    }

                    PIA_SAMPLE_PRINT("Call JoinJointSessionAsync.\n");
                    result = nn::pia::session::Session::GetInstance()->JoinJointSessionAsync(&joinSessionSetting);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("Call JoinJointSessionAsync is failure.\n");
                        PIA_SAMPLE_ASSERT(false);
                        isExit = true;
                    }
                    else
                    {
                        g_SessionMigrationEventChecker.CheckBegin(EventChecker::CheckTarget_SessionMigration);
                        g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_StartJoinJointSession);
                        g_SessionMigrationEventChecker.SetExpectedEventFlag(EventChecker::EventBit_EndJoinJointSession);
                        waitingProcess = SampleWaitingAsyncProcessing_JoinJointSession;
                    }
                }
            }
        }
        break;
        case SampleWaitingAsyncProcessing_JoinJointSession:
        {
            if (nn::pia::session::Session::GetInstance()->IsJoinJointSessionCompleted())
            {
                isExit = true;
                result = nn::pia::session::Session::GetInstance()->GetJoinJointSessionResult();
            }
        }
        break;
        case SampleWaitingAsyncProcessing_LeaveJointSession:
        {
            if (nn::pia::session::Session::GetInstance()->IsLeaveJointSessionCompleted())
            {
                isExit = true;
                result = nn::pia::session::Session::GetInstance()->GetLeaveJointSessionResult();
            }
        }
        break;
#endif
        case SampleWaitingAsyncProcessing_DestroyJointSession:
        {
            if (nn::pia::session::Session::GetInstance()->IsDestroyJointSessionCompleted())
            {
                isExit = true;
                result = nn::pia::session::Session::GetInstance()->GetDestroyJointSessionResult();
            }
        }
        break;
        case SampleWaitingAsyncProcessing_Passive:
        {
            // 非同期処理を自分で開始していない（ホストの指示で受動的に処理開始した）クライアント
            // セッション移行処理中フラグで処理の進捗をチェック
            if (!IsSessionMigrating())
            {
                isExit = true;
                result = nn::pia::session::Session::GetInstance()->CheckConnectionError();
            }
        }
        break;
#if PIA_TEST_CODE
        case SampleWaitingAsyncProcessing_LeaveSession:
        {
            if (nn::pia::session::Session::GetInstance()->IsLeaveSessionCompleted())
            {
                isExit = true;
                result = nn::pia::session::Session::GetInstance()->GetLeaveSessionResult();
            }
        }
        break;
#endif
        default:
        {
        }
        break;
        }

        //画面描画処理
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

        {
            //自分自身の役割チェック
            int roleIndex = 0;  //Unknown;
            nn::pia::StationId localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
            if (localStationId != nn::pia::StationIdInvalid)
            {
                if (nn::pia::session::Session::GetInstance()->IsHost())
                {
                    roleIndex = 1;  // 1:Host
                }
                else
                {
                    roleIndex = 2;  // 2:Client
                }
            }

            int line = 0;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Changing Session");
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationNum:%2d  Role=%s", nn::pia::session::Session::GetInstance()->GetValidStationIdNum(), GetSessionRoleString(roleIndex));
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationId:0x%016llX", localStationId);
            line += FontSize;
            line += FontSize;

            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   Processing is in progress...");
            line += FontSize;

            line = ScreenHeight - FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
            line += FontSize;
        }

        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
        frameNo++;
    }

    if (result.IsFailure())
    {
        if (result == nn::pia::ResultSessionIsNotFound())
        {
            // サンプルの都合上、ハンドリングタイプを直接指定する
            handlingType = nn::pia::Result::HandlingType_CleanupWithLeave;
        }
        else
        {
            handlingType = result.GetHandlingType();
        }
        PIA_SAMPLE_PRINT("Process(%d) ErrorCode:%d\n", waitingProcess, result.GetErrorCode());

        SetCommunicationErrorType(SampleErrorType_Matchmake);
        SetNetworkErrorResult(result);

        PIA_SAMPLE_PRINT("HandlingType=%d\n", handlingType);

        // このサンプルデモでは、これらのハンドリングタイプの Result を本関数に渡されることはない
        if (handlingType == nn::pia::Result::HandlingType_Void ||
            handlingType == nn::pia::Result::HandlingType_Ignorable ||
            handlingType == nn::pia::Result::HandlingType_Retry ||
            handlingType == nn::pia::Result::HandlingType_ProgrammingError)
        {
            PIA_SAMPLE_ASSERT(false);
        }
        else
        {
            if (handlingType == nn::pia::Result::HandlingType_CleanupWithLeave)
            {
                nn::pia::Result leaveResult = nns::pia::Session_LeaveSession();
                if (leaveResult.IsSuccess())
                {
                    SetResetOfNetworkIsNecessary(false);
                }
            }
            else if (handlingType == nn::pia::Result::HandlingType_Cleanup)
            {
                SetResetOfNetworkIsNecessary(false);
            }
            else if (handlingType == nn::pia::Result::HandlingType_LogoutWithLeave)
            {
                nns::pia::Session_LeaveSession();
            }
            else if (handlingType == nn::pia::Result::HandlingType_ProgrammingError)
            {
                PIA_SAMPLE_ASSERT(false);
            }
            nns::pia::Session_CleanupSession();
            return SampleSceneNo_NetworkError;
        }
    }

    // 処理後の SessionStatus でシーン遷移先を分岐
    switch (nn::pia::session::Session::GetInstance()->GetStatus())
    {
    case nn::pia::session::Session::Status_NotConnected:
    {
        // チーム選択へ戻る
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_SelectTeam;
    }
    case nn::pia::session::Session::Status_ConnectedSession:
    case nn::pia::session::Session::Status_ConnectedJointSession:
    {
        switch (g_GameState)
        {
        case GameState_PreparingGame:
        {
            if (nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedSession)
            {
                // セッションマイグレーションが連続で起きた？
                PIA_SAMPLE_PRINT("Session migrated continuously? GameState_PreparingGame, Status_ConnectedSession\n");
                PIA_SAMPLE_PRINT("SceneChangingSession: GameState %d -> %d\n", g_GameState, GameState_Team);
                g_GameState = GameState_Team;
                return SampleSceneNo_SessionMigrationPostprocess;
            }
            PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedJointSession);
            PIA_SAMPLE_PRINT("SceneChangingSession: GameState %d -> %d\n", g_GameState, GameState_Game);
            g_GameState = GameState_Game;
            return SampleSceneNo_SessionMigrationPostprocess;
        }
        case GameState_PreparingTeam:
        {
            if (nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedJointSession)
            {
                // セッションマイグレーションが連続で起きた？
                PIA_SAMPLE_PRINT("Session migrated continuously? GameState_PreparingTeam, Status_ConnectedJointSession\n");
                PIA_SAMPLE_PRINT("SceneChangingSession: GameState %d -> %d\n", g_GameState, GameState_Game);
                g_GameState = GameState_Game;
                return SampleSceneNo_SessionMigrationPostprocess;
            }
            PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetStatus() == nn::pia::session::Session::Status_ConnectedSession);
            PIA_SAMPLE_PRINT("SceneChangingSession: GameState %d -> %d\n", g_GameState, GameState_Team);
            g_GameState = GameState_Team;
            return SampleSceneNo_SessionMigrationPostprocess;
        }
        default:
        {
            PIA_SAMPLE_ASSERT(false);
            nns::pia::Session_CleanupSession();
            return SampleSceneNo_NetworkError;
        }
        }
    }
    case nn::pia::session::Session::Status_MigratingSession:
    {
        PIA_SAMPLE_PRINT("Status_MigratingSession\n");
        // ジョイントセッション参加直後に、ジョイントセッション解散が開始された。
        PIA_SAMPLE_ASSERT(waitingProcess == SampleWaitingAsyncProcessing_JoinRandomJointSession || 
                          waitingProcess == SampleWaitingAsyncProcessing_CreateJointSession || 
                          waitingProcess == SampleWaitingAsyncProcessing_JoinJointSession);
        // 後処理を飛ばしてゲーム中の通信シーンに遷移する
        PIA_SAMPLE_PRINT("GameState %d -> %d\n", g_GameState, GameState_Game);
        PIA_SAMPLE_ASSERT(g_GameState == GameState_PreparingGame);
        g_GameState = GameState_Game;
        return SampleSceneNo_CommunicateGame;
    }
    case nn::pia::session::Session::Status_DisconnectedSession:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedSession\n");
        SetCommunicationErrorType(SampleErrorType_SessionLost);
        SetNetworkErrorResult(nn::pia::ResultSessionConnectionIsLost());
        nn::pia::Result leaveResult = nns::pia::Session_LeaveSession();
        if (leaveResult.IsSuccess())
        {
            // Session のクリーンアップ＆再スタートアップをすればマッチメイク処理に戻れる状態であるため、ネットワークのリセット不要状態にしておく
            SetResetOfNetworkIsNecessary(false);
        }
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetwork:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetwork\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultNetworkConnectionIsLost());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetworkByDuplicateLogin:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetworkByDuplicateLogin\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetworkByWifiOff:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetworkByWifiOff\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultWifiOff());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetworkBySleep:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetworkBySleep\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultSleep());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    default:
    {
        // このデモの仕様上、ありえないはず
        PIA_SAMPLE_ASSERTMSG(false, "Unexpected MatchmakeStatus:%d", nn::pia::session::Session::GetInstance()->GetStatus());

        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    }
}

// セッションマイグレーション完了後の処理
SampleSceneNo SceneChangingSessionPostprocessing()
{
    nn::pia::Result result = nn::pia::ResultSuccess();

    uint32_t frameNo = 0;
    bool isExit = false;

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_SessionMigrationPostprocess,
        true,
        false,
        0,
        0);

    nn::pia::StationId localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();

    nn::pia::Result::HandlingType handlingType = nn::pia::Result::HandlingType_Void;
    SampleWaitingAsyncProcessing waitingProcess = SampleWaitingAsyncProcessing_Passive;

    while (!isExit)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        // CloneProtocol のクロックを更新
        nns::pia::Clone_GetCloneProtocol()->UpdateClock();

        // Dispatch 処理
        nn::pia::common::Scheduler::GetInstance()->Dispatch();
        nns::pia::Nex_SchedulerDispatch();

        if (waitingProcess == SampleWaitingAsyncProcessing_Passive)
        {
            result = nn::pia::session::Session::GetInstance()->CheckConnectionError();
            if (result.IsFailure())
            {
                isExit = true;
            }
            // CloneProtocol が Active になるのを待つ
            else if (nns::pia::Clone_GetCloneProtocol()->GetState() == nn::pia::clone::CloneProtocol::State_Active)
            {
                switch (g_GameState)
                {
                case GameState_Team:
                {  // チームセッションへ移行完了

                    // セッション参加／拒否判定コールバックを解除
                    nn::pia::session::Session::GetInstance()->UnregisterJoiningApprovalCallback();

#if PIA_TEST_CODE
#else
                    if (nn::pia::session::Session::GetInstance()->IsHost() &&
                        nn::pia::session::Session::GetInstance()->GetSessionOpenStatus() != nn::pia::session::Session::SessionOpenStatus_Open)
                    {
                        PIA_SAMPLE_PRINT("Call OpenSessionAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->OpenSessionAsync();
                        if (result.IsFailure())
                        {
                            PIA_SAMPLE_PRINT("Call OpenSessionAsync is failed.\n");
                            PIA_SAMPLE_ASSERT(false);
                            isExit = true;
                        }
                        else
                        {
                            waitingProcess = SampleWaitingAsyncProcessing_OpenSession;
                        }
                    }
                    else
#endif
                    {
                        isExit = true;
                    }
                }
                break;
                case GameState_Game:
                {  // ゲームセッションへ移行完了

                    // セッション参加／拒否判定コールバックを解除
                    nn::pia::session::Session::GetInstance()->UnregisterJoiningApprovalCallback();

#if PIA_TEST_CODE
#else
                    if (nn::pia::session::Session::GetInstance()->IsJointSessionHost() &&
                        nn::pia::session::Session::GetInstance()->GetJointSessionOpenStatus() != nn::pia::session::Session::SessionOpenStatus_Open)
                    {
                        PIA_SAMPLE_PRINT("Call OpenJointSessionAsync.\n");
                        result = nn::pia::session::Session::GetInstance()->OpenJointSessionAsync();
                        if (result.IsFailure())
                        {
                            PIA_SAMPLE_PRINT("Call OpenJointSessionAsync is failed.\n");
                            PIA_SAMPLE_ASSERT(false);
                            isExit = true;
                        }
                        else
                        {
                            waitingProcess = SampleWaitingAsyncProcessing_OpenJointSession;
                        }
                    }
                    else
#endif
                    {
                        isExit = true;
                    }
                }
                break;
                default:
                {
                    PIA_SAMPLE_ASSERT(false);
                }
                break;
                }
            }
            // CloneProtocol が Active になるのを待ってる間にセッション移行処理が開始された
            else if (IsSessionMigrating())
            {
                isExit = true;
            }
        }

#if PIA_TEST_CODE
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(TestParamIndex_JointSessionSequence, nn::pia::session::Session::GetInstance()->GetJointSequence());

        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));
            PIA_SAMPLE_PRINT("[JointSequence] %d\n", nn::pia::session::Session::GetInstance()->GetJointSequence());
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                pMgr->SetSendThreadPacketLossRatio(100);
                pMgr->SetReceiveThreadPacketLossRatio(10);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 100%, Receive 10%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                pMgr->SetSendThreadPacketLossRatio(10);
                pMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 10%, Receive 100%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                pMgr->SetSendThreadPacketLossRatio(10);
                pMgr->SetReceiveThreadPacketLossRatio(10);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 10%).\n");
            }
            else
            {
                pMgr->SetSendThreadPacketLossRatio(100);
                pMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 100%).\n");
            }
        }
#endif

        // 非同期処理の完了チェック
        switch (waitingProcess)
        {
        case SampleWaitingAsyncProcessing_OpenSession:
        {
            if (nn::pia::session::Session::GetInstance()->IsOpenSessionCompleted())
            {
                result = nn::pia::session::Session::GetInstance()->GetOpenSessionResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("OpenSession is success\n");
                }
                else
                {
                    PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                                      result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                                      result == nn::pia::ResultWifiOff() ||
                                      result == nn::pia::ResultSleep());
                }
                isExit = true;
            }
        }
        break;
        case SampleWaitingAsyncProcessing_OpenJointSession:
        {
            if (nn::pia::session::Session::GetInstance()->IsOpenJointSessionCompleted())
            {
                result = nn::pia::session::Session::GetInstance()->GetOpenJointSessionResult();
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("OpenJointSession is success\n");
                }
                else
                {
                    PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                                      result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
                                      result == nn::pia::ResultWifiOff() ||
                                      result == nn::pia::ResultSleep());
                }
                isExit = true;
            }
        }
        break;
        default:
        {
        }
        break;
        }

        UpdateStatusManager();

        //画面描画処理
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

        {
            //自分自身の役割チェック
            int roleIndex = 0;  //Unknown;
            localStationId = nn::pia::session::Session::GetInstance()->GetLocalStationId();
            if (localStationId != nn::pia::StationIdInvalid)
            {
                if (nn::pia::session::Session::GetInstance()->IsHost())
                {
                    roleIndex = 1;  // 1:Host
                }
                else
                {
                    roleIndex = 2;  // 2:Client
                }
            }

            int line = 0;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Changing Session Postprocessing");
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationNum:%2d  Role=%s", nn::pia::session::Session::GetInstance()->GetValidStationIdNum(), GetSessionRoleString(roleIndex));
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  StationId:%llu", localStationId);
            line += FontSize;
            line += FontSize;

            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   Processing is in progress...");
            line += FontSize;

            line = ScreenHeight - FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
            line += FontSize;
        }

        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
        frameNo++;
    }

    if (result.IsFailure())
    {
        if (result == nn::pia::ResultNetworkConnectionIsLost() ||
            result == nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin() ||
            result == nn::pia::ResultWifiOff() ||
            result == nn::pia::ResultSleep())
        {
            SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        }
        else if (result == nn::pia::ResultSessionConnectionIsLost())
        {
            SetCommunicationErrorType(SampleErrorType_SessionLost);
        }
        else if (result == nn::pia::ResultSessionIsNotFound())
        {
            SetCommunicationErrorType(SampleErrorType_SessionLost);
        }
        else if (result == nn::pia::ResultNexInternalError() ||
            result == nn::pia::ResultGameServerProcessAborted())
        {
            SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        }
        else
        {
            PIA_TRACE_RESULT_ALWAYS(result);
            PIA_SAMPLE_ASSERT(false);
        }
        SetNetworkErrorResult(result);
        handlingType = result.GetHandlingType();
        PIA_SAMPLE_PRINT("HandlingType=%d\n", handlingType);

        // このサンプルデモでは、これらのハンドリングタイプの Result を本関数に渡されることはない
        if (handlingType == nn::pia::Result::HandlingType_Void ||
            handlingType == nn::pia::Result::HandlingType_Ignorable ||
            handlingType == nn::pia::Result::HandlingType_Retry ||
            handlingType == nn::pia::Result::HandlingType_ProgrammingError)
        {
            PIA_SAMPLE_ASSERT(false);
        }
        else
        {
            if (handlingType == nn::pia::Result::HandlingType_CleanupWithLeave)
            {
                nn::pia::Result leaveResult = nns::pia::Session_LeaveSession();
                if (leaveResult.IsSuccess())
                {
                    SetResetOfNetworkIsNecessary(false);
                }
            }
            else if (handlingType == nn::pia::Result::HandlingType_Cleanup)
            {
                SetResetOfNetworkIsNecessary(false);
            }
            else if (handlingType == nn::pia::Result::HandlingType_LogoutWithLeave)
            {
                nns::pia::Session_LeaveSession();
            }
            else if (handlingType == nn::pia::Result::HandlingType_ProgrammingError)
            {
                PIA_SAMPLE_ASSERT(false);
            }
            nns::pia::Session_CleanupSession();
            return SampleSceneNo_NetworkError;
        }
    }

    // 処理後の SessionStatus でシーン遷移先を分岐
    switch (nn::pia::session::Session::GetInstance()->GetStatus())
    {
    case nn::pia::session::Session::Status_NotConnected:
    {
        // チーム選択へ戻る
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_SelectTeam;
    }
    case nn::pia::session::Session::Status_ConnectedSession:
    case nn::pia::session::Session::Status_ConnectedJointSession:
    case nn::pia::session::Session::Status_MigratingSession:
    {
        switch (g_GameState)
        {
        case GameState_Game:
        {
            return SampleSceneNo_CommunicateGame;
        }
        case GameState_Team:
        {
            return SampleSceneNo_CommunicateTeam;
        }
        default:
        {
            PIA_SAMPLE_ASSERT(false);
            nns::pia::Session_CleanupSession();
            return SampleSceneNo_NetworkError;
        }
        }
    }
    case nn::pia::session::Session::Status_DisconnectedSession:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedSession\n");
        SetCommunicationErrorType(SampleErrorType_SessionLost);
        SetNetworkErrorResult(nn::pia::ResultSessionConnectionIsLost());
        nn::pia::Result leaveResult = nns::pia::Session_LeaveSession();
        if (leaveResult.IsSuccess())
        {
            // Session のクリーンアップ＆再スタートアップをすればマッチメイク処理に戻れる状態であるため、ネットワークのリセット不要状態にしておく
            SetResetOfNetworkIsNecessary(false);
        }
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetwork:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetwork\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultNetworkConnectionIsLost());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetworkByDuplicateLogin:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetworkByDuplicateLogin\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultNetworkConnectionIsLostByDuplicateLogin());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetworkByWifiOff:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetworkByWifiOff\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultWifiOff());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    case nn::pia::session::Session::Status_DisconnectedNetworkBySleep:
    {
        PIA_SAMPLE_PRINT("Status_DisconnectedNetworkBySleep\n");
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(nn::pia::ResultSleep());
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    default:
    {
        // このデモの仕様上、ありえないはず
        PIA_SAMPLE_ASSERTMSG(false, "Unexpected MatchmakeStatus:%d", nn::pia::session::Session::GetInstance()->GetStatus());

        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }
    }
}

#endif
