﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include <nn/pia/common.h>
#include <nn/pia/common/common_Event.h>
#include <nn/pia/inet.h>
#include <nn/pia/session.h>

#include "SampleLib/Defines.h"
#include "SampleLib/ErrorHandling.h"
#include "SampleLib/SampleLib.h"

#if NNS_PIA_ENABLE_INET

#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Session_Setup.h"

#include "SampleConfig.h"
#include "EventChecker.h"
#include "TeamConstitutionManager.h"

namespace
{
SampleTeamType s_CurrentTeamType = SampleTeamType_Alpha;

const nns::pia::sampleframework::Color cTeamColorAlpha(1.0f, 1.0f, 0.0f, 1.0f);
const nns::pia::sampleframework::Color cTeamColorBravo(1.0f, 0.0f, 1.0f, 1.0f);
const nns::pia::sampleframework::Color cTeamColorCharlie(0.0f, 0.0f, 1.0f, 1.0f);
const nns::pia::sampleframework::Color cTeamColorDelta(1.0f, 0.0f, 0.0f, 1.0f);
const nns::pia::sampleframework::Color cTeamColor[] = {
    cTeamColorAlpha, cTeamColorBravo, cTeamColorCharlie, cTeamColorDelta
};
const char* cTeamName[] = {
    "TEAM ALPHA", "TEAM BRAVO", "TEAM CHARLIE", "TEAM DELTA"
};

const nns::pia::sampleframework::Color cSelectMarkColor(0.8f, 0.8f, 0.8f, 1.0f);
const int32_t cSelectMarkSize = 20;

#if PIA_TEST_CODE
uint32_t s_MaxSilenceTime = nn::pia::session::SessionMaxSilenceTimeDefault;
#endif

void DrawTeamExplanatoryNotes(
    int32_t posLeftTopX,
    int32_t posLeftTopY,
    const char* teamNameStr,
    const nns::pia::sampleframework::Color& teamColor)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(teamColor);
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
        posLeftTopX + 5, posLeftTopY + 6, teamNameStr);
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
}

void DrawSelectMark(int32_t posLeftTopX, int32_t posLeftTopY)
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(
        posLeftTopX - cSelectMarkSize + 5, posLeftTopY + 6, ">");
}
}

// 通信中のシーンで、チームタイプ関連情報を利用するための関数
SampleTeamType GetCurrentTeamType()
{
    return s_CurrentTeamType;
}

const nns::pia::sampleframework::Color GetColorOfTeamType(SampleTeamType teamType)
{
    if (teamType < SampleTeamType_Max)
    {
        return cTeamColor[teamType];
    }
    return nns::pia::sampleframework::ColorGray;
}


SampleSceneNo SceneSelectTeam()
{
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_SelectTeam,
        false,
        false,
        0);

    uint32_t frameNo = 0;
    bool isExit = false;
    nn::pia::Result errorResult = nn::pia::ResultSuccess();
    bool logout = false;

    s_CurrentTeamType = SampleTeamType_Alpha;
    SampleTeamType selectTeamBuf = s_CurrentTeamType;

#if PIA_TEST_CODE
    SetHostMigrationEnabled(true);
    SetMaxPlayerNum(GamePlayerMax);
    s_MaxSilenceTime = nn::pia::session::SessionMaxSilenceTimeDefault;
#endif

    while (!isExit)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        // UP/DOWN: Change team type
        const uint32_t cTeamTypeNum = static_cast<uint32_t>(TeamMax);
        uint32_t selectMarkPosBuf = static_cast<uint32_t>(selectTeamBuf);
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            selectMarkPosBuf = (selectMarkPosBuf + cTeamTypeNum - 1) % cTeamTypeNum;
        }
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            selectMarkPosBuf = (selectMarkPosBuf + 1) % cTeamTypeNum;
        }
        selectTeamBuf = static_cast<SampleTeamType>(selectMarkPosBuf);

        // A: Decide team
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            isExit = true;
        }
        // START: Logout
        else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonStart))
        {
            logout = true;
            isExit = true;
        }
#if PIA_TEST_CODE
        else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            SetHostMigrationEnabled(!IsHostMigrationEnabled());
            nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(TestParamIndex_HostMigration, IsHostMigrationEnabled() ? 1 : 0);
            PIA_SAMPLE_PRINT("Set Host Migration %s\n", IsHostMigrationEnabled() ? "true" : "false");
        }
        else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            SetMaxPlayerNum(GamePlayerMax / TeamMax + 1);
            PIA_SAMPLE_PRINT("Set Max Player Num: %d\n", GetMaxPlayerNum());
        }
        else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            s_MaxSilenceTime = 60 * 1000;
            PIA_SAMPLE_PRINT("Set Max Silence Time: %d\n", s_MaxSilenceTime);
        }
#endif

        // Dispatch 処理
        nns::pia::Nex_SchedulerDispatch();

        // AC のネットワーク接続状態、ゲームサーバとの接続状態のチェック
        if (!nns::pia::Net_IsConnected())
        {
            errorResult = nn::pia::ResultNetworkConnectionIsLost();
            isExit = true;
        }
        else if (nns::pia::Nex_IsNgsConnected().IsFailure())
        {
            errorResult = nns::pia::Nex_IsNgsConnected();
            isExit = true;
        }

        //画面描画処理
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        int line = 0;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Select Team.");
        line += FontSize;
        line += FontSize;

        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   Please select your team type.");
        line += FontSize;
        line += FontSize;

        int32_t xBuf, yBuf;
        xBuf = 30;
        yBuf = line;

        for (uint32_t i = 0; i < cTeamTypeNum; i++)
        {
            DrawTeamExplanatoryNotes(xBuf, yBuf, cTeamName[i], cTeamColor[i]);
            if (selectTeamBuf == static_cast<SampleTeamType>(i))
            {
                DrawSelectMark(xBuf, yBuf);
            }
            yBuf += cSelectMarkSize;
        }

        line = ScreenHeight - (FontSize * 6);
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Manual ]");
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  UP/DOWN: Change team type.");
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  A: Decide team type.");
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  START: Logout.");
        line += FontSize;
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
        line += FontSize;

        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
        ++frameNo;
    }

    s_CurrentTeamType = selectTeamBuf;

    if (errorResult.IsFailure())
    {
        SetCommunicationErrorType(SampleErrorType_ConnectionLost);
        SetNetworkErrorResult(errorResult);
        return SampleSceneNo_NetworkError;
    }
    if (logout)
    {
        return SampleSceneNo_Reset;
    }

    return SampleSceneNo_MatchmakeTeam;
}


SampleSceneNo SceneMatchmake()
{
    PIA_SAMPLE_PRINT("SceneMatchmake: GameState %d -> %d\n", g_GameState, GameState_PreparingTeam);
    g_GameState = GameState_PreparingTeam;
    g_SessionMigrationEventChecker.Reset();
    g_HostMigrationEventChecker.Reset();

    ResetErrorOccurredFlag();

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_MatchmakeTeam,
        false,
        false,
        0);

    PIA_SAMPLE_PRINT("ScenMatchmake start.\n");

    // セッション参加／拒否判定コールバックを解除
    nn::pia::session::Session::GetInstance()->UnregisterJoiningApprovalCallback();

    // トークンに、チーム情報を入れておく
    SampleTokenData tokenData;
#if 0
    tokenData.sessionId = nn::pia::session::Session::GetInstance()->GetSessionId();
#endif
    tokenData.teamType = static_cast<uint8_t>(GetCurrentTeamType());
    nn::pia::transport::Station::IdentificationToken token;
    memcpy(token.data, &tokenData, sizeof(tokenData));

    nn::pia::inet::NexSessionStartupSetting startupSetting;
    startupSetting.pToken = &token;
    startupSetting.cryptoMode = nn::pia::common::CryptoSetting::Mode_Aes128;
#if PIA_TEST_CODE
    startupSetting.isHostMigrationEnabled = IsHostMigrationEnabled();
    startupSetting.maxSilenceTime = s_MaxSilenceTime;
#else
    startupSetting.isHostMigrationEnabled = HostMigrationSetting;
#endif
    nn::pia::transport::Station::PlayerInfo playerName;
    nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
    startupSetting.playerInfo[0].Copy(playerName);

    nn::pia::Result r = nns::pia::Session_StartupSession(startupSetting);
    if (r.IsFailure())
    {
        SetNetworkErrorResult(r);
        SetCommunicationErrorType(SampleErrorType_Matchmake);
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }

    // 作成するチーム用マッチメイクセッションの設定
    nn::pia::inet::NexCreateSessionSetting createSessionSetting;
    createSessionSetting.SetGameMode(MatchmakeGameModeForTeam);  // ゲームモードは "チーム用" にする
    createSessionSetting.SetMaxParticipantNum(TeamMemberMax);
    createSessionSetting.SetMinParticipantNum(1);
    createSessionSetting.SetAttribute(MatchmakeAttributeIndexTeamType, static_cast<uint32_t>(s_CurrentTeamType));
    createSessionSetting.SetAttribute(MatchmakeAttributeIndexUserPassword, 0);
    createSessionSetting.SetOpenSession(true);

    // オートマッチの検索条件の設定
    const uint8_t SearchCriteriaNum = 1;
    nn::pia::inet::NexSessionSearchCriteria nexSearchCriteriaArray[SearchCriteriaNum];
    nexSearchCriteriaArray[0].SetGameMode(MatchmakeGameModeForTeam);
    nexSearchCriteriaArray[0].SetMaxParticipantNum(TeamMemberMax);
    // 自分と同じ TeamType を検索する
    nexSearchCriteriaArray[0].SetAttribute(MatchmakeAttributeIndexTeamType, static_cast<uint32_t>(s_CurrentTeamType));
    nexSearchCriteriaArray[0].SetAttribute(MatchmakeAttributeIndexUserPassword, 0);

    nn::pia::inet::NexJoinRandomSessionSetting joinRandomSessionSetting;
    joinRandomSessionSetting.SetCreateSessionSetting(&createSessionSetting);
    joinRandomSessionSetting.SetSessionSearchCriteria(nexSearchCriteriaArray, SearchCriteriaNum);

#if PIA_TEST_CODE
    bool isDelayDispatch = false;
    bool isStopDispatch = false;
    int delayCount = 0;
#endif

    uint32_t frameNo = 0;
    bool isExit = false;
    bool backToSelectTeam = false;
    bool logout = false;
    nn::pia::Result errorResult = nn::pia::ResultSuccess();

    bool asyncProcessingFlag = false;

    while (!isExit)
    {
        if (asyncProcessingFlag)
        {
            if (nn::pia::session::Session::GetInstance()->IsJoinRandomSessionCompleted())
            {
                isExit = true;
                if (nn::pia::session::Session::GetInstance()->GetJoinRandomSessionResult().IsFailure())
                {
                    errorResult = nn::pia::session::Session::GetInstance()->GetJoinRandomSessionResult();
                    PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetJoinRandomSessionResult() != nn::pia::ResultMeshIsFull());
                }
            }
#if PIA_TEST_CODE
            else if (nn::pia::session::Session::GetInstance()->IsCreateSessionCompleted())
            {
                isExit = true;
                if (nn::pia::session::Session::GetInstance()->GetCreateSessionResult().IsFailure())
                {
                    errorResult = nn::pia::session::Session::GetInstance()->GetCreateSessionResult();
                    PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetCreateSessionResult() != nn::pia::ResultMeshIsFull());
                }
            }
            else if (nn::pia::session::Session::GetInstance()->IsBrowseSessionCompleted())
            {
                if (nn::pia::session::Session::GetInstance()->GetBrowseSessionResult().IsFailure())
                {
                    isExit = true;
                    errorResult = nn::pia::session::Session::GetInstance()->GetBrowseSessionResult();
                }
                else if (nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->GetSize() > 0)
                {
                    nn::pia::inet::NexJoinSessionSetting joinSessionSetting;
                    joinSessionSetting.SetSessionInfoPtr(*(nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->Begin()));
                    joinSessionSetting.SetSessionUserPassword(UserPasswordString[s_CurrentTeamType]);

                    nn::pia::Result result = nn::pia::session::Session::GetInstance()->JoinSessionAsync(&joinSessionSetting);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("call JoinSession is failure.\n");
                        errorResult = result;
                        PIA_SAMPLE_ASSERT(false);
                        isExit = true;
                    }
                }
                else
                {
                    PIA_SAMPLE_PRINT("Not found.\n");
                    asyncProcessingFlag = false;
                }
            }
            else if (nn::pia::session::Session::GetInstance()->IsJoinSessionCompleted())
            {
                isExit = true;
                if (nn::pia::session::Session::GetInstance()->GetJoinSessionResult().IsFailure())
                {
                    errorResult = nn::pia::session::Session::GetInstance()->GetJoinSessionResult();
                    PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetJoinSessionResult() != nn::pia::ResultMeshIsFull());
                }
            }

            nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ProgramParamGetJoinSequence, nn::pia::session::Session::GetInstance()->GetJoinAutoMatchmakeSequence());
#endif
        }
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        if (!asyncProcessingFlag)
        {
            // A: Start AutoMatchmake
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                nn::pia::Result result = nn::pia::session::Session::GetInstance()->JoinRandomSessionAsync(&joinRandomSessionSetting);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("call JoinRandomSessionAsync is failure.\n");
                    errorResult = result;
                    PIA_SAMPLE_ASSERT(false);
                    isExit = true;
                }
                else
                {
                    PIA_SAMPLE_PRINT("call JoinRandomSessionAsync is success.\n");
                    asyncProcessingFlag = true;
                }
            }
#if PIA_TEST_CODE
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonX))
            {
                if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
                {
                    createSessionSetting.SetAttribute(MatchmakeAttributeIndexUserPassword, 1);
                    createSessionSetting.SetSessionUserPassword(UserPasswordString[s_CurrentTeamType]);

                    nn::pia::Result result = nn::pia::session::Session::GetInstance()->CreateSessionAsync(&createSessionSetting);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("call CreateSession is failure.\n");
                        errorResult = result;
                        PIA_SAMPLE_ASSERT(false);
                        isExit = true;
                    }
                    else
                    {
                        asyncProcessingFlag = true;
                    }
                }
                else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonDown))
                {
                    nexSearchCriteriaArray[0].SetAttribute(MatchmakeAttributeIndexUserPassword, 1);
                    nexSearchCriteriaArray[0].SetExcludeUserPasswordSet(false);
                    nexSearchCriteriaArray[0].SetResultRange(0, 4);
                    nn::pia::Result result = nn::pia::session::Session::GetInstance()->BrowseSessionAsync(nexSearchCriteriaArray);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("call BrowseSession is failure.\n");
                        errorResult = result;
                        PIA_SAMPLE_ASSERT(false);
                        isExit = true;
                    }
                    else
                    {
                        asyncProcessingFlag = true;
                    }
                }
            }
#endif
            // B: Back to SelectTeam
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonB))
            {
                backToSelectTeam = true;
                isExit = true;
            }
            // START: Logout
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonStart))
            {
                logout = true;
                isExit = true;
            }
        }


#if PIA_TEST_CODE
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonRight) &&
            nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonLeft))
        {
            isDelayDispatch = !isDelayDispatch;
            PIA_SAMPLE_PRINT("matchmake Dispatch delay %s\n", isDelayDispatch ? "enable" : "disable");
            delayCount = 0;
            isStopDispatch = false;
        }
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp) &&
            nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            isStopDispatch = !isStopDispatch;
            PIA_SAMPLE_PRINT("matchmake Dispatch %s.\n", isStopDispatch ? "stop" : "resume");
            isDelayDispatch = false;
            delayCount = 0;
        }
#endif


#if PIA_TEST_CODE
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));
            PIA_SAMPLE_PRINT("[JointSequence] %d\n", nn::pia::session::Session::GetInstance()->GetJointSequence());
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonUp))
            {
                pMgr->SetSendThreadPacketLossRatio(100);
                pMgr->SetReceiveThreadPacketLossRatio(10);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 100%, Receive 10%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonDown))
            {
                pMgr->SetSendThreadPacketLossRatio(10);
                pMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send 10%, Receive 100%).\n");
            }
            else if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsHold(nns::pia::sampleframework::ButtonRight))
            {
                pMgr->SetSendThreadPacketLossRatio(0);
                pMgr->SetReceiveThreadPacketLossRatio(0);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 10%).\n");
            }
            else
            {
                pMgr->SetSendThreadPacketLossRatio(100);
                pMgr->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("Enable packet loss emulation (Send/Receive 100%).\n");
            }
        }
#endif

#if PIA_TEST_CODE
        // テストのためディスパッチの頻度を下げて、特定のタイミングを狙い易くします。
        if (isDelayDispatch)
        {
            ++delayCount;
            delayCount = delayCount % DispatchDelayCount;
        }

        if ((!isDelayDispatch || (isDelayDispatch && delayCount == DispatchDelayCount - 1)) && !isStopDispatch)
        {
#endif
            // Dispatch 処理
            nns::pia::Nex_SchedulerDispatch();
            nn::pia::common::Scheduler::GetInstance()->Dispatch();
#if PIA_TEST_CODE
        }
#endif

        //画面描画処理
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        int line = 0;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %08X", ProgramNameString, frameNo);
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Matchmake.");
        line += FontSize;
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  CurrentTeamType:");
        line += FontSize;
        const int32_t xBuf = 30;
        switch (s_CurrentTeamType)
        {
        case SampleTeamType_Alpha:
            DrawTeamExplanatoryNotes(xBuf, line, "TEAM ALPHA", cTeamColorAlpha);
            break;
        case SampleTeamType_Bravo:
            DrawTeamExplanatoryNotes(xBuf, line, "TEAM BRAVO", cTeamColorBravo);
            break;
        case SampleTeamType_Charlie:
            DrawTeamExplanatoryNotes(xBuf, line, "TEAM CHARLIE", cTeamColorCharlie);
            break;
        case SampleTeamType_Delta:
            DrawTeamExplanatoryNotes(xBuf, line, "TEAM DELTA", cTeamColorDelta);
            break;
        default:
            break;
        }
        line += cSelectMarkSize + FontSize;

        if (!asyncProcessingFlag)
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   Please select operation.");
            line += FontSize;
        }
        else
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   Processing is in progress...");
            line += FontSize;
        }

        line = ScreenHeight - (FontSize * 8);
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Manual ]");
        line += FontSize;
        if (!asyncProcessingFlag)
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  A: Start JoinRandomSession.");
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  B: Back to SelectTeam.");
            line += FontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  START: Logout.");
            line += FontSize;
        }
        else
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Can not accept operation now.");
            line += FontSize;
            line += FontSize * 2;
        }
        line += FontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", GetPiaVersionString());
        line += FontSize;

        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
        ++frameNo;
    }

    if (backToSelectTeam)
    {
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_SelectTeam;
    }
    if (logout)
    {
        return SampleSceneNo_Reset;
    }
    if (errorResult.IsFailure())
    {
        SetNetworkErrorResult(errorResult);
        SetCommunicationErrorType(SampleErrorType_Matchmake);
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }

    nn::pia::session::Session::Status sessionStatus = nn::pia::session::Session::GetInstance()->GetStatus();
    // セッション参加に成功したと同時に NicDown が発動し、送受信スレッド側でソケットエラーを検出する場合がある
    if (sessionStatus == nn::pia::session::Session::Status_DisconnectedNetwork)
    {
        return SampleSceneNo_Reset;
    }
    // セッション参加処理を開始した直後にセッションマイグレーションが開始されると
    // セッションマイグレーションを開始することができない場合がある
    // セッションマイグレーション中のセッション再構築とセッション参加成功が
    // 同時に起こった場合、参加処理に成功してもセッション状態が切断状態の場合がある
    if (sessionStatus == nn::pia::session::Session::Status_DisconnectedSession)
    {
        nn::pia::Result leaveResult = nns::pia::Session_LeaveSession();
        if (leaveResult == nn::pia::ResultSuccess())
        {
            // Session のクリーンアップ＆再スタートアップをすればマッチメイク処理に戻れる状態であるため、ネットワークのリセット不要状態にしておく
            SetResetOfNetworkIsNecessary(false);
        }
        SetNetworkErrorResult(nn::pia::ResultSessionConnectionIsLost());
        SetCommunicationErrorType(SampleErrorType_Matchmake);
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_NetworkError;
    }

    PIA_SAMPLE_PRINT("SceneMatchmake: GameState %d -> %d\n", g_GameState, GameState_Team);
    g_GameState = GameState_Team;
    return SampleSceneNo_CommunicateTeam;
}

#endif
