﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include "SampleConfig.h"

#if NNS_PIA_ENABLE_INET

class TeamConstitutionManager
{
public:
    TeamConstitutionManager();

    struct TeamConstitution
    {
        TeamConstitution()
        {
            Clear();
        }

        void Clear()
        {
            for (int i = 0; i < TeamMax; ++i)
            {
                sessionIdTable[i] = nn::pia::session::InvalidSessionId;
                for (int j = 0; j < TeamMemberMax; ++j)
                {
                    stationIdTable[i][j] = nn::pia::StationIdInvalid;
                }
            }
        }

        void Trace() const
        {
            PIA_SAMPLE_PRINT("TeamConstitution Trace\n");
            for (int i = 0; i < TeamMax; ++i)
            {
                PIA_SAMPLE_PRINT("stationIdTable[%d] : ", i);
                for (int j = 0; j < TeamMemberMax; ++j)
                {
                    PIA_SAMPLE_PRINT("[%d]%llu ", j, stationIdTable[i][j]);
                }
                PIA_SAMPLE_PRINT("\n");
            }
        }

        uint32_t sessionIdTable[TeamMax];
        uint64_t stationIdTable[TeamMax][TeamMemberMax];
    };

    void Setup();
    void Reset();
    void Update();

    bool JoinEvent(nn::pia::StationId stationId);
    void LeaveEvent(nn::pia::StationId stationId);
    bool SessionHostChangedEvent();

    nn::pia::StationId GetStationId(int teamIndex, int memberIndex) const;
    int GetTeamIndex(nn::pia::StationId stationId) const;
    int GetMemberIndex(nn::pia::StationId stationId) const;
    uint32_t GetTeamSessionId(int teamIndex) const;
    int GetTeamMemberNum(int teamIndex) const;
    int GetTeamNum() const;

private:
    bool AssignTeamMember(nn::pia::StationId stationId);
    bool CancelTeamMember(nn::pia::StationId stationId);

    nn::pia::clone::AtomicSharingClone clone;
    nn::pia::clone::ReliableCloneElement<TeamConstitution> m_ElementR;
    TeamConstitution m_TeamConstitution;
    bool m_IsNeedSend;
    nn::pia::clone::ClockValue m_LatestInfoClock;
};

extern TeamConstitutionManager g_TeamConstitutionManager;

#endif
