﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#include <stdlib.h>

#include <nn/pia/common/common_Report.h>

#include <nn/pia/framework/framework_Api.h>
#include <nn/pia/framework/framework_Framework.h>

#include <nn/pia/clone.h>
#include <nn/pia/session.h>
#include <nn/pia/lan.h>
#include <nn/pia/transport.h>


#if NN_PIA_NINTENDOSDK
#include <nn/account.h>
#include <nn/account/account_Selector.h>
#include <nn/err.h>
#include <nn/err/err_Types.h>
#endif

#include "SampleLib/SampleLib.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Local_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Session_Setup.h"
#include "SampleFramework/include/SampleFramework.h"

#define NN_PIA_BROWSE_ONLY 1        // 0: ランダムマッチメイク、1: ブラウズマッチメイク

#define NN_PIA_FRAMEWORK_INET_JOINT_SESSION 0

const uint32_t MatchmakeGameModeOffset = 0;
const uint32_t MatchmakeGameModeBase = 65100;
const uint32_t MatchmakeGameMode = MatchmakeGameModeBase + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;

#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
const uint32_t TeamMatchmakeGameModeBase = 66100;
const uint32_t TeamMatchmakeGameMode = TeamMatchmakeGameModeBase + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;

// チームタイプに使用する属性のインデックス
const uint32_t MatchmakeAttributeIndexTeamType = 0;

/* ---< 所属するチームを区別するための列挙型 >--- */
enum SampleTeamType
{
    SampleTeamType_Alpha = 0,
    SampleTeamType_Bravo,
    SampleTeamType_Charlie,
    SampleTeamType_Delta,
    SampleTeamType_Max
};

uint32_t s_CurrentTeamTypeIndex = 0;

const nns::pia::sampleframework::Color cTeamColorAlpha(1.0f, 1.0f, 0.0f, 1.0f);
const nns::pia::sampleframework::Color cTeamColorBravo(1.0f, 0.0f, 1.0f, 1.0f);
const nns::pia::sampleframework::Color cTeamColorCharlie(0.0f, 0.0f, 1.0f, 1.0f);
const nns::pia::sampleframework::Color cTeamColorDelta(1.0f, 0.0f, 0.0f, 1.0f);

const char* cTeamName[] = {
    "TEAM ALPHA", "TEAM BRAVO", "TEAM CHARLIE", "TEAM DELTA"
};
#endif

static const uint16_t MaxStationNum = 8;  // 最大 Station 数
static const int MaxBrowsedSessionNum = 8; 
static const nns::pia::sampleframework::Color CursorColor(1.0f, 0.3f, 0.3f, 1.0f);

#if NN_PIA_CTR
static const int DataIdNum = 4;  // nn::pia::sync::SyncProtocol::DataIdNum で取得できるが、画面表示の都合で 4 とする。
#else
static const int DataIdNum = nn::pia::sync::SyncProtocol::DataIdNum;
#endif

// PiaSync 用
static const int Delay = 4;
static const int SendPeriod = 1;

// PiaClone 用
static nn::pia::clone::SendClone s_SendClone;
static nn::pia::clone::ReceiveClone s_ReceiveClone[MaxStationNum];
static nn::pia::clone::UnreliableCloneElement<int> s_ElementForSend;
static nn::pia::clone::UnreliableCloneElement<int> s_ElementForReceive[MaxStationNum];

// ローカル用
#if NN_PIA_NINTENDOSDK || NN_PIA_CTR
#include <nn/pia/local.h>
static const char PassPhrase[] = "FrameworkCloneAndSync";
static const int PassPhraseSize = sizeof(PassPhrase);
#if NN_PIA_NINTENDOSDK
static uint16_t LocalSceneId = MatchmakeGameModeOffset + 11000;
#elif NN_PIA_CTR
static const uint32_t LocalTitleId = 0xFF351;
static uint8_t LocalSubId = MatchmakeGameModeOffset;
#endif
#endif

bool CheckStationNum();

enum State
{
    State_Title,
    State_InitializeFramework,
    State_RandomSession,
    State_CreateOrJoinSession,
    State_BrowseSession,
    State_Session,
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
    State_JoinRandomJointSession,
    State_SessionMigration,
    State_JointSession,
    State_DestroyJointSession,
    State_LeaveJointSession,
#endif
    State_Leave,
    State_FinalizeFramework,
    State_Max
};

static bool s_IsSyncMode = false;    // false: PiaClone のテスト、true: PiaSync のテスト

nn::pia::StationIndex ConvertStationIdToStationIndex(nn::pia::StationId stationId)
{
    nn::pia::StationIndex stationIndex;
    nn::pia::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&stationIndex, stationId);
    if (result.IsFailure())
    {
        stationIndex = nn::pia::StationIndex_Invalid;
    }
    return stationIndex;
}

nns::pia::sampleframework::SampleFramework* GetSampleFramework()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = NULL;
    pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    if (!pSampleFramework)
    {
        PIA_SAMPLE_ASSERT(false);
    }
    return pSampleFramework;
}

// PiaFramework の非同期処理が完了するまで（Framework::IsXxxCompleted() が true を返すまで）ブロックします
void WaitForAsyncProcess(const char* asyncProcessName, bool (nn::pia::framework::Framework::*checkCompletionFunc)() const, State state)
{
    nn::pia::Result result;
    nns::pia::sampleframework::SampleFramework* pSampleFramework = GetSampleFramework();
    int frameNo = 0;
    while (!(nn::pia::framework::Framework::GetInstance()->*checkCompletionFunc)())
    {
        pSampleFramework->BeginFrame();
        pSampleFramework->DrawString(10, 20, "Sample state : %d", state);
        pSampleFramework->DrawString(10, 30, "PiaFramework state : %d", nn::pia::framework::Framework::GetInstance()->GetState());
        pSampleFramework->DrawString(10, 40, "%s... (frame count: %d)", asyncProcessName, frameNo++);
        result = nn::pia::framework::Framework::GetInstance()->Update();
        pSampleFramework->EndFrame();
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->Update() failed.\n");
        }
    }
}

// 失敗した Result が与えられた場合、 HandleErrorAsync でエラーハンドリングを行う。
void ResultCheck(nn::pia::Result result, State* pState, const nn::pia::framework::Framework::State& transitionState)
{
    if (result.IsFailure())
    {
#if NN_PIA_NINTENDOSDK
        // エラービューアで表示すべきエラーのハンドリング
        if (result.GetViewerType() == nn::pia::Result::ViewerType_ShouldUse)
        {
            if (result == nn::pia::ResultSdkViewerResultError())
            {
                nn::err::ShowError(nn::pia::framework::Framework::GetInstance()->GetLastSdkResult());
            }
            else
            {
                nn::err::ErrorCode errorCode;
                result.GetErrorCode(&errorCode);
                nn::err::ShowError(errorCode);
            }
        }
#endif
        PIA_SAMPLE_PRINT("----------------ErrorHandlingAsync start---------------\n");
        result = nn::pia::framework::Framework::GetInstance()->HandleErrorAsync(transitionState, result);
        if (result.IsFailure())
        {
            *pState = State_FinalizeFramework;
            return;
        }
        
        WaitForAsyncProcess("HandleErrorAsync", &nn::pia::framework::Framework::IsHandleErrorCompleted, *pState);

        result = nn::pia::framework::Framework::GetInstance()->GetHandleErrorResult();
        PIA_SAMPLE_ASSERTMSG(result.IsSuccess(), "Result(%08x)\n    Description: (%4d)\n    HandlingType:(%2d)", result.GetValue(), result.GetDescription(), result.GetHandlingType());
        if (nn::pia::framework::Framework::GetInstance()->GetState() >= nn::pia::framework::Framework::State_SessionJoined)
        {
            *pState = State_Leave;
        }
        else
        {
            *pState = State_FinalizeFramework;
        }
        PIA_SAMPLE_PRINT(" ----------------ErrorHandlingAsync end---------------\n");
    }
}

// 失敗した Result が与えられた場合、次に遷移すべき状態を求めて HandleErrorAsync でエラーハンドリングを行う。
void ResultCheck(nn::pia::Result result, State* pState)
{
    if (result.IsFailure())
    {
        const nn::pia::framework::Framework::State transitionState = nn::pia::framework::Framework::GetInstance()->ConvertHandlingTypeToState(result.GetHandlingType());
        ResultCheck(result, pState, transitionState);
    }
}

// ChangeStateAsync による通信の状態遷移を行う。
// ChangeStateAsync は非同期処理なので、この関数で終了を待つ。
nn::pia::Result ChangeState(State &state, const nn::pia::framework::Framework::State& transitionState)
{
    nn::pia::Result result;
    PIA_SAMPLE_PRINT("----------------ChangeState start---------------\n");

    // 状態遷移の開始
    result = nn::pia::framework::Framework::GetInstance()->ChangeStateAsync(transitionState);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->ChangeStateAsync() failed.\n");
        return result;
    }

    WaitForAsyncProcess("ChangeStateAsync", &nn::pia::framework::Framework::IsChangeStateCompleted, state);

    result = nn::pia::framework::Framework::GetInstance()->GetChangeStateResult();
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->GetChangeStateResult() failed.\n");
        return result;
    }
    else
    {
        PIA_SAMPLE_ASSERTMSG(result.IsSuccess(), "Result(%08x)\n    Description: (%4d)\n    HandlingType:(%2d)", result.GetValue(), result.GetDescription(), result.GetHandlingType());
    }

    PIA_SAMPLE_PRINT("----------------ChangeState end---------------\n");

    return nn::pia::ResultSuccess();
}

// セッションに参加しているステーションの数が妥当な値かチェックします。
bool CheckStationNum()
{
    const nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    if (pSession != NULL)
    {
        int stationNum = pSession->GetStationNum();
        int stationCnt = 0;
        for (nn::pia::session::Session::StationIdConstIterator it = pSession->Begin(), itEnd = pSession->End(); it != itEnd; ++it)
        {
            ++stationCnt;
        }
        if (stationCnt != stationNum)
        {
            PIA_SAMPLE_PRINT("error : stationNum:%d, stationCnt:%d", stationNum, stationCnt);
            return false;
        }
    }

    return true;
}

// 各 setting の初期化
void InitializeSetting(nn::pia::framework::NetworkType networkType)
{
    // framework 設定
    {
        const int RootHeapSize = 10 * 1024 * 1024;
        PIA_ATTRIBUTE_ALIGN(8) static uint8_t s_memory[RootHeapSize];
        nn::pia::framework::InitializeFrameworkSetting setting;
        setting.piaBuffer = s_memory;
        setting.piaBufferSize = RootHeapSize;
        setting.networkType = networkType;
        setting.traceFlags = nn::pia::common::TraceFlagFramework |
                             nn::pia::common::TraceFlagSession |
                             nn::pia::common::TraceFlagSessionConnect |
                             nn::pia::common::TraceFlagSessionLeave |
                             nn::pia::common::TraceFlagSessionInit |
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
                             nn::pia::common::TraceFlagSessionJoint |
#endif
                             nn::pia::common::TraceFlagLocal |
                             nn::pia::common::TraceFlagInet |
                             nn::pia::common::TraceFlagLan |
                             nn::pia::common::TraceFlagPlugin;
        PIA_SAMPLE_PRINT("---------------Initialize start----------------\n");
        nn::pia::Result result = nn::pia::framework::Initialize(setting);
        if (result.IsFailure())
        {
            PIA_SAMPLE_ASSERT(false);
        }
    }
    // Network 設定
    {
        nn::pia::framework::StartupNetworkSetting setting;
        setting.isAutoInitializeNetworkInterface = true;
        setting.isAutoInitializeSocket = true;
#if NN_PIA_CTR
        const int32_t SocketSessionMax = 5;
        const int32_t SocketBufferSize = 64 * 1024;                                      // １つのソケットの送受信バッファの割り当てに使用するサイズ
        const uint32_t SocketBufferTotalSize = SocketBufferSize * SocketSessionMax;      // ソケットライブラリ用ワークメモリのサイズ
        PIA_ATTRIBUTE_ALIGN(4096) static uint8_t s_SocketBuffer[SocketBufferTotalSize];  // ソケットライブラリに渡すワークメモリ（4096Byteアライメントされたバッファが必要
        setting.memoryPool = s_SocketBuffer;
        setting.allocatorPoolSize = SocketBufferSize;
        setting.memoryPoolSize = sizeof(s_SocketBuffer);
#elif NN_PIA_NINTENDOSDK
        const uint32_t SocketBufferTotalSize = 6 * 1024 * 1024;                          // ソケットライブラリ用ワークメモリのサイズ(とりあえず 6MB)
        PIA_ATTRIBUTE_ALIGN(4096) static uint8_t s_SocketBuffer[SocketBufferTotalSize];  // ソケットライブラリに渡すワークメモリ（4096Byteアライメントされたバッファが必要）
        setting.memoryPool = s_SocketBuffer;
        setting.allocatorPoolSize = nn::socket::DefaultSocketAllocatorSize;
        setting.memoryPoolSize = sizeof(s_SocketBuffer);
#endif
        nn::pia::framework::Framework::GetInstance()->RegisterStartupNetworkSetting(setting);
    }
    // inet 設定
    {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
        const size_t MemoryBlockSizeForNexEmergency = 50 * 1024;
        const size_t TotalMemoryBlockSizeForNex = 1 * 1024 * 1024 + MemoryBlockSizeForNexEmergency;
        PIA_ATTRIBUTE_ALIGN(4096) static uint8_t s_NexMemory[TotalMemoryBlockSizeForNex];
        nn::pia::framework::InitializeInetSetting setting;
        setting.isAutoInitializeAndLogin = true;
        setting.nexMemory = s_NexMemory;          // NEX で使用するメモリ
        setting.emergencyMemorySize = MemoryBlockSizeForNexEmergency;  // ヒープ枯渇時に使用する緊急メモリのサイズ
        setting.totalMemorySize = TotalMemoryBlockSizeForNex;
        setting.gameId = NNS_PIA_LOGIN_GAME_ID;
#if NN_PIA_CTR
        setting.accessKey.Copy(NNS_PIA_LOGIN_ACCESS_KEY);
#elif NN_PIA_NINTENDOSDK
#if NN_PIA_NINTENDOSDK_WIN
        setting.hostName.Copy(NNS_PIA_DEBUGLOGIN_HOST_ADDRESS);
        setting.port = static_cast<uint16_t>(NNS_PIA_DEBUGLOGIN_HOST_PORT);
        setting.hostUri.Copy(NNS_PIA_DEBUGLOGIN_HOST_URL);
        nn::nex::StringStream name;
        name << "testuser_" << nn::nex::Platform::GetRandomNumber(10000);
        setting.userName.Copy(name.CStr());
        setting.password.Copy(NNS_PIA_DEBUGLOGIN_USER_PASSWORD);
#endif
        setting.accessKey.Copy(NNS_PIA_LOGIN_ACCESS_KEY);
        nns::pia::UserAccountInfo* accountInfo = nns::pia::Net_GetUserAccountInfo(0);
        PIA_SAMPLE_ASSERTMSG(accountInfo != NULL, "accountInfo is not found.");
        setting.userHandle = accountInfo->userHandle;
#endif
        nn::pia::framework::Framework::GetInstance()->RegisterInitializeInetSetting(setting);
#endif
    }
    // local 設定
    {
        nn::pia::framework::InitializeLocalSetting setting;
        nn::pia::framework::Framework::GetInstance()->RegisterInitializeLocalSetting(setting);
    }
    // lan 設定
    {
        nn::pia::framework::InitializeLanSetting setting;
        nn::pia::framework::Framework::GetInstance()->RegisterInitializeLanSetting(setting);
    }
    // transport 設定
    {
        nn::pia::framework::InitializeTransportSetting setting;
        nn::pia::framework::Framework::GetInstance()->RegisterInitializeTransportSetting(setting);
    }
    // JoinRandomSessionSetting 設定
    {
        switch (networkType)
        {
        case nn::pia::framework::NetworkType_Local:
            break;
        case nn::pia::framework::NetworkType_Inet:
        {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
            nn::pia::framework::NexJoinRandomSessionSetting setting;
            nn::pia::inet::NexCreateSessionSetting nexCreateSessionSetting;
            nexCreateSessionSetting.SetGameMode(MatchmakeGameMode);                // ゲームモード
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
            nexCreateSessionSetting.SetAttribute(MatchmakeAttributeIndexTeamType, s_CurrentTeamTypeIndex);
#endif
            nexCreateSessionSetting.SetMaxParticipantNum(MaxStationNum);  // 最大参加人数
            nexCreateSessionSetting.SetOpenSession(true);                 // 作成されてすぐに参加可能状態にします。
            nexCreateSessionSetting.SetSessionType(nn::pia::inet::SessionType_Anybody);

            // 検索条件の設定
            nn::pia::inet::NexSessionSearchCriteria nexSessionSearchCriteria;
            nexSessionSearchCriteria.SetGameMode(MatchmakeGameMode);                    // ゲームモードの値
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
            nexSessionSearchCriteria.SetAttribute(MatchmakeAttributeIndexTeamType, s_CurrentTeamTypeIndex);
#endif
            nexSessionSearchCriteria.SetMaxParticipantNum(MaxStationNum);      // 最大参加人数
            nexSessionSearchCriteria.SetResultRange(0, MaxBrowsedSessionNum);  // 検索結果取得数

            setting.nexJoinRandomSessionSetting.SetCreateSessionSetting(&nexCreateSessionSetting);
            setting.nexJoinRandomSessionSetting.SetSessionSearchCriteria(&nexSessionSearchCriteria, 1);

            nn::pia::framework::Framework::GetInstance()->RegisterJoinRandomSessionSetting(setting);

#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
            nn::pia::framework::NexJoinRandomSessionSetting jointSetting;
            nn::pia::inet::NexCreateSessionSetting nexCreateTeamSessionSetting;
            nexCreateTeamSessionSetting.SetGameMode(TeamMatchmakeGameMode);                // ゲームモード
            nexCreateTeamSessionSetting.SetMaxParticipantNum(MaxStationNum);  // 最大参加人数
            nexCreateTeamSessionSetting.SetOpenSession(true);                 // 作成されてすぐに参加可能状態にします。
            nexCreateTeamSessionSetting.SetSessionType(nn::pia::inet::SessionType_Anybody);
            nexCreateTeamSessionSetting.SetAttribute(MatchmakeAttributeIndexTeamType, s_CurrentTeamTypeIndex);

            // 検索条件の設定
            nn::pia::inet::NexSessionSearchCriteria nexTeamSessionSearchCriteria;
            nexTeamSessionSearchCriteria.SetGameMode(TeamMatchmakeGameMode);                    // ゲームモードの値
            nexTeamSessionSearchCriteria.SetMaxParticipantNum(MaxStationNum);      // 最大参加人数
            nexTeamSessionSearchCriteria.SetResultRange(0, MaxBrowsedSessionNum);  // 検索結果取得数
            uint32_t oppositeTeamList[SampleTeamType_Max - 1];
            int oppositeTeamListIndex = 0;
            for (int i = 0; i < (int)SampleTeamType_Max - 1; ++i)
            {
                if (i != s_CurrentTeamTypeIndex)
                {
                    oppositeTeamList[oppositeTeamListIndex] = i;
                    ++oppositeTeamListIndex;
                }
            }
            nexTeamSessionSearchCriteria.SetAttribute(MatchmakeAttributeIndexTeamType, oppositeTeamList, SampleTeamType_Max - 1);

            jointSetting.nexJoinRandomSessionSetting.SetCreateSessionSetting(&nexCreateTeamSessionSetting);
            jointSetting.nexJoinRandomSessionSetting.SetSessionSearchCriteria(&nexTeamSessionSearchCriteria, 1);

            nn::pia::framework::Framework::GetInstance()->RegisterJoinRandomJointSessionSetting(jointSetting);
            PIA_SAMPLE_PRINT("--------------- RegisterJoinRandomJointSessionSetting ----------------\n");
#endif
#endif
        }
        break;
        case nn::pia::framework::NetworkType_Lan:
        {
            nn::pia::framework::LanJoinRandomSessionSetting setting;
            nn::pia::lan::LanCreateSessionSetting lanCreateSessionSetting;
            lanCreateSessionSetting.SetGameMode(MatchmakeGameMode);                // ゲームモード
            lanCreateSessionSetting.SetMaxParticipantNum(MaxStationNum);  // 最大参加人数
            lanCreateSessionSetting.SetOpenSession(true);                 // 作成されてすぐに参加可能状態にします。

            // 検索条件の設定
            nn::pia::lan::LanSessionSearchCriteria lanSessionSearchCriteria;
            lanSessionSearchCriteria.SetGameMode(MatchmakeGameMode);                    // ゲームモードの値
            lanSessionSearchCriteria.SetMaxParticipantNum(MaxStationNum);      // 最大参加人数
            lanSessionSearchCriteria.SetResultRange(0, MaxBrowsedSessionNum);  // 検索結果取得数

            setting.lanJoinRandomSessionSetting.SetCreateSessionSetting(&lanCreateSessionSetting);
            setting.lanJoinRandomSessionSetting.SetSessionSearchCriteria(&lanSessionSearchCriteria, 1);

            nn::pia::framework::Framework::GetInstance()->RegisterJoinRandomSessionSetting(setting);
        }
        break;
        }
    }
    // InitializeSessionSetting 設定
    {
        nn::pia::framework::InitializeSessionSetting setting;
        setting.browsedSessionInfoListNum = MaxBrowsedSessionNum;
        nn::pia::framework::Framework::GetInstance()->RegisterInitializeSessionSetting(setting);
    }
    if (s_IsSyncMode)
    {
        // InitializeSyncSetting 設定
        {
            nn::pia::framework::InitializeSyncSetting setting;
            // 同期通信中の遅延変更機能を使用するかを設定します。
            // 使用する場合は、毎フレーム PiaPluginSync.RequestToChangeDelay を呼ぶ必要があります。
            setting.isChangeDelayEnabled = true;
            for (int i = 0; i < DataIdNum; ++i)
            {
                setting.dataUnitSize[i] = 32;
            }
            setting.isAutoInitialize = true;
            nn::pia::framework::Framework::GetInstance()->RegisterInitializeSyncSetting(setting);
        }
    }
    // SessionStartupSetting 設定
    {
        nn::pia::framework::StartupSessionSetting setting;

        nns::pia::Session_GetLocalPlayerInfo(&setting.playerInfo, 1);
        setting.signatureKeyData.Copy("CloneAndSync Key");
        setting.signatureKeyDataSize = sizeof("CloneAndSync Key");
#if NN_PIA_NINTENDOSDK
        setting.isAutoInitializeLdn = true;
#endif
        nn::pia::framework::Framework::GetInstance()->RegisterStartupSessionSetting(setting);
    }
    // InitializeCloneSetting 設定
    {
        nn::pia::framework::InitializeCloneSetting setting;
        setting.isAutoInitialize = true;
        nn::pia::framework::Framework::GetInstance()->RegisterInitializeCloneSetting(setting);
    }
}

int compairForSort(const void *a, const void *b)
{
    return *(int*)a < *(int*)b ? 1 : -1;
}

void SceneTitle(State* pState, nn::pia::framework::NetworkType* pNetworkType)
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = GetSampleFramework();

    bool isExit = false;
    while (!isExit)
    {
        pSampleFramework->BeginFrame();
        pSampleFramework->DrawString(10, 20, "state : State_Title");
#if NN_PIA_BROWSE_ONLY
            pSampleFramework->DrawString(10, 40, "A : [Inet]State_BrowseSession");
            pSampleFramework->DrawString(10, 50, "Y : [Lan]State_BrowseSession");
#else
            pSampleFramework->DrawString(10, 40, "A : [Inet]State_RandomSession");
            pSampleFramework->DrawString(10, 50, "Y : [Lan]State_RandomSession");
#endif
            pSampleFramework->DrawString(10, 60, "X : [Local]State_CreateOrJoinSession");
        if (s_IsSyncMode)
        {
            pSampleFramework->DrawString(10, 70, "LEFT or RIGHT : select Sync or Clone [Sync]");
        }
        else
        {
            pSampleFramework->DrawString(10, 70, "LEFT or RIGHT : select Sync or Clone [Clone]");
        }
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
        pSampleFramework->DrawString(10, 90, "L, R : Change Team (%s)", cTeamName[s_CurrentTeamTypeIndex]);
#endif
        pSampleFramework->DrawString(10, 100, "START : End");
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
            *pState = State_InitializeFramework;
            *pNetworkType = nn::pia::framework::NetworkType_Inet;
            isExit = true;
#endif
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            *pState = State_InitializeFramework;
            *pNetworkType = nn::pia::framework::NetworkType_Lan;
            isExit = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            *pState = State_InitializeFramework;
            *pNetworkType = nn::pia::framework::NetworkType_Local;
            isExit = true;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
        {
            s_IsSyncMode = false;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
        {
            s_IsSyncMode = true;
        }
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
        {
            s_CurrentTeamTypeIndex = (s_CurrentTeamTypeIndex + 1) % SampleTeamType_Max;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
        {
            s_CurrentTeamTypeIndex = (s_CurrentTeamTypeIndex + SampleTeamType_Max - 1) % SampleTeamType_Max;
        }
#endif
    pSampleFramework->EndFrame();
    }
}

void SceneInitializeFramework(State* pState, nn::pia::framework::NetworkType networkType)
{
    nn::pia::Result result;
    // アカウントライブラリの初期化、アカウント選択
#if NN_PIA_NINTENDOSDK
    nns::pia::NetworkType nnsNetworkType = nns::pia::NetworkType_Internet;
    switch (networkType)
    {
    case nn::pia::framework::NetworkType_Inet:
        nnsNetworkType = nns::pia::NetworkType_Internet;
        break;
    case nn::pia::framework::NetworkType_Lan:
        nnsNetworkType = nns::pia::NetworkType_Lan;
        break;
    case nn::pia::framework::NetworkType_Local:
        nnsNetworkType = nns::pia::NetworkType_Local;
        break;
    default:
        PIA_SAMPLE_ASSERT(false);
        break;
    }

    nns::pia::NetResult netResult = nns::pia::Net_InitializeAccount(nnsNetworkType);
    if (netResult != nns::pia::NetResult_Success)
    {
        *pState = State_FinalizeFramework;
        return;
    }
#endif

    // 各 setting の初期化
    InitializeSetting(networkType);

    // State_PiaStartedUp まで進める
    result = ChangeState(*pState, nn::pia::framework::Framework::State_SessionStartedUp);
    ResultCheck(result, pState);
    if (result.IsFailure())
    {
        return;
    }

    PIA_SAMPLE_PRINT("---------------Initialize end----------------\n");
    if (networkType == nn::pia::framework::NetworkType_Local)
    {
        *pState = State_BrowseSession;
    }
    else
    {
#if NN_PIA_BROWSE_ONLY
        *pState = State_BrowseSession;
#else
        *pState = State_RandomSession;
#endif
    }
}

void SceneRandomSession(State* pState)
{
    nn::pia::Result result;
    PIA_SAMPLE_PRINT("----------------RandomSession start---------------\n");
    // 通信の進行状態を State_SessionJoined に遷移すると自動で RandomSession が行われる
    result = ChangeState(*pState, nn::pia::framework::Framework::State_SessionJoined);
    ResultCheck(result, pState);
    if (result.IsFailure())
    {
        return;
    }
    if (!s_IsSyncMode)
    {
        // cloneProtocol の設定
        nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
        result = pCloneProtocol->RegisterSendClone(&s_SendClone, 0x0001);
        ResultCheck(result, pState);
        for (int i = 0; i < MaxStationNum; ++i)
        {
            result = pCloneProtocol->RegisterReceiveClone(&s_ReceiveClone[i], static_cast<nn::pia::StationIndex>(i), 0x0001);
            ResultCheck(result, pState);
        }
        result = pCloneProtocol->Start();
        ResultCheck(result, pState);
    }
    if (result.IsSuccess())
    {
        *pState = State_Session;
    }
    PIA_SAMPLE_PRINT("----------------RandomSession end---------------\n");
}

void SceneBrowseSession(State* pState, nn::pia::framework::NetworkType networkType)
{
    PIA_SAMPLE_PRINT("----------------BrowseSession start---------------\n");
    nn::pia::Result result;

    // まずはセッションを探索する
    switch (networkType)
    {
    case nn::pia::framework::NetworkType_Local:
#if NN_PIA_NINTENDOSDK
    {
        nn::pia::local::LdnSessionSearchCriteria criteria;
        {
            criteria.SetLocalCommunicationId(nns::pia::LocalCommunicationId);  // ローカル通信ID
            criteria.SetSceneId(LocalSceneId);                                 // 通信モード識別用 ID です
            criteria.SetMaxParticipantNum(MaxStationNum);                      // 最大参加人数
            criteria.SetResultRange(0, MaxBrowsedSessionNum);                  // 検索結果取得数
        }
        result = nn::pia::framework::Framework::GetInstance()->BrowseSessionAsync(&criteria);
    }
#elif NN_PIA_CTR
    {
        nn::pia::local::UdsSessionSearchCriteria criteria;
        {
            criteria.SetLocalCommunicationId(nn::pia::framework::Framework::GetInstance()->CreateLocalCommunicationId(LocalTitleId));  // ローカル通信ID
            criteria.SetSubId(LocalSubId);                           // 通信モード識別用 ID です
            criteria.SetMaxParticipantNum(MaxStationNum);            // 最大参加人数
            criteria.SetResultRange(0, MaxBrowsedSessionNum);        // 検索結果取得数
        }
        result = nn::pia::framework::Framework::GetInstance()->BrowseSessionAsync(&criteria);
    }
#endif
    break;
    case nn::pia::framework::NetworkType_Inet:
    {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
        nn::pia::inet::NexSessionSearchCriteria criteria;
        criteria.SetGameMode(MatchmakeGameMode);                    // ゲームモードの値
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
        criteria.SetAttribute(MatchmakeAttributeIndexTeamType, s_CurrentTeamTypeIndex);
#endif
        criteria.SetMaxParticipantNum(MaxStationNum);      // 最大参加人数
        criteria.SetResultRange(0, MaxBrowsedSessionNum);  // 検索結果取得数
        result = nn::pia::framework::Framework::GetInstance()->BrowseSessionAsync(&criteria);
#endif
    }
    break;
    case nn::pia::framework::NetworkType_Lan:
    {
        nn::pia::lan::LanSessionSearchCriteria criteria;
        criteria.SetGameMode(MatchmakeGameMode);                    // ゲームモードの値
        criteria.SetMaxParticipantNum(MaxStationNum);      // 最大参加人数
        criteria.SetResultRange(0, MaxBrowsedSessionNum);  // 検索結果取得数
        result = nn::pia::framework::Framework::GetInstance()->BrowseSessionAsync(&criteria);
    }
    break;
    }

    ResultCheck(result, pState);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->BrowseSessionAsync() failed. line: %d\n", __LINE__);
        return;
    }

    // 非同期処理 BrowseSessionAsync の終了を待つ
    WaitForAsyncProcess("BrowseSessionAsync", &nn::pia::framework::Framework::IsBrowseSessionCompleted, *pState);

    if (*pState == State_BrowseSession)
    {
        *pState = State_CreateOrJoinSession;
    }
    PIA_SAMPLE_PRINT("----------------BrowseSession end---------------\n");
}

#if NN_PIA_NINTENDOSDK || NN_PIA_CTR
void SceneCreateOrJoinSession(State* pState, nn::pia::framework::NetworkType networkType)
{
    PIA_SAMPLE_PRINT("----------------CreateOrJoinSession start---------------\n");
    nn::pia::Result result;
    nns::pia::sampleframework::SampleFramework* pSampleFramework = GetSampleFramework();

    bool isExit = false;
    int32_t targetSession = 0;

    while (!isExit)
    {
        result = nn::pia::framework::Framework::GetInstance()->GetBrowseSessionResult();
        ResultCheck(result, pState);
        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT(" nn::pia::framework::Framework::GetInstance()->GetBrowseSessionResult() failed. line: %d\n", __LINE__);
            break;
        }

        nn::pia::session::ISessionInfoList* pSessionInfoList = nn::pia::framework::Framework::GetInstance()->GetBrowsedSessionInfoList();
        nn::pia::session::ISessionInfoList::Iterator it = pSessionInfoList->Begin();

        if (result.IsSuccess() && pSessionInfoList != NULL)
        {
            pSampleFramework->BeginFrame();
            pSampleFramework->DrawString(10, 20, "state : State_CreateOrJoinSession");
            pSampleFramework->DrawString(10, 30, "[A] JoinSession");
            pSampleFramework->DrawString(10, 40, "[X] CreateSession");
            pSampleFramework->DrawString(10, 50, "[B] Back");
            pSampleFramework->EndFrame();

            for (int i = 0; i < MaxBrowsedSessionNum; ++i)
            {
                if (i == targetSession)
                {
                    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(CursorColor);
                }

                if (i < pSessionInfoList->GetSize())
                {
                    pSampleFramework->DrawString(10, 70 + i * 10, "  GID=%08X [%02d/%02d]", (*it)->GetSessionId(), (*it)->GetCurrentParticipantNum(), (*it)->GetMaxParticipantNum());
                }
                else
                {
                    pSampleFramework->DrawString(10, 70 + i * 10, "  -----");
                }
                nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
            }
        }
        else
        {
            return;
        }

        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            PIA_SAMPLE_PRINT("Back\n");
            *pState = State_FinalizeFramework;
            return;
        }

        if (pSessionInfoList->GetSize() > 0)
        {
            // 上下: セッションの選択
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                targetSession = (MaxBrowsedSessionNum + targetSession - 1) % MaxBrowsedSessionNum;
            }
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                targetSession = (targetSession + 1) % MaxBrowsedSessionNum;
            }

            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA) && targetSession < pSessionInfoList->GetSize())
            {
                PIA_SAMPLE_PRINT("Join session\n");

                nn::pia::session::ISessionInfoList::Iterator targetSessionInfoPtr = pSessionInfoList->Begin();
                for (int i = 0; i < pSessionInfoList->GetSize(); ++i)
                {
                    if (i == targetSession)
                    {
                        break;
                    }
                    ++targetSessionInfoPtr;
                }

                // セッションが見つかった場合は、見つけたセッションに参加する
                switch (networkType)
                {
                case nn::pia::framework::NetworkType_Local:
                {
#if NN_PIA_NINTENDOSDK
                    nn::pia::local::LdnJoinSessionSetting joinSessionSetting;
#elif NN_PIA_CTR
                    nn::pia::local::UdsJoinSessionSetting joinSessionSetting;
#endif
                    joinSessionSetting.SetWirelessCryptoKey(PassPhrase, PassPhraseSize);
                    joinSessionSetting.SetSessionInfoPtr(*(targetSessionInfoPtr));
                    joinSessionSetting.SetMaxParticipantNum(MaxStationNum);
                    result = nn::pia::framework::Framework::GetInstance()->JoinSessionAsync(&joinSessionSetting);
                }
                break;
                case nn::pia::framework::NetworkType_Inet:
                {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
                    nn::pia::inet::NexJoinSessionSetting joinSessionSetting;
                    joinSessionSetting.SetSessionInfoPtr(*(targetSessionInfoPtr));
                    joinSessionSetting.SetMaxParticipantNum(MaxStationNum);
                    result = nn::pia::framework::Framework::GetInstance()->JoinSessionAsync(&joinSessionSetting);
#endif
                }
                break;
                case nn::pia::framework::NetworkType_Lan:
                {
                    nn::pia::lan::LanJoinSessionSetting joinSessionSetting;
                    joinSessionSetting.SetSessionInfoPtr(*(targetSessionInfoPtr));
                    joinSessionSetting.SetMaxParticipantNum(MaxStationNum);
                    result = nn::pia::framework::Framework::GetInstance()->JoinSessionAsync(&joinSessionSetting);
                }
                break;
                }

                ResultCheck(result, pState);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT(" JoinSessionAsync failed.\n");
                    break;
                }

                WaitForAsyncProcess("JoinSessionAsync", &nn::pia::framework::Framework::IsJoinSessionCompleted, *pState);

                result = nn::pia::framework::Framework::GetInstance()->GetJoinSessionResult();
                ResultCheck(result, pState);
                if (result.IsSuccess())
                {
                    PIA_SAMPLE_PRINT("JoinSession is success.\n");
                    isExit = true;
                }
                else
                {
                    return;
                }
            }
        
        }

        if (!isExit && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
            PIA_SAMPLE_PRINT("Create session\n");
            // セッションが見つからなかった場合は、セッションを作成する
            switch (networkType)
            {
            case nn::pia::framework::NetworkType_Local:
            {
#if NN_PIA_NINTENDOSDK
                nn::pia::local::LdnCreateSessionSetting createSessionSetting;
                {
                    createSessionSetting.SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    createSessionSetting.SetSceneId(LocalSceneId);
                    createSessionSetting.SetWirelessCryptoKey(PassPhrase, PassPhraseSize);
                    createSessionSetting.SetMaxParticipantNum(MaxStationNum);
                }
#elif NN_PIA_CTR
                nn::pia::local::UdsCreateSessionSetting createSessionSetting;
                {
                    createSessionSetting.SetLocalCommunicationId(nn::pia::framework::Framework::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    createSessionSetting.SetSubId(LocalSubId);
                    createSessionSetting.SetWirelessCryptoKey(PassPhrase, PassPhraseSize);
                    createSessionSetting.SetMaxParticipantNum(MaxStationNum);
                }
#endif
                result = nn::pia::framework::Framework::GetInstance()->CreateSessionAsync(&createSessionSetting);
            }
            break;
            case nn::pia::framework::NetworkType_Inet:
            {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
                nn::pia::inet::NexCreateSessionSetting createSessionSetting;
                createSessionSetting.SetGameMode(MatchmakeGameMode);
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
                createSessionSetting.SetAttribute(MatchmakeAttributeIndexTeamType, s_CurrentTeamTypeIndex);
#endif
                createSessionSetting.SetMaxParticipantNum(MaxStationNum);
                createSessionSetting.SetOpenSession(true);
                result = nn::pia::framework::Framework::GetInstance()->CreateSessionAsync(&createSessionSetting);
#endif
            }
            break;
            case nn::pia::framework::NetworkType_Lan:
            {
                nn::pia::lan::LanCreateSessionSetting createSessionSetting;
                createSessionSetting.SetGameMode(MatchmakeGameMode);
                createSessionSetting.SetMaxParticipantNum(MaxStationNum);
                createSessionSetting.SetOpenSession(true);
                result = nn::pia::framework::Framework::GetInstance()->CreateSessionAsync(&createSessionSetting);
            }
            break;
            }

            ResultCheck(result, pState);
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->CreateSessionAsync() failed.\n");
                break;
            }

            // CreateSessionAsync の終了を待つ
            WaitForAsyncProcess("CreateSessionAsync", &nn::pia::framework::Framework::IsCreateSessionCompleted, *pState);
            PIA_SAMPLE_PRINT("CreateSession has been completed.\n");

            result = nn::pia::framework::Framework::GetInstance()->GetCreateSessionResult();
            ResultCheck(result, pState);
            if (result.IsSuccess())
            {
                isExit = true;
            }
            else
            {
                PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->GetCreateSessionResult() failed.\n");
                break;
            }
        }

        if (isExit && *pState == State_CreateOrJoinSession)
        {
            if (!s_IsSyncMode)
            {
                // cloneProtocol の設定
                nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
                PIA_SAMPLE_ASSERT(pCloneProtocol != NULL);
                result = pCloneProtocol->RegisterSendClone(&s_SendClone, 0x0001);
                ResultCheck(result, pState);
                for (int i = 0; i < MaxStationNum; ++i)
                {
                    result = pCloneProtocol->RegisterReceiveClone(&s_ReceiveClone[i], static_cast<nn::pia::StationIndex>(i), 0x0001);
                    ResultCheck(result, pState);
                }
                result = pCloneProtocol->Start();
                ResultCheck(result, pState);
            }
            *pState = State_Session;
        }
    }
    PIA_SAMPLE_PRINT("----------------CreateOrJoinSession end---------------\n");
}
#endif

void SceneSession(State* pState, nn::pia::framework::NetworkType networkType)
{
    nn::pia::Result result;
    nns::pia::sampleframework::SampleFramework* pSampleFramework = GetSampleFramework();

    // RequestSessionInfoAsync の処理中かどうか
    bool isAsyncProcessing = false;

    int32_t appData = 1;

    // Sync 用
    int32_t syncAllData[MaxStationNum][DataIdNum];
    int32_t syncSendData = 0;
    nn::pia::StationId stationIdList[MaxStationNum];
    uint32_t stationNum;
    uint32_t usingDataIdBitmap = 3;
    uint32_t changeDelay = Delay;
    bool isDelayChanged = false;
    if (s_IsSyncMode)
    {
        for (int i = 0; i < MaxStationNum; ++i)
        {
            stationIdList[i] = nn::pia::StationIdInvalid;
        }
        result = nn::pia::framework::Framework::GetInstance()->GetStationIdList(stationIdList, static_cast<uint32_t>(MaxStationNum), stationNum);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
        qsort(stationIdList, MaxStationNum, sizeof(nn::pia::StationId), compairForSort);
    }

    // Clone 用
    nn::pia::framework::Framework::CloneUpdateResult cloneUpdateResult;

    uint32_t menuMode = 1;

    int errorHandlingCase = 0;

    bool isExit = false;

    while (!isExit)
    {
        pSampleFramework->BeginFrame();

        bool isHost = nn::pia::session::Session::GetInstance()->IsHost();

        // セッション参加時の処理
        switch (networkType)
        {
        case nn::pia::framework::NetworkType_Inet:
            pSampleFramework->DrawString(10, 10, "[Inet] state : State_Session");
            break;
        case nn::pia::framework::NetworkType_Lan:
            pSampleFramework->DrawString(10, 10, "[Lan] state : State_Session");
            break;
        case nn::pia::framework::NetworkType_Local:
            pSampleFramework->DrawString(10, 10, "[Local] state : State_Session");
            break;
        }
        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
            menuMode = 1 - menuMode;
        }
        pSampleFramework->DrawString(10, 40, "A: Switch menu [%d]", menuMode);
        switch (menuMode)
        {
        case 0:
        {
            pSampleFramework->DrawString(10, 60, "B: Leave session");
            if (isHost)
            {
                pSampleFramework->DrawString(10, 70, "L: UpdateSessionSetting");
            }
            pSampleFramework->DrawString(10, 80, "R: RequestSessionInfo");
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
            if (networkType == nn::pia::framework::NetworkType_Inet && isHost)
            {
                pSampleFramework->DrawString(10, 90, "Start: JoinJointSession");
            }
#endif
            if (isHost)
            {
                if ((nn::pia::session::Session::GetInstance()->GetSessionOpenStatus() == nn::pia::session::Session::SessionOpenStatus_Open))
                {
                    pSampleFramework->DrawString(10, 100, "X: session is open.");
                }
                else
                {
                    pSampleFramework->DrawString(10, 100, "X: session is close.");
                }
            }
#if NN_PIA_CTR
            pSampleFramework->DrawString(10, 110, "errorHandlingCase : %d", errorHandlingCase);
            pSampleFramework->DrawString(10, 120, "  (right, left, Y)");
#else
            pSampleFramework->DrawString(10, 110, "errorHandlingCase(right, left, Y) : %d", errorHandlingCase);
#endif
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
            {
                --errorHandlingCase;
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                ++errorHandlingCase;
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
            {
                pSampleFramework->EndFrame();
                pSampleFramework->DrawString(10, 60, "State_Leave...");
                *pState = State_Leave;
                return;
            }
            else if (isHost && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
            {
                ++appData;
                result = nn::pia::ResultNotImplemented();
                switch (networkType)
                {
                case nn::pia::framework::NetworkType_Local:
                {
#if NN_PIA_NINTENDOSDK
                    nn::pia::local::LdnUpdateSessionSetting updateSessionSetting;
                    updateSessionSetting.SetApplicationData(&appData, sizeof(appData));
                    result = nn::pia::framework::Framework::GetInstance()->UpdateSessionSettingAsync(&updateSessionSetting);
#elif NN_PIA_CTR
                    nn::pia::local::UdsUpdateSessionSetting updateSessionSetting;
                    updateSessionSetting.SetApplicationData(&appData, sizeof(appData));
                    result = nn::pia::framework::Framework::GetInstance()->UpdateSessionSettingAsync(&updateSessionSetting);
#endif
                }
                break;
                case nn::pia::framework::NetworkType_Inet:
                {
#if NN_PIA_FRAMEWORK_ENABLE_NEX
                    nn::pia::inet::NexUpdateSessionSetting updateSessionSetting;
                    updateSessionSetting.SetApplicationData(&appData, sizeof(appData));
                    result = nn::pia::framework::Framework::GetInstance()->UpdateSessionSettingAsync(&updateSessionSetting);
#endif
                }
                break;
                case nn::pia::framework::NetworkType_Lan:
                {
                    nn::pia::lan::LanUpdateSessionSetting updateSessionSetting;
                    updateSessionSetting.SetApplicationData(&appData, sizeof(appData));
                    result = nn::pia::framework::Framework::GetInstance()->UpdateSessionSettingAsync(&updateSessionSetting);
                }
                break;
                }
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->UpdateSessionSettingAsync() failed.\n");
                }
            }
            else if (!isAsyncProcessing && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
            {
                result = nn::pia::framework::Framework::GetInstance()->RequestSessionInfoAsync();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->RequestSessionInfoAsync() failed.\n");
                }
                else
                {
                    isAsyncProcessing = true;
                }
            }
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                pSampleFramework->EndFrame();

                // Yボタンで HandleErrorAsync を用いた任意のエラーに対するエラーハンドリングをテストします
                switch (errorHandlingCase)
                {
                case 0:
                    // HandlingType_Cleanup
                    ResultCheck(nn::pia::ResultCancelled(), pState);
                    break;
                case 1:
                    // HandlingType_LogoutWithLeave
                    ResultCheck(nn::pia::ResultGameServerMaintenance(), pState);
                    break;
                case 2:
                    // HandlingType_CleanupWithLeave
                    ResultCheck(nn::pia::ResultSessionConnectionIsLost(), pState);
                    break;
                case 3:
                    // HandlingType_ShutdownNetwork
                    ResultCheck(nn::pia::ResultNetworkConnectionIsLost(), pState);
                    break;
                case 4:
                    // HandlingType_Finalize
                    ResultCheck(nn::pia::ResultNexInternalError(), pState);
                    break;
                case 5:
                    // success -> State_LoggedIn
                    result = ChangeState(*pState, nn::pia::framework::Framework::State_LoggedIn);
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_ASSERT(false);
                    }
                    *pState = State_Leave;
                    break;
                case 6:
                    // HandlingType_LogoutWithLeave -> State_LoggedIn
                    ResultCheck(nn::pia::ResultGameServerMaintenance(), pState, nn::pia::framework::Framework::State_LoggedIn);
                    break;
                case 7:
                    // HandlingType_Cleanup -> State_LoggedIn
                    ResultCheck(nn::pia::ResultCancelled(), pState, nn::pia::framework::Framework::State_LoggedIn);
                    break;
                case 8:
#if NN_PIA_CTR
                    nn::math::RandomGenerator random;
                    random.SetSeed(pSampleFramework->GetFrameCount());

                    switch (random.Generate(5))
                    {
                    case 0:
                        result = nn::pia::ResultCancelled();
                        break;
                    case 1:
                        result = nn::pia::ResultGameServerMaintenance();
                        break;
                    case 2:
                        result = nn::pia::ResultSessionConnectionIsLost();
                        break;
                    case 3:
                        result = nn::pia::ResultNetworkConnectionIsLost();
                        break;
                    case 4:
                        result = nn::pia::ResultNexInternalError();
                        break;
                    default:
                        result = nn::pia::ResultSuccess();
                        break;
                    }
                    nn::pia::framework::Framework::State transitionState = static_cast<nn::pia::framework::Framework::State>(random.Generate(8));
                    PIA_SAMPLE_PRINT("result : %d, state : %d -> %d", result.GetHandlingType(), nn::pia::framework::Framework::GetInstance()->GetState(), transitionState);
                    ResultCheck(result, pState, transitionState);
#endif
                    break;
                }
                return;
            }
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
            else if (networkType == nn::pia::framework::NetworkType_Inet && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
            {
                if (isHost)
                {
                    pSampleFramework->DrawString(10, 90, "State_JoinRandomJointSession...");
                    pSampleFramework->EndFrame();
                    *pState = State_JoinRandomJointSession;
                    return;
                }
            }
#endif
            else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonX))
            {
                // セッションの開閉状態を切り替える
                nn::pia::session::Session::SessionOpenStatus sessionOpenStatus = nn::pia::session::Session::GetInstance()->GetSessionOpenStatus();
                switch (sessionOpenStatus)
                {
                case nn::pia::session::Session::SessionOpenStatus_Close:
                    result = nn::pia::framework::Framework::GetInstance()->OpenSessionAsync();
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("OpenSessionAsync() failed.");
                        PIA_SAMPLE_ASSERT(false);
                    }
                    break;
                case nn::pia::session::Session::SessionOpenStatus_Open:
                    result = nn::pia::framework::Framework::GetInstance()->CloseSessionAsync();
                    if (result.IsFailure())
                    {
                        PIA_SAMPLE_PRINT("CloseSessionAsync() failed.");
                        PIA_SAMPLE_ASSERT(false);
                    }
                    break;
                default:
                    // クライアントの場合ここに来る
                    break;
                }
            }
            break;
        }
        case 1:
        {
            if (s_IsSyncMode)
            {
                pSampleFramework->DrawString(10, 20, "syncState : %d", nn::pia::framework::Framework::GetInstance()->GetSyncState());
                if (nn::pia::framework::Framework::GetInstance()->GetSyncState() == nn::pia::sync::SyncProtocol::State_NotSynchronized)
                {
                    pSampleFramework->DrawString(10, 70, "B: Sync Start!");
#if NN_PIA_CTR
                    pSampleFramework->DrawString(10, 90, "L/R: usingDataIdBitmap");
                    pSampleFramework->DrawString(10, 100,"      = %d", usingDataIdBitmap);
#else
                    pSampleFramework->DrawString(10, 90, "L/R : usingDataIdBitmap(%d)", usingDataIdBitmap);
#endif

                    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
                    {
                        nn::pia::framework::Framework::GetInstance()->SyncStart(usingDataIdBitmap, Delay, SendPeriod);
                    }
                    else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonL))
                    {
                        --usingDataIdBitmap;
                    }
                    else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
                    {
                        ++usingDataIdBitmap;
                    }
                }
                else if (nn::pia::framework::Framework::GetInstance()->GetSyncState() == nn::pia::sync::SyncProtocol::State_Synchronized)
                {
                    pSampleFramework->DrawString(10, 70, "Left/Right: Change delay");
                    pSampleFramework->DrawString(10, 80, "B: End");
                    pSampleFramework->DrawString(10, 90, "Y: EndAlone");

                    if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
                    {
                        nn::pia::framework::Framework::GetInstance()->SyncEnd();
                    }
                    else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
                    {
                        nn::pia::framework::Framework::GetInstance()->SyncEndAlone();
                    }
                }

                if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
                {
                    ++syncSendData;
                }
                else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
                {
                    --syncSendData;
                }
                else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
                {
                    changeDelay -= changeDelay > 1 ? 1 : 0;
                    PIA_SAMPLE_PRINT("changeDelay %d\n", changeDelay);
                    isDelayChanged = true;
                }
                else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
                {
                    changeDelay += (changeDelay + 1) <= nn::pia::framework::Framework::GetInstance()->GetMaxDelay() ? 1 : 0;
                    PIA_SAMPLE_PRINT("changeDelay %d\n", changeDelay);
                    isDelayChanged = true;
                }

            }
            else
            {
                if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
                {
                    s_ElementForSend.SetValue(s_ElementForSend.GetValue() + 1);
                }
                else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
                {
                    s_ElementForSend.SetValue(s_ElementForSend.GetValue() - 1);
                }
            }
            pSampleFramework->DrawString(10, 60, "Up/Down : change data");
            break;
        }
        default:
            break;
        }
#if NN_PIA_CTR
        pSampleFramework->SetFontSize(8);
#endif

        result = nn::pia::framework::Framework::GetInstance()->Update();
        ResultCheck(result, pState);
        if (result.IsFailure())
        {
            pSampleFramework->EndFrame();
            break;
        }

#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
        // ジョイントセッション使用時はクライアントは随時チェック
        if (networkType == nn::pia::framework::NetworkType_Inet && !isHost)
        {
            if (nn::pia::framework::Framework::GetInstance()->IsSessionMigrating())
            {
                pSampleFramework->EndFrame();
                if (!s_IsSyncMode)
                {
                    // JointSession 移行前に cloneProtocol の設定を解除
                    nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
                    PIA_SAMPLE_ASSERT(pCloneProtocol != NULL);
                    if (pCloneProtocol != NULL)
                    {
                        result = pCloneProtocol->UnregisterSendClone(&s_SendClone);
                        ResultCheck(result, pState);
                        for (int i = 0; i < MaxStationNum; ++i)
                        {
                            result = pCloneProtocol->UnregisterReceiveClone(&s_ReceiveClone[i]);
                            ResultCheck(result, pState);
                        }
                        result = pCloneProtocol->Stop();
                        ResultCheck(result, pState);
                    }
                }

                *pState = State_SessionMigration;
                return;
            }
        }
#endif

        if (s_IsSyncMode)
        {
            // 1 フレーム進めます
            result = nn::pia::framework::Framework::GetInstance()->SyncStep();
            if (result.IsFailure())
            {
                if (result == nn::pia::ResultDataIsNotArrivedYet())
                {
                    // まだ同期データが届いていないのでゲームフレームをドロップさせます
                    // データを設定してからフレームが進んでいないので、ここで設定する必要はありません。
                    pSampleFramework->EndFrame();
                    continue;
                }
                else if (result == nn::pia::ResultDataIsNotSet())
                {
                    // 既に同期データを設定してあるので、この Result にはなりません。
                    PIA_SAMPLE_ASSERT(false);
                }
                else if (result == nn::pia::ResultInvalidState())
                {
                    // 通信中なので、この Result にはなりません。
                    PIA_SAMPLE_ASSERT(false);
                }
                else if (result == nn::pia::ResultTemporaryUnavailable())
                {
                    // 単独同期終了処理中には、この Result が返りますが、ハンドリングの必要はありません。
                }
                else
                {
                    // その他の Result が返ることはありません。
                    PIA_SAMPLE_ASSERT(false);
                }
            }

            for (int i = 0; i < MaxStationNum; ++i)
            {
                stationIdList[i] = nn::pia::StationIdInvalid;
            }
            result = nn::pia::framework::Framework::GetInstance()->GetStationIdList(stationIdList, static_cast<uint32_t>(MaxStationNum), stationNum);
            PIA_SAMPLE_ASSERT(result.IsSuccess());
            qsort(stationIdList, MaxStationNum, sizeof(nn::pia::StationId), compairForSort);

            for (int i = 0; i < MaxStationNum; ++i)
            {
                // 初期化時に SyncProtocol::Setting::dataUnitSize を設定して有効にしたデータ ID について確認します
                for (int j = 0; j < DataIdNum; ++j)
                {
                    result = nn::pia::framework::Framework::GetInstance()->GetSyncData(stationIdList[i], j, &syncAllData[i][j]);
                    if (result.IsFailure())
                    {
                        syncAllData[i][j] = 0;
                    }
                }
            }
            // 送信する同期データを用意して SyncProtocol に設定します。
            // 初期化時に SyncProtocol::Setting::dataUnitSize を設定して有効にしたデータ ID について確認します
            for (int i = 0; i < DataIdNum; ++i)
            {
                result = nn::pia::framework::Framework::GetInstance()->SetSyncData(i, &syncSendData);
            }

            // 入力遅延変更依頼（送信する同期データの設定後に、毎フレーム呼ぶ）
            if (isDelayChanged)
            {
                nn::pia::framework::Framework::GetInstance()->RequestToChangeDelay(changeDelay);
                isDelayChanged = false;
            }
            else
            {
                nn::pia::framework::Framework::GetInstance()->RequestToChangeDelay(0);
                isDelayChanged = false;
            }

            // 同期データの送信処理をすぐに行うために、もう一度 nn::pia::common::Scheduler::Dispatch() を呼びます。
            nn::pia::common::Scheduler::GetInstance()->Dispatch();
        }
        else
        {
            cloneUpdateResult = nn::pia::framework::Framework::GetInstance()->CloneUpdate();
            if (cloneUpdateResult.cloneProtocolState == nn::pia::clone::CloneProtocol::State_Inactive)
            {
                pSampleFramework->EndFrame();
                PIA_SAMPLE_PRINT("warning : cloneProtocolState == State_Inactive");
                *pState = State_Leave;
                return;
            }
        }
   
        if (isHost)
        {
            pSampleFramework->DrawString(10, 130, "I'm host.");
        }
        else
        {
            pSampleFramework->DrawString(10, 130, "I'm client.");
        }
        nn::pia::session::Session* session = nn::pia::session::Session::GetInstance();
        pSampleFramework->DrawString(10, 150, "sta. num : %u(MyID : %d)", session->GetStationNum(), ConvertStationIdToStationIndex(session->GetLocalStationId()));
        pSampleFramework->DrawString(10, 160, "hostId : %u", session->GetHostStationId());
        pSampleFramework->DrawString(10, 170, "localId : %u", session->GetLocalStationId());
#if NN_PIA_FRAMEWORK_ENABLE_NEX
        if (networkType == nn::pia::framework::NetworkType_Inet)
        {
            pSampleFramework->DrawString(10, 180, "GetLocalURLs : %d", nn::nex::Network::GetInstance()->GetLocalURLs().size());
        }
#endif
        pSampleFramework->DrawString(10, 190, "ApplicationData : %d", appData);
        if (s_IsSyncMode)
        {
            pSampleFramework->DrawString(10, 200, "delay : %d", nn::pia::framework::Framework::GetInstance()->GetDelay());
        }
        else
        {
            pSampleFramework->DrawString(10, 200, "cloneProtocolState : %d", cloneUpdateResult.cloneProtocolState);
            pSampleFramework->DrawString(10, 210, "errorType : %d", cloneUpdateResult.errorType);
            pSampleFramework->DrawString(10, 220, "clock : %u", cloneUpdateResult.clock);
        }

        if (s_IsSyncMode)
        {
            pSampleFramework->DrawString(195, 20, "sendData : %d", syncSendData);
            for (int i = 0; i < MaxStationNum; ++i)
            {
                nn::pia::common::StringStream ss;
                for (int j = 0; j < DataIdNum; ++j)
                {
                    ss << syncAllData[i][j] << ",";
                }
                pSampleFramework->DrawString(195, 40 + (i * 10), "recv[%2d]: %s", i, ss.CStr());
            }
            pSampleFramework->DrawString(195, 30, "---------------------");
        }
        else
        {
            pSampleFramework->DrawString(250, 20, "sendData : %d", s_ElementForSend.GetValue());
            pSampleFramework->DrawString(250, 30, "---------------------");
            for (int i = 0; i < MaxStationNum; ++i)
            {
                pSampleFramework->DrawString(250, 40 + (i * 10), "recvData[%d] : %d", i, s_ElementForReceive[i].GetValue());
            }
        }      

        if (isAsyncProcessing && nn::pia::framework::Framework::GetInstance()->IsRequestSessionInfoCompleted())
        {
            const nn::pia::session::ISessionInfo* pSessionInfo = nn::pia::framework::Framework::GetInstance()->GetSessionInfo();
            result = pSessionInfo->GetApplicationData(&appData, sizeof(appData));
            PIA_SAMPLE_ASSERT(result.IsSuccess());
            isAsyncProcessing = false;
        }

        result = nn::pia::framework::Framework::GetInstance()->Update();
        pSampleFramework->EndFrame();

        ResultCheck(result, pState);
        if (result.IsFailure())
        {
            break;
        }
    }
}

void SceneLeave(State* pState)
{
    nn::pia::Result result;

    // セッションから離脱します
    PIA_SAMPLE_PRINT("---------------LeaveSession start----------------\n");
    result = ChangeState(*pState, nn::pia::framework::Framework::State_SessionLeft);
    ResultCheck(result, pState);
    PIA_SAMPLE_PRINT("---------------LeaveSession end----------------\n");

    *pState = State_FinalizeFramework;
}

void SceneFinalizeFramework(State* pState)
{
    PIA_SAMPLE_PRINT("---------------Finalize start----------------\n");
    if (!s_IsSyncMode)
    {
        nn::pia::Result result;
        // cloneProtocol の設定
        nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
        if (pCloneProtocol != NULL)
        {
            result = pCloneProtocol->UnregisterSendClone(&s_SendClone);
            ResultCheck(result, pState);
            for (int i = 0; i < MaxStationNum; ++i)
            {
                result = pCloneProtocol->UnregisterReceiveClone(&s_ReceiveClone[i]);
                ResultCheck(result, pState);
            }
            result = pCloneProtocol->Stop();
            ResultCheck(result, pState);
        }
        else
        {
            // エラーハンドリングで既に CloneProtocol が破棄されている場合
            PIA_SAMPLE_PRINT("---------------CloneProtocol is NULL. ----------------\n");
        }
    }

    nn::pia::framework::Finalize();
#if NN_PIA_NINTENDOSDK
    nns::pia::Net_FinalizeAccount();
#endif
    PIA_SAMPLE_PRINT("---------------Finalize end----------------\n");
    *pState = State_Title;
}

#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
void SceneJoinRandomJointSession(State* pState)
{
    nn::pia::Result result;
    if (!s_IsSyncMode)
    {
        // JointSession 移行前に cloneProtocol の設定を解除
        nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
        PIA_SAMPLE_ASSERT(pCloneProtocol != NULL);
        if (pCloneProtocol != NULL)
        {
            result = pCloneProtocol->UnregisterSendClone(&s_SendClone);
            ResultCheck(result, pState);
            for (int i = 0; i < MaxStationNum; ++i)
            {
                result = pCloneProtocol->UnregisterReceiveClone(&s_ReceiveClone[i]);
                ResultCheck(result, pState);
            }
            result = pCloneProtocol->Stop();
            ResultCheck(result, pState);
        }
    }
    PIA_SAMPLE_PRINT("%d\n", __LINE__);

    PIA_SAMPLE_PRINT("---------------JoinRandomJointSession start----------------\n");
    result = ChangeState(*pState, nn::pia::framework::Framework::State_JointSessionJoined);
    ResultCheck(result, pState);

    // 非同期処理の成功時に State_JointSession へ遷移
    if (result.IsSuccess())
    {
        if (!s_IsSyncMode)
        {
            nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
            PIA_SAMPLE_ASSERT(pCloneProtocol != NULL);

            // JointSession 用に cloneProtocol の設定
            result = pCloneProtocol->RegisterSendClone(&s_SendClone, 0x0001);
            ResultCheck(result, pState);
            for (int i = 0; i < MaxStationNum; ++i)
            {
                result = pCloneProtocol->RegisterReceiveClone(&s_ReceiveClone[i], static_cast<nn::pia::StationIndex>(i), 0x0001);
                ResultCheck(result, pState);
            }
            result = pCloneProtocol->Start();
            ResultCheck(result, pState);
        }

        *pState = State_JointSession;
    }
}

void SceneSessionMigration(State* pState)
{
    nn::pia::Result result;
    result = nn::pia::framework::Framework::GetInstance()->Update();
    ResultCheck(result, pState);

    if (!nn::pia::framework::Framework::GetInstance()->IsSessionMigrating())
    {
        if (nn::pia::framework::Framework::GetInstance()->GetState() == nn::pia::framework::Framework::State_JointSessionJoined)
        {
            if (!s_IsSyncMode)
            {
                nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
                PIA_SAMPLE_ASSERT(pCloneProtocol != NULL);
                result = pCloneProtocol->RegisterSendClone(&s_SendClone, 0x0001);
                ResultCheck(result, pState);
                for (int i = 0; i < MaxStationNum; ++i)
                {
                    result = pCloneProtocol->RegisterReceiveClone(&s_ReceiveClone[i], static_cast<nn::pia::StationIndex>(i), 0x0001);
                    ResultCheck(result, pState);
                }
                result = pCloneProtocol->Start();
                ResultCheck(result, pState);
            }
            *pState = State_JointSession;
        }
        else if (nn::pia::framework::Framework::GetInstance()->GetState() == nn::pia::framework::Framework::State_SessionJoined)
        {
            if (!s_IsSyncMode)
            {
                // cloneProtocol の設定
                nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
                PIA_SAMPLE_ASSERT(pCloneProtocol != NULL);
                result = pCloneProtocol->RegisterSendClone(&s_SendClone, 0x0001);
                ResultCheck(result, pState);
                for (int i = 0; i < MaxStationNum; ++i)
                {
                    result = pCloneProtocol->RegisterReceiveClone(&s_ReceiveClone[i], static_cast<nn::pia::StationIndex>(i), 0x0001);
                    ResultCheck(result, pState);
                }
                result = pCloneProtocol->Start();
                ResultCheck(result, pState);
            }
            *pState = State_Session;
        }
        PIA_SAMPLE_PRINT("---------------Wait SessionMigration end----------------\n");
    }
}

void SceneJointSession(State* pState, nn::pia::framework::NetworkType networkType)
{
    PIA_SAMPLE_PRINT("---------------SceneJointSession begin----------------\n");
    nn::pia::Result result;
    nns::pia::sampleframework::SampleFramework* pSampleFramework = GetSampleFramework();

    // Sync 用
    int32_t syncAllData[MaxStationNum][DataIdNum];
    int32_t syncSendData = 0;
    nn::pia::StationId stationIdList[MaxStationNum];
    uint32_t stationNum;
    uint32_t usingDataIdBitmap = 3;
    if (s_IsSyncMode)
    {
        for (int i = 0; i < MaxStationNum; ++i)
        {
            stationIdList[i] = nn::pia::StationIdInvalid;
        }
        result = nn::pia::framework::Framework::GetInstance()->GetStationIdList(stationIdList, static_cast<uint32_t>(MaxStationNum), stationNum);
        PIA_SAMPLE_ASSERT(result.IsSuccess());
        qsort(stationIdList, MaxStationNum, sizeof(nn::pia::StationId), compairForSort);
    }

    // Clone 用
    nn::pia::framework::Framework::CloneUpdateResult cloneUpdateResult;

    int errorHandlingCase = 0;

    bool isExit = false;

    while (!isExit)
    {
        pSampleFramework->BeginFrame();

        pSampleFramework->DrawString(10, 20, "state : State_JointSession");
        if (s_IsSyncMode)
        {
            pSampleFramework->DrawString(10, 30, "syncState : %d", nn::pia::framework::Framework::GetInstance()->GetSyncState());
            if (nn::pia::framework::Framework::GetInstance()->GetSyncState() == nn::pia::sync::SyncProtocol::State_NotSynchronized)
            {
                pSampleFramework->DrawString(10, 40, "A: Sync Start!");
                if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonA))
                {
                    nn::pia::framework::Framework::GetInstance()->SyncStart(usingDataIdBitmap, Delay, SendPeriod);
                }
            }
        }
        pSampleFramework->DrawString(10, 50, "B: LeaveJointSession");
        if (nn::pia::session::Session::GetInstance()->IsJointSessionHost())
        {
#if NN_PIA_CTR
            pSampleFramework->DrawString(10, 60, "Start: Destroy");
            pSampleFramework->DrawString(10, 70, "       JointSession");
#else
            pSampleFramework->DrawString(10, 60, "Start: DestroyJointSession");
#endif
        }
        pSampleFramework->DrawString(10, 80, "up/down : change data");

        result = nn::pia::framework::Framework::GetInstance()->Update();
        if (result.IsFailure())
        {
            pSampleFramework->EndFrame();
            ResultCheck(result, pState);
            return;
        }

        // ジョイントセッション使用時はクライアントは随時チェック
        if (!nn::pia::session::Session::GetInstance()->IsJointSessionHost())
        {
            if (nn::pia::framework::Framework::GetInstance()->IsSessionMigrating())
            {
                pSampleFramework->EndFrame();
                if (!s_IsSyncMode)
                {
                    // JointSession 用 cloneProtocol の設定を解除
                    nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
                    PIA_SAMPLE_ASSERT(pCloneProtocol != NULL);
                    if (pCloneProtocol != NULL)
                    {
                        ResultCheck(result, pState);
                        result = pCloneProtocol->UnregisterSendClone(&s_SendClone);
                        ResultCheck(result, pState);
                        for (int i = 0; i < MaxStationNum; ++i)
                        {
                            result = pCloneProtocol->UnregisterReceiveClone(&s_ReceiveClone[i]);
                            ResultCheck(result, pState);
                        }
                        result = pCloneProtocol->Stop();
                        ResultCheck(result, pState);
                    }
                }
                *pState = State_SessionMigration;
                return;
            }
        }

        if (s_IsSyncMode)
        {
            // 1 フレーム進めます
            result = nn::pia::framework::Framework::GetInstance()->SyncStep();
            if (result.IsFailure())
            {
                if (result == nn::pia::ResultDataIsNotArrivedYet())
                {
                    // まだ同期データが届いていないのでゲームフレームをドロップさせます
                    // データを設定してからフレームが進んでいないので、ここで設定する必要はありません。
                    pSampleFramework->EndFrame();
                    continue;
                }
                else if (result == nn::pia::ResultDataIsNotSet())
                {
                    // 既に同期データを設定してあるので、この Result にはなりません。
                    PIA_SAMPLE_ASSERT(false);
                }
                else if (result == nn::pia::ResultInvalidState())
                {
                    // 通信中なので、この Result にはなりません。
                    PIA_SAMPLE_ASSERT(false);
                }
                else if (result == nn::pia::ResultTemporaryUnavailable())
                {
                    // 単独同期終了処理中には、この Result が返りますが、ハンドリングの必要はありません。
                }
                else
                {
                    // その他の Result が返ることはありません。
                    PIA_SAMPLE_ASSERT(false);
                }
            }

            for (int i = 0; i < MaxStationNum; ++i)
            {
                stationIdList[i] = nn::pia::StationIdInvalid;
            }
            result = nn::pia::framework::Framework::GetInstance()->GetStationIdList(stationIdList, static_cast<uint32_t>(MaxStationNum), stationNum);
            PIA_SAMPLE_ASSERT(result.IsSuccess());
            qsort(stationIdList, MaxStationNum, sizeof(nn::pia::StationId), compairForSort);

            for (int i = 0; i < MaxStationNum; ++i)
            {
                // 初期化時に SyncProtocol::Setting::dataUnitSize を設定して有効にしたデータ ID について確認します
                for (int j = 0; j < DataIdNum; ++j)
                {
                    result = nn::pia::framework::Framework::GetInstance()->GetSyncData(stationIdList[i], j, &syncAllData[i][j]);
                    if (result.IsFailure())
                    {
                        syncAllData[i][j] = 0;
                    }
                }
            }
            // 送信する同期データを用意して SyncProtocol に設定します。
            // 初期化時に SyncProtocol::Setting::dataUnitSize を設定して有効にしたデータ ID について確認します
            for (int i = 0; i < DataIdNum; ++i)
            {
                result = nn::pia::framework::Framework::GetInstance()->SetSyncData(i, &syncSendData);
            }

            // 同期データの送信処理をすぐに行うために、もう一度 nn::pia::common::Scheduler::Dispatch() を呼びます。
            nn::pia::common::Scheduler::GetInstance()->Dispatch();
        }
        else
        {
            cloneUpdateResult = nn::pia::framework::Framework::GetInstance()->CloneUpdate();
            if (cloneUpdateResult.cloneProtocolState == nn::pia::clone::CloneProtocol::State_Inactive)
            {
                pSampleFramework->EndFrame();
                PIA_SAMPLE_PRINT("warning : cloneProtocolState == State_Inactive");
                *pState = State_Leave;
                return;
            }

        }
        {
            if (nn::pia::session::Session::GetInstance()->IsJointSessionHost())
            {
                pSampleFramework->DrawString(10, 120, "I'm joint session host.");
            }
            else
            {
                pSampleFramework->DrawString(10, 120, "I'm client.");
            }
            nn::pia::session::Session* session = nn::pia::session::Session::GetInstance();
            pSampleFramework->DrawString(10, 130, "station num : %u(MyID : %d)", session->GetStationNum(), ConvertStationIdToStationIndex(session->GetLocalStationId()));
            pSampleFramework->DrawString(10, 140, "JointSessionHostId : %u", session->GetJointSessionHostStationId());
            pSampleFramework->DrawString(10, 150, "sessionId: %u, localId : %u", session->GetJointSessionId(), session->GetLocalStationId());
#if NN_PIA_ENABLE_NEX
            if (networkType == nn::pia::framework::NetworkType_Inet)
            {
                pSampleFramework->DrawString(10, 160, "GetLocalURLs : %d", nn::nex::Network::GetInstance()->GetLocalURLs().size());
            }
#endif

            if (!s_IsSyncMode)
            {
                pSampleFramework->DrawString(10, 170, "cloneProtocolState : %d", cloneUpdateResult.cloneProtocolState);
                pSampleFramework->DrawString(10, 180, "errorType : %d", cloneUpdateResult.errorType);
                pSampleFramework->DrawString(10, 190, "clock : %u", cloneUpdateResult.clock);
            }

            if (nn::pia::session::Session::GetInstance()->IsHost())
            {
                if ((nn::pia::session::Session::GetInstance()->GetSessionOpenStatus() == nn::pia::session::Session::SessionOpenStatus_Open))
                {
                    pSampleFramework->DrawString(10, 210, "[L] session is open.");
                }
                else
                {
                    pSampleFramework->DrawString(10, 210, "[L] session is close.");
                }
            }
            if (s_IsSyncMode)
            {
                pSampleFramework->DrawString(190, 20, "sendData : %d", syncSendData);
                for (int i = 0; i < MaxStationNum; ++i)
                {
                    nn::pia::common::StringStream ss;
                    for (int j = 0; j < DataIdNum; ++j)
                    {
                        ss << syncAllData[i][j] << ",";
                    }
                    pSampleFramework->DrawString(190, 40 + (i * 10), "recv[%2d]: %s", i, ss.CStr());
                }
                pSampleFramework->DrawString(190, 30, "---------------------");
            }
            else
            {
                pSampleFramework->DrawString(250, 20, "sendData : %d", s_ElementForSend.GetValue());
                pSampleFramework->DrawString(250, 30, "---------------------");
                for (int i = 0; i < MaxStationNum; ++i)
                {
                    pSampleFramework->DrawString(250, 40 + (i * 10), "recvData[%d] : %d", i, s_ElementForReceive[i].GetValue());
                }
            }
        }

        if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonLeft))
        {
            errorHandlingCase--;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonRight))
        {
            errorHandlingCase++;
        }
        else if (s_IsSyncMode && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            ++syncSendData;
        }
        else if (s_IsSyncMode && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            --syncSendData;
        }
        else if (!s_IsSyncMode && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonUp))
        {
            s_ElementForSend.SetValue(s_ElementForSend.GetValue() + 1);
        }
        else if (!s_IsSyncMode && pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonDown))
        {
            s_ElementForSend.SetValue(s_ElementForSend.GetValue() - 1);
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonR))
        {
            pSampleFramework->SetProgramStatus(0, true, true, 1, 0);
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonY))
        {
            pSampleFramework->EndFrame();
            // Yボタンで HandleErrorAsync を用いた任意のエラーに対するエラーハンドリングをテストします
            switch (errorHandlingCase)
            {
            case 0:
                // HandlingType_Cleanup
                ResultCheck(nn::pia::ResultCancelled(), pState);
                break;
            case 1:
                // HandlingType_LogoutWithLeave
                ResultCheck(nn::pia::ResultGameServerMaintenance(), pState);
                break;
            case 2:
                // HandlingType_CleanupWithLeave
                ResultCheck(nn::pia::ResultSessionConnectionIsLost(), pState);
                break;
            case 3:
                // HandlingType_ShutdownNetwork
                ResultCheck(nn::pia::ResultNetworkConnectionIsLost(), pState);
                break;
            case 4:
                // HandlingType_Finalize
                ResultCheck(nn::pia::ResultNexInternalError(), pState);
                break;
            case 5:
                // success -> State_LoggedIn
                result = ChangeState(*pState, nn::pia::framework::Framework::State_LoggedIn);
                if (result.IsFailure())
                {
                    PIA_SAMPLE_ASSERT(false);
                }
                *pState = State_Leave;
                break;
            case 6:
                // HandlingType_LogoutWithLeave -> State_LoggedIn
                ResultCheck(nn::pia::ResultGameServerMaintenance(), pState, nn::pia::framework::Framework::State_LoggedIn);
                break;
            case 7:
                // HandlingType_Cleanup -> State_LoggedIn
                ResultCheck(nn::pia::ResultCancelled(), pState, nn::pia::framework::Framework::State_LoggedIn);
                break;
            case 8:
#if NN_PIA_CTR
                nn::math::RandomGenerator random;
                random.SetSeed(pSampleFramework->GetFrameCount());

                switch (random.Generate(5))
                {
                case 0:
                    result = nn::pia::ResultCancelled();
                    break;
                case 1:
                    result = nn::pia::ResultGameServerMaintenance();
                    break;
                case 2:
                    result = nn::pia::ResultSessionConnectionIsLost();
                    break;
                case 3:
                    result = nn::pia::ResultNetworkConnectionIsLost();
                    break;
                case 4:
                    result = nn::pia::ResultNexInternalError();
                    break;
                default:
                    result = nn::pia::ResultSuccess();
                    break;
                }
                nn::pia::framework::Framework::State transitionState = static_cast<nn::pia::framework::Framework::State>(random.Generate(8));
                PIA_SAMPLE_PRINT("result : %d, state : %d -> %d", result.GetHandlingType(), nn::pia::framework::Framework::GetInstance()->GetState(), transitionState);
                ResultCheck(result, pState, transitionState);
#endif
                break;
            }
            return;
        }
        else if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            pSampleFramework->EndFrame();
            pSampleFramework->DrawString(10, 60, "State_LeaveSession...");
            PIA_SAMPLE_PRINT("---------------LeaveJointSession start----------------\n");
            *pState = State_LeaveJointSession;
            return;
        }
        else if (nn::pia::session::Session::GetInstance()->IsJointSessionHost())
        {
            if (pSampleFramework->IsTrigger(nns::pia::sampleframework::ButtonStart))
            {
                pSampleFramework->EndFrame();
                pSampleFramework->DrawString(10, 60, "State_DestroyJoint...");
                *pState = State_DestroyJointSession;
                return;
            }
        }

        pSampleFramework->EndFrame();
    }
    PIA_SAMPLE_PRINT("---------------SceneJointSession end----------------\n");
}

void SceneDestroyJoint(State* pState)
{
    nn::pia::Result result;
    if (!s_IsSyncMode)
    {
        // JointSession 用 cloneProtocol の設定を解除
        nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
        PIA_SAMPLE_ASSERT(pCloneProtocol != NULL);
        if (pCloneProtocol != NULL)
        {
            result = pCloneProtocol->UnregisterSendClone(&s_SendClone);
            ResultCheck(result, pState);
            for (int i = 0; i < MaxStationNum; ++i)
            {
                result = pCloneProtocol->UnregisterReceiveClone(&s_ReceiveClone[i]);
                ResultCheck(result, pState);
            }
            result = pCloneProtocol->Stop();
            ResultCheck(result, pState);
        }
    }

    PIA_SAMPLE_PRINT("---------------DestroyJointSession start----------------\n");
    result = nn::pia::framework::Framework::GetInstance()->DestroyJointSessionAsync();
    ResultCheck(result, pState);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT("nn::pia::framework::Framework::GetInstance()->DestroyJointSessionAsync() failed.\n");
        return;
    }

    WaitForAsyncProcess("DestroyJointSessionAsync", &nn::pia::framework::Framework::IsDestroyJointSessionCompleted, *pState);

    if (result.IsSuccess())
    {
        if (!s_IsSyncMode)
        {
            // cloneProtocol の設定
            nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
            result = pCloneProtocol->RegisterSendClone(&s_SendClone, 0x0001);
            ResultCheck(result, pState);
            for (int i = 0; i < MaxStationNum; ++i)
            {
                result = pCloneProtocol->RegisterReceiveClone(&s_ReceiveClone[i], static_cast<nn::pia::StationIndex>(i), 0x0001);
                ResultCheck(result, pState);
            }
            result = pCloneProtocol->Start();
            ResultCheck(result, pState);
        }
        *pState = State_Session;
    }
}

void SceneLeaveJoint(State* pState)
{
    nn::pia::Result result;
    if (!s_IsSyncMode)
    {
        // JointSession 用 cloneProtocol の設定を解除
        nn::pia::clone::CloneProtocol* pCloneProtocol = nn::pia::framework::Framework::GetInstance()->GetCloneProtocol();
        PIA_SAMPLE_ASSERT(pCloneProtocol != NULL);
        if (pCloneProtocol != NULL)
        {
            result = pCloneProtocol->UnregisterSendClone(&s_SendClone);
            ResultCheck(result, pState);
            for (int i = 0; i < MaxStationNum; ++i)
            {
                result = pCloneProtocol->UnregisterReceiveClone(&s_ReceiveClone[i]);
                ResultCheck(result, pState);
            }
            result = pCloneProtocol->Stop();
            ResultCheck(result, pState);
        }
    }

    PIA_SAMPLE_PRINT("---------------LeaveJointSession start----------------\n");
    ChangeState(*pState, nn::pia::framework::Framework::State_JointSessionLeft);

    *pState = State_FinalizeFramework;
}
#endif

// エントリポイント。
PIA_SAMPLE_MAIN_FUNC()
{
    nn::pia::framework::NetworkType networkType = nn::pia::framework::NetworkType_Inet;

    nns::pia::sampleframework::SampleFramework* pSampleFramework = NULL;
    nns::pia::sampleframework::SampleFramework::Setting setting;
    setting.isUpperScreenOnly = true;
    nns::pia::sampleframework::SampleFramework::CreateInstance(setting);
    pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    if (!pSampleFramework)
    {
        PIA_SAMPLE_ASSERT(false);
    }
    pSampleFramework->SetFontColor(nns::pia::sampleframework::ColorWhite);
    pSampleFramework->SetFramePerSecond(60);

    if (!s_IsSyncMode)
    {
        // Clone 用
        s_SendClone.RegisterElement(&s_ElementForSend, 0x0001);
        for (int i = 0; i < MaxStationNum; ++i)
        {
            s_ReceiveClone[i].RegisterElement(&s_ElementForReceive[i], 0x0001);
        }
    }

    State state = State_Title;

    while (true)
    {
        switch (state)
        {
        case State_Title:
        {
            SceneTitle(&state, &networkType);
        }
        break;
        case State_InitializeFramework:
        {
            SceneInitializeFramework(&state, networkType);
        }
        break;
        case State_RandomSession:
        {
            SceneRandomSession(&state);
        }
        break;
        case State_BrowseSession:
        {
            SceneBrowseSession(&state, networkType);
        }
        break;
        case State_CreateOrJoinSession:
#if NN_PIA_NINTENDOSDK || NN_PIA_CTR
        {
            SceneCreateOrJoinSession(&state, networkType);
        }
#endif
        break;
        case State_Session:
        {
            SceneSession(&state, networkType);
        }
        break;
#if NN_PIA_FRAMEWORK_INET_JOINT_SESSION
        case State_JoinRandomJointSession:
        {
            SceneJoinRandomJointSession(&state);
        }
        break;
        // 受動的にセッション移行を開始したステーションは Framework::Update() を定期的に呼んで Framework::IsSessionMigrating() が false になるのを待つ
        case State_SessionMigration:
        {
            SceneSessionMigration(&state);
        }
        break;
        case State_JointSession:
        {
            SceneJointSession(&state, networkType);
        }
        break;
        case State_DestroyJointSession:
        {
            SceneDestroyJoint(&state);
        }
        break;
        case State_LeaveJointSession:
        {
            SceneLeaveJoint(&state);
        }
        break;
#endif
        case State_Leave:
        {
            SceneLeave(&state);
        }
        break;
        case State_FinalizeFramework:
        {
            SceneFinalizeFramework(&state);
        }
        break;
        default:
            break;
        }
    }
}
