﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <nn/pia/pia_Platform.h>
#include <nn/pia/common.h>
#include <nn/pia/session.h>

#include <SampleLib/SampleLib.h>
#include <SampleLib/Net_Setup.h>

#define NNS_PIA_ENABLE_FRIENDS (NNS_PIA_ENABLE_INET && !NN_PIA_NINTENDOSDK_WIN && (NN_PIA_CTR || NN_PIA_CAFE || NN_PIA_NINTENDOSDK))
#define NNS_PIA_ENABLE_DIRECT_JOIN_FROM_FRIENDLIST (NNS_PIA_ENABLE_FRIENDS && (NN_PIA_CTR || NN_PIA_CAFE))

#ifdef PIA_TEST_BUILD
// 自動検証用のコードです。サンプルとしては無視してください。
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif

#define TEST_PIA_LOCAL_USER_PASSWORD 0

const int32_t FramePerSecond = 60;
const char ProgramNameString[] = "session/advanced";

/*!
    @brief サンプルプログラムのシーンを示す列挙型です。
 */
enum SampleSceneNo
{
    SampleSceneNo_Start,              //!< 最初のシーンです。通信とは無関係です。
    SampleSceneNo_PrepareLan,         //!< LAN マッチメイクを行う準備を行うシーンです。通信とは無関係です。
    SampleSceneNo_PrepareInet,        //!< インターネット通信を行う準備を行うシーンです。通信とは無関係です。
    SampleSceneNo_PrepareLocal,       //!< ローカル通信を行う準備を行うシーンです。通信とは無関係です。
    SampleSceneNo_SelectSessionType,  //!< セッションの構築方法を指定するシーンです。
    SampleSceneNo_Community,          //!< コミュニティの構築や参加を行うシーンです。
    SampleSceneNo_JoinRandomSession,  //!< 自動マッチメイク機能を使用してセッションに参加するシーンです。
    SampleSceneNo_JoinSession,        //!< 指定したネットワーク種別毎に、参加するセッションを作る/作成されたセッションを検索し、参加する処理を行うシーンです。
    SampleSceneNo_JoinFriendSession,  //!< フレンドプレゼンス機能を使用してセッションに参加するシーンです。
    SampleSceneNo_Communication,      //!< セッション内の相手と通信を行うシーンです。
    SampleSceneNo_Error,              //!< 問題が発生した場合に遷移するシーンです。実際のアプリにおける エラーメッセージウィンドウ をイメージした処理を行います。
    SampleSceneNo_NetworkError,       //!< ネットワークエラーコードを表示させるシーンです。本来はこのシーンでエラー・EULAアプレットを起動することを想定しています。
    SampleSceneNo_DummySession,       //!< デバッグ用ダミーセッションの構築、破棄、更新を行うシーンです。
    SampleSceneNo_Exit                //!< デモを終了します。
};

/*!
    @brief セッション構築方法の種別を示す列挙型です。
 */
enum MatchmakeType
{
    MatchmakeType_Random = 10000000,  //!< ランダムマッチメイク機能を使用します。
    MatchmakeType_Browse = 20000000,  //!< 参加するセッションを作ります/作成されたセッションを検索し、参加します。
    MatchmakeType_Friends = 30000000  //!< フレンドプレゼンス機能を使用し、フレンドがオーナーのセッションに参加します。
};

/*!
    @brief サンプルプログラムでハンドリングする通信エラーの種別です。
 */
enum SampleErrorType
{
    SampleErrorType_Unknown = 0,                     //!< 不明なエラーです
    SampleErrorType_InitializeByNetworkInterfaceUp,  //!< ネットワークインターフェースの起動に失敗（CTR の場合は無線 OFF モードも含まれます）
    SampleErrorType_InitializeByAccount,             //!< ユーザーアカウントが必要な場合に、有効なアカウントが存在しなかった
    SampleErrorType_InitializeByOtherReason,         //!< その他の理由で、初期化に失敗
    SampleErrorType_Login,                           //!< ゲームサーバーへのログインに失敗しました
    SampleErrorType_ConnectionLost,                  //!< ネットワーク接続が断絶しました
    SampleErrorType_ConnectionLostBeforeStartup,     //!< セッションのスタートアップ前にネットワーク接続が断絶しました
    SampleErrorType_SessionLost,                     //!< 通信中にセッションから切断されました
    SampleErrorType_Startup,
    SampleErrorType_Create,
    SampleErrorType_Browse,
    SampleErrorType_Join,
    SampleErrorType_Leave,
    SampleErrorType_Destroy,
    SampleErrorType_Community
};

enum MatchmakeCommand
{
    MatchmakeCommand_None = 0,
    MatchmakeCommand_Create,
    MatchmakeCommand_Browse,
    MatchmakeCommand_BrowseOwner,
    MatchmakeCommand_BrowseParticipant,
    MatchmakeCommand_BrowseSessionId,
    MatchmakeCommand_Join,
    MatchmakeCommand_Rejoin,
    MatchmakeCommand_Back,
    MatchmakeCommand_JoinCommunity
};

enum CommunityCommand
{
    CommunityCommand_None = 0,
    CommunityCommand_Create,
    CommunityCommand_Browse,
    CommunityCommand_Join,
    CommunityCommand_Update,
    CommunityCommand_Destroy,
    CommunityCommand_Leave
};

enum LocalMultiPlayType
{
    LocalMultiPlayType_None = 0,        //!< 同じ端末から参加するプレイヤーは一人です。
#if NN_PIA_NINTENDOSDK
    LocalMultiPlayType_Participants,    //!< 同じ端末から複数プレイヤーがマッチメイクセッションに参加します。
    LocalMultiPlayType_Stations,        //!< 同じ端末から複数プレイヤーが参加しますが、マッチメイクセッション上は 1 台としてカウントされます。
#endif
    LocalMultiPlayType_Max
};

enum MatchmakeKeywordType
{
    MatchmakeKeywordType_None = 0,      //!< あいことば無し
    MatchmakeKeywordType_A,             //!< あいことばA
    MatchmakeKeywordType_B,             //!< あいことばB
    MatchmakeKeywordType_Max
};

#if NNS_PIA_ENABLE_INET
extern const SampleStringChar_t* SampleMatchmakeKeyword[];
#endif

nns::pia::sampleframework::SampleFramework* GetFramework();
void SetCommunicationErrorType(SampleErrorType error);  //通信エラーの内容をセットする関数です。
void ClearErrorResult();
void SetPiaErrorResult(nn::pia::Result result);  //Pia API 呼び出しが失敗した場合に返される nn::pia::Result 値をセットする関数です。
SampleSceneNo GetNextSceneByErrorResult(nn::pia::Result result, SampleErrorType errorType);
SampleSceneNo HandlingErrorResult(nn::pia::Result::HandlingType handlingType, SampleErrorType errorType);

uint32_t GetIncrementalGameModeValueByBitRateCheckMode();

#if NNS_PIA_ENABLE_FRIENDS
void ResetGameMode();
void SetFriendErrorResult(uint32_t result);  //非 Pia API 呼び出しが失敗した場合にエラー値をセットする関数です。
#if NN_PIA_NINTENDOSDK
void DeclareOpenOnlinePlaySession();
void DeclareCloseOnlinePlaySession();
#endif
#if NNS_PIA_ENABLE_DIRECT_JOIN_FROM_FRIENDLIST
void CheckJoinFromFriendList();
bool IsJoinFromFriendList();
#endif
#endif

void Dispatch();
void SampleSessionCallbackForDispatch();

void SetUnreliableProtocolHandle(uint32_t handle);
uint32_t GetUnreliableProtocolHandle();
void SampleSessionCallback(nn::pia::session::Session::EventType eventType, nn::pia::StationId stationId, void* pApplicationParam);
uint32_t GetJoinedCommunityId();
const nn::pia::common::String& GetSystemPassword();
uint32_t GetSessionId();
void ResetRejoinSettings();

void Logout();
void CleanupLan();
void CleanupLocal();
void CleanupPia();
void FinalizeInet();
void FinalizeLan();
void FinalizeLocal();
void FinalizePia();

uint16_t GetMaxParticipantNum();
uint32_t GetSendingInterval(uint16_t stationNum);

/* ---< 処理別シーン >--- */
SampleSceneNo SceneStart();
SampleSceneNo ScenePrepareLan();
#if NNS_PIA_ENABLE_INET
SampleSceneNo ScenePrepareInternet();
SampleSceneNo SceneDummySession();
#endif
#if NNS_PIA_ENABLE_LOCAL
SampleSceneNo ScenePrepareLocal();
#endif
SampleSceneNo SceneSelectMatchmakeType();
SampleSceneNo SceneCommunity();
SampleSceneNo SceneRandomMatchmake();
SampleSceneNo SceneBrowseMatchmake();
SampleSceneNo SceneFriendsMatchmake();
SampleSceneNo SceneCommunication();
SampleSceneNo SceneError();
SampleSceneNo SceneNetworkError();

/* セッション関連 */
const uint16_t MaxParticipantNum = 16;
const uint16_t MaxBrowsedSessionNum = 5; //!< セッションの表示数最大
const uint8_t MaxFriendSessionNum = 5;   //!< フレンドの表示数最大
const nn::pia::common::String DescriptionRandomSession(SAMPLE_STRING("JoinRandomSession"));
const nn::pia::common::String DescriptionJoinSession(SAMPLE_STRING("JoinSession"));
const nn::pia::common::String DescriptionJoinFriendSession(SAMPLE_STRING("JoinFriendSession"));

/* NEX 関連設定 */
const uint32_t MatchmakeGameModeOffset = 0;
const uint32_t MatchmakeGameModeBase = 35300;
const uint32_t MatchmakeGameMode = MatchmakeGameModeBase + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MatchmakeGameModeOffset;
const uint32_t MatchmakeAttributeIndex = 0;
const uint32_t MatchmakeAttributeIndexHostMigration = 1;
const uint32_t MatchmakeAttributeIndexNetworkTopology = 2;

/* メモリ関連 */
const uint32_t MemoryBlockSizeForPia = 10 * 1024 * 1024;     //!< Pia に与えるメモリのサイズです。
const uint32_t MemoryBlockSizeForNex = 1024 * 1024;         //!< NEX に与えるメモリのサイズです。
const uint32_t MemoryBlockSizeForNexEmergency = 50 * 1024;  //!< NEX に与える緊急時用メモリのサイズです。

/* ローカル関連 */
#if NNS_PIA_ENABLE_LOCAL
#if NN_PIA_CTR
const uint32_t LocalTitleId = 0xFF311;    //!< 本プログラムのユニーク ID です。
const uint8_t LocalSubId = MatchmakeGameModeOffset + 100 * PIA_TEST_CODE;          //!< サブ ID です。
const char LocalWirelessCryptoKey[] = __FILE__;  //!< UDS 通信の暗号鍵を生成するためのパスフレーズです。実アプリでは "類推されにくい"、"十分な長さ" のパスフレーズを設定するようにしてください。
#elif NN_PIA_NINTENDOSDK
const uint16_t LocalSceneId = MatchmakeGameModeOffset + 30000 * PIA_TEST_CODE;
const char LocalWirelessCryptoKey[] = __FILE__;
#endif
const char SignatureKey[] = "signature";

struct SampleBeaconApplicationData
{
    char programName[64];
    char playerName[nn::pia::MaxPlayerNameLength + 1];
};
#endif

/* 描画関連 */
const int UpperScreenWidth = 400;   //!< 上画面の解像度(横)
const int UpperScreenHeight = 240;  //!< 上画面の解像度(縦)
const int LowerScreenWidth = 320;   //!< 下画面の解像度(横)
const int LowerScreenHeight = 240;  //!< 下画面の解像度(縦)
const int FontSize = 8;             //!< 描画するフォントのサイズ

const char* GetPiaVersionString();  //!< Piaライブラリバージョン情報を描画するための文字列取得関数

// Global 変数宣言
extern uint32_t g_FrameNo;
extern nns::pia::NetworkType g_NetworkType;
extern nn::pia::session::NetworkTopology g_NetworkTopologyType;
extern bool g_IsHostMigrationEnabled;
extern bool g_IsBitRateCheckEnabled;

#if TEST_PIA_LOCAL_USER_PASSWORD
// ローカルのユーザーパスワード変更テスト用
extern uint8_t g_UserPasswordType;
#endif

#if NN_PIA_NINTENDOSDK
extern nn::Result g_LoginError;
#else
extern int32_t g_LoginError;
#endif
extern LocalMultiPlayType g_LocalMultiPlayType;
extern MatchmakeKeywordType g_MatchmakeKeywordType;

#if PIA_TEST_CODE
extern bool g_IsLatencyEmulationEnabled;
void piaTestCallback();
const int ProgramParamStationId = 0;
const int ProgramParamBrowsedSessionInfoListSize = 1;
const int ProgramParamGetJoinSequence = 101;
const int ProgramParamMatchmakeSessionParticipant = 102;
const int ProgramParamIsCloseSessionCompleted = 103;
const int ProgramParamNetworkTopologyType = 104;
const int ProgramParamHostMigrationSequence = 109;
const int ProgramParamHostMigrationState = 110;
const int ProgramParamLanMatchmakeState = 111;
const int ProgramParamSessionTime = 112;
const int ProgramParamMinParticipant = 120;
const int ProgramParamMaxParticipant = 121;
const int ProgramParamSessionType = 122;
const int ProgramParamAutoClose = 123;
const int ProgramParamAttribute0 = 124;
const int ProgramParamAttribute1 = 125;
const int ProgramParamAttribute2 = 126;
const int ProgramParamAttribute3 = 127;
const int ProgramParamAttribute4 = 128;
const int ProgramParamAttribute5 = 129;
const int ProgramParamUserPassword = 130;
const int ProgramParamPriority = 131;
const int ProgramParamAppData = 132;
const int ProgramParamStartYear = 140;
const int ProgramParamStartMonth = 141;
const int ProgramParamStartDay = 142;
const int ProgramParamStartHour = 143;
const int ProgramParamStartMin = 144;
const int ProgramParamStartSec = 145;
#endif
