﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"
#include "SampleLib/SampleLib.h"
#include "SampleLib/Session_Setup.h"
#include <nn/pia/lan.h>

#if NNS_PIA_ENABLE_INET
#include <nn/pia/inet.h>
#endif
#if NNS_PIA_ENABLE_LOCAL
#include <nn/pia/local.h>
#include "SampleLib/Local_Setup.h"
#endif

#include <cstdlib>

namespace
{
// 検索結果の履歴数
uint16_t s_SessionInfoHistoryNum = 0;
// SessionId を履歴として使用するかどうか（インターネットマッチメイク、LAN マッチメイク）
bool s_IsSessionIdHistoryUsed = false;
// SessionInfo を履歴として使用するかどうか（インターネットマッチメイク、ローカルマッチメイク、LAN マッチメイク）
bool s_IsSessionInfoHistoryUsed = false;

// 検索結果の履歴保持用の配列
nn::pia::lan::LanSessionInfo s_LanSessionInfoPtrArray[MaxBrowsedSessionNum];
#if NNS_PIA_ENABLE_INET
nn::pia::inet::NexSessionInfo s_NexSessionInfoPtrArray[MaxBrowsedSessionNum];
#endif
#if NNS_PIA_ENABLE_LOCAL
#if NN_PIA_CTR
nn::pia::local::UdsSessionInfo s_LocalSessionInfoPtrArray[MaxBrowsedSessionNum];
#elif NN_PIA_NINTENDOSDK
nn::pia::local::LdnSessionInfo s_LocalSessionInfoPtrArray[MaxBrowsedSessionNum];
#endif
#endif
uint32_t s_SessionIdBufferArray[MaxBrowsedSessionNum];

#if NNS_PIA_ENABLE_LOCAL
#if NN_PIA_CTR
// UserName を取得する際の共通処理
void CheckAndGetUserName(char userNameDst[], const nn::cfg::UserName& userNameSrc, uint32_t dstMaxSize)
{
    if (userNameSrc.isNgUserName)
    {
        // NG ワード
        ::std::strncpy(userNameDst, "NG_WORD", dstMaxSize);
    }
    else
    {
        // UserName は画面表示用にワイド文字列からマルチバイト文字列へ変換しておく
        // （PiaSampleFramework はワイド文字の画面表示に非対応のため）
        int ret = ::std::wcstombs(userNameDst, userNameSrc.userName, dstMaxSize);
        if (ret == -1)
        {
            // 変換失敗
            ::std::strncpy(userNameDst, "NOT ASCII", dstMaxSize);
        }
    }

    // 安全の為に明示的に終端文字を入れておく
    userNameDst[dstMaxSize - 1] = '\0';
}
#endif
#endif
#if PIA_TEST_CODE
void SampleSessionCallbackForEndFrame()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    PIA_ASSERT(PIA_IS_VALID_POINTER(pSampleFramework));

    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pSession));

    GetFramework()->SetProgramParam(ProgramParamGetJoinSequence, pSession->GetJoinSequence());
}
#endif
}


SampleSceneNo SceneBrowseMatchmakeLoop(
    nn::pia::session::CreateSessionSetting* pCreateSessionSetting,
    nn::pia::session::JoinSessionSetting* pJoinSessionSetting,
    nn::pia::session::SessionSearchCriteria* pSearchCriteria)
{
    SampleSceneNo nextSceneNo = SampleSceneNo_JoinSession;
    nn::pia::Result result;

    while (nextSceneNo == SampleSceneNo_JoinSession)
    {
        uint8_t targetSession = 0;
        MatchmakeCommand command = MatchmakeCommand_None;

        // 参加していたセッションへの再参加
        if (GetSessionId() != nn::pia::session::InvalidSessionId)
        {
            // 再参加用の情報をクリア
            ResetRejoinSettings();

            // システムパスワードによる再参加はインターネット通信のみ
            if (g_NetworkType == nns::pia::NetworkType_Internet)
            {
                command = MatchmakeCommand_Join;
            }
        }

        while (command == MatchmakeCommand_None)
        {
            GetFramework()->BeginFrame();

            // 上下: セッションの選択
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                targetSession = (MaxBrowsedSessionNum + targetSession - 1) % MaxBrowsedSessionNum;
            }
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                targetSession = (targetSession + 1) % MaxBrowsedSessionNum;
            }
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonLeft))
            {
                // 検索結果の履歴を使用するかどうか
                s_IsSessionInfoHistoryUsed = (s_IsSessionInfoHistoryUsed) ? false : true;
                s_IsSessionIdHistoryUsed = false;
            }
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                if (g_NetworkType == nns::pia::NetworkType_Internet || g_NetworkType == nns::pia::NetworkType_Lan)
                {
                    // 検索結果の履歴としてセッションIDを使用するかどうか
                    s_IsSessionIdHistoryUsed = (s_IsSessionIdHistoryUsed) ? false : true;
                    s_IsSessionInfoHistoryUsed = false;
                }
            }
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonStart))
            {
                // 検索結果の履歴をクリア
                for (uint16_t i = 0; i < MaxBrowsedSessionNum; i++)
                {
                    s_LanSessionInfoPtrArray[i].Clear();
#if NNS_PIA_ENABLE_INET
                    s_NexSessionInfoPtrArray[i].Clear();
#endif
#if NNS_PIA_ENABLE_LOCAL
                    s_LocalSessionInfoPtrArray[i].Clear();
#endif
                    s_SessionIdBufferArray[i] = nn::pia::session::InvalidSessionId;
                }
                s_SessionInfoHistoryNum = 0;
            }

#if TEST_PIA_LOCAL_USER_PASSWORD
            // ローカルで空パスワードを設定し直すテスト
            if (g_NetworkType == nns::pia::NetworkType_Local)
            {
                if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonR))
                {
                    SampleBeaconApplicationData beaconApplicationData;
                    memset(&beaconApplicationData, 0x00, sizeof(beaconApplicationData));
                    memcpy(beaconApplicationData.programName, ProgramNameString, sizeof(ProgramNameString));

#if NN_PIA_CTR
                    static_cast<nn::pia::local::UdsCreateSessionSetting*>(pCreateSessionSetting)->Reset();
                    static_cast<nn::pia::local::UdsCreateSessionSetting*>(pCreateSessionSetting)->SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
                    static_cast<nn::pia::local::UdsCreateSessionSetting*>(pCreateSessionSetting)->SetSubId(LocalSubId);
                    static_cast<nn::pia::local::UdsCreateSessionSetting*>(pCreateSessionSetting)->SetChannel(0);
                    static_cast<nn::pia::local::UdsCreateSessionSetting*>(pCreateSessionSetting)->SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
                    static_cast<nn::pia::local::UdsCreateSessionSetting*>(pCreateSessionSetting)->SetApplicationData(&beaconApplicationData, sizeof(beaconApplicationData));
                    static_cast<nn::pia::local::UdsCreateSessionSetting*>(pCreateSessionSetting)->SetMaxParticipantNum(GetMaxParticipantNum());
#elif NN_PIA_NINTENDOSDK
                    uint8_t wirelessCryptoKeySize = sizeof(LocalWirelessCryptoKey);
                    if (wirelessCryptoKeySize > nn::pia::local::LdnWirelessCryptoKeySizeMax)
                    {
                        wirelessCryptoKeySize = nn::pia::local::LdnWirelessCryptoKeySizeMax;
                    }
                    static_cast<nn::pia::local::LdnCreateSessionSetting*>(pCreateSessionSetting)->Reset();
                    static_cast<nn::pia::local::LdnCreateSessionSetting*>(pCreateSessionSetting)->SetLocalCommunicationId(nns::pia::LocalCommunicationId);
                    static_cast<nn::pia::local::LdnCreateSessionSetting*>(pCreateSessionSetting)->SetSceneId(LocalSceneId);
                    static_cast<nn::pia::local::LdnCreateSessionSetting*>(pCreateSessionSetting)->SetChannel(0);
                    static_cast<nn::pia::local::LdnCreateSessionSetting*>(pCreateSessionSetting)->SetWirelessCryptoKey(LocalWirelessCryptoKey, wirelessCryptoKeySize);
                    static_cast<nn::pia::local::LdnCreateSessionSetting*>(pCreateSessionSetting)->SetApplicationData(&beaconApplicationData, sizeof(beaconApplicationData));
                    static_cast<nn::pia::local::LdnCreateSessionSetting*>(pCreateSessionSetting)->SetMaxParticipantNum(GetMaxParticipantNum());
#endif
                    PIA_SAMPLE_PRINT("set empty UserPassword\n");
                }
            }
#endif

#if PIA_TEST_CODE
            // L/R: パケットロスのエミュレート
            nn::pia::transport::ThreadStreamManager* pThreadStreamManager = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pThreadStreamManager));
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonR))
            {
                pThreadStreamManager->SetSendThreadPacketLossRatio(100);
                pThreadStreamManager->SetReceiveThreadPacketLossRatio(100);
                PIA_SAMPLE_PRINT("set PacketLossRatio to 100.\n");
            }
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonL))
            {
                pThreadStreamManager->SetSendThreadPacketLossRatio(0);
                pThreadStreamManager->SetReceiveThreadPacketLossRatio(0);
                PIA_SAMPLE_PRINT("set PacketLossRatio to 0.\n");
            }
#else
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonR) && g_NetworkType == nns::pia::NetworkType_Internet)
            {
                command = MatchmakeCommand_BrowseOwner;
            }
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonL) && g_NetworkType == nns::pia::NetworkType_Internet)
            {
                command = MatchmakeCommand_BrowseParticipant;
            }
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonStart) && g_NetworkType == nns::pia::NetworkType_Internet)
            {
                command = MatchmakeCommand_BrowseSessionId;
            }
#endif
            // B: リセット
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonB))
            {
                command = MatchmakeCommand_Back;
            }
            // X: 新規にセッションを構築
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonX))
            {
                command = MatchmakeCommand_Create;
            }
            // Y: セッション検索を開始
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                command = MatchmakeCommand_Browse;
            }
            // A: 選択したセッションに参加
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                // インターネット通信の場合、セッションIDだけを検索結果の履歴として残しておいて参加時に指定できます
                if (s_IsSessionIdHistoryUsed)
                {
                    for (uint8_t i = 0; i < s_SessionInfoHistoryNum; i++)
                    {
                        if (i == targetSession && s_SessionIdBufferArray[i] != nn::pia::session::InvalidSessionId)
                        {
#if NNS_PIA_ENABLE_INET
                            if (g_NetworkType == nns::pia::NetworkType_Internet)
                            {
                                static_cast<nn::pia::inet::NexJoinSessionSetting*>(pJoinSessionSetting)->SetSessionId(s_SessionIdBufferArray[i]);
                                command = MatchmakeCommand_Join;
                                break;
                            }
#endif
                            if (g_NetworkType == nns::pia::NetworkType_Lan)
                            {
                                static_cast<nn::pia::lan::LanJoinSessionSetting*>(pJoinSessionSetting)->SetSessionId(s_SessionIdBufferArray[i]);
                                command = MatchmakeCommand_Join;
                                break;
                            }
                        }
                    }
                }
                else
                {
                    nn::pia::session::ISessionInfo* pTargetSessionInfo = NULL;
                    if (s_IsSessionInfoHistoryUsed)
                    {
                        for (uint8_t i = 0; i < s_SessionInfoHistoryNum; i++)
                        {
                            if (i == targetSession)
                            {
#if NNS_PIA_ENABLE_LOCAL
                                if (g_NetworkType == nns::pia::NetworkType_Local)
                                {
                                    pTargetSessionInfo = &s_LocalSessionInfoPtrArray[i];
                                }
#endif
#if NNS_PIA_ENABLE_INET
                                if (g_NetworkType == nns::pia::NetworkType_Internet)
                                {
                                    pTargetSessionInfo = &s_NexSessionInfoPtrArray[i];
                                }
#endif
                                if (g_NetworkType == nns::pia::NetworkType_Lan)
                                {
                                    pTargetSessionInfo = &s_LanSessionInfoPtrArray[i];
                                }
                            }
                        }
                    }
                    else
                    {
                        nn::pia::session::ISessionInfoList::Iterator it = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->Begin();
                        uint8_t foundSessionNum = static_cast<uint8_t>(nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->GetSize());
                        for (uint8_t i = 0; i < foundSessionNum; i++)
                        {
                            if (i == targetSession)
                            {
                                pTargetSessionInfo = *it;
                            }
                            ++it;
                        }
                    }
                    if (pTargetSessionInfo)
                    {
                        pJoinSessionSetting->SetSessionInfoPtr(pTargetSessionInfo);
                        command = MatchmakeCommand_Join;
#if NNS_PIA_ENABLE_INET
                        if (g_NetworkType == nns::pia::NetworkType_Internet)
                        {
                            uint8_t nexApplicationDataBuffer[nn::pia::inet::ApplicationDataBufferSizeMax];
                            uint32_t appDataSize = pTargetSessionInfo->GetApplicationDataSize();
                            PIA_SAMPLE_ASSERT(appDataSize <= nn::pia::inet::ApplicationDataBufferSizeMax);
                            if (appDataSize <= sizeof(nexApplicationDataBuffer))
                            {
                                result = pTargetSessionInfo->GetApplicationData(nexApplicationDataBuffer, sizeof(nexApplicationDataBuffer));
                                PIA_SAMPLE_ASSERT(result.IsSuccess());
                                if (result.IsSuccess())
                                {
                                    PIA_SAMPLE_PRINT("GetApplicationData is success.\n");
                                }
                            }
                        }
#endif
                        if (g_NetworkType == nns::pia::NetworkType_Lan)
                        {
                            nn::pia::lan::LanSessionInfo* pInfo = static_cast<nn::pia::lan::LanSessionInfo*>(pTargetSessionInfo);

                            uint8_t lanApplicationDataBuffer[nn::pia::lan::LanApplicationDataBufferSizeMax];
                            uint32_t appDataSize = pInfo->GetApplicationDataSize();
                            PIA_SAMPLE_ASSERT(appDataSize <= nn::pia::lan::LanApplicationDataBufferSizeMax);
                            if (appDataSize <= sizeof(lanApplicationDataBuffer))
                            {
                                result = pInfo->GetApplicationData(lanApplicationDataBuffer, sizeof(lanApplicationDataBuffer));
                                PIA_SAMPLE_ASSERT(result.IsSuccess());
                                if (result.IsSuccess())
                                {
                                    PIA_SAMPLE_PRINT("GetApplicationData is success.\n");
                                }
                            }

                            nn::pia::lan::LanStationInfo stationInfoList[MaxParticipantNum];
                            result = pInfo->GetSessionStationInfo(stationInfoList, GetMaxParticipantNum());
                            if (result.IsSuccess())
                            {
                                PIA_SAMPLE_PRINT("[ UserName List ]\n");
                                for (uint32_t i = 0; i < pInfo->GetSessionStationInfoSize(); ++i)
                                {
                                    char userName[nn::pia::MaxUserNameLength + 1];
                                    nns::pia::StringUtil::ConvertToAscii(userName, sizeof(userName), stationInfoList[i].GetUserName());
                                    PIA_SAMPLE_PRINT(" %-6s:%s\n",
                                                   stationInfoList[i].GetRole() == nn::pia::session::SessionStationInfo::StationInfoRole_Host ? "Host" : "Client",
                                                   userName);
                                }
                            }
                            else
                            {
                                PIA_SAMPLE_PRINT("GetSessionStationInfo is failed\n");
                            }
                        }
#if NNS_PIA_ENABLE_LOCAL
                        if (g_NetworkType == nns::pia::NetworkType_Local)
                        {
                            // ローカルマッチメイク専用の情報を取得するため、LocalSessionInfo にキャスト
                            nn::pia::local::LocalSessionInfo* pTargetLocalSessionInfo = static_cast<nn::pia::local::LocalSessionInfo*>(pTargetSessionInfo);
                            // 参加するセッションのビーコンデータを取得
                            SampleBeaconApplicationData beaconApplicationData;
                            result = pTargetLocalSessionInfo->GetApplicationData(&beaconApplicationData, sizeof(beaconApplicationData));
                            if (result.IsSuccess())
                            {
                                PIA_SAMPLE_PRINT("Get BeaconData: %s, %s\n", beaconApplicationData.programName, beaconApplicationData.playerName);
                            }
                            else
                            {
                                PIA_SAMPLE_PRINT("GetApplicationData Failure\n");
                            }
#if NN_PIA_CTR
                            // 参加するセッションに参加中のステーション情報を取得
                            nn::pia::local::UdsStationInfo udsStationInfoList[MaxParticipantNum];
                            result = pTargetLocalSessionInfo->GetSessionStationInfo(udsStationInfoList, GetMaxParticipantNum());
                            if (result.IsSuccess())
                            {
                                PIA_SAMPLE_PRINT("[ UserName List ]\n");
                                char userName[nn::cfg::CFG_USER_NAME_LENGTH];
                                for (uint16_t i = 0; i < GetMaxParticipantNum(); i++)
                                {
                                    if (udsStationInfoList[i].GetRole() != nn::pia::local::LocalStationInfo::StationInfoRole_None)
                                    {
                                        bool isHost = udsStationInfoList[i].GetRole() == nn::pia::local::LocalStationInfo::StationInfoRole_Host;
                                        CheckAndGetUserName(userName, udsStationInfoList[i].userName, nn::cfg::CFG_USER_NAME_LENGTH);
                                        PIA_SAMPLE_PRINT(" %-6s:%s\n", (isHost ? "Host" : "Client"), userName);
                                        NN_PIA_UNUSED_IN_RELEASE(isHost);
                                    }
                                }
                            }
                            else
                            {
                                PIA_SAMPLE_PRINT("GetSessionStationInfo Failure\n");
                            }
#elif NN_PIA_NINTENDOSDK
                            //// 参加するセッションに参加中のステーション情報を取得
                            nn::pia::local::LdnStationInfo ldnStationInfoList[MaxParticipantNum];
                            result = pTargetLocalSessionInfo->GetSessionStationInfo(ldnStationInfoList, GetMaxParticipantNum());
                            if (result.IsSuccess())
                            {
                                PIA_SAMPLE_PRINT("GetSessionStationInfo Success\n");
                                // 登録したアカウントのニックネームで取得される
                                PIA_SAMPLE_PRINT("[ UserName List ]\n");
                                for (uint16_t i = 0; i < GetMaxParticipantNum(); i++)
                                {
                                    if (ldnStationInfoList[i].GetRole() != nn::pia::local::LocalStationInfo::StationInfoRole_None)
                                    {
                                        bool isHost = ldnStationInfoList[i].GetRole() == nn::pia::local::LocalStationInfo::StationInfoRole_Host;
                                        //CheckAndGetUserName(userName, udsStationInfoList[i].userName, nn::cfg::CFG_USER_NAME_LENGTH);
                                        PIA_SAMPLE_PRINT(" %-6s:%s\n", (isHost ? "Host" : "Client"), ldnStationInfoList[i].userName.GetCharArray());
                                        NN_PIA_UNUSED_IN_RELEASE(isHost);
                                    }
                                }
                            }
                            else
                            {
                                PIA_SAMPLE_PRINT("GetSessionStationInfo Failure\n");
                            }
#endif
                        }
#endif
                    }
                }
            }

#if PIA_TEST_CODE
            GetFramework()->SetProgramParam(
                ProgramParamBrowsedSessionInfoListSize,
                nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList()->GetSize());
#endif

            // 下画面
            GetFramework()->ChangeRenderTarget(false);
            GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
            int line = 0;
            GetFramework()->DrawString(0, line, "[%s Demo] %d", ProgramNameString, g_FrameNo);
            line += FontSize;
            GetFramework()->DrawString(0, line, "  Scene: BrowseMatchmake.");
            line += FontSize;
            line += FontSize;

            nn::pia::session::ISessionInfoList* pSessionInfoList = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList();
            GetFramework()->DrawString(0, line, "[ Session List ] Found:%01d ", pSessionInfoList->GetSize());
            line += FontSize;
            nn::pia::session::ISessionInfoList::Iterator it = pSessionInfoList->Begin();

            for (uint16_t i = 0; i < MaxBrowsedSessionNum; i++)
            {
                if (s_IsSessionIdHistoryUsed)
                {
                    if (i == targetSession)
                    {
                        GetFramework()->SetFontColor(nns::pia::sampleframework::ColorRed);
                    }
                    else
                    {
                        GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                    }
                    if (i < s_SessionInfoHistoryNum)
                    {
                        GetFramework()->DrawString(0, line, " GID=%d", s_SessionIdBufferArray[i]);
                    }
                    else
                    {
                        GetFramework()->DrawString(0, line, "  -----");
                    }
                    line += FontSize;
                    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                }
                else
                {
                    nn::pia::session::ISessionInfo* pSessionInfo = NULL;
                    if (s_IsSessionInfoHistoryUsed)
                    {
                        if (i < s_SessionInfoHistoryNum)
                        {
#if NNS_PIA_ENABLE_LOCAL
                            if (g_NetworkType == nns::pia::NetworkType_Local)
                            {
                                pSessionInfo = &s_LocalSessionInfoPtrArray[i];
                            }
#endif
#if NNS_PIA_ENABLE_INET
                            if (g_NetworkType == nns::pia::NetworkType_Internet)
                            {
                                pSessionInfo = &s_NexSessionInfoPtrArray[i];
                            }
#endif
                            if (g_NetworkType == nns::pia::NetworkType_Lan)
                            {
                                pSessionInfo = &s_LanSessionInfoPtrArray[i];
                            }
                        }
                    }
                    else
                    {
                        if (it != pSessionInfoList->End())
                        {
                            pSessionInfo = (*it);
                        }
                    }
                    if (pSessionInfo != NULL)
                    {
                        if (i == targetSession)
                        {
#if NNS_PIA_ENABLE_INET
                            if (g_NetworkType == nns::pia::NetworkType_Internet)
                            {
                                // インターネットマッチメイク専用の情報を取得するため、NexSessionInfo にキャスト
                                nn::pia::inet::NexSessionInfo* pInfo = static_cast<nn::pia::inet::NexSessionInfo*>(pSessionInfo);
                                uint8_t appData[nn::pia::inet::ApplicationDataBufferSizeMax];
                                uint32_t appDataSize = pInfo->GetApplicationDataSize();
                                uint32_t size = (appDataSize < sizeof(appData)) ? appDataSize : sizeof(appData);
                                result = pInfo->GetApplicationData(appData, size);
                                PIA_SAMPLE_ASSERT(result.IsSuccess());

                                uint32_t attribute[3];
                                result = pInfo->GetAttribute(MatchmakeAttributeIndex, &attribute[0]);
                                PIA_SAMPLE_ASSERT(result.IsSuccess());
                                result = pInfo->GetAttribute(MatchmakeAttributeIndexHostMigration, &attribute[1]);
                                PIA_SAMPLE_ASSERT(result.IsSuccess());
                                result = pInfo->GetAttribute(MatchmakeAttributeIndexNetworkTopology, &attribute[2]);
                                PIA_SAMPLE_ASSERT(result.IsSuccess());

                                GetFramework()->SetFontColor(nns::pia::sampleframework::ColorRed);
                                GetFramework()->DrawString(0,
                                                           line,
                                                           "%sMode=%d GID=%d [%d/%d-%d] [%s]",
                                                           pInfo->IsRestrictedByUserPassword() ? "*" : " ",
                                                           pInfo->GetGameMode(),
                                                           pInfo->GetSessionId(),
                                                           pInfo->GetCurrentParticipantNum(),
                                                           pInfo->GetMinParticipantNum(),
                                                           pInfo->GetMaxParticipantNum(),
                                                           pInfo->IsOpened() ? "Opened" : "Closed");
                                line += FontSize;

                                GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                                GetFramework()->DrawString(0,
                                                           LowerScreenHeight - (FontSize * 14),
                                                           "Type=%d/%d Host=%llu Prior=%d",
                                                           pInfo->GetSessionType(),
                                                           pInfo->IsAutoCloseSession(),
                                                           pInfo->GetHostPrincipalId(),
                                                           pInfo->GetSelectionPriority());
                                GetFramework()->DrawString(0,
                                                           LowerScreenHeight - (FontSize * 13),
                                                           "Date:%04d/%02d/%02d-%02d:%02d:%02d",
                                                           pInfo->GetStartedTime().GetYear(),
                                                           pInfo->GetStartedTime().GetMonth(),
                                                           pInfo->GetStartedTime().GetDay(),
                                                           pInfo->GetStartedTime().GetHour(),
                                                           pInfo->GetStartedTime().GetMinute(),
                                                           pInfo->GetStartedTime().GetSecond());
                                GetFramework()->DrawString(0, LowerScreenHeight - (FontSize * 12), "ApplicationData=[%d %d %d %d]", appData[0], appData[1], appData[2], appData[3]);
                                GetFramework()->DrawString(0, LowerScreenHeight - (FontSize * 11), "Attribute=[%d %d %d]", attribute[0], attribute[1], attribute[2]);
                                GetFramework()->DrawString(0, LowerScreenHeight - (FontSize * 10), "Community=%d Target=%llu", pInfo->GetCommunityId(), pInfo->GetTargetPrincipalId());
                            }
#endif
                            if (g_NetworkType == nns::pia::NetworkType_Lan)
                            {
                                // LANマッチメイク専用の情報を取得するため、LanSessionInfo にキャスト
                                nn::pia::lan::LanSessionInfo* pInfo = static_cast<nn::pia::lan::LanSessionInfo*>(pSessionInfo);
                                uint8_t appData[nn::pia::lan::LanApplicationDataBufferSizeMax];
                                uint32_t appDataSize = pInfo->GetApplicationDataSize();
                                uint32_t size = (appDataSize < sizeof(appData)) ? appDataSize : sizeof(appData);
                                result = pInfo->GetApplicationData(appData, size);
                                PIA_SAMPLE_ASSERT(result.IsSuccess());

                                uint32_t attribute[3];
                                result = pInfo->GetAttribute(MatchmakeAttributeIndex, &attribute[0]);
                                PIA_SAMPLE_ASSERT(result.IsSuccess());
                                result = pInfo->GetAttribute(MatchmakeAttributeIndexHostMigration, &attribute[1]);
                                PIA_SAMPLE_ASSERT(result.IsSuccess());
                                result = pInfo->GetAttribute(MatchmakeAttributeIndexNetworkTopology, &attribute[2]);
                                PIA_SAMPLE_ASSERT(result.IsSuccess());

                                GetFramework()->SetFontColor(nns::pia::sampleframework::ColorRed);
                                GetFramework()->DrawString(0,
                                                           line,
                                                           "Mode=%d GID=%d [%d/%d-%d] [%s]",
                                                           pInfo->GetGameMode(),
                                                           pInfo->GetSessionId(),
                                                           pInfo->GetCurrentParticipantNum(),
                                                           pInfo->GetMinParticipantNum(),
                                                           pInfo->GetMaxParticipantNum(),
                                                           pInfo->IsOpened() ? "Opened" : "Closed");
                                line += FontSize;

                                GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                                GetFramework()->DrawString(0, LowerScreenHeight - (FontSize * 12), "Type=%d", pInfo->GetSessionType());
                                GetFramework()->DrawString(0, LowerScreenHeight - (FontSize * 11), "ApplicationData=[%d %d %d %d]", appData[0], appData[1], appData[2], appData[3]);
                                GetFramework()->DrawString(0, LowerScreenHeight - (FontSize * 10), "Attribute=[%d %d %d]", attribute[0], attribute[1], attribute[2]);
                            }
#if NNS_PIA_ENABLE_LOCAL
                            else if (g_NetworkType == nns::pia::NetworkType_Local)
                            {
                                // ローカルマッチメイク専用の情報を取得するため、LocalSessionInfo にキャスト
                                nn::pia::local::LocalSessionInfo* pInfo = static_cast<nn::pia::local::LocalSessionInfo*>(pSessionInfo);
                                // 選択されているセッションの情報表示
                                SampleBeaconApplicationData beaconApplicationData;
                                beaconApplicationData.playerName[0] = '\0';
                                beaconApplicationData.programName[0] = '\0';

                                uint32_t appDataSize = pInfo->GetApplicationDataSize();
                                PIA_SAMPLE_ASSERT(appDataSize == sizeof(beaconApplicationData));
                                if (appDataSize == sizeof(beaconApplicationData))
                                {
                                    result = pInfo->GetApplicationData(&beaconApplicationData, sizeof(beaconApplicationData));
                                    if (result.IsFailure())
                                    {
                                        PIA_SAMPLE_PRINT("LocalSessionInfo::GetApplicationData is failed. [%08X]\n", result.GetValue());
                                    }
                                }

                                GetFramework()->SetFontColor(nns::pia::sampleframework::ColorRed);
                                GetFramework()->DrawString(0,
                                                           line,
                                                           "%sMode=%d GID=%d [%d/%d-%d] [%s]",
                                                           pInfo->IsRestrictedByUserPassword() ? "*" : " ",
                                                           pInfo->GetGameMode(),
                                                           pInfo->GetSessionId(),
                                                           pInfo->GetCurrentParticipantNum(),
                                                           pInfo->GetMinParticipantNum(),
                                                           pInfo->GetMaxParticipantNum(),
                                                           pInfo->IsOpened() ? "Opened" : "Closed");
                                line += FontSize;

                                GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                                uint8_t bssid[nn::pia::local::BssIdSize];
                                pInfo->GetBssid(bssid, sizeof(bssid));
                                GetFramework()->DrawString(0, LowerScreenHeight - (FontSize * 12), " RF=%d BSSID=%02x:%02x:%02x:%02x:%02x:%02x", pInfo->GetRadioStrength(), bssid[0], bssid[1], bssid[2], bssid[3], bssid[4], bssid[5]);
                                GetFramework()->DrawString(0, LowerScreenHeight - (FontSize * 11), " Title=%s", beaconApplicationData.programName);
                                GetFramework()->DrawString(0, LowerScreenHeight - (FontSize * 10), " PlayerName(Host) =%s", beaconApplicationData.playerName);
                            }
#endif
                        }
                        else
                        {
                            GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                            GetFramework()->DrawString(0,
                                                       line,
                                                       " Mode=%d GID=%d [%d/%d-%d] [%s]",
                                                       pSessionInfo->GetGameMode(),
                                                       pSessionInfo->GetSessionId(),
                                                       pSessionInfo->GetCurrentParticipantNum(),
                                                       pSessionInfo->GetMinParticipantNum(),
                                                       pSessionInfo->GetMaxParticipantNum(),
                                                       pSessionInfo->IsOpened() ? "Opened" : "Closed");
                            line += FontSize;
                        }
                        ++it;
                    }
                    else
                    {
                        if (i == targetSession)
                        {
                            GetFramework()->SetFontColor(nns::pia::sampleframework::ColorRed);
                        }
                        else
                        {
                            GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                        }
                        GetFramework()->DrawString(0, line, "  -----");
                        GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                        line += FontSize;
                    }
                }
            }

            line = LowerScreenHeight - (FontSize * 9);
            GetFramework()->DrawString(0, line, "[ Manual ]");
            line += FontSize;
            GetFramework()->DrawString(0, line, "  UP/DOWN: Select Session");
            line += FontSize;
            GetFramework()->DrawString(0, line, "  X: Create New Session");
            line += FontSize;
            GetFramework()->DrawString(0, line, "  Y: Browse Session");
            line += FontSize;
            GetFramework()->DrawString(0, line, "  A: Join this Session");
            line += FontSize;
            GetFramework()->DrawString(0, line, "  B: Back");
            line += FontSize;
            GetFramework()->DrawString(0, line, "  LEFT : Use SessionInfoHistory [%s]", s_IsSessionInfoHistoryUsed ? "on" : "off");
            line += FontSize;
            if (g_NetworkType == nns::pia::NetworkType_Internet || g_NetworkType == nns::pia::NetworkType_Lan)
            {
                GetFramework()->DrawString(0, line, "  RIGHT: Use SessionIdHistory [%s]", s_IsSessionIdHistoryUsed ? "on" : "off");
                line += FontSize;
            }
            if (s_IsSessionInfoHistoryUsed || s_IsSessionIdHistoryUsed)
            {
                GetFramework()->DrawString(0, line, "  START: Clear SessionInfoHistory");
                line += FontSize;
            }

            GetFramework()->SwapBuffers();

            // 上画面
            GetFramework()->ChangeRenderTarget(true);
            GetFramework()->SwapBuffers();

            ++g_FrameNo;
            Dispatch();

            GetFramework()->EndFrame();

            // 接続状態チェック
            result = nn::pia::session::Session::GetInstance()->CheckConnectionError();
            if (result.IsFailure())
            {
                // セッション参加前に ResultSessionConnectionIsLost が返ることはない
                PIA_SAMPLE_ASSERT(result != nn::pia::ResultSessionConnectionIsLost());
                return GetNextSceneByErrorResult(result, SampleErrorType_ConnectionLost);
            }

        }  // while (command == MatchmakeCommand_None)

        if (command == MatchmakeCommand_Create)
        {
            result = nns::pia::Session_CreateSession(pCreateSessionSetting);
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT_RESULT(result);
                nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Create);
            }
            else
            {
                nextSceneNo = SampleSceneNo_Communication;
            }
        }
        if (command == MatchmakeCommand_Browse)
        {
            result = nns::pia::Session_BrowseSession(pSearchCriteria);
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT_RESULT(result);
                nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Browse);
            }
            else
            {
                nn::pia::session::ISessionInfoList* sessionInfoList = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList();
                for (nn::pia::session::ISessionInfoList::Iterator it = sessionInfoList->Begin(); it != sessionInfoList->End(); ++it)
                {
                    if (s_SessionInfoHistoryNum < MaxBrowsedSessionNum)
                    {
#if NNS_PIA_ENABLE_LOCAL
                        if (g_NetworkType == nns::pia::NetworkType_Local)
                        {
#if NN_PIA_CTR
                            nn::pia::local::UdsSessionInfo* pInfo = static_cast<nn::pia::local::UdsSessionInfo*>(*it);
#elif NN_PIA_NINTENDOSDK
                            nn::pia::local::LdnSessionInfo* pInfo = static_cast<nn::pia::local::LdnSessionInfo*>(*it);
#endif
                            s_LocalSessionInfoPtrArray[s_SessionInfoHistoryNum].Copy(*pInfo);
                        }
#endif
#if NNS_PIA_ENABLE_INET
                        if (g_NetworkType == nns::pia::NetworkType_Internet)
                        {
                            nn::pia::inet::NexSessionInfo* pInfo = static_cast<nn::pia::inet::NexSessionInfo*>(*it);
                            s_NexSessionInfoPtrArray[s_SessionInfoHistoryNum].Copy(*pInfo);
                            s_SessionIdBufferArray[s_SessionInfoHistoryNum] = pInfo->GetSessionId();
                        }
#endif
                        if (g_NetworkType == nns::pia::NetworkType_Lan)
                        {
                            nn::pia::lan::LanSessionInfo* pInfo = static_cast<nn::pia::lan::LanSessionInfo*>(*it);
                            s_LanSessionInfoPtrArray[s_SessionInfoHistoryNum].Copy(*pInfo);
                            s_SessionIdBufferArray[s_SessionInfoHistoryNum] = pInfo->GetSessionId();
                        }
                        ++s_SessionInfoHistoryNum;
                    }
                }
            }
        }
        if (command == MatchmakeCommand_BrowseOwner)
        {
#if NNS_PIA_ENABLE_INET
#if NN_PIA_NINTENDOSDK
            const nn::pia::PrincipalId TargetPrincipalId = 8255496466044702730;
#elif NN_PIA_CTR
            const nn::pia::PrincipalId TargetPrincipalId = 1800164835;
#elif NN_PIA_CAFE
            const nn::pia::PrincipalId TargetPrincipalId = 1799852706;
#else
            const nn::pia::PrincipalId TargetPrincipalId = 0;
#endif
            // マッチメイクセッションのオーナーを指定して検索します（指定するプリンシパルIDは適切な値に変更してください）
            nn::pia::inet::NexSessionSearchCriteriaOwner searchCriteria;
            searchCriteria.SetOwnerPrincipalId(TargetPrincipalId);
            searchCriteria.SetResultRange(0, MaxBrowsedSessionNum);

            result = nns::pia::Session_BrowseSession(&searchCriteria);
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT_RESULT(result);
                nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Browse);
            }
            else
            {
                nn::pia::session::ISessionInfoList* sessionInfoList = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList();
                for (nn::pia::session::ISessionInfoList::Iterator it = sessionInfoList->Begin(); it != sessionInfoList->End(); ++it)
                {
                    if (s_SessionInfoHistoryNum < MaxBrowsedSessionNum)
                    {
                        nn::pia::inet::NexSessionInfo* pInfo = static_cast<nn::pia::inet::NexSessionInfo*>(*it);
                        s_NexSessionInfoPtrArray[s_SessionInfoHistoryNum].Copy(*pInfo);
                        s_SessionIdBufferArray[s_SessionInfoHistoryNum] = pInfo->GetSessionId();
                        ++s_SessionInfoHistoryNum;
                    }
                }
            }
#endif
        }
        if (command == MatchmakeCommand_BrowseParticipant)
        {
#if NNS_PIA_ENABLE_INET
#if NN_PIA_NINTENDOSDK
            const nn::pia::PrincipalId TargetPrincipalId[3] = { 8255496466044702730, 803932690101079849, 4827284217682470995 };
#elif NN_PIA_CTR
            const nn::pia::PrincipalId TargetPrincipalId[3] = { 1800164835, 1800164834, 1800164809 };
#elif NN_PIA_CAFE
            const nn::pia::PrincipalId TargetPrincipalId[3] = { 1799852706, 1799916875, 1799916873 };
#else
            const nn::pia::PrincipalId TargetPrincipalId[3] = { 0, 0, 0 };
#endif
            // ユーザーを指定して検索します（指定するプリンシパルIDは適切な値に変更してください）
            nn::pia::inet::NexSessionSearchCriteriaParticipant searchCriteria;
            searchCriteria.SetParticipantPrincipalId(TargetPrincipalId, 3);

            result = nns::pia::Session_BrowseSession(&searchCriteria);
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT_RESULT(result);
                nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Browse);
            }
            else
            {
                nn::pia::session::ISessionInfoList* sessionInfoList = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList();
                for (nn::pia::session::ISessionInfoList::Iterator it = sessionInfoList->Begin(); it != sessionInfoList->End(); ++it)
                {
                    if (s_SessionInfoHistoryNum < MaxBrowsedSessionNum)
                    {
                        nn::pia::inet::NexSessionInfo* pInfo = static_cast<nn::pia::inet::NexSessionInfo*>(*it);
                        s_NexSessionInfoPtrArray[s_SessionInfoHistoryNum].Copy(*pInfo);
                        s_SessionIdBufferArray[s_SessionInfoHistoryNum] = pInfo->GetSessionId();
                        ++s_SessionInfoHistoryNum;
                    }
                }
            }
#endif
        }
        if (command == MatchmakeCommand_BrowseSessionId)
        {
#if NNS_PIA_ENABLE_INET
            const uint32_t TargetSessionId = 1084047;

            // セッション ID を指定して検索します（指定するセッション ID は適切な値に変更してください）
            nn::pia::inet::NexSessionSearchCriteriaSessionId searchCriteria;
            searchCriteria.SetSessionId(TargetSessionId);

            result = nns::pia::Session_BrowseSession(&searchCriteria);
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT_RESULT(result);
                nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Browse);
            }
            else
            {
                nn::pia::session::ISessionInfoList* sessionInfoList = nn::pia::session::Session::GetInstance()->GetBrowsedSessionInfoList();
                for (nn::pia::session::ISessionInfoList::Iterator it = sessionInfoList->Begin(); it != sessionInfoList->End(); ++it)
                {
                    if (s_SessionInfoHistoryNum < MaxBrowsedSessionNum)
                    {
                        nn::pia::inet::NexSessionInfo* pInfo = static_cast<nn::pia::inet::NexSessionInfo*>(*it);
                        s_NexSessionInfoPtrArray[s_SessionInfoHistoryNum].Copy(*pInfo);
                        s_SessionIdBufferArray[s_SessionInfoHistoryNum] = pInfo->GetSessionId();
                        ++s_SessionInfoHistoryNum;
                    }
                }
            }
#endif
        }
        if (command == MatchmakeCommand_Join)
        {
#if PIA_TEST_CODE
            nns::pia::RegisterEndFrameSampleSessionCallback(SampleSessionCallbackForEndFrame);
#endif
            nns::pia::RegisterDispatchSampleSessionCallback(SampleSessionCallbackForDispatch);
            result = nns::pia::Session_JoinSession(pJoinSessionSetting);
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT_RESULT(result);
                nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Join);
            }
            else
            {
                nextSceneNo = SampleSceneNo_Communication;
            }
#if PIA_TEST_CODE
            nns::pia::RegisterEndFrameSampleSessionCallback(NULL);
#endif
        }
        if (command == MatchmakeCommand_Back)
        {
#if NNS_PIA_ENABLE_LOCAL
            if (g_NetworkType == nns::pia::NetworkType_Local)
            {
                // ローカルの場合はスタートシーンに戻す
                CleanupPia();
                nns::pia::Net_FinalizeNetwork();
                FinalizePia();
                nextSceneNo = SampleSceneNo_Start;
            }
            else
#endif
            {
                nextSceneNo = GetNextSceneByErrorResult(nn::pia::ResultCancelled(), SampleErrorType_Unknown);
            }
        }
    }  // while (nextSceneNo == SampleSceneNo_JoinSession)

    return nextSceneNo;
}

SampleSceneNo SceneBrowseMatchmake()
{
    nn::pia::Result result;

    GetFramework()->SetProgramStatus(
        SampleSceneNo_JoinSession,
        false,
        false,
        0);

#if TEST_PIA_LOCAL_USER_PASSWORD
    // ローカルのユーザーパスワードを変更するためのテスト用コード
    if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonR))
    {
        g_UserPasswordType = 1;
    }
    else if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonL))
    {
        g_UserPasswordType = 2;
    }
    PIA_SAMPLE_PRINT("change UserPassword: %d\n", g_UserPasswordType);
#endif

    std::srand(static_cast<uint32_t>(nn::pia::common::Time::GetTime().GetTick()));

#if NNS_PIA_ENABLE_LOCAL
    if (g_NetworkType == nns::pia::NetworkType_Local)
    {
#if NN_PIA_CTR
        // 作成するセッションの設定
        SampleBeaconApplicationData beaconApplicationData;
        ::std::memset(&beaconApplicationData, 0x00, sizeof(beaconApplicationData));
        ::std::memcpy(beaconApplicationData.programName, ProgramNameString, sizeof(ProgramNameString));

        nn::pia::transport::Station::PlayerInfo playerInfo;
        nns::pia::Session_GetLocalPlayerInfo(&playerInfo, 1);

        // ascii 以外は ? に置換
        nns::pia::StringUtil::ConvertToAscii(beaconApplicationData.playerName, nn::pia::MaxPlayerNameLength, playerInfo.screenName);

        nn::pia::local::UdsCreateSessionSetting createSessionSetting;
        createSessionSetting.SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
        createSessionSetting.SetSubId(LocalSubId);
        createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
        createSessionSetting.SetApplicationData(&beaconApplicationData, sizeof(beaconApplicationData));
        createSessionSetting.SetMaxParticipantNum(GetMaxParticipantNum());
#if TEST_PIA_LOCAL_USER_PASSWORD
        // ローカルのユーザーパスワードを変更するためのテスト用コード
        if (g_UserPasswordType == 0)
        {
            PIA_SAMPLE_PRINT("Dont SetPass\n");
        }
        else if (g_UserPasswordType == 1)
        {
            createSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("1234")));
            PIA_SAMPLE_PRINT("SetPass: 1234\n");
        }
        else if (g_UserPasswordType == 2)
        {
            createSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("5678")));
            PIA_SAMPLE_PRINT("SetPass: 5678\n");
        }
#else
        createSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("1234")));
#endif

        // 参加するセッションの設定（SessionId 以外）
        nn::pia::local::UdsJoinSessionSetting joinSessionSetting;
#if TEST_PIA_LOCAL_USER_PASSWORD
        // ローカルのユーザーパスワードを変更するためのテスト用コード
        if (g_UserPasswordType == 0)
        {
            PIA_SAMPLE_PRINT("Dont SetPass\n");
        }
        else if (g_UserPasswordType == 1)
        {
            joinSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("1234")));
            PIA_SAMPLE_PRINT("SetPass: 1234\n");
        }
        else if (g_UserPasswordType == 2)
        {
            joinSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("5678")));
            PIA_SAMPLE_PRINT("SetPass: 5678\n");
        }
#else
        joinSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("1234")));
#endif

        result = joinSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, sizeof(LocalWirelessCryptoKey));
        PIA_SAMPLE_ASSERT(result.IsSuccess());

        // 検索条件の設定
        nn::pia::local::UdsSessionSearchCriteria searchCriteria;
        {
            searchCriteria.SetLocalCommunicationId(nn::pia::local::LocalFacade::GetInstance()->CreateLocalCommunicationId(LocalTitleId));
            searchCriteria.SetSubId(LocalSubId);
            searchCriteria.SetMaxParticipantNum(GetMaxParticipantNum());
            searchCriteria.SetVacantOnly(true);
            searchCriteria.SetOpenedOnly(true);
            searchCriteria.SetResultRange(0, MaxBrowsedSessionNum);
        }
#elif NN_PIA_NINTENDOSDK
        uint8_t wirelessCryptoKeySize = sizeof(LocalWirelessCryptoKey);
        if (wirelessCryptoKeySize > nn::pia::local::LdnWirelessCryptoKeySizeMax)
        {
            wirelessCryptoKeySize = nn::pia::local::LdnWirelessCryptoKeySizeMax;
        }

        // 作成するセッションの設定
        SampleBeaconApplicationData beaconApplicationData;
        memset(&beaconApplicationData, 0x00, sizeof(beaconApplicationData));
        memcpy(beaconApplicationData.programName, ProgramNameString, sizeof(ProgramNameString));

        nn::pia::transport::Station::PlayerInfo playerInfo;
        nns::pia::Session_GetLocalPlayerInfo(&playerInfo, 1);

        // ascii 以外は ? に置換
        nns::pia::StringUtil::ConvertToAscii(beaconApplicationData.playerName, nn::pia::MaxPlayerNameLength, playerInfo.screenName);

        PIA_SAMPLE_PRINT("createSessionSetting applicationDataSize: %d\n", sizeof(beaconApplicationData));

        nn::pia::local::LdnCreateSessionSetting createSessionSetting;
        createSessionSetting.SetLocalCommunicationId(nns::pia::LocalCommunicationId);
        createSessionSetting.SetSceneId(LocalSceneId);
        createSessionSetting.SetChannel(0);
        createSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, wirelessCryptoKeySize);
        createSessionSetting.SetApplicationData(&beaconApplicationData, sizeof(beaconApplicationData));
        createSessionSetting.SetMaxParticipantNum(GetMaxParticipantNum());
#if TEST_PIA_LOCAL_USER_PASSWORD
        // ローカルのユーザーパスワードを変更するためのテスト用コード
        if (g_UserPasswordType == 0)
        {
            PIA_SAMPLE_PRINT("Dont SetPass\n");
        }
        else if (g_UserPasswordType == 1)
        {
            createSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("1234")));
            PIA_SAMPLE_PRINT("SetPass: 1234\n");
        }
        else if (g_UserPasswordType == 2)
        {
            createSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("5678")));
            PIA_SAMPLE_PRINT("SetPass: 5678\n");
        }
#else
        createSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("1234")));
#endif

        // 参加するセッションの設定
        nn::pia::local::LdnJoinSessionSetting joinSessionSetting;
        joinSessionSetting.SetWirelessCryptoKey(LocalWirelessCryptoKey, wirelessCryptoKeySize);
#if TEST_PIA_LOCAL_USER_PASSWORD
        // ローカルのユーザーパスワードを変更するためのテスト用コード
        if (g_UserPasswordType == 0)
        {
            PIA_SAMPLE_PRINT("Dont SetPass\n");
        }
        else if (g_UserPasswordType == 1)
        {
            joinSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("1234")));
            PIA_SAMPLE_PRINT("SetPass: 1234\n");
        }
        else if (g_UserPasswordType == 2)
        {
            joinSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("5678")));
            PIA_SAMPLE_PRINT("SetPass: 5678\n");
        }
#else
        joinSessionSetting.SetSessionUserPassword(nn::pia::common::String(SAMPLE_STRING("1234")));
#endif

        // 検索条件の設定
        nn::pia::local::LdnSessionSearchCriteria searchCriteria;
        {
            searchCriteria.SetScanFilterType(nn::pia::local::LdnScanFilterType_LocalCommunicationIdAndSceneId);
            searchCriteria.SetLocalCommunicationId(nns::pia::LocalCommunicationId);
            searchCriteria.SetSceneId(LocalSceneId);
            searchCriteria.SetChannel(0);
            searchCriteria.SetMaxParticipantNum(GetMaxParticipantNum());
            searchCriteria.SetVacantOnly(true);
            searchCriteria.SetOpenedOnly(true);
            searchCriteria.SetResultRange(0, MaxBrowsedSessionNum);
        }
#endif
        return SceneBrowseMatchmakeLoop(&createSessionSetting, &joinSessionSetting, &searchCriteria);
    }
    else
#endif
    {
        if (g_NetworkType == nns::pia::NetworkType_Lan)
        {
            // LAN マッチメイクの場合
            nn::pia::lan::LanCreateSessionSetting createSessionSetting;

            // 作成するマッチメイクセッションの設定
            createSessionSetting.SetSessionType(nn::pia::lan::LanSessionType_Anybody);
            createSessionSetting.SetGameMode(MatchmakeGameMode + MatchmakeType_Browse + GetIncrementalGameModeValueByBitRateCheckMode());
            createSessionSetting.SetMaxParticipantNum(GetMaxParticipantNum());
            createSessionSetting.SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
            createSessionSetting.SetAttribute(MatchmakeAttributeIndexNetworkTopology, g_NetworkTopologyType);
            createSessionSetting.SetOpenSession(true);
            uint8_t lanApplicationDataBuffer[nn::pia::lan::LanApplicationDataBufferSizeMax];

            for (uint32_t i = 0; i < nn::pia::lan::LanApplicationDataBufferSizeMax; i++)
            {
                lanApplicationDataBuffer[i] = static_cast<uint8_t>(std::rand());
            }
            createSessionSetting.SetApplicationData(lanApplicationDataBuffer, sizeof(lanApplicationDataBuffer));
            // 設定したアプリケーションデータのチェック
            {
                uint8_t checkBuffer[nn::pia::lan::LanApplicationDataBufferSizeMax];
                createSessionSetting.GetApplicationData(checkBuffer, sizeof(checkBuffer));
                PIA_SAMPLE_ASSERT(createSessionSetting.GetApplicationDataSize() == nn::pia::lan::LanApplicationDataBufferSizeMax);
                PIA_SAMPLE_ASSERT(!memcmp(checkBuffer, lanApplicationDataBuffer, nn::pia::lan::LanApplicationDataBufferSizeMax));
            }

            // セッション参加時の設定
            nn::pia::lan::LanJoinSessionSetting joinSessionSetting;

            // 検索条件の設定
            nn::pia::lan::LanSessionSearchCriteria searchCriteriaList[2];
            // Attribute 単体指定
            searchCriteriaList[0].SetGameMode(MatchmakeGameMode + MatchmakeType_Browse + GetIncrementalGameModeValueByBitRateCheckMode());
            searchCriteriaList[0].SetMaxParticipantNum(GetMaxParticipantNum());
            searchCriteriaList[0].SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
            searchCriteriaList[0].SetAttribute(MatchmakeAttributeIndexNetworkTopology, g_NetworkTopologyType);
            searchCriteriaList[0].SetResultRange(0, MaxBrowsedSessionNum);

            // Attribute レンジ指定
            // (複数指定方法を示すコード例)
            searchCriteriaList[1].SetGameMode(MatchmakeGameMode + MatchmakeType_Browse + GetIncrementalGameModeValueByBitRateCheckMode());
            searchCriteriaList[1].SetMaxParticipantNumRange(GetMaxParticipantNum(), GetMaxParticipantNum());
            searchCriteriaList[1].SetAttributeRange(MatchmakeAttributeIndex, 2, 72);
            searchCriteriaList[1].SetResultRange(0, MaxBrowsedSessionNum);

            return SceneBrowseMatchmakeLoop(&createSessionSetting, &joinSessionSetting, searchCriteriaList);
        }
#if NNS_PIA_ENABLE_INET
        else
        {
            // 作成するマッチメイクセッションの設定
            nn::pia::inet::NexCreateSessionSetting createSessionSetting;

            createSessionSetting.SetSessionType(nn::pia::inet::SessionType_Anybody);
            createSessionSetting.SetGameMode(MatchmakeGameMode + MatchmakeType_Browse + GetIncrementalGameModeValueByBitRateCheckMode());
            createSessionSetting.SetMaxParticipantNum(GetMaxParticipantNum());
            createSessionSetting.SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
            createSessionSetting.SetAttribute(MatchmakeAttributeIndexNetworkTopology, g_NetworkTopologyType);
            createSessionSetting.SetOpenSession(true);
            // コミュニティ ID を設定
            if (GetJoinedCommunityId() != 0)
            {
                createSessionSetting.SetCommunityId(GetJoinedCommunityId());
            }
            if (g_MatchmakeKeywordType != MatchmakeKeywordType_None)
            {
                createSessionSetting.SetSessionMatchmakeKeyword(SampleMatchmakeKeyword[g_MatchmakeKeywordType]);
            }

            uint8_t nexApplicationDataBuffer[nn::pia::inet::ApplicationDataBufferSizeMax];

            for (uint32_t i = 0; i < nn::pia::inet::ApplicationDataBufferSizeMax; i++)
            {
                nexApplicationDataBuffer[i] = static_cast<uint8_t>(std::rand());
            }
            createSessionSetting.SetApplicationData(nexApplicationDataBuffer, sizeof(nexApplicationDataBuffer));
            // 設定したアプリケーションデータのチェック
            {
                uint8_t checkBuffer[nn::pia::inet::ApplicationDataBufferSizeMax];
                createSessionSetting.GetApplicationData(checkBuffer, sizeof(checkBuffer));
                PIA_SAMPLE_ASSERT(createSessionSetting.GetApplicationDataSize() == nn::pia::inet::ApplicationDataBufferSizeMax);
                PIA_SAMPLE_ASSERT(!memcmp(checkBuffer, nexApplicationDataBuffer, nn::pia::inet::ApplicationDataBufferSizeMax));
            }

            // セッション参加時の設定
            nn::pia::inet::NexJoinSessionSetting joinSessionSetting;
            if (GetSessionId() != nn::pia::session::InvalidSessionId)
            {
                joinSessionSetting.SetSessionId(GetSessionId());
                joinSessionSetting.SetSessionSystemPassword(GetSystemPassword());
            }

            // 検索条件の設定
            nn::pia::inet::NexSessionSearchCriteria searchCriteriaList[2];

            // Attribute 単体指定
            searchCriteriaList[0].SetGameMode(MatchmakeGameMode + MatchmakeType_Browse + GetIncrementalGameModeValueByBitRateCheckMode());
            searchCriteriaList[0].SetMaxParticipantNum(GetMaxParticipantNum());
            searchCriteriaList[0].SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
            searchCriteriaList[0].SetAttribute(MatchmakeAttributeIndexNetworkTopology, g_NetworkTopologyType);
            searchCriteriaList[0].SetResultRange(0, MaxBrowsedSessionNum);
            if (GetJoinedCommunityId() != 0)
            {
                searchCriteriaList[0].SetCommunityId(GetJoinedCommunityId());
            }
            if (g_MatchmakeKeywordType != MatchmakeKeywordType_None)
            {
                searchCriteriaList[0].SetSessionMatchmakeKeyword(SampleMatchmakeKeyword[g_MatchmakeKeywordType]);
            }

            // Attribute レンジ指定
            // (複数指定方法を示すコード例)
            searchCriteriaList[1].SetGameMode(MatchmakeGameMode + MatchmakeType_Browse + GetIncrementalGameModeValueByBitRateCheckMode());
            searchCriteriaList[1].SetMaxParticipantNumRange(GetMaxParticipantNum(), GetMaxParticipantNum());
            searchCriteriaList[1].SetAttributeRange(MatchmakeAttributeIndex, 2, 72);
            searchCriteriaList[1].SetResultRange(0, MaxBrowsedSessionNum);
            if (GetJoinedCommunityId() != 0)
            {
                searchCriteriaList[1].SetCommunityId(GetJoinedCommunityId());
            }
            if (g_MatchmakeKeywordType != MatchmakeKeywordType_None)
            {
                searchCriteriaList[1].SetSessionMatchmakeKeyword(SampleMatchmakeKeyword[g_MatchmakeKeywordType]);
            }

            return SceneBrowseMatchmakeLoop(&createSessionSetting, &joinSessionSetting, searchCriteriaList);
        }
#else
        nns::pia::Session_CleanupSession();
        return SampleSceneNo_SelectSessionType;
#endif
    }
}
