﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/

#include "SampleConfig.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Session_Setup.h"

#if NNS_PIA_ENABLE_INET
#include <nn/pia/inet.h>
#endif

namespace
{
static uint32_t s_JoinedCommunityId = 0;
}

uint32_t GetJoinedCommunityId()
{
    return s_JoinedCommunityId;
}

SampleSceneNo SceneCommunity()
{
    GetFramework()->SetProgramStatus(
        SampleSceneNo_Community,
        false,
        false,
        0);

#if NNS_PIA_ENABLE_INET
    SampleSceneNo nextSceneNo = SampleSceneNo_Community;
    nn::pia::Result result;

    CommunityCommand command = CommunityCommand_None;

    uint8_t cursolRow = 0;
    uint8_t currentMaxRow = 0;
    const uint8_t RowMax = 17;
    const uint32_t ApplicationDataBufferSize = 16;
    nn::pia::session::ICommunityInfo* targetCommunityInfo = NULL;

    nn::pia::PrincipalId localPrincipalId = nn::pia::inet::NexFacade::GetInstance()->GetLocalPrincipalId();
    PIA_SAMPLE_PRINT("Local PID:%llu\n", localPrincipalId);

    bool isAsyncProcessing = false;
    while (nextSceneNo == SampleSceneNo_Community)
    {
        GetFramework()->BeginFrame();

        // コミュニティリスト取得
        nn::pia::session::ICommunityInfoList* pCommunityList = nn::pia::session::Session::GetInstance()->GetBrowsedCommunityInfoList();
        currentMaxRow = pCommunityList->GetSize() > RowMax ? RowMax : static_cast<uint8_t>(pCommunityList->GetSize());

        if (isAsyncProcessing)
        {
            if (command == CommunityCommand_Create)
            {
                if (nn::pia::session::Session::GetInstance()->IsCreateCommunityCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetCreateCommunityResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("Create Community Success.\n");
                    }
                    else
                    {
                        PIA_TRACE_RESULT_ALWAYS(result);
                        PIA_SAMPLE_PRINT("Create Community Failed.\n");
                    }
                    isAsyncProcessing = false;
                    command = CommunityCommand_None;
                }
            }
            else if (command == CommunityCommand_Join)
            {
                if (nn::pia::session::Session::GetInstance()->IsJoinCommunityCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetJoinCommunityResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("Join Community Success.\n");
                    }
                    else
                    {
                        PIA_TRACE_RESULT_ALWAYS(result);
                        PIA_SAMPLE_PRINT("Join Community Failed.\n");
                    }
                    isAsyncProcessing = false;
                    command = CommunityCommand_None;
                }
            }
            else if (command == CommunityCommand_Update)
            {
                if (nn::pia::session::Session::GetInstance()->IsUpdateCommunityCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetUpdateCommunityResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("Update Community Success.\n");
                    }
                    else
                    {
                        PIA_TRACE_RESULT_ALWAYS(result);
                        PIA_SAMPLE_PRINT("Update Community Failed.\n");
                    }
                    isAsyncProcessing = false;
                    command = CommunityCommand_None;
                }
            }
            else if (command == CommunityCommand_Leave)
            {
                if (nn::pia::session::Session::GetInstance()->IsLeaveCommunityCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetLeaveCommunityResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("Leave Community Success.\n");
                    }
                    else
                    {
                        PIA_TRACE_RESULT_ALWAYS(result);
                        PIA_SAMPLE_PRINT("Leave Community Failed.\n");
                    }
                    isAsyncProcessing = false;
                    command = CommunityCommand_None;
                }
            }
            else if (command == CommunityCommand_Destroy)
            {
                if (nn::pia::session::Session::GetInstance()->IsDestroyCommunityCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetDestroyCommunityResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("Destroy Community Success.\n");
                    }
                    else
                    {
                        PIA_TRACE_RESULT_ALWAYS(result);
                        PIA_SAMPLE_PRINT("Destroy Community Failed.\n");
                    }
                    isAsyncProcessing = false;
                    command = CommunityCommand_None;
                }
            }
            else if (command == CommunityCommand_Browse)
            {
                if (nn::pia::session::Session::GetInstance()->IsBrowseCommunityCompleted())
                {
                    result = nn::pia::session::Session::GetInstance()->GetBrowseCommunityResult();
                    if (result.IsSuccess())
                    {
                        PIA_SAMPLE_PRINT("Browse Community Success.\n");
                    }
                    else
                    {
                        PIA_TRACE_RESULT_ALWAYS(result);
                        PIA_SAMPLE_PRINT("Browse Community Failed.\n");
                    }
                    isAsyncProcessing = false;
                    command = CommunityCommand_None;

                    // コミュニティリスト更新
                    pCommunityList = nn::pia::session::Session::GetInstance()->GetBrowsedCommunityInfoList();
                    currentMaxRow = pCommunityList->GetSize() > RowMax ? RowMax : static_cast<uint8_t>(pCommunityList->GetSize());
                    cursolRow = 0;
                    if (pCommunityList->GetSize() == 0)
                    {
                        targetCommunityInfo = NULL;
                    }
                }
            }
        }
        else
        {
            // B: 戻る
            // B+L: 離脱
            // B+R: 破棄
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonB))
            {
                if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonL))
                {
                    if (targetCommunityInfo != NULL)
                    {
                        result = nn::pia::session::Session::GetInstance()->LeaveCommunityAsync(targetCommunityInfo->GetCommunityId());
                        if (result.IsSuccess())
                        {
                            command = CommunityCommand_Leave;
                            isAsyncProcessing = true;
                        }
                        else
                        {
                            PIA_TRACE_RESULT_ALWAYS(result);
                            PIA_SAMPLE_PRINT("LeaveCommunityAsync failed.\n");
                        }
                    }
                }
                else if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonR))
                {
                    if (targetCommunityInfo != NULL)
                    {
                        if (targetCommunityInfo->GetOwnerPrincipalId() == localPrincipalId)
                        {
                            result = nn::pia::session::Session::GetInstance()->DestroyCommunityAsync(targetCommunityInfo->GetCommunityId());
                            if (result.IsSuccess())
                            {
                                command = CommunityCommand_Destroy;
                                isAsyncProcessing = true;
                            }
                            else
                            {
                                PIA_TRACE_RESULT_ALWAYS(result);
                                PIA_SAMPLE_PRINT("DestroyCommunityAsync failed.\n");
                            }
                        }
                    }
                }
                else
                {
                    nextSceneNo = SampleSceneNo_SelectSessionType;
                }
            }
            // A: コミュニティに参加
            // A+L: コミュニティ更新
            else if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                if (targetCommunityInfo != NULL)
                {
                    if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonL))
                    {
                        nn::pia::inet::NexUpdateCommunitySetting communitySetting;

                        // 更新するコミュニティIDをセット
                        communitySetting.SetCommunityId(targetCommunityInfo->GetCommunityId());

                        // 更新しない項目は現在の値をセット
                        communitySetting.SetMinParticipantNum(targetCommunityInfo->GetMinParticipantNum());
                        communitySetting.SetMaxParticipantNum(targetCommunityInfo->GetMaxParticipantNum());
                        uint32_t attributes[nn::pia::inet::AttributeSizeMax];
                        for (uint32_t i = 0; i < nn::pia::inet::AttributeSizeMax; ++i)
                        {
                            targetCommunityInfo->GetAttribute(i, &attributes[i]);
                        }
                        communitySetting.SetAttributes(attributes);

                        // 更新する項目をセット
                        nn::pia::common::DateTime startDateTime(2017, 4, 25, 0, 0, 0);
                        nn::pia::common::DateTime endDateTime(2017, 6, 25, 12, 34, 56);
                        communitySetting.SetParticipationStartDate(startDateTime);
                        communitySetting.SetParticipationEndDate(endDateTime);

                        // ユーザーパスワードが設定されている場合のみパスワードは更新可能
                        if (targetCommunityInfo->IsRestrictedByPassword())
                        {
                            communitySetting.SetCommunityUserPassword(SAMPLE_STRING("password2"));
                        }
                        communitySetting.SetDescription(SAMPLE_STRING("description2"));

                        result = nn::pia::session::Session::GetInstance()->UpdateCommunityAsync(&communitySetting);
                        if (result.IsSuccess())
                        {
                            command = CommunityCommand_Update;
                            isAsyncProcessing = true;
                        }
                        else
                        {
                            PIA_TRACE_RESULT_ALWAYS(result);
                            PIA_SAMPLE_PRINT("UpdateCommunityAsync failed.\n");
                        }
                    }
                    else
                    {
                        nn::pia::inet::NexJoinCommunitySetting joinCommunitySetting;
                        joinCommunitySetting.SetCommunityId(targetCommunityInfo->GetCommunityId());
                        if (targetCommunityInfo->IsRestrictedByPassword())
                        {
                            joinCommunitySetting.SetCommunityUserPassword(SAMPLE_STRING("password"));
                        }
                        {
                            const uint32_t sampleDescriptionSize = 16;
                            char sampleDescription[sampleDescriptionSize];
                            NN_PIA_UNUSED_IN_RELEASE(sampleDescription);

                            const SampleStringChar_t* pStr = static_cast<const SampleStringChar_t*>(targetCommunityInfo->GetDescription().GetBuffer());
                            for (uint32_t i = 0; i < targetCommunityInfo->GetDescription().GetStringLength(); i++)
                            {
                                if (*pStr >= 0x20 && *pStr <= 0x7F)
                                {
                                    sampleDescription[i] = static_cast<char>(*pStr);
                                }
                                else
                                {
                                    sampleDescription[i] = '?';
                                }
                                ++pStr;
                            }
                            PIA_SAMPLE_PRINT("Join community. Id:%d Code:%llu Desc:%s\n",
                                             targetCommunityInfo->GetCommunityId(),
                                             nn::pia::session::Session::GetInstance()->ConvertCommunityIdToCommunityCode(targetCommunityInfo->GetCommunityId()),
                                             sampleDescription);
                        }
                        result = nn::pia::session::Session::GetInstance()->JoinCommunityAsync(&joinCommunitySetting);
                        if (result.IsSuccess())
                        {
                            command = CommunityCommand_Join;
                            isAsyncProcessing = true;
                            PIA_SAMPLE_PRINT("JoinCommunityAsync Success.\n");
                        }
                        else
                        {
                            PIA_TRACE_RESULT_ALWAYS(result);
                            PIA_SAMPLE_PRINT("JoinCommunityAsync failed.\n");
                        }
                    }
                }
            }
            // Y: 自身が参加しているコミュニティ検索
            // Y+L: 公式コミュニティ検索
            // Y+R: コミュニティID 検索
            // Y+START: 自身がオーナーのコミュニティ検索
            else if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                const uint32_t communityArraySize = 16;
                uint64_t communityCodeArray[communityArraySize] = {
                    45870251919276, 66743792977837, 6910603577262, 33371897087920,
                    67654326044663, 10402411988984, 36734856480761, 38719131371514,
                    15015206864891, 25378962950141, 10857678522364, 32611687876606,
                    58299886139628, 31911608181857, 30867931128957, 35424891434855
                };
                uint32_t communityIdArray[communityArraySize];
                for (int i = 0; i < communityArraySize; ++i)
                {
                    communityIdArray[i] = nn::pia::session::Session::GetInstance()->ConvertCommunityCodeToCommunityId(communityCodeArray[i]);
                    PIA_SAMPLE_PRINT("Id[%d]: %d (%llu)\n", i, communityIdArray[i], communityCodeArray[i]);
                }
                nn::pia::inet::NexCommunitySearchCriteriaCommunityId searchCriteriaCommunityId(communityIdArray, communityArraySize);
                nn::pia::inet::NexCommunitySearchCriteriaOfficial searchCriteriaOfficial(true, 0, 16);
                nn::pia::inet::NexCommunitySearchCriteriaParticipant searchCriteriaParticipant(localPrincipalId, 0, 16);
                nn::pia::inet::NexCommunitySearchCriteriaOwner searchCriteriaOwner(localPrincipalId, 0, 16);
                nn::pia::session::CommunitySearchCriteria* pSearchCriteria;

                if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonL))
                {
                    pSearchCriteria = &searchCriteriaOfficial;
                }
                else if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonR))
                {
                    pSearchCriteria = &searchCriteriaCommunityId;
                }
                else if (GetFramework()->IsHold(nns::pia::sampleframework::ButtonStart))
                {
                    pSearchCriteria = &searchCriteriaOwner;
                }
                else
                {
                    pSearchCriteria = &searchCriteriaParticipant;
                }
                pSearchCriteria->SetResultRange(0, 16);
                // 指定された検索条件でコミュニティを検索
                result = nn::pia::session::Session::GetInstance()->BrowseCommunityAsync(pSearchCriteria);
                if (result.IsSuccess())
                {
                    command = CommunityCommand_Browse;
                    isAsyncProcessing = true;
                    PIA_SAMPLE_PRINT("BrowseCommunityAsync Success.\n");
                }
                else
                {
                    PIA_TRACE_RESULT_ALWAYS(result);
                    PIA_SAMPLE_PRINT("BrowseCommunityAsync failed.\n");
                }
            }
            // X: コミュニティ作成
            else if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonX))
            {
                nn::pia::inet::NexCreateCommunitySetting communitySetting;
                nn::pia::common::DateTime startDateTime(2015, 12, 25, 0, 0, 0);
                nn::pia::common::DateTime endDateTime(2017, 12, 25, 12, 34, 56);

                communitySetting.SetParticipationStartDate(startDateTime);
                communitySetting.SetParticipationEndDate(endDateTime);

                communitySetting.SetCommunityUserPassword(SAMPLE_STRING("password"));
                communitySetting.SetDescription(SAMPLE_STRING("description"));

                result = nn::pia::session::Session::GetInstance()->CreateCommunityAsync(&communitySetting);
                if (result.IsSuccess())
                {
                    command = CommunityCommand_Create;
                    isAsyncProcessing = true;
                }
                else
                {
                    PIA_TRACE_RESULT_ALWAYS(result);
                    PIA_SAMPLE_PRINT("CreateCommunityAsync failed.\n");
                }
            }
            else if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                cursolRow = (cursolRow + currentMaxRow - 1) % currentMaxRow;
            }
            else if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                cursolRow = ++cursolRow % currentMaxRow;
            }
        }

        // 下画面
        GetFramework()->ChangeRenderTarget(false);
        GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        int line = 0;
        GetFramework()->DrawString(0, line, "[%s Demo] %d", ProgramNameString, g_FrameNo);
        line += FontSize;
        GetFramework()->DrawString(0, line, "  Scene: Community.");
        line += FontSize;
        GetFramework()->DrawString(0, line, "[ Community List ] Found:%01d ", pCommunityList->GetSize());
        line += FontSize;

        nn::pia::session::ICommunityInfoList::Iterator it = pCommunityList->Begin();
        for (int i = 0; i < currentMaxRow; ++i)
        {
            if (it != pCommunityList->End())
            {
                GetFramework()->DrawString(0,
                                           line,
                                           "%s Code:%llu Owner:%llu [%d:%d-%d] descSize:%d",
                                           i == cursolRow ? ">" : " ",
                                           nn::pia::session::Session::GetInstance()->ConvertCommunityIdToCommunityCode((*it)->GetCommunityId()),
                                           (*it)->GetOwnerPrincipalId(),
                                           (*it)->GetCurrentParticipantNum(),
                                           (*it)->GetMinParticipantNum(),
                                           (*it)->GetMaxParticipantNum(),
                                           (*it)->GetDescription().GetStringLength());

                line += FontSize;

                if (i == cursolRow)
                {
                    targetCommunityInfo = (*it);
                }
            }
            ++it;
        }

        line = LowerScreenHeight - (FontSize * 11);
        GetFramework()->DrawString(0, line, "[ Manual ]");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  X:       Create community");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  Y:       Browse community by participant");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  Y+L:     Browse official community");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  Y+R:     Browse community by communityId");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  Y+START: Browse community by owner");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  A:       Join community");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  A+L:     Update community");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  B+L:     Leave community");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  B+R:     Destroy community");
        line += FontSize;
        GetFramework()->DrawString(0, line, "  B:       Back");
        line += FontSize;

        GetFramework()->SwapBuffers();

        // 上画面
        GetFramework()->ChangeRenderTarget(true);
        line = FontSize;

        if (targetCommunityInfo != NULL)
        {
            // コミュニティコードとオーナーPID
            GetFramework()->DrawString(0,
                                       line,
                                       "Code:%llu OwnerPID:%llu",
                                       nn::pia::session::Session::GetInstance()->ConvertCommunityIdToCommunityCode(targetCommunityInfo->GetCommunityId()),
                                       targetCommunityInfo->GetOwnerPrincipalId());
            line += FontSize;

            // ApplicationData
            uint8_t applicationData[ApplicationDataBufferSize];
            uint32_t appDataSize = 0;
            if (targetCommunityInfo->GetApplicationDataSize(&appDataSize).IsFailure())
            {
                appDataSize = 0;
            }
            uint32_t size = (appDataSize < sizeof(applicationData)) ? appDataSize : sizeof(applicationData);
            result = targetCommunityInfo->GetApplicationData(applicationData, size);
            if (result.IsFailure())
            {
                memset(applicationData, 0, sizeof(applicationData));
            }
            GetFramework()->DrawString(0, line, "  appDataSize:%d", size);
            line += FontSize;

            //
            GetFramework()->DrawString(0,
                                       line,
                                       "  Official:%s, Password:%s, RelatedSessionCount:%d",
                                       targetCommunityInfo->IsOfficialCommunity() ? "t" : "f",
                                       targetCommunityInfo->IsRestrictedByPassword() ? "t" : "f",
                                       targetCommunityInfo->GetRelatedSessionCount());
            line += FontSize;

            // 参加開始日時
            const nn::pia::common::DateTime& startDateTime = targetCommunityInfo->GetParticipationStartDate();
            if (!startDateTime.IsRegistered())
            {
                GetFramework()->DrawString(0, line, "  Start Participation Time: No Condition.");
            }
            else
            {
                GetFramework()->DrawString(0,
                                           line,
                                           "  Start Participation Time: %d/%d/%d %d-%d-%d.",
                                           startDateTime.GetYear(),
                                           startDateTime.GetMonth(),
                                           startDateTime.GetDay(),
                                           startDateTime.GetHour(),
                                           startDateTime.GetMinute(),
                                           startDateTime.GetSecond());
            }
            line += FontSize;

            // 参加終了日時
            const nn::pia::common::DateTime& endDateTime = targetCommunityInfo->GetParticipationEndDate();
            if (!endDateTime.IsRegistered())
            {
                GetFramework()->DrawString(0, line, "  End Participation Time: No Condition.");
            }
            else
            {
                GetFramework()->DrawString(0,
                                           line,
                                           "  End Participation Time: %d/%d/%d %d-%d-%d.",
                                           endDateTime.GetYear(),
                                           endDateTime.GetMonth(),
                                           endDateTime.GetDay(),
                                           endDateTime.GetHour(),
                                           endDateTime.GetMinute(),
                                           endDateTime.GetSecond());
            }
            line += FontSize;

            for (uint32_t i = 0; i < nn::pia::inet::AttributeSizeMax; ++i)
            {
                uint32_t attributeBuf;
                targetCommunityInfo->GetAttribute(static_cast<uint8_t>(i), &attributeBuf);
                GetFramework()->DrawString(0, line, "  Attribute[%d]: %d", i, attributeBuf);
                line += FontSize;
            }
        }
        GetFramework()->SwapBuffers();

        ++g_FrameNo;
        Dispatch();

        GetFramework()->EndFrame();

        // 接続状態チェック
        if (!nns::pia::Net_IsConnected())
        {
            PIA_SAMPLE_PRINT("nns::pia::Inet_IsConnected Disconnected \n");
            SetCommunicationErrorType(SampleErrorType_ConnectionLost);
            nextSceneNo = SampleSceneNo_Error;
        }
    }

    nns::pia::Session_CleanupSession();
    return nextSceneNo;
#else
    nns::pia::Session_CleanupSession();
    return SampleSceneNo_SelectSessionType;
#endif
}
