﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"

#if NNS_PIA_ENABLE_INET && NN_PIA_UNDER_DEVELOP

#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include <nn/pia/inet/inet_NexDebugSession.h>

#endif

namespace
{
#if NNS_PIA_ENABLE_INET && NN_PIA_UNDER_DEVELOP
const int DummySessionMaxSize = 20;
uint32_t s_DummySessionIdArray[DummySessionMaxSize];
uint32_t s_DummySessionIdSize = 0;

const int DummySessionCommand_Create = 1;
const int DummySessionCommand_Destroy = 2;
const int DummySessionCommand_Update = 3;
const int DummySessionCommand_Back = 4;
#endif
};

SampleSceneNo SceneDummySession()
{
    SampleSceneNo nextSceneNo = SampleSceneNo_DummySession;

    GetFramework()->SetProgramStatus(SampleSceneNo_DummySession, false, false, 0);

#if NNS_PIA_ENABLE_INET && NN_PIA_UNDER_DEVELOP
    nn::pia::Result result;

    bool isLoggedIn = false;

    // NEX の初期化
    result = nns::pia::Nex_InitializeNex(
        MemoryBlockSizeForNex,
        MemoryBlockSizeForNexEmergency);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    PIA_SAMPLE_PRINT("Nex_InitializeNex is success.\n");

    // PiaInet の初期化
    result = nn::pia::inet::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::inet::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    // NexFacade のシングルトン生成
    result = nn::pia::inet::NexFacade::CreateInstance();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    // NexDebugSession のシングルトン生成
    result = nn::pia::inet::NexDebugSession::CreateInstance();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::inet::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    PIA_SAMPLE_PRINT("PiaInet Initialize is success.\n");

    // ネットワーク接続の初期化
    nns::pia::NetResult netResult = nns::pia::Net_InitializeNetwork(nns::pia::NetworkType_Internet);
    if (netResult != nns::pia::NetResult_Success)
    {
        PIA_SAMPLE_PRINT("Net_InitializeNetwork is failed.\n");
        nextSceneNo = SampleSceneNo_Start;
    }
    // ゲームサーバーへのログイン
    else if (!nns::pia::Inet_Login())
    {
        PIA_SAMPLE_PRINT("Inet_Login is failed.\n");
        nextSceneNo = SampleSceneNo_Start;
    }
    else
    {
        PIA_SAMPLE_PRINT("Inet_Login is success.\n");
        isLoggedIn = true;
        // ゲームサーバーへのログインに成功したら NexDebugSession をスタートアップ
        result = nn::pia::inet::NexDebugSession::GetInstance()->Startup();
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
        PIA_SAMPLE_PRINT("NexDebugSession is started up.\n");
    }

    while (nextSceneNo == SampleSceneNo_DummySession)
    {
        int command = 0;
        while (command == 0)
        {
            GetFramework()->BeginFrame();

            // A: 新規にダミーセッションを構築
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                command = DummySessionCommand_Create;
            }
            // X: ダミーセッションを破棄
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonX))
            {
                command = DummySessionCommand_Destroy;
            }
            // Y: ダミーセッションの設定更新
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                command = DummySessionCommand_Update;
            }
            // B: 戻る
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonB))
            {
                command = DummySessionCommand_Back;
                nextSceneNo = SampleSceneNo_Start;
            }

            // 下画面
            GetFramework()->ChangeRenderTarget(false);
            GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
            int line = 0;
            GetFramework()->DrawString(0, line, "[%s Demo] %d", ProgramNameString, g_FrameNo);
            line += FontSize;
            GetFramework()->DrawString(0, line, "  Scene: DummySession.");
            line += FontSize;
            line += FontSize;

            line = LowerScreenHeight - (FontSize * 5);
            GetFramework()->DrawString(0, line, "[ Manual ]");
            line += FontSize;
            GetFramework()->DrawString(0, line, "  A: Create DummySession");
            line += FontSize;
            GetFramework()->DrawString(0, line, "  X: Destroy DummySession");
            line += FontSize;
            GetFramework()->DrawString(0, line, "  Y: Update DummySession");
            line += FontSize;
            GetFramework()->DrawString(0, line, "  B: Back");
            line += FontSize;

            GetFramework()->SwapBuffers();

            // 上画面
            GetFramework()->ChangeRenderTarget(true);
            GetFramework()->SwapBuffers();

            GetFramework()->EndFrame();
            ++g_FrameNo;

            Dispatch();

            // 接続状態チェック
            if (!nns::pia::Net_IsConnected())
            {
                PIA_SAMPLE_PRINT("nns::pia::Inet_IsConnected Disconnected \n");
                nextSceneNo = SampleSceneNo_Start;
                break;
            }
        }
        if (command == DummySessionCommand_Create)
        {
                // 作成するマッチメイクセッションの設定
                nn::pia::inet::NexCreateSessionSetting createSessionSetting;

                createSessionSetting.SetSessionType(nn::pia::inet::SessionType_Anybody);
                createSessionSetting.SetGameMode(MatchmakeGameMode + MatchmakeType_Browse + GetIncrementalGameModeValueByBitRateCheckMode());
                createSessionSetting.SetMaxParticipantNum(GetMaxParticipantNum());
                createSessionSetting.SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
                createSessionSetting.SetAttribute(MatchmakeAttributeIndexNetworkTopology, g_NetworkTopologyType);
                createSessionSetting.SetOpenSession(true);

                result = nn::pia::inet::NexDebugSession::GetInstance()->CreateDummySessionAsync(createSessionSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                // CreateDummySessionAsync の非同期処理完了待ち
                int loopCounter = 0;
                while (true)
                {
                    GetFramework()->BeginFrame();
                    GetFramework()->DrawString(10, 20, "creating dummy session... %d", loopCounter++);
                    GetFramework()->EndFrame();
                    ++g_FrameNo;

                    Dispatch();
                    if (nn::pia::inet::NexDebugSession::GetInstance()->IsCreateDummySessionCompleted())
                    {
                        result = nn::pia::inet::NexDebugSession::GetInstance()->GetCreateDummySessionResult();
                        if (result.IsFailure())
                        {
                            PIA_SAMPLE_PRINT("CreateDummySessionAsync failed\n");
                            PIA_SAMPLE_PRINT_RESULT(result);
                            nextSceneNo = SampleSceneNo_Start;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("CreateDummySessionAsync success. SessionId=%d\n",
                                nn::pia::inet::NexDebugSession::GetInstance()->GetCreatedDummySessionId());
                            s_DummySessionIdArray[s_DummySessionIdSize] = nn::pia::inet::NexDebugSession::GetInstance()->GetCreatedDummySessionId();
                            ++s_DummySessionIdSize;
                        }
                        break;
                    }
                }
            }
        if (command == DummySessionCommand_Destroy)
        {
            if (s_DummySessionIdSize > 0)
            {
                result = nn::pia::inet::NexDebugSession::GetInstance()->DestroyDummySessionAsync(s_DummySessionIdArray[s_DummySessionIdSize - 1]);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                // DestroyDummySessionAsync の非同期処理完了待ち
                int loopCounter = 0;
                while (true)
                {
                    GetFramework()->BeginFrame();
                    GetFramework()->DrawString(10, 20, "destroying dummy session... %d", loopCounter++);
                    GetFramework()->EndFrame();
                    ++g_FrameNo;

                    Dispatch();
                    if (nn::pia::inet::NexDebugSession::GetInstance()->IsDestroyDummySessionCompleted())
                    {
                        result = nn::pia::inet::NexDebugSession::GetInstance()->GetDestroyDummySessionResult();
                        if (result.IsFailure())
                        {
                            PIA_SAMPLE_PRINT("DestroyDummySessionAsync failed\n");
                            PIA_SAMPLE_PRINT_RESULT(result);
                            nextSceneNo = SampleSceneNo_Start;
                        }
                        else
                        {
                            PIA_SAMPLE_PRINT("CreateDummySession: DestroyDummySessionAsync success. SessionId=%d\n",
                                           s_DummySessionIdArray[s_DummySessionIdSize - 1]);
                            --s_DummySessionIdSize;
                            s_DummySessionIdArray[s_DummySessionIdSize] = 0;
                        }
                        break;
                    }
                }
            }
        }
        if (command == DummySessionCommand_Update)
        {
            if (s_DummySessionIdSize > 0)
            {
                const uint32_t UpdateSessionSettingSize = 6;
                nn::pia::inet::NexUpdateSessionSetting updateSessionSetting[UpdateSessionSettingSize];
                updateSessionSetting[1].SetMinParticipantNum(2);
                updateSessionSetting[2].SetMaxParticipantNum(6);
                uint8_t appData[4] = { 0xDE, 0xAD, 0xBE, 0xEF };
                updateSessionSetting[3].SetApplicationData(appData, sizeof(appData));
                uint32_t attributes[6] = { 0xf, 0xe, 0xe, 0xd, 0xb, 0xa };
                updateSessionSetting[4].SetAttributes(attributes);
                updateSessionSetting[5].SetMinParticipantNum(1);
                updateSessionSetting[5].SetMaxParticipantNum(8);
                uint32_t attributes2[6] = { 0xc, 0xa, 0xf, 0xe, 0xb, 0xa };
                updateSessionSetting[5].SetAttributes(attributes2);
                appData[0] = 0xCA;
                appData[1] = 0xFE;
                updateSessionSetting[5].SetApplicationData(appData, sizeof(appData));

                for (uint32_t i = 0; i < s_DummySessionIdSize && i < UpdateSessionSettingSize && nextSceneNo == SampleSceneNo_DummySession; i++)
                {
                    uint32_t sessionId = s_DummySessionIdArray[i];
                    result = nn::pia::inet::NexDebugSession::GetInstance()->UpdateDummySessionAsync(sessionId, updateSessionSetting[i]);
                    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

                    // UpdateDummySessionAsync の非同期処理完了待ち
                    int loopCounter = 0;
                    while (true)
                    {
                        GetFramework()->BeginFrame();
                        GetFramework()->DrawString(10, 20, "updating dummy session... %d", loopCounter++);
                        GetFramework()->EndFrame();
                        ++g_FrameNo;

                        Dispatch();
                        if (nn::pia::inet::NexDebugSession::GetInstance()->IsUpdateDummySessionCompleted())
                        {
                            result = nn::pia::inet::NexDebugSession::GetInstance()->GetUpdateDummySessionResult();
                            if (result.IsFailure())
                            {
                                PIA_SAMPLE_PRINT("UpdateDummySessionAsync failed. sessionId=%d\n", sessionId);
                                PIA_SAMPLE_PRINT_RESULT(result);
                                nextSceneNo = SampleSceneNo_Start;
                            }
                            else
                            {
                                PIA_SAMPLE_PRINT("UpdateDummySessionAsync success. SessionId=%d\n", sessionId);
                            }
                            break;
                        }
                    }
                }
            }
        }
    }

    if (isLoggedIn)
    {
        // ゲームサーバーからログアウトする前に NexDebugSession をクリーンアップ
        nn::pia::inet::NexDebugSession::GetInstance()->Cleanup();
    }
    // ゲームサーバーからログアウト
    nns::pia::Inet_Logout();
    // ネットワーク接続の終了処理
    nns::pia::Net_FinalizeNetwork();
    // NexDebugSession のシングルトンの破棄
    nn::pia::inet::NexDebugSession::DestroyInstance();
    // NexFacade のシングルトンの破棄
    nn::pia::inet::NexFacade::DestroyInstance();
    // PiaInet の終了処理
    nn::pia::inet::Finalize();
    // NEX の終了処理
    nns::pia::Nex_FinalizeNex();
#else
    nextSceneNo = SampleSceneNo_Start;
#endif

    return nextSceneNo;
}
