﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Local_Setup.h"
#include "SampleLib/Net_Setup.h"
#include "SampleLib/Nex_Setup.h"
#include "SampleLib/Lan_Setup.h"
#include "SampleLib/Session_Setup.h"

namespace
{
uint8_t s_TokenBrowse[nn::pia::transport::Station::IdentificationTokenMaxDataSize] = "Browse";
uint8_t s_TokenRandom[nn::pia::transport::Station::IdentificationTokenMaxDataSize] = "Random";
#if NNS_PIA_ENABLE_FRIENDS
uint8_t s_TokenFriend[nn::pia::transport::Station::IdentificationTokenMaxDataSize] = "Friend";
#endif
};


SampleSceneNo SceneSelectMatchmakeType()
{
    nn::pia::Result result;
    SampleSceneNo nextSceneNo = SampleSceneNo_SelectSessionType;

    GetFramework()->SetProgramStatus(SampleSceneNo_SelectSessionType, false, false, 0);

#if PIA_TEST_CODE
    GetFramework()->SetProgramParam(
        ProgramParamLanMatchmakeState,
        (g_NetworkType == nns::pia::NetworkType_Lan));
#endif

    // 参加していたセッションへの再参加用情報を保持している場合はブラウズマッチメイクシーンを経由してセッションに参加
    if (GetSessionId() != nn::pia::session::InvalidSessionId)
    {
        nextSceneNo = SampleSceneNo_JoinSession;
    }
    // ローカル通信はブラウズマッチメイクのみ
    else if (g_NetworkType == nns::pia::NetworkType_Local)
    {
        nextSceneNo = SampleSceneNo_JoinSession;
        g_IsHostMigrationEnabled = true;
    }
#if NNS_PIA_ENABLE_DIRECT_JOIN_FROM_FRIENDLIST
    // フレンドプレゼンス + HOME メニューからの起動の場合はフレンド合流マッチメイクシーンに自動遷移
    else if (IsJoinFromFriendList())
    {
        nextSceneNo = SampleSceneNo_JoinFriendSession;
    }
#endif
    else
    {
        // マッチメイクタイプを選択するループ
        while (nextSceneNo == SampleSceneNo_SelectSessionType)
        {
            GetFramework()->BeginFrame();
            // Y:
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonY))
            {
                nextSceneNo = SampleSceneNo_JoinSession;
                g_IsHostMigrationEnabled = true;
            }
            // B:
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonB))
            {
                nextSceneNo = SampleSceneNo_Start;
            }
            // A:
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonA))
            {
                nextSceneNo = SampleSceneNo_JoinRandomSession;
                g_IsHostMigrationEnabled = true;
            }
#if NNS_PIA_ENABLE_FRIENDS
            // X:
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonX))
            {
                // サンプルデモの実装上の都合でフレンド合流はフルメッシュ固定にしています。
                if (g_NetworkTopologyType == nn::pia::session::NetworkTopology_FullMesh && g_NetworkType == nns::pia::NetworkType_Internet)
                {
                    nextSceneNo = SampleSceneNo_JoinFriendSession;
                    // このサンプルではフレンドマッチメイクはホストマイグレーションをオフすることにします
                    // フレンドマッチメイクはホストとフレンド関係になっている場合にマッチメイクできるため、
                    // ホストが変更されるとマッチメイクできるユーザーが変わってしまいます。
                    g_IsHostMigrationEnabled = false;
                }
            }
#endif
            // START:
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonStart))
            {
                if (g_NetworkType == nns::pia::NetworkType_Internet)
                {
                    nextSceneNo = SampleSceneNo_Community;
                }
            }
            // L:
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonL))
            {
                if (g_NetworkType == nns::pia::NetworkType_Internet)
                {
                    g_MatchmakeKeywordType = static_cast<MatchmakeKeywordType>(g_MatchmakeKeywordType + 1);
                    if (g_MatchmakeKeywordType == MatchmakeKeywordType_Max)
                    {
                        g_MatchmakeKeywordType = MatchmakeKeywordType_None;
                    }
                }
            }
            // R:
            if (GetFramework()->IsTrigger(nns::pia::sampleframework::ButtonR))
            {
                g_LocalMultiPlayType = static_cast<LocalMultiPlayType>(g_LocalMultiPlayType + 1);
                if (g_LocalMultiPlayType == LocalMultiPlayType_Max)
                {
                    g_LocalMultiPlayType = LocalMultiPlayType_None;
                }
            }

            // 下画面
            GetFramework()->ChangeRenderTarget(false);
            GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
            int line = 0;
            GetFramework()->DrawString(0, line, "[%s Demo] %d", ProgramNameString, g_FrameNo);
            line += FontSize;
            GetFramework()->DrawString(0, line, "  Scene: SelectMatchmake.");
            line += FontSize;
            if (g_NetworkType == nns::pia::NetworkType_Internet)
            {
                GetFramework()->DrawString(0, line, "  MatchmakeKeywordTYpe: %d", g_MatchmakeKeywordType);
                line += FontSize;
                GetFramework()->DrawString(0, line, "  LocalMultiPlayType: %d", g_LocalMultiPlayType);
                line += FontSize;
            }
            line = LowerScreenHeight - (FontSize * 9);
            GetFramework()->DrawString(0, line, "[ Manual ]");
            line += FontSize;
            GetFramework()->DrawString(0, line, "      B: Back");
            line += FontSize;
            GetFramework()->DrawString(0, line, "      Y: Start BrowseMatchmake");
            line += FontSize;
            GetFramework()->DrawString(0, line, "      A: Start RandomMatchmake");
            line += FontSize;
            if (g_NetworkType == nns::pia::NetworkType_Internet)
            {
                GetFramework()->DrawString(0, line, "      X: Start FriendsMatchmake");
                line += FontSize;
                GetFramework()->DrawString(0, line, "  START: Start Community");
                line += FontSize;
                GetFramework()->DrawString(0, line, "      L: Change MatchmakeKeyword");
                line += FontSize;
            }
            GetFramework()->DrawString(0, line, "      R: Change LocalMultiPlayType");
            line += FontSize;
            GetFramework()->SwapBuffers();

            // 上画面
            GetFramework()->ChangeRenderTarget(true);
            GetFramework()->SwapBuffers();

            GetFramework()->EndFrame();
            ++g_FrameNo;

            Dispatch();

            // 接続状態チェック
            if (!nns::pia::Net_IsConnected())
            {
                PIA_SAMPLE_PRINT("nns::pia::Net_IsConnected Disconnected \n");
                return GetNextSceneByErrorResult(nn::pia::ResultNetworkConnectionIsLost(), SampleErrorType_ConnectionLost);
            }
#if NNS_PIA_ENABLE_INET
            if (nns::pia::Nex_IsNgsConnected().IsFailure() && g_NetworkType == nns::pia::NetworkType_Internet)
            {
                PIA_SAMPLE_PRINT("nns::pia::Nex_IsNgsConnected Disconnected \n");
                return GetNextSceneByErrorResult(nns::pia::Nex_IsNgsConnected(), SampleErrorType_ConnectionLostBeforeStartup);
            }
#endif
        }
    }
    if (nextSceneNo == SampleSceneNo_Start)
    {
#if NNS_PIA_ENABLE_INET
        if (g_NetworkType == nns::pia::NetworkType_Internet)
        {
            Logout();
        }
#endif
        nns::pia::Net_FinalizeNetwork();
        FinalizePia();
    }
    else
    {
#if NNS_PIA_ENABLE_FRIENDS
        if (g_NetworkType == nns::pia::NetworkType_Internet)
        {
            uint32_t param1 = static_cast<uint32_t>(nn::pia::inet::NexFacade::GetInstance()->GetLocalPrincipalId() >> 32);
            uint32_t param2 = static_cast<uint32_t>(nn::pia::inet::NexFacade::GetInstance()->GetLocalPrincipalId() & 0x00000000ffffffff);
            nn::pia::common::String stringParam(SAMPLE_STRING("MatchmakeType is decided"));
            nn::nex::NotificationEvents::NotificationEvents eventType;
            if (nextSceneNo == SampleSceneNo_JoinRandomSession)
            {
                eventType = nn::nex::NotificationEvents::GameNotificationType1;
            }
            else if (nextSceneNo == SampleSceneNo_JoinSession)
            {
                eventType = nn::nex::NotificationEvents::GameNotificationType2;
            }
            else if (nextSceneNo == SampleSceneNo_JoinFriendSession)
            {
                eventType = nn::nex::NotificationEvents::GameNotificationType3;
            }
            else 
            {
                eventType = nn::nex::NotificationEvents::GameNotificationType4;
            }
            PIA_SAMPLE_PRINT("nn::pia::inet::NexFacade::UpdateNotificationDataAsync is called. EventType:%d param1:0x%08x param2:0x%08x\n", eventType, param1, param2);
            result = nn::pia::inet::NexFacade::GetInstance()->UpdateNotificationDataAsync(eventType, param1, param2, stringParam);
            if (result.IsFailure())
            {
                PIA_SAMPLE_PRINT("Calling nn::pia::inet::NexFacade::UpdateNotificationDataAsync is failed.\n");
                return GetNextSceneByErrorResult(result, SampleErrorType_Unknown);
            }
            else
            {
                do
                {
                    GetFramework()->BeginFrame();
                    // 下画面
                    GetFramework()->ChangeRenderTarget(false);
                    GetFramework()->SetFontColor(nns::pia::sampleframework::ColorWhite);
                    int line = 0;
                    GetFramework()->DrawString(0, line, "[%s Demo] %d", ProgramNameString, g_FrameNo);
                    line += FontSize;
                    GetFramework()->DrawString(0, line, "  Scene: SelectMatchmake.");
                    line += FontSize;
                    line += FontSize;

                    GetFramework()->DrawString(0, line, "Updating NotificationData...");
                    line += FontSize;

                    GetFramework()->SwapBuffers();

                    // 上画面
                    GetFramework()->ChangeRenderTarget(true);
                    GetFramework()->SwapBuffers();

                    GetFramework()->EndFrame();
                    ++g_FrameNo;

                    Dispatch();

                    if (!nns::pia::Net_IsConnected())
                    {
                        PIA_SAMPLE_PRINT("nns::pia::Net_IsConnected Disconnected \n");
                        return GetNextSceneByErrorResult(nn::pia::ResultNetworkConnectionIsLost(), SampleErrorType_ConnectionLost);
                    }
                } while (!nn::pia::inet::NexFacade::GetInstance()->IsUpdateNotificationDataCompleted());

                if (nn::pia::inet::NexFacade::GetInstance()->GetUpdateNotificationDataResult().IsSuccess())
                {
                    PIA_SAMPLE_PRINT("nn::pia::inet::NexFacade::UpdateNotificationDataAsync process is success.\n");
                }
                else
                {
                    PIA_SAMPLE_PRINT("nn::pia::inet::NexFacade::UpdateNotificationDataAsync process is failed.\n");
                }
            }
        }
#endif

        nn::pia::transport::Station::IdentificationToken token;
        nn::pia::transport::Station::IdentificationToken* pTokenData = NULL;
        switch (nextSceneNo)
        {
        case SampleSceneNo_JoinSession:
        {
            memcpy(token.data, s_TokenBrowse, nn::pia::transport::Station::IdentificationTokenMaxDataSize);
            pTokenData = &token;
        }
        break;
        case SampleSceneNo_JoinRandomSession:
        {
            memcpy(token.data, s_TokenRandom, nn::pia::transport::Station::IdentificationTokenMaxDataSize);
            pTokenData = &token;
        }
        break;
#if NNS_PIA_ENABLE_FRIENDS
        case SampleSceneNo_JoinFriendSession:
        {
            memcpy(token.data, s_TokenFriend, nn::pia::transport::Station::IdentificationTokenMaxDataSize);
            pTokenData = &token;
        }
        break;
        case SampleSceneNo_Community:
        {
            pTokenData = NULL;
        }
        break;
#endif
        default:
        {
        }
        break;
        }

        switch (g_NetworkType)
        {
        case nns::pia::NetworkType_Local:
        {
#if NNS_PIA_ENABLE_LOCAL
            bool isSuccess = nns::pia::Local_InitializeLocalCommunication();
            if (!isSuccess)
            {
                PIA_SAMPLE_PRINT("Local_InitializeLocalCommunication failed\n");
                return GetNextSceneByErrorResult(nn::pia::ResultNetworkConnectionIsLost(), SampleErrorType_Startup);
            }
#endif
        }
        break;
        case nns::pia::NetworkType_Lan:
        {
            result = nns::pia::Lan_StartupPiaLan();
            if (result.IsFailure())
            {
                if (result == nn::pia::ResultNetworkConnectionIsLost())
                {
                    return GetNextSceneByErrorResult(result, SampleErrorType_InitializeByNetworkInterfaceUp);
                }
                else
                {
                    return GetNextSceneByErrorResult(result, SampleErrorType_InitializeByOtherReason);
                }
            }
        }
        break;
        default:
            break;
        }

        nn::pia::transport::Station::PlayerInfo playerInfoList[nn::pia::MaxPlayerInfoSize];
        switch (g_NetworkType)
        {
        case nns::pia::NetworkType_Local:
        {
#if NNS_PIA_ENABLE_LOCAL
            nn::pia::local::LocalSessionStartupSetting startupSetting;

#if NN_PIA_NINTENDOSDK
            uint8_t playerInfoNum = nns::pia::Session_GetLocalPlayerInfo(playerInfoList, 1, true);
            NN_PIA_UNUSED_IN_RELEASE(playerInfoNum);
            PIA_SAMPLE_ASSERT(playerInfoNum == 1);
            startupSetting.playerInfo.Copy(playerInfoList[0]);
#endif

            memcpy(startupSetting.signatureKeyData, SignatureKey, sizeof(SignatureKey));
            startupSetting.signatureKeyDataSize = sizeof(SignatureKey);

            startupSetting.pToken = pTokenData;

            result = nn::pia::session::Session::GetInstance()->Startup(startupSetting);
#endif
        }
        break;
        case nns::pia::NetworkType_Internet:
        {
#if NNS_PIA_ENABLE_INET
            nn::pia::inet::NexSessionStartupSetting startupSetting;

            uint8_t playerInfoNum = nns::pia::Session_GetLocalPlayerInfo(playerInfoList, nn::pia::MaxPlayerInfoSize, true);
            NN_PIA_UNUSED(playerInfoNum);

#if NN_PIA_CAFE
            startupSetting.playerInfo[0].Copy(playerInfoList[0]);
            startupSetting.isPlayersCountedAsParticipants = true;
#elif NN_PIA_NINTENDOSDK
            if (g_LocalMultiPlayType == LocalMultiPlayType_Participants)
            {
                for (uint8_t i = 0; i < playerInfoNum; ++i)
                {
                    startupSetting.playerInfo[i].Copy(playerInfoList[i]);
                }
                startupSetting.isPlayersCountedAsParticipants = true;
            }
            else if (g_LocalMultiPlayType == LocalMultiPlayType_Stations)
            {
                for (uint8_t i = 0; i < playerInfoNum; ++i)
                {
                    startupSetting.playerInfo[i].Copy(playerInfoList[i]);
                }
                startupSetting.isPlayersCountedAsParticipants = false;
            }
            else
            {
                startupSetting.playerInfo[0].Copy(playerInfoList[0]);
                startupSetting.isPlayersCountedAsParticipants = true;
            }
#endif

            if (g_IsBitRateCheckEnabled)
            {
                startupSetting.uplinkBitRateLowerLimit = 200 * 1000;
                startupSetting.bitRateCheckPacketSize = 1024;
            }
            startupSetting.isHostMigrationEnabled = g_IsHostMigrationEnabled;
            startupSetting.pToken = pTokenData;
            startupSetting.cryptoMode = nn::pia::common::CryptoSetting::Mode_Aes128;

            result = nn::pia::session::Session::GetInstance()->Startup(startupSetting);
#endif
        }
        break;
        case nns::pia::NetworkType_Lan:
        {
            nn::pia::lan::LanSessionStartupSetting startupSetting;

            uint8_t playerInfoNum = nns::pia::Session_GetLocalPlayerInfo(playerInfoList, nn::pia::MaxPlayerInfoSize, false);
            NN_PIA_UNUSED(playerInfoNum);

#if NN_PIA_CAFE
            startupSetting.playerInfo[0].Copy(playerInfoList[0]);
            startupSetting.isPlayersCountedAsParticipants = true;
#elif NN_PIA_NINTENDOSDK
            if (g_LocalMultiPlayType == LocalMultiPlayType_Participants)
            {
                for (uint8_t i = 0; i < playerInfoNum; ++i)
                {
                    startupSetting.playerInfo[i].Copy(playerInfoList[i]);
                }
                startupSetting.isPlayersCountedAsParticipants = true;
            }
            else if (g_LocalMultiPlayType == LocalMultiPlayType_Stations)
            {
                for (uint8_t i = 0; i < playerInfoNum; ++i)
                {
                    startupSetting.playerInfo[i].Copy(playerInfoList[i]);
                }
                startupSetting.isPlayersCountedAsParticipants = false;
            }
            else
            {
                startupSetting.playerInfo[0].Copy(playerInfoList[0]);
                startupSetting.isPlayersCountedAsParticipants = true;
        }
#endif

            startupSetting.pToken = pTokenData;
#if NN_PIA_A
            startupSetting.cryptoMode = nn::pia::common::CryptoSetting::Mode_Nothing;
#else
            startupSetting.cryptoMode = nn::pia::common::CryptoSetting::Mode_Aes128;
#endif

            result = nn::pia::session::Session::GetInstance()->Startup(startupSetting);
        }
        break;
        }

        if (result.IsFailure())
        {
            PIA_SAMPLE_PRINT("Session_StartupSession: Startup failed\n");
            PIA_SAMPLE_PRINT_RESULT(result);
#if NN_PIA_NINTENDOSDK
            PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() || 
                              result == nn::pia::ResultWifiOff() || 
                              result == nn::pia::ResultSleep());
#endif
            return GetNextSceneByErrorResult(result, SampleErrorType_Startup);
        }

        //同期クロック用のパルス間隔設定
        //デフォルトは2000msに設定されています。基本的に設定を変更する必要はありませんが、ここでは1000msに設定しています。
        nn::pia::session::Session::GetInstance()->SetSyncClockPulseInterval(1000);
        PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetSyncClockPulseInterval() == 1000);

        //リレー接続の設定
        if (g_NetworkTopologyType == nn::pia::session::NetworkTopology_RelayMesh)
        {
            //ステーション1台に割り当てられる転送依頼数の最大制限値
            //デフォルトは [メッシュ最大参加数] * [メッシュ最大参加数] に設定されています。ここでは 4 に設定しています。
            nn::pia::session::Session::GetInstance()->SetMaxNumOfRelayRoutesAssignedToStation(4);

            //1つのリレー接続経路の合計 RTT の最大制限値
            //デフォルトは nn::pia::session::SessionMaxRttDefault に設定されています。ここでは 400ms に設定しています。
            nn::pia::session::Session::GetInstance()->SetMaxRtt(400);
        }
    }
    return nextSceneNo;
}
