﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"
#include "SampleLib/Inet_Setup.h"
#include "SampleLib/Lan_Setup.h"
#include "SampleLib/Session_Setup.h"

namespace
{
#if PIA_TEST_CODE
void SampleSessionCallbackForEndFrame()
{
    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();
    PIA_ASSERT(PIA_IS_VALID_POINTER(pSampleFramework));

    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pSession));

    GetFramework()->SetProgramParam(ProgramParamGetJoinSequence, pSession->GetJoinAutoMatchmakeSequence());
}
#endif
}

SampleSceneNo SceneRandomMatchmake()
{
    nn::pia::Result result;
    SampleSceneNo nextSceneNo = SampleSceneNo_Communication;

    GetFramework()->SetProgramStatus(SampleSceneNo_JoinRandomSession, false, false, 0);

    nn::pia::session::JoinRandomSessionSetting* pJoinRandomSessionSetting = NULL;
    const uint8_t SearchCriteriaNum = 2;

    // 作成するマッチメイクセッションと検索条件の設定
    nn::pia::lan::LanCreateSessionSetting lanCreateSessionSetting;
    nn::pia::lan::LanSessionSearchCriteria lanSearchCriteriaList[SearchCriteriaNum];
    nn::pia::lan::LanJoinRandomSessionSetting lanJoinRandomSessionSetting;
#if NNS_PIA_ENABLE_INET
    nn::pia::inet::NexCreateSessionSetting nexCreateSessionSetting;
    nn::pia::inet::NexSessionSearchCriteria nexSearchCriteriaList[SearchCriteriaNum];
    nn::pia::inet::NexJoinRandomSessionSetting nexJoinRandomSessionSetting;
#endif
    if (g_NetworkType == nns::pia::NetworkType_Lan)
    {
        lanCreateSessionSetting.SetSessionType(nn::pia::lan::LanSessionType_Anybody);
        lanCreateSessionSetting.SetGameMode(MatchmakeGameMode + MatchmakeType_Random + GetIncrementalGameModeValueByBitRateCheckMode());
        lanCreateSessionSetting.SetMaxParticipantNum(GetMaxParticipantNum());
        lanCreateSessionSetting.SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
        lanCreateSessionSetting.SetAttribute(MatchmakeAttributeIndexNetworkTopology, g_NetworkTopologyType);
        lanCreateSessionSetting.SetOpenSession(true);  // 作成されてすぐに参加可能状態にします。


        // Attribute 単体指定
        lanSearchCriteriaList[0].SetGameMode(MatchmakeGameMode + MatchmakeType_Random + GetIncrementalGameModeValueByBitRateCheckMode());
        lanSearchCriteriaList[0].SetMaxParticipantNum(GetMaxParticipantNum());
        lanSearchCriteriaList[0].SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
        lanSearchCriteriaList[0].SetAttribute(MatchmakeAttributeIndexNetworkTopology, g_NetworkTopologyType);

        // Attribute 複数指定
        // (複数指定方法を示すコード例だが、上記の単体指定で検索にかかるので、実際の検索に利用されることは無い)
        lanSearchCriteriaList[1].SetGameMode(MatchmakeGameMode + MatchmakeType_Random + GetIncrementalGameModeValueByBitRateCheckMode());
        lanSearchCriteriaList[1].SetMaxParticipantNumRange(GetMaxParticipantNum(), GetMaxParticipantNum());
        const uint32_t arraySize = 5;
        const uint32_t attributeArray[arraySize] = { 2, 4, 6, 11, 72 };
        lanSearchCriteriaList[1].SetAttribute(MatchmakeAttributeIndex, attributeArray, arraySize);

        lanJoinRandomSessionSetting.SetCreateSessionSetting(&lanCreateSessionSetting);
        lanJoinRandomSessionSetting.SetSessionSearchCriteria(lanSearchCriteriaList, SearchCriteriaNum);

        pJoinRandomSessionSetting = &lanJoinRandomSessionSetting;
    }
#if NNS_PIA_ENABLE_INET
    else
    {
        nexCreateSessionSetting.SetSessionType(nn::pia::inet::SessionType_Anybody);
        nexCreateSessionSetting.SetGameMode(MatchmakeGameMode + MatchmakeType_Random + GetIncrementalGameModeValueByBitRateCheckMode());
        nexCreateSessionSetting.SetMaxParticipantNum(GetMaxParticipantNum());
        nexCreateSessionSetting.SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
        nexCreateSessionSetting.SetAttribute(MatchmakeAttributeIndexNetworkTopology, g_NetworkTopologyType);
        nexCreateSessionSetting.SetOpenSession(true);  // 作成されてすぐに参加可能状態にします。
        if (g_MatchmakeKeywordType != MatchmakeKeywordType_None)
        {
            nexCreateSessionSetting.SetSessionMatchmakeKeyword(SampleMatchmakeKeyword[g_MatchmakeKeywordType]);
        }

        // コミュニティ ID を設定
        if (GetJoinedCommunityId() != 0)
        {
            nexCreateSessionSetting.SetCommunityId(GetJoinedCommunityId());
        }

        // Attribute 単体指定
        nexSearchCriteriaList[0].SetGameMode(MatchmakeGameMode + MatchmakeType_Random + GetIncrementalGameModeValueByBitRateCheckMode());
        nexSearchCriteriaList[0].SetMaxParticipantNum(GetMaxParticipantNum());
        nexSearchCriteriaList[0].SetAttribute(MatchmakeAttributeIndexHostMigration, g_IsHostMigrationEnabled);
        nexSearchCriteriaList[0].SetAttribute(MatchmakeAttributeIndexNetworkTopology, g_NetworkTopologyType);
        if (GetJoinedCommunityId() != 0)
        {
            nexSearchCriteriaList[0].SetCommunityId(GetJoinedCommunityId());
        }
        if (g_MatchmakeKeywordType != MatchmakeKeywordType_None)
        {
            nexSearchCriteriaList[0].SetSessionMatchmakeKeyword(SampleMatchmakeKeyword[g_MatchmakeKeywordType]);
        }

        // Attribute 複数指定
        // (複数指定方法を示すコード例だが、上記の単体指定で検索にかかるので、実際の検索に利用されることは無い)
        nexSearchCriteriaList[1].SetGameMode(MatchmakeGameMode + MatchmakeType_Random + GetIncrementalGameModeValueByBitRateCheckMode());
        nexSearchCriteriaList[1].SetMaxParticipantNumRange(GetMaxParticipantNum(), GetMaxParticipantNum());
        const uint32_t arraySize = 5;
        const uint32_t attributeArray[arraySize] = { 2, 4, 6, 11, 72 };
        nexSearchCriteriaList[1].SetAttribute(MatchmakeAttributeIndex, attributeArray, arraySize);
        if (GetJoinedCommunityId() != 0)
        {
            nexSearchCriteriaList[1].SetCommunityId(GetJoinedCommunityId());
        }
        if (g_MatchmakeKeywordType != MatchmakeKeywordType_None)
        {
            nexSearchCriteriaList[1].SetSessionMatchmakeKeyword(SampleMatchmakeKeyword[g_MatchmakeKeywordType]);
        }

        nexJoinRandomSessionSetting.SetCreateSessionSetting(&nexCreateSessionSetting);
        nexJoinRandomSessionSetting.SetSessionSearchCriteria(nexSearchCriteriaList, SearchCriteriaNum);
        nexJoinRandomSessionSetting.SetMatchmakeUniqueKeyword(true);

        pJoinRandomSessionSetting = &nexJoinRandomSessionSetting;
    }
#endif

#if PIA_TEST_CODE
    nns::pia::RegisterEndFrameSampleSessionCallback(SampleSessionCallbackForEndFrame);
#endif
    nns::pia::RegisterDispatchSampleSessionCallback(SampleSessionCallbackForDispatch);
    // マッチメイク処理を行うループ
    result = nns::pia::Session_JoinRandomSession(pJoinRandomSessionSetting);
    if (result.IsFailure())
    {
        PIA_SAMPLE_PRINT_RESULT(result);
        nextSceneNo = GetNextSceneByErrorResult(result, SampleErrorType_Join);
    }
    else
    {
        nextSceneNo = SampleSceneNo_Communication;
    }
#if PIA_TEST_CODE
    nns::pia::RegisterEndFrameSampleSessionCallback(NULL);
#endif
    return nextSceneNo;
}
