﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"

#if NN_PIA_CTR
#include <nn/os.h>
#include <nn/ac.h>
#elif NN_PIA_CAFE
#include <nn/ac.h>
#endif


/*!

    @brief マッチメイクを行う前の準備処理をまとめた *.cpp

*/

namespace
{
bool s_IsPiaInitialized = false;
bool s_IsLogined = false;

// InitializePia の進捗を画面表示する
void InitializingProcessRender(uint32_t processingStep)
{
    // 電波強度情報の取得
    uint32_t linkLevel = GetLinkLevel();
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_Preparation,
        false,
        false,
        0,
        linkLevel);

    nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

    // 下画面
    nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(false);
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
    int line = 0;
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo]", ProgramNameString);
    line += nns::pia::BasicFontSize;
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Inet Initializing process.");
    line += nns::pia::BasicFontSize;
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(nns::pia::BasicLowerScreenWidth - (nns::pia::BasicFontSize * 9), line, "LinkLv:%d", linkLevel);
    line += nns::pia::BasicFontSize;

    line += nns::pia::BasicFontSize;

    // 進捗の表示
    // 処理済みステップを白、未処理ステップを赤で表示
    {
        const char* StepString[] = {
            "Initialize NEX",
            "Initialize inet module",
            "Initialize Network",
            "Login server"
        };  // 4 step
        const uint32_t MaxStep = 4;

        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   START");
        line += nns::pia::BasicFontSize;
        for (uint32_t step = 0; step < MaxStep; step++)
        {
            if (step == processingStep)
            {
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "     |");
                line += nns::pia::BasicFontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorRed);
            }
            else
            {
                line += nns::pia::BasicFontSize;
            }
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   %s", StepString[step]);
            line += nns::pia::BasicFontSize;
        }
        line += nns::pia::BasicFontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "   END");
        line += nns::pia::BasicFontSize;
    }
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);

    nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

    // 上画面
    nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(true);

    line = nns::pia::BasicUpperScreenHeight - nns::pia::BasicFontSize;
    nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", nns::pia::Basic_GetPiaVersionString());
    line += nns::pia::BasicFontSize;

    nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
}

}  // end of anonymous namespace

#if NN_PIA_CTR
// AP との通信電波強度を取得し、0-3 の値に変換して返す関数
uint32_t GetInternetLinkLevel()
{
    nn::ac::CTR::LinkLevel acLinkLevel = nn::ac::CTR::GetLinkLevel();

    switch (acLinkLevel)
    {
    case nn::ac::CTR::LINK_LEVEL_3:
        return 3;
    case nn::ac::CTR::LINK_LEVEL_2:
        return 2;
    case nn::ac::CTR::LINK_LEVEL_1:
        return 1;
    case nn::ac::CTR::LINK_LEVEL_0:
    default:
        return 0;
    }
}
#endif

void InitializePiaSession()
{
    PIA_SAMPLE_PRINT("InitializePiaTransport\n");

    nn::pia::Result result;

#if PIA_TEST_CODE
    // デバッグ機能設定用（パケロスエミュレーション機能）
    // 製品版 ROM では呼び出さないように注意してください
    nn::pia::transport::Transport::DebugSetting transportDebugSetting;
    transportDebugSetting.isPacketLossEmulationEnabled = true;
    nn::pia::transport::Transport::SetDebugSetting(transportDebugSetting);
#endif

    // ここでは各パラメータを transport のデモから拝借しています。
    // 各パラメータの詳細については プログラミングマニュアル を参照して下さい。
    nn::pia::transport::Transport::Setting transportSetting;

#if ENABLE_LAN_MATCHMAKE
    transportSetting.pFactory = nns::pia::Lan_GetNetworkFactory();
#else
    transportSetting.pFactory = nns::pia::Inet_GetNetworkFactory();
#endif
    transportSetting.maxStationNum = MaxParticipantNum;
    transportSetting.maxSendThreadBufferNum = nns::pia::MaxSendThreadBufferNum;
    transportSetting.maxReceiveThreadBufferNum = nns::pia::MaxReceiveThreadBufferNum;

    result = nn::pia::transport::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::transport::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::transport::Transport::CreateInstance(transportSetting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    //ポート番号は 0 決め打ち
    SetUnreliableProtocolHandle(nn::pia::transport::Transport::GetInstance()->CreateProtocol<nn::pia::transport::UnreliableProtocol>(0));
    result = nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::UnreliableProtocol>(GetUnreliableProtocolHandle())->Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::transport::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

#if NN_PIA_CTR || NN_PIA_CAFE
    //送受信スレッドの優先度設定 (優先度については SampleConfig.h 参照)
    //送受信スレッドの優先度は必ずしも変更する必要はありませんが、ここでは変更方法を提示する意味を込めて
    //優先度を変更しています。
    nn::pia::transport::ThreadStreamManager* pThreadStreamManager = nn::pia::transport::ThreadStreamManager::GetInstance();
    PIA_SAMPLE_ASSERT(pThreadStreamManager);  //transport の初期化が終わっていればインスタンスが生成されているはず
    pThreadStreamManager->SetSendThreadPriority(SendThreadPriority);
    pThreadStreamManager->SetReceiveThreadPriority(ReceiveThreadPriority);
#endif

    PIA_SAMPLE_PRINT("InitializePiaSession\n");

#if ENABLE_MATCHMAKE_ANALYSE
    {
        // デバッグ機能設定用（マッチメイク分析機能）
        // 製品版 ROM では呼び出さないように注意してください
        nn::pia::session::Session::DebugSetting debugSetting;
        debugSetting.isMatchmakeAnalysisEnabled = true;
        result = nn::pia::session::Session::SetDebugSetting(debugSetting);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    }
#endif

    nn::pia::session::Session::Setting sessionSetting;

#if ENABLE_LAN_MATCHMAKE
    sessionSetting.pNetworkFactory = nns::pia::Lan_GetNetworkFactory();
#else
    sessionSetting.pNetworkFactory = nns::pia::Inet_GetNetworkFactory();
#endif

    // ネットワークトポロジーをフルメッシュに設定
    sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;

#if ENABLE_BIT_RATE_CHECK
    // 帯域幅の測定をセッション確立時に行う
    sessionSetting.bitRateCheckMode = nn::pia::session::BitRateCheckMode_Enable;
#else
    sessionSetting.bitRateCheckMode = nn::pia::session::BitRateCheckMode_Disable;
#endif

    result = nn::pia::session::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::session::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::session::Session::CreateInstance(sessionSetting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::session::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // セッションイベントコールバックを登録
    nn::pia::session::Session::GetInstance()->RegisterSessionEventCallback(SampleSessionCallback);

#if ENABLE_MATCHMAKE_ANALYSE
    nn::pia::session::Session::GetInstance()->RegisterJoiningApprovalCallback(SampleJoinApprovalCallback);
#endif
}

/* ---< Pia モジュールで利用する機能のセットアップ >--- */
// インターネット接続処理、ゲームサーバへのログインなどを行います。
// nn::ac::Connect などは長期間ブロックするため、メインスレッドでの実行は推奨されませんが
// このサンプルプログラムでは処理の可読性を重視して、ここで行います。
bool InitializePia()
{
    nn::pia::Result result;

    // 長時間ブロックする処理が続くため、進捗を画面表示しておく
    uint32_t step = 0;
    InitializingProcessRender(step);
    step++;

    if (!s_IsPiaInitialized)
    {
#if ENABLE_LAN_MATCHMAKE
        InitializingProcessRender(step);
        step++;

        // PiaLan の初期化処理
        result = nns::pia::Lan_InitializePiaLan();
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
#else
        // NEX 関連初期化処理
        result = nns::pia::Nex_InitializeNex(
            MemoryBlockSizeForNex,
            MemoryBlockSizeForNexEmergency);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
        PIA_SAMPLE_PRINT("NEX initialize completed.\n");

        InitializingProcessRender(step);
        step++;

        // PiaInet の初期化処理
        result = nns::pia::Inet_InitializePiaInet();
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
        PIA_SAMPLE_PRINT("nns::pia::Inet_InitializePiaInet completed.\n");
#endif

        // PiaTransport, PiaSession の初期化処理
        InitializePiaSession();
        PIA_SAMPLE_PRINT("InitializePiaSession completed.\n");

        InitializingProcessRender(step);
        step++;
    }
    s_IsPiaInitialized = true;

    // ネットワークインターフェース起動処理
    if (!nns::pia::Net_IsInitialized())
    {
        nns::pia::NetworkType networkType = (ENABLE_LAN_MATCHMAKE) ? nns::pia::NetworkType_Lan : nns::pia::NetworkType_Internet;
        nns::pia::NetResult netResult = nns::pia::Net_InitializeNetwork(networkType);
        if (netResult != nns::pia::NetResult_Success)
        {
            if (netResult == nns::pia::NetResult_AccountError)
            {
                SetCommunicationErrorType(SampleErrorType_InitializeByAccount);
            }
            else if (netResult == nns::pia::NetResult_AcError || netResult == nns::pia::NetResult_NicDown || netResult == nns::pia::NetResult_SocketError)
            {
                SetCommunicationErrorType(SampleErrorType_InitializeByNetworkInterfaceUp);
            }
            else
            {
                SetCommunicationErrorType(SampleErrorType_InitializeByOtherReason);
            }
            PIA_SAMPLE_PRINT("nns::pia::Net_InitializeNetwork failed.\n");
            SetNetworkErrorResult(netResult);
            return false;
        }
        PIA_SAMPLE_PRINT("nns::pia::Net_InitializeNetwork completed.\n");
    }

    InitializingProcessRender(step);
    step++;

    if (!Login())
    {
        return false;
    }

    InitializingProcessRender(step);
    step++;

    return true;
}

bool Login()
{
    if (!s_IsLogined)
    {
#if !ENABLE_LAN_MATCHMAKE
        // サーバへのログイン処理
        if (!nns::pia::Inet_Login())
        {
            // ログイン処理に失敗
            SetCommunicationErrorType(SampleErrorType_InitializeByOtherReason);
            PIA_SAMPLE_PRINT("nns::pia::Inet_Login failed.\n");
            return false;
        }
        PIA_SAMPLE_PRINT("nns::pia::Inet_Login completed.\n");
#endif
        s_IsLogined = true;
    }
    return true;
}

void Logout()
{
#if !ENABLE_LAN_MATCHMAKE
    nns::pia::Inet_Logout();
#endif
    s_IsLogined = false;
}

void CleanupPia()
{
    nns::pia::Session_CleanupSession();
#if ENABLE_LAN_MATCHMAKE
    nns::pia::Lan_CleanupPiaLan();
#endif
}

//終了処理は乱暴に行っても問題ないようにします。
void FinalizePia()
{
    // Finalize nn::pia::session
    PIA_SAMPLE_PRINT("FinalizePiaSession\n");
    nns::pia::Session_FinalizePiaSession();

    PIA_SAMPLE_PRINT("FinalizePiaTransport\n");
    if (nn::pia::transport::Transport::GetInstance() != NULL)
    {
        //Finalize nn::pia::transport::UnreliableProtocol
        if (nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::UnreliableProtocol>(GetUnreliableProtocolHandle()) != NULL)
        {
            nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::transport::UnreliableProtocol>(GetUnreliableProtocolHandle())->Finalize();
            nn::pia::transport::Transport::GetInstance()->DestroyProtocol(GetUnreliableProtocolHandle());
            SetUnreliableProtocolHandle(0);
        }
}

    //Finalize nn::pia::Transport
    nn::pia::transport::Transport::DestroyInstance();  //transport のインスタンスの破棄は、生成していない状態でも実行して問題ありません
    nn::pia::transport::Finalize();

#if ENABLE_LAN_MATCHMAKE
    nns::pia::Lan_FinalizePiaLan();
#else
    nns::pia::Inet_FinalizePiaInet();
    nns::pia::Nex_FinalizeNex();
#endif
    s_IsPiaInitialized = false;
}
