﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"

#if NN_PIA_CAFE
#include <nn/act.h>
#endif

namespace
{
enum SampleSessionSelectMethod
{
    SampleSessionSelectionMethod_Random = 0,
    SampleSessionSelectionMethod_BroadenRangeWithSelectionPriority,
    SampleSessionSelectionMethod_ScoreBased,
    SampleSessionSelectionMethod_Max
};

const char* SelectionMethodString[SampleSessionSelectionMethod_Max] = {
    "Random",
    "BroadenRange+Priority",
    "ScoreBased"
};

#if !ENABLE_LAN_MATCHMAKE
const uint32_t RatingValueMin = 800;
const uint32_t RatingValueMax = 3000;

enum SampleScoreSettingIndex
{
    SampleScoreSettingIndex_Rating = MinScoreSettingIndex,
    SampleScoreSettingIndex_DisconnectionRate,
    SampleScoreSettingIndex_ViolationRate,
    SampleScoreSettingIndex_GeoIp,
    SampleScoreSettingIndex_CountryCode,
    SampleScoreSettingIndex_SessionDuration,
    SampleScoreSettingIndex_SelectionPriority,
    MaxScoreSettingIndex = SampleScoreSettingIndex_SelectionPriority
};

const uint32_t ScoreSettingIndexSize = MaxScoreSettingIndex - MinScoreSettingIndex + 1;
const char* ScoreSettingString[ScoreSettingIndexSize] = {
    "Rating",
    "Disconnecion Rate",
    "Violation Rate",
    "Geo Ip",
    "Country Code",
    "Session Duration",
    "Selection Priority"
};

const uint32_t MaxGeoIpLocation = 9;
const char* GeoIpLocationString[MaxGeoIpLocation] = {
    "Tokyo",
    "Osaka",
    "New York",
    "San Flancisco",
    "Vancouver",
    "London",
    "Paris",
    "Berlin",
    "Sydney"
};

const nn::pia::common::InetAddress GeoIpLocationList[MaxGeoIpLocation] = {
    nn::pia::common::InetAddress(0xCAE8E069, 0),  // ("202.232.224.105")  Tokyo
    nn::pia::common::InetAddress(0x85018A90, 0),  // ("133.1.138.144")    Osaka
    nn::pia::common::InetAddress(0x803B6918, 0),  // ("128.59.105.24")    New York
    nn::pia::common::InetAddress(0x82D40ACC, 0),  // ("130.212.10.204")   San Flancisco
    nn::pia::common::InetAddress(0x89528231, 0),  // ("137.82.130.49")    Vancouver
    nn::pia::common::InetAddress(0x805682C3, 0),  // ("128.86.130.195")   London
    nn::pia::common::InetAddress(0xC1376079, 0),  // ("193.55.96.121")    Paris
    nn::pia::common::InetAddress(0xA02DAA0A, 0),  // ("160.45.170.10")    Berlin
    nn::pia::common::InetAddress(0x814E050B, 0)   // ("129.78.5.11")      Sydney
};

const uint32_t MaxMatchmakeKeyword = 4;
const SampleStringChar_t* MatchmakeKeywordString[MaxMatchmakeKeyword] = {
    SAMPLE_STRING(""),
    SAMPLE_STRING("Boo"),
    SAMPLE_STRING("Foo"),
    SAMPLE_STRING("Woo")
};
#endif
}


SampleSceneNo SceneMatchmake()
{
    nn::pia::Result result;

    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SampleSceneNo_Matchmake,
        false,
        false,
        0);

#if ENABLE_LAN_MATCHMAKE
    // LAN マッチメイクのスタートアップ処理
    result = nns::pia::Lan_StartupPiaLan();
    if (result.IsFailure())
    {
        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost());
        SetCommunicationErrorType(SampleErrorType_InitializeByNetworkInterfaceUp);
        SetNetworkErrorResult(result);
        return SampleSceneNo_ShowErrorViewer;
    }
#endif

// Session スタートアップ
#if ENABLE_LAN_MATCHMAKE
    nn::pia::lan::LanSessionStartupSetting startupSetting;
#else
    nn::pia::inet::NexSessionStartupSetting startupSetting;
    startupSetting.isHostMigrationEnabled = IsHostMigrationEnabled;
#if ENABLE_BIT_RATE_CHECK
    startupSetting.uplinkBitRateLowerLimit = 200 * 1000;
    startupSetting.bitRateCheckPacketSize = 1024;
#endif
#endif
    startupSetting.cryptoMode = nn::pia::common::CryptoSetting::Mode_Nothing;  // 通信暗号化は使用しない
    startupSetting.maxSilenceTime = 8000;                                      // 最大無通信時間は 8000ms に設定
    startupSetting.keepAliveSendingInterval = 1500;                            // キープアライブ送信間隔は 1500ms に設定
    startupSetting.pToken = NULL;                                              // 識別トークンは設定しない

    nn::pia::transport::Station::PlayerInfo playerName;
    nns::pia::Session_GetLocalPlayerInfo(&playerName, 1);
    startupSetting.playerInfo[0].Copy(playerName);

    result = nns::pia::Session_StartupSession(startupSetting);
    if (result.IsFailure())
    {
        // エラーシーン用に result 通知
        PIA_SAMPLE_ASSERT(result == nn::pia::ResultNetworkConnectionIsLost() ||
                          result == nn::pia::ResultWifiOff() ||
                          result == nn::pia::ResultSleep());
        SetCommunicationErrorType(SampleErrorType_InitializeByOtherReason);
        SetNetworkErrorResult(result);
        return SampleSceneNo_ShowErrorViewer;
    }

    SampleSessionSelectMethod selectionMethodType = SampleSessionSelectionMethod_Random;

#if ENABLE_LAN_MATCHMAKE
    // 作成するマッチメイクセッションの設定
    nn::pia::lan::LanCreateSessionSetting createSessionSetting;
    createSessionSetting.SetGameMode(MatchmakeGameMode);
    createSessionSetting.SetMaxParticipantNum(MaxParticipantNum);
    createSessionSetting.SetAttribute(MatchmakeAttributeIndex, MatchmakeAttributeValue);
    createSessionSetting.SetOpenSession(true);  // 作成されてすぐに参加可能状態にします。

    // 検索条件の設定
    const uint32_t SearchSettingNum = nn::pia::lan::LanSessionSearchCriteriaListSizeMax;
    nn::pia::lan::LanSessionSearchCriteria sessionSearchCriteriaList[SearchSettingNum];

    // Attribute 単体指定
    sessionSearchCriteriaList[0].SetGameMode(MatchmakeGameMode);
    sessionSearchCriteriaList[0].SetMaxParticipantNum(MaxParticipantNum);
    sessionSearchCriteriaList[0].SetAttribute(MatchmakeAttributeIndex, MatchmakeAttributeValue);

    // Attribute 複数指定
    // (複数指定方法を示すコード例だが、上記の単体指定で検索にかかるので、実際の検索に利用されることは無い)
    sessionSearchCriteriaList[1].SetGameMode(MatchmakeGameMode);
    sessionSearchCriteriaList[1].SetMaxParticipantNumRange(MaxParticipantNum, MaxParticipantNum);
    const uint32_t arraySize = 5;
    uint32_t attributeArray[arraySize] = { 2, 4, 6, 11, 28 };
    sessionSearchCriteriaList[1].SetAttribute(MatchmakeAttributeIndex, attributeArray, arraySize);
#else
    uint32_t keywordIndex = 0;
    uint32_t rating = InitialRatingValue;

    uint32_t scoreSettingIndex = MinScoreSettingIndex;
    uint32_t disconnectionRate = 0;
    uint32_t violationRate = 0;
    uint32_t geoIpIndex = 0;
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
    nn::pia::common::FixedString<nn::pia::inet::CountryCodeLengthMax> countryCode(SAMPLE_STRING("JP"));
#else
    uint32_t countryCode = 1;  // 1: Japan
#endif
    uint8_t selectionPriority = 0;

    uint32_t scoreBasicParam = 0;

    // 作成するマッチメイクセッションの設定
    nn::pia::inet::NexCreateSessionSetting createSessionSetting;
    createSessionSetting.SetGameMode(MatchmakeGameMode);
    createSessionSetting.SetMaxParticipantNum(MaxParticipantNum);
    createSessionSetting.SetAttribute(MatchmakeAttributeIndex, MatchmakeAttributeValue);
    createSessionSetting.SetOpenSession(true);  // 作成されてすぐに参加可能状態にします。
    createSessionSetting.SetSelectionPriority(InitialSelectionPriority);

    // 検索条件の設定
    const uint32_t SearchSettingNum = nn::pia::inet::SearchCriteriaListSizeMax;
    nn::pia::inet::NexSessionSearchCriteria sessionSearchCriteriaList[SearchSettingNum];

    // Attribute 単体指定
    sessionSearchCriteriaList[0].SetGameMode(MatchmakeGameMode);
    sessionSearchCriteriaList[0].SetMaxParticipantNum(MaxParticipantNum);
    sessionSearchCriteriaList[0].SetAttribute(MatchmakeAttributeIndex, MatchmakeAttributeValue);

    // Attribute 複数指定
    // (複数指定方法を示すコード例だが、上記の単体指定で検索にかかるので、実際の検索に利用されることは無い)
    sessionSearchCriteriaList[1].SetGameMode(MatchmakeGameMode);
    sessionSearchCriteriaList[1].SetMaxParticipantNumRange(MaxParticipantNum, MaxParticipantNum);
    const uint32_t arraySize = 5;
    uint32_t attributeArray[arraySize] = { 2, 4, 6, 11, 28 };
    sessionSearchCriteriaList[1].SetAttribute(MatchmakeAttributeIndex, attributeArray, arraySize);
#endif

    uint32_t frameNo = 0;
    bool isExit = false;
    SampleSceneNo nextScene = SampleSceneNo_Matchmake;

    while (!isExit)
    {
        nns::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        // 下画面
        nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(false);
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nns::pia::sampleframework::ColorWhite);
        int line = 0;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[%s Demo] %d", ProgramNameString, frameNo);
        line += nns::pia::BasicFontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Scene: Matchmake.");
        line += nns::pia::BasicFontSize;
        line += nns::pia::BasicFontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Selection Method:");
        line += nns::pia::BasicFontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(10, line, "  %s.", SelectionMethodString[selectionMethodType]);
        line += nns::pia::BasicFontSize;
#if !ENABLE_LAN_MATCHMAKE
        if (selectionMethodType == SampleSessionSelectionMethod_BroadenRangeWithSelectionPriority)
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  Rating: %u", rating);
            line += nns::pia::BasicFontSize;
        }
        else if (selectionMethodType == SampleSessionSelectionMethod_ScoreBased)
        {
            line += nns::pia::BasicFontSize;
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "    Score Setting: %d %s", scoreSettingIndex, ScoreSettingString[scoreSettingIndex - MinScoreSettingIndex]);
            line += nns::pia::BasicFontSize;
            switch (scoreSettingIndex)
            {
            case SampleScoreSettingIndex_Rating:
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "    Rating: %u", rating);
                line += nns::pia::BasicFontSize;
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "    Geo IP: %u. %s", geoIpIndex, GeoIpLocationString[geoIpIndex]);
                line += nns::pia::BasicFontSize;
                break;
            case SampleScoreSettingIndex_DisconnectionRate:
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "    DisconnectionRate: %u", disconnectionRate);
                line += nns::pia::BasicFontSize;
                break;
            case SampleScoreSettingIndex_ViolationRate:
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "    ViolationRate: %u", violationRate);
                line += nns::pia::BasicFontSize;
                break;
            case SampleScoreSettingIndex_SessionDuration:
                // 表示は特になし
                break;
            case SampleScoreSettingIndex_SelectionPriority:
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "    SelectionPriority: %u", selectionPriority);
                line += nns::pia::BasicFontSize;
                break;
            case SampleScoreSettingIndex_GeoIp:
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "    Geo IP: %u. %s", geoIpIndex, GeoIpLocationString[geoIpIndex]);
                line += nns::pia::BasicFontSize;
                break;
            case SampleScoreSettingIndex_CountryCode:
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "    Country Code: %s", countryCode.GetCharArray());
#else
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "    Country Code: %u", countryCode);
#endif
                line += nns::pia::BasicFontSize;
                break;
            default:
                break;
            }
        }
#endif
        line = nns::pia::BasicLowerScreenHeight - (nns::pia::BasicFontSize * 8);
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Manual ]");
        line += nns::pia::BasicFontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  A: Start RandomMatchmake");
        line += nns::pia::BasicFontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  B: Logout");
        line += nns::pia::BasicFontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  X: Change Matchmake Keyword");
        line += nns::pia::BasicFontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  R: Change Selection Method");
        line += nns::pia::BasicFontSize;
#if !ENABLE_LAN_MATCHMAKE
        if (selectionMethodType != SampleSessionSelectionMethod_Random)
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  UP, DOWN: Parameter Adjustment");
            line += nns::pia::BasicFontSize;
        }
        if (selectionMethodType == SampleSessionSelectionMethod_ScoreBased)
        {
            nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  RIGHT, LEFT: Change Score Setting");
            line += nns::pia::BasicFontSize;
            if (scoreSettingIndex == SampleScoreSettingIndex_Rating)
            {
                nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "  L: Change GeoIp");
                line += nns::pia::BasicFontSize;
            }
        }
#endif
        nns::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

        ++frameNo;

        // 上画面
        nns::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(true);

        line = nns::pia::BasicUpperScreenHeight - nns::pia::BasicFontSize;
        nns::pia::sampleframework::SampleFramework::GetInstance()->DrawString(0, line, "[ Version ] %s", nns::pia::Basic_GetPiaVersionString());
        line += nns::pia::BasicFontSize;

        nns::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

#if !ENABLE_LAN_MATCHMAKE
        nns::pia::Nex_SchedulerDispatch();
#endif
        nn::pia::common::Scheduler::GetInstance()->Dispatch();

#if PIA_TEST_CODE
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonL))
        {
            // わざと検索条件が合致しないようにする
            attributeArray[4] = 50;
            sessionSearchCriteriaList[1].SetAttribute(MatchmakeAttributeIndex, attributeArray, arraySize);
        }
#endif
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonA))
        {
#if !ENABLE_LAN_MATCHMAKE
            if (selectionMethodType != SampleSessionSelectionMethod_ScoreBased)
            {
                createSessionSetting.SetAttribute(nn::pia::inet::SessionBroadenRangeAttributeIndex, rating);

                sessionSearchCriteriaList[0].SetAttribute(nn::pia::inet::SessionBroadenRangeAttributeIndex, rating);
                sessionSearchCriteriaList[1].SetAttribute(nn::pia::inet::SessionBroadenRangeAttributeIndex, rating);
            }
            else if (selectionMethodType == SampleSessionSelectionMethod_ScoreBased)
            {
                PIA_SAMPLE_PRINT("score Setting Index: %d\n", scoreSettingIndex);

                createSessionSetting.SetRatingValue(rating);
                createSessionSetting.SetDisconnectionRate(disconnectionRate);
                createSessionSetting.SetViolationRate(violationRate);
                createSessionSetting.SetSelectionPriority(selectionPriority);

                sessionSearchCriteriaList[0].SetScoreSettingIndex(scoreSettingIndex);
                sessionSearchCriteriaList[0].SetRatingValue(rating);
                sessionSearchCriteriaList[0].SetDisconnectionRate(disconnectionRate);
                sessionSearchCriteriaList[0].SetViolationRate(violationRate);

                sessionSearchCriteriaList[1].SetScoreSettingIndex(scoreSettingIndex);
                sessionSearchCriteriaList[1].SetRatingValue(rating);
                sessionSearchCriteriaList[1].SetDisconnectionRate(disconnectionRate);
                sessionSearchCriteriaList[1].SetViolationRate(violationRate);

                if (scoreSettingIndex == SampleScoreSettingIndex_GeoIp)
                {
                    nn::pia::common::InetAddress geoIpLocation(GeoIpLocationList[geoIpIndex]);

                    createSessionSetting.SetUseGeoIp(true);
                    createSessionSetting.SetOverrideIpAddress(geoIpLocation);
                    createSessionSetting.SetCountryCode(countryCode);

                    sessionSearchCriteriaList[0].SetUseGeoIp(true);
                    sessionSearchCriteriaList[0].SetOverrideIpAddress(geoIpLocation);
                    sessionSearchCriteriaList[0].SetCountryCode(countryCode);

                    sessionSearchCriteriaList[1].SetUseGeoIp(true);
                    sessionSearchCriteriaList[1].SetOverrideIpAddress(geoIpLocation);
                    sessionSearchCriteriaList[1].SetCountryCode(countryCode);
                }
                else if (scoreSettingIndex == SampleScoreSettingIndex_CountryCode)
                {
                    createSessionSetting.SetUseGeoIp(false);
                    createSessionSetting.SetCountryCode(countryCode);

                    sessionSearchCriteriaList[0].SetUseGeoIp(false);
                    sessionSearchCriteriaList[0].SetCountryCode(countryCode);

                    sessionSearchCriteriaList[1].SetUseGeoIp(false);
                    sessionSearchCriteriaList[1].SetCountryCode(countryCode);
                }
                else if (scoreSettingIndex == SampleScoreSettingIndex_Rating)
                {
                    nn::pia::common::InetAddress geoIpLocation(GeoIpLocationList[geoIpIndex]);

                    createSessionSetting.SetUseGeoIp(true);
                    createSessionSetting.SetOverrideIpAddress(geoIpLocation);

                    sessionSearchCriteriaList[0].SetUseGeoIp(true);
                    sessionSearchCriteriaList[0].SetOverrideIpAddress(geoIpLocation);

                    sessionSearchCriteriaList[1].SetUseGeoIp(true);
                    sessionSearchCriteriaList[1].SetOverrideIpAddress(geoIpLocation);
                }
            }
#endif

#if ENABLE_LAN_MATCHMAKE
            nn::pia::lan::LanJoinRandomSessionSetting joinRandomSessionSetting;
            joinRandomSessionSetting.SetCreateSessionSetting(&createSessionSetting);
            joinRandomSessionSetting.SetSessionSearchCriteria(sessionSearchCriteriaList, SearchSettingNum);
            bool scoreBasedEnabled = false;
#else
            nn::pia::inet::NexJoinRandomSessionSetting joinRandomSessionSetting;
            joinRandomSessionSetting.SetCreateSessionSetting(&createSessionSetting);
            joinRandomSessionSetting.SetSessionSearchCriteria(sessionSearchCriteriaList, SearchSettingNum);
            bool scoreBasedEnabled = (selectionMethodType == SampleSessionSelectionMethod_ScoreBased);
#endif
            result = nns::pia::Session_JoinRandomSession(&joinRandomSessionSetting, scoreBasedEnabled);
            if (result.IsSuccess())
            {
                nextScene = SampleSceneNo_Communicate;
                isExit = true;
                continue;
            }
            else if (result == nn::pia::ResultCancelled())
            {
                CleanupPia();
                nextScene = SampleSceneNo_Matchmake;
                isExit = true;
            }
            else
            {
                nn::pia::Result::ViewerType viewerType = result.GetViewerType();
                if (viewerType == nn::pia::Result::ViewerType_ShouldUse)
                {
                    nextScene = SampleSceneNo_ShowErrorViewer;
                }
                else
                {
                    nextScene = SampleSceneNo_ShowErrorApp;
                }

                SetCommunicationErrorType(SampleErrorType_Matchmake);
                SetNetworkErrorResult(result);
                isExit = true;
            }
        }

        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonB))
        {
            CleanupPia();
            Logout();
            nextScene = SampleSceneNo_Preparation;
            isExit = true;
        }
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonX))
        {
#if !ENABLE_LAN_MATCHMAKE
            keywordIndex = (keywordIndex + 1) % MaxMatchmakeKeyword;
            createSessionSetting.SetSessionMatchmakeKeyword(MatchmakeKeywordString[keywordIndex]);
            for (uint32_t i = 0; i < SearchSettingNum; ++i)
            {
                sessionSearchCriteriaList[i].SetSessionMatchmakeKeyword(MatchmakeKeywordString[keywordIndex]);
            }
#endif
        }
        if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonR))
        {
            selectionMethodType = static_cast<SampleSessionSelectMethod>((selectionMethodType + 1) % SampleSessionSelectionMethod_Max);
#if !ENABLE_LAN_MATCHMAKE
            switch (selectionMethodType)
            {
            case SampleSessionSelectionMethod_Random:
                rating = InitialRatingValue;
                sessionSearchCriteriaList[0].SetRandomSessionSelectionMethod(nn::pia::inet::SelectionMethod_Random);
                sessionSearchCriteriaList[1].SetRandomSessionSelectionMethod(nn::pia::inet::SelectionMethod_Random);

                scoreBasicParam = rating;
                break;
            case SampleSessionSelectionMethod_BroadenRangeWithSelectionPriority:
                rating = InitialRatingValue;
                sessionSearchCriteriaList[0].SetRandomSessionSelectionMethod(nn::pia::inet::SelectionMethod_BroadenRangeWithSelectionPriority);
                sessionSearchCriteriaList[1].SetRandomSessionSelectionMethod(nn::pia::inet::SelectionMethod_BroadenRangeWithSelectionPriority);

                scoreBasicParam = rating;
                break;
            case SampleSessionSelectionMethod_ScoreBased:
                scoreSettingIndex = SampleScoreSettingIndex_Rating;
                rating = InitialRatingValue;
                disconnectionRate = 0;
                violationRate = 0;
                geoIpIndex = 0;
                selectionPriority = 0;
                sessionSearchCriteriaList[0].SetRandomSessionSelectionMethod(nn::pia::inet::SelectionMethod_ScoreBased);
                sessionSearchCriteriaList[1].SetRandomSessionSelectionMethod(nn::pia::inet::SelectionMethod_ScoreBased);

                scoreBasicParam = rating;
                break;
            default:
                break;
            }
#endif
        }
#if !ENABLE_LAN_MATCHMAKE
        if (selectionMethodType == SampleSessionSelectionMethod_BroadenRangeWithSelectionPriority)
        {
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                rating = rating + 100;
                if (rating > RatingValueMax)
                {
                    rating = RatingValueMax;
                }
                scoreBasicParam = rating;
            }

            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                rating = rating - 100;
                if (rating < RatingValueMin)
                {
                    rating = RatingValueMin;
#if PIA_TEST_CODE
                    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(0, rating);
#endif
                }
                scoreBasicParam = rating;
            }
        }
        else if (selectionMethodType == SampleSessionSelectionMethod_ScoreBased)
        {
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonRight))
            {
                ++scoreSettingIndex;
                if (scoreSettingIndex > MaxScoreSettingIndex)
                {
                    scoreSettingIndex = MinScoreSettingIndex;
                }
            }

            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonLeft))
            {
                --scoreSettingIndex;
                if (scoreSettingIndex < MinScoreSettingIndex)
                {
                    scoreSettingIndex = MaxScoreSettingIndex;
                }
            }

            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonUp))
            {
                switch (scoreSettingIndex)
                {
                case SampleScoreSettingIndex_Rating:
                    rating = rating + 100;
                    if (rating > RatingValueMax)
                    {
                        rating = RatingValueMax;
                    }
                    break;
                case SampleScoreSettingIndex_DisconnectionRate:
                    // 0-1%, 1-3%, 3-5%, 5-100% の 4段階で評価しているため 2% 刻みで増加、8% になったら一周するようにしている
                    disconnectionRate = (disconnectionRate + 2) % 8;
                    break;
                case SampleScoreSettingIndex_ViolationRate:
                    // 0-1%, 1-3%, 3-5%, 5-100% の 4段階で評価しているため 2% 刻みで増加、8% になったら一周するようにしている
                    violationRate = (violationRate + 2) % 8;
                    break;
                case SampleScoreSettingIndex_SelectionPriority:
                    selectionPriority = (selectionPriority + 2) % 8;
                    break;
                case SampleScoreSettingIndex_GeoIp:
                    geoIpIndex = (geoIpIndex + 1) % MaxGeoIpLocation;
                    break;
                case SampleScoreSettingIndex_CountryCode:
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
                    if (countryCode == nn::pia::common::String(SAMPLE_STRING("JP")))
                    {
                        countryCode.Copy(nn::pia::common::String(SAMPLE_STRING("US")));
                    }
                    else
                    {
                        countryCode.Copy(nn::pia::common::String(SAMPLE_STRING("JP")));
                    }
#else
                    countryCode = 50 - countryCode;  // 49(US) <-> 1(JP)
#endif
                    break;
                default:
                    break;
                }
            }

            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonDown))
            {
                switch (scoreSettingIndex)
                {
                case SampleScoreSettingIndex_Rating:
                    rating = rating - 100;
                    if (rating < RatingValueMin)
                    {
                        rating = RatingValueMin;
#if PIA_TEST_CODE
                        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(0, rating);
#endif
                    }
                    break;
                case SampleScoreSettingIndex_DisconnectionRate:
                    disconnectionRate = (disconnectionRate + 6) % 8;
                    break;
                case SampleScoreSettingIndex_ViolationRate:
                    violationRate = (violationRate + 6) % 8;
                    break;
                case SampleScoreSettingIndex_GeoIp:
                    geoIpIndex = (geoIpIndex + MaxGeoIpLocation - 1) % MaxGeoIpLocation;
                    break;
                case SampleScoreSettingIndex_SelectionPriority:
                    selectionPriority = (selectionPriority + 6) % 8;
                    break;
                case SampleScoreSettingIndex_CountryCode:
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
                    if (countryCode == nn::pia::common::String(SAMPLE_STRING("JP")))
                    {
                        countryCode.Copy(nn::pia::common::String(SAMPLE_STRING("US")));
                    }
                    else
                    {
                        countryCode.Copy(nn::pia::common::String(SAMPLE_STRING("JP")));
                    }
#else
                    countryCode = 50 - countryCode;  // 49(US) <-> 1(JP)
#endif
                    break;
                default:
                    break;
                }
            }

            // SampleScoreSettingIndex_Rating のときのみ GeoIp による距離が 5000(km) から 40000(km) 合ったらスコアを減算して 0 としている
            if (nns::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nns::pia::sampleframework::ButtonL))
            {
                if (scoreSettingIndex == SampleScoreSettingIndex_Rating)
                {
                    geoIpIndex = (geoIpIndex + MaxGeoIpLocation - 1) % ((MaxGeoIpLocation - 1) * 2);
                }
            }

            switch (scoreSettingIndex)
            {
            case SampleScoreSettingIndex_Rating:
                scoreBasicParam = rating;
                break;
            case SampleScoreSettingIndex_DisconnectionRate:
                scoreBasicParam = disconnectionRate;
                break;
            case SampleScoreSettingIndex_ViolationRate:
                scoreBasicParam = violationRate;
                break;
            case SampleScoreSettingIndex_GeoIp:
                scoreBasicParam = geoIpIndex;
                break;
            case SampleScoreSettingIndex_SelectionPriority:
                scoreBasicParam = selectionPriority;
                break;
            case SampleScoreSettingIndex_CountryCode:
#if NN_PIA_ENABLE_ISO_COUNTRY_CODE
                if (countryCode == nn::pia::common::String(SAMPLE_STRING("JP")))
                {
                    scoreBasicParam = 1;
                }
                else
                {
                    scoreBasicParam = 49;
                }
#else
                scoreBasicParam = countryCode;
#endif
                break;
            default:
                break;
            }
        }

#if PIA_TEST_CODE
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ProgramParamMatchmakeSessionSelectionMethod, selectionMethodType + (scoreSettingIndex * 100));
        nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ProgramParamMatchmakeSessionScoreBasedParam, scoreBasicParam);
#else
        NN_PIA_UNUSED(scoreBasicParam);
#endif
#endif // !ENABLE_LAN_MATCHMAKE

        // 接続状態チェック
        if (!nns::pia::Net_IsConnected())
        {
            PIA_SAMPLE_PRINT("nns::pia::Net_IsConnected Disconnected \n");
#if NN_PIA_ENABLE_CACHEDPRINT
            nn::pia::common::CachedPrint::GetInstance()->Flush();
#endif // NN_PIA_ENABLE_CACHEDPRINT
            nextScene = SampleSceneNo_ShowErrorViewer;
            SetNetworkErrorResult(nn::pia::ResultNetworkConnectionIsLost());
            SetCommunicationErrorType(SampleErrorType_ConnectionLost);
            isExit = true;
        }
#if !ENABLE_LAN_MATCHMAKE
        if (nns::pia::Nex_IsNgsConnected().IsFailure())
        {
            nextScene = SampleSceneNo_ShowErrorViewer;
            SetNetworkErrorResult(nns::pia::Nex_IsNgsConnected());
            SetCommunicationErrorType(SampleErrorType_ConnectionLost);
            isExit = true;
        }
#endif
    }

    return nextScene;
}
