﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include <nn/pia/pia_Version.h>

#include "SampleConfig.h"


namespace
{
// Pia に与えるメモリ領域です。 4バイト整合されていないといけません。
// このサンプルでは十分余裕を持たせた数値にしています。
PIA_ATTRIBUTE_ALIGN(8) uint8_t s_PiaMemoryBlock[MemoryBlockSizeForPia];

// true にするとPiaライブラリのトレースフラグを多数有効にした場合の処理速度への影響を抑えられます。
// ただし、他のライブラリ(NEXなど)のログとの順序関係が入れ替わる可能性があります
const bool IsUsedCachePrint = true;

}  // end of anonymous namespace

#if NN_PIA_CTR
// 通信方式に合わせて電波強度を取得する関数
uint32_t GetInternetLinkLevel();
#endif

// 電波強度を取得するための処理をまとめた関数
uint32_t GetLinkLevel()
{
#if NN_PIA_CTR
    // 通信方式に合わせて電波強度を取得し、0-3 の数値として返します
    return GetInternetLinkLevel();
#elif NN_PIA_WIN || NN_PIA_NINTENDOSDK
    // 常に 0 とします
    return 0;
#elif NN_PIA_CAFE
    // 電波強度を取得する方法が無いため、常に 0 とします
    return 0;
#else
#error "illegal platform."
#endif
}

PIA_SAMPLE_MAIN_FUNC()
{
    PIA_SAMPLE_PRINT("Sample program Start.\n");

    nn::pia::Result result;

    // Pia のサンプルプログラム用描画フレームワークの初期化
    nns::pia::sampleframework::SampleFramework::CreateInstance();
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetFramePerSecond(FramePerSecond);
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetClearColor(nns::pia::sampleframework::Color(0.0f, 0.0f, 0.2f, 1.0f));
    nns::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(SampleSceneNo_Preparation, false);

    nns::pia::Basic_SetProgramTitle(ProgramNameString);

    while (1)
    {
        SampleSceneNo nextSceneNo = SampleSceneNo_ShowErrorApp;

        // PiaCommon モジュールで利用する機能のセットアップ
        result = nns::pia::Common_InitializePiaCommon(
            s_PiaMemoryBlock,
            MemoryBlockSizeForPia,
            nn::pia::common::TraceFlagCommon |
#if PIA_TEST_CODE
                nn::pia::common::TraceFlagCommonSocket |
#endif
                nn::pia::common::TraceFlagTransport |
#if PIA_TEST_CODE
                nn::pia::common::TraceFlagTransportPacketSend |
                nn::pia::common::TraceFlagTransportPacketReceive |
#endif
                nn::pia::common::TraceFlagSession |
#if PIA_TEST_CODE
                nn::pia::common::TraceFlagSessionConnect |
                nn::pia::common::TraceFlagSessionLeave |
                nn::pia::common::TraceFlagSessionUpdate |
                nn::pia::common::TraceFlagSessionMigration |
#endif
                nn::pia::common::TraceFlagInet |
#if PIA_TEST_CODE
                nn::pia::common::TraceFlagInetNat |
                nn::pia::common::TraceFlagInetNatDetail |
                nn::pia::common::TraceFlagInetNatProbe |
#endif
                nn::pia::common::TraceFlagLan |
                nn::pia::common::TraceFlagAlways,
            IsUsedCachePrint,
            true);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

        // このサンプルでは機能ごとの処理を明確に分けるため、メインスレッドの処理をいくつかのシーンに分割しています。
        bool isExit = false;

        nextSceneNo = SampleSceneNo_Preparation;
        //bool isSessionHost = false;

        /* ゲームループ */
        while (isExit == false)
        {
            switch (nextSceneNo)
            {
            case SampleSceneNo_Preparation:
            {
                if (InitializePia())
                {
                    nextSceneNo = SampleSceneNo_Matchmake;
                }
                else
                {
                    nextSceneNo = SampleSceneNo_ShowErrorViewer;
                }
            }
            break;
            case SampleSceneNo_Matchmake:
            {
                nextSceneNo = SceneMatchmake();
            }
            break;
            case SampleSceneNo_Communicate:
            {
                nextSceneNo = SceneCommunication();
            }
            break;
            case SampleSceneNo_ShowErrorApp:
            {
                nextSceneNo = SceneShowErrorByApp();
            }
            break;
            case SampleSceneNo_ShowErrorViewer:
            {
                nextSceneNo = SceneShowErrorByViewr();
            }
            break;
            case SampleSceneNo_Reset:
            {
                isExit = true;
            }
            break;
            }
        }

        /* PiaCommon 関連の終了処理 */

        nns::pia::Common_FinalizePiaCommon();
        /* end of PiaCommon 関連の終了処理 */
    }
    //ここには到達しません
    //nns::pia::sampleframework::SampleFramework::DestroyInstance();
    //PIA_SAMPLE_PRINT("Sample program end.\n");

}  // end of nnMain()
