﻿/*--------------------------------------------------------------------------------*
  Copyright (C)Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#include "SampleConfig.h"
#include "ControllerData.h"

#include "SampleLib/SampleLib.h"
#include <string.h>  // for memset()

// コントローラデータのクリア
void ControllerData::Clear()
{
    memset(&m_KeyData, 0, sizeof(m_KeyData));
    memset(&m_TouchData, 0, sizeof(m_TouchData));
    memset(&m_StickData, 0, sizeof(m_StickData));

#if PIA_TEST_CODE
    memset(&m_SyncCheckData, 0, sizeof(m_SyncCheckData));
#endif
}


// フレームワークのボタンIDをキーに変換
uint32_t ControllerData::KeyToButton(Key key)
{
    switch (key)
    {
    case Key_Up:
        return nns::pia::sampleframework::ButtonUp;
    case Key_Down:
        return nns::pia::sampleframework::ButtonDown;
    case Key_Left:
        return nns::pia::sampleframework::ButtonLeft;
    case Key_Right:
        return nns::pia::sampleframework::ButtonRight;
    case Key_A:
        return nns::pia::sampleframework::ButtonA;
    case Key_B:
        return nns::pia::sampleframework::ButtonB;
    case Key_X:
        return nns::pia::sampleframework::ButtonX;
    case Key_Y:
        return nns::pia::sampleframework::ButtonY;
    default:
        return 0;
    }
}


// 送信するコントローラデータへのアクセス
const void* ControllerReader::GetData(DataId dataId) const
{
    switch (dataId)
    {
    case DataId_Key:
        return &m_KeyData;
    case DataId_Touch:
        return &m_TouchData;
    case DataId_Stick:
        return &m_StickData;

#if PIA_TEST_CODE
    case DataId_SyncCheck:
        return &m_SyncCheckData;
#endif

    default:
        return NULL;
    }
}


// フレームワークからコントローラデータを読み込んで送信データを作る
void ControllerReader::Read()
{
    Clear();

    nns::pia::sampleframework::SampleFramework* pSampleFramework = nns::pia::sampleframework::SampleFramework::GetInstance();

    for (uint32_t i = 0; i < Key_Max; ++i)
    {
        uint32_t button = KeyToButton(static_cast<Key>(i));
        uint32_t flag = 0x1 << i;
        if (pSampleFramework->IsHold(button))
        {
            m_KeyData.holdFlag |= flag;
        }
        if (pSampleFramework->IsTrigger(button))
        {
            m_KeyData.triggerFlag |= flag;
        }
        if (pSampleFramework->IsRelease(button))
        {
            m_KeyData.releaseFlag |= flag;
        }
    }

    if (pSampleFramework->IsPenTouched())
    {
        m_TouchData.touch = 1;
        pSampleFramework->GetLatestPenPosition(&m_TouchData.x, &m_TouchData.y);
    }

    pSampleFramework->GetAnalog(&m_StickData.x, &m_StickData.y);
}

#if PIA_TEST_CODE
// 同期ずれ検知用データをセット
void ControllerReader::SetSyncCheckKeyData(uint32_t n, uint32_t holdFlag, uint32_t triggerFlag, uint32_t releaseFlag)
{
    m_SyncCheckData.keyDataArray[n].holdFlag = holdFlag;
    m_SyncCheckData.keyDataArray[n].triggerFlag = triggerFlag;
    m_SyncCheckData.keyDataArray[n].releaseFlag = releaseFlag;
}

void ControllerReader::SetSyncCheckTouchData(uint32_t n, uint16_t x, uint16_t y, uint8_t touch)
{
    m_SyncCheckData.touchDataArray[n].x = x;
    m_SyncCheckData.touchDataArray[n].y = y;
    m_SyncCheckData.touchDataArray[n].touch = touch;
}

void ControllerReader::SetSyncCheckStickData(uint32_t n, float x, float y)
{
    m_SyncCheckData.stickDataArray[n].x = x;
    m_SyncCheckData.stickDataArray[n].y = y;
}
#endif

// 受信したコントローラデータの保存先を取得
void* ControllerAccessor::GetDataPtr(DataId dataId)
{
    switch (dataId)
    {
    case DataId_Key:
        return &m_KeyData;
    case DataId_Touch:
        return &m_TouchData;
    case DataId_Stick:
        return &m_StickData;

#if PIA_TEST_CODE
    case DataId_SyncCheck:
        return &m_SyncCheckData;
#endif

    default:
        return NULL;
    }
}
